#ifndef _ALTICONFIGURATION_H_
#define _ALTICONFIGURATION_H_

//******************************************************************************
// file: AltiConfiguration.h
// desc: configuration for the ALTI module
// auth: 28-NOV-2017 P. Kuzmanovic
//*****************************************************************************

#include "ALTI/AltiModule.h"

#include <string>
#include <iostream>

namespace LVL1 {

class AltiConfiguration {

  public:

    typedef AltiModule::ECR_GENERATION ECR_GENERATION;

    // public constant
    static const unsigned int   STRING_LENGTH           = 1024;

    // constructor and destructor
    AltiConfiguration();
    AltiConfiguration(const std::string &);
   ~AltiConfiguration();
    AltiConfiguration &operator=(const AltiConfiguration &);

    // methods
    void clearFileNames();
    void setDefaultFileNames();
    void addFileNamePrefix(const std::string &);
    void config(const bool);
    void print(std::ostream & = std::cout) const;
    int read(const std::string &);
    int write(const std::string &) const;
  
    void readSWXIndividualToBundleTTYP();

  public:

    bool                                          clk_config;
    AltiModule::CLK_TYPE                          clk_select;
    AltiModule::CLK_TYPE                          clk_jc_select;
    std::string                                   clk_jc_file;
    int                                           clk_phase_shift;
  
    bool                                          sig_config;
    bool                                          sig_fp_bgo2enable;
    bool                                          sig_fp_bgo3enable;
    AltiModule::SIGNAL_SOURCE                     sig_swx[AltiModule::SIGNAL_NUMBER_ROUTED][AltiModule::SIGNAL_DESTINATION_NUMBER];
    AltiModule::SIGNAL_SOURCE                     sig_swx_ttyp[AltiModule::SIGNAL_DESTINATION_NUMBER];
  
    bool                                          sig_eqz_ctp_in_enable;
    bool                                          sig_eqz_alti_in_enable;
    AltiModule::EQUALIZER_CONFIG                  sig_eqz_ctp_in_mode;
    AltiModule::EQUALIZER_CONFIG                  sig_eqz_alti_in_mode;
   
    std::vector<bool>                             sig_io_sync_enable;
    std::vector<unsigned int>                     sig_io_sync_phase;
    std::vector<bool>                             sig_io_shape_enable;
  
    bool                                          bsy_config;
    std::vector<bool>                             bsy_in_select;
    std::vector<AltiModule::BUSY_SOURCE>          bsy_out_select;
    bool                                          bsy_data_vme;
    bool                                          bsy_masking_l1a_enable;
    AltiModule::BUSY_LEVEL                        bsy_level;

    bool                                          crq_config;
    std::vector<AltiModule::CALREQ_INPUT>         crq_in_select;
    std::vector<AltiModule::CALREQ_SOURCE>        crq_out_select[AltiModule::CALREQ_OUTPUT_NUMBER];
    unsigned int                                  crq_data_vme;

    bool                                          mem_config;
    bool                                          mem_pg_enable;
    std::vector<bool>                             mem_pg_config;
    bool                                          mem_pg_repeat;
    unsigned int                                  mem_pg_start_addr;
    unsigned int                                  mem_pg_stop_addr;
    std::string                                   mem_pg_file;
    std::vector<bool>                             mem_snap_masked;

    bool                                          ttc_config;
    bool                                          ttc_tx_enable[AltiModule::TRANSMITTER_NUMBER];
    std::vector<bool>                             ttc_l1a_source;
    std::vector<unsigned int>                     ttc_bgo_inhibit_width;
    std::vector<unsigned int>                     ttc_bgo_inhibit_delay;
    unsigned int                                  ttc_ttyp_delay;
    unsigned int                                  ttc_ttyp_addr;
    unsigned int                                  ttc_ttyp_subaddr;
    AltiModule::TTC_FRAME_TYPE                    ttc_ttyp_frame_type;
    AltiModule::TTC_ADDRESS_SPACE                 ttc_ttyp_address_space;
    std::vector<AltiModule::TTC_BGO_MODE>         ttc_bgo_mode;
    std::vector<bool>                             ttc_bgo_busy_gate;
    std::vector<bool>                             ttc_fifo_retransmit;


    bool                                          cnt_config;
    bool                                          cnt_enable[AltiModule::CNT_NUMBER];
    std::string                                   cnt_ttyp_lut_file;

    bool                                          ctp_config;
    ECR_GENERATION                                ctp_ecr_generation;


    // private method;
 private:

    // private method
    int readNextRecord(std::istream&, std::string&, std::string&);

    // private utility methods
    static std::string bool2string(const bool);
    static std::string phase2string(const unsigned);

};      // class AltiConfiguration

}       // namespace LVL1

#endif  // _ALTICONFIGURATION_H_
