#include <AccessManager/xacml/impl/RMResource.h>

namespace daq
{

namespace am
{

// the resource category is "ResourceManager"
#define VALUE_RESOURCE_CATEGORY		"ResourceManager"

// the resource id is "rm"
#define VALUE_RESOURCE_ID			"rm"

// the rm partition name identifier;
// It has ":partition" attached to the resource type identifier.
#define IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME 	"urn:oasis:names:tc:xacml:1.0:resource:resource-type:partition"

// the RM possible actions: any
const RMAction RMResource::ACTION_ANY(RMAction::getValueForActionANY());

RMResource::RMResource(const std::string& partitionName, const RMAction& action):
	Resource(){

	//TYPES INITIALIZATION

	//VALUES INITIALIZATION
	initValues();

	// the partition name resource type
	if (partitionName.length() > 0){
		addTypes(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME);
		addValue(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME, partitionName);
	}

	//ACTIONS INITIALIZATION
	//initialize the actions for the resource id
	addAction(getResourceId(), &action);

	// set the description attribute
	this->description = "RM Resource";
}

/*
 * Populate the list and maps from parent class with known values;
 */
void RMResource::initValues(){
	// set the RM resource category value
	addValue(getResourceCategory(), VALUE_RESOURCE_CATEGORY);
	// set the RM resource id value
	addValue(getResourceId(), VALUE_RESOURCE_ID);
}

RMResource::~RMResource(){

}

const RMResource *RMResource::getInstanceForActionANY(const std::string& partitionName) {
	return new RMResource(partitionName, RMResource::ACTION_ANY);
}

std::unique_ptr<const RMResource> RMResource::getAutoptrInstanceForActionANY(const std::string& partitionName) {
	return std::unique_ptr<const RMResource>(RMResource::getInstanceForActionANY(partitionName));
}



}

}
