#include <AccessManager/xacml/impl/RunControlResource.h>

// the resource category is "RunControl"
#define VALUE_RESOURCE_CATEGORY                 "RunControl"

// the resource id is "rc"
#define VALUE_RESOURCE_ID                       "rc"

/**
 * the run control's command name resource identifier;
 * This is command to be executed by the run controller
 * It has ":command" attached to the resource type identifier.
 */
#define IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME   "urn:oasis:names:tc:xacml:1.0:resource:resource-type:command"

/**
 * the run control's partition name resource identifier;
 * This is the partition affected by the run control command execution
 * It has ":partition" attached to the resource type identifier.
 */
#define IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME "urn:oasis:names:tc:xacml:1.0:resource:resource-type:partition"

namespace daq
{

  namespace am
  {

    // the Run Control action
    const RunControlAction RunControlResource::rcAction;

    RunControlResource::RunControlResource(const std::string& commandName, const std::string& partitionName) :
        Resource()
    {

      //TYPES INITIALIZATION
      // add the command name and partition name
      // ... later when the values are checked

      //VALUES INITIALIZATION
      initValues();

      if (commandName.length() > 0)
        {
          addTypes(IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME);
          addValue(IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME, commandName);
        }

      if (partitionName.length() > 0)
        {
          addTypes(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME);
          addValue(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME, partitionName);
        }

      //ACTIONS INITIALIZATION
      //initialize the actions for the resource id
      addAction(getResourceId(), &rcAction);

      // set the description attribute
      this->description = "Run Control Resource";
    }

    RunControlResource::~RunControlResource()
    {

    }

    /*
     * Populate the list and maps from parent class with known values;
     */
    void
    RunControlResource::initValues()
    {
      // set the Run Control category value
      addValue(getResourceCategory(), VALUE_RESOURCE_CATEGORY);
      addValue(getResourceId(), VALUE_RESOURCE_ID);
    }

    /*
     * The string value of TriggerCommander actions.
     */
    const std::string RunControlResource::ACTION_VALUES[ACTIONS_COUNT] =
      { "PUBLISH", "PUBLISHSTATS", "PUBLISHSTATISTICS", "USER", "USERBROADCAST" };

  }

}
