#include <AccessManager/xacml/impl/TriggerCommanderResource.h>

// the resource category is "TriggerCommander"
#define VALUE_RESOURCE_CATEGORY                 "TriggerCommander"

// the resource id is "rc"
#define VALUE_RESOURCE_ID                       "tc"

    /**
     * the trigger commander's command name resource identifier;
     * This is command to be executed by the CHIP
     * It has ":command" attached to the resource type identifier.
     */
#define IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME   "urn:oasis:names:tc:xacml:1.0:resource:resource-type:command"

    /**
     * the run control's partition name resource identifier;
     * This is the partition affected by the trigger commander command execution
     * It has ":partition" attached to the resource type identifier.
     */
#define IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME "urn:oasis:names:tc:xacml:1.0:resource:resource-type:partition"

namespace daq
{

  namespace am
  {

    // the Trigger Commander action
    const TriggerCommanderAction TriggerCommanderResource::tcAction;

    TriggerCommanderResource::TriggerCommanderResource(const std::string& commandName, const std::string& partitionName) :
        Resource()
    {

      //TYPES INITIALIZATION
      // add the command name and partition name
      // ... later when the values are checked

      //VALUES INITIALIZATION
      initValues();

      if (commandName.length() > 0)
        {
          addTypes(IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME);
          addValue(IDENTIFIER_RESOURCE_TYPE_COMMAND_NAME, commandName);
        }

      if (partitionName.length() > 0)
        {
          addTypes(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME);
          addValue(IDENTIFIER_RESOURCE_TYPE_PARTITION_NAME, partitionName);
        }

      //ACTIONS INITIALIZATION
      //initialize the actions for the resource id
      addAction(getResourceId(), &tcAction);

      // set the description attribute
      this->description = "Trigger Commander Resource";
    }

    TriggerCommanderResource::~TriggerCommanderResource()
    {

    }

    /*
     * Populate the list and maps from parent class with known values;
     */
    void
    TriggerCommanderResource::initValues()
    {
      // set the Run Control category value
      addValue(getResourceCategory(), VALUE_RESOURCE_CATEGORY);
      addValue(getResourceId(), VALUE_RESOURCE_ID);
    }

    /*
     * The string value of TriggerCommander actions.
     */
    const std::string TriggerCommanderResource::ACTION_VALUES[ACTIONS_COUNT] =
      { "HOLD", "RESUME", "SETPRESCALES", "SETL1PRESCALES", "SETHLTPRESCALES", "SETBUNCHGROUP", "SETLUMIBLOCKLEN", "SETMINLUMIBLOCKLEN", "INCREASELUMIBLOCK", "SETCONDITIONSUPDATE" };

  }

}
