package autoprescaleeditor;

//pc needed for CtpControllerL1PSKey from L1CTISData.jar
import TTCInfo.CtpControllerL1PSKey;
//import L1CT.L1CTISData.*;
import TTCInfo.TrigConfSmKeyNamed;
import TTCInfo.TrigConfSmKey;
import TTCInfo.AutoPrescaleCTPNamed;
import TTCInfo.AutoPrescaleCTP;
import Igui.IguiLogger;
import TRIGGER.AutoPrescalerServant;
import TRIGGER.TriggerCommander;
import triggertool.L1Records.CutToValueConverter;
import triggerdb.Entities.L1.L1Prescale;
import L1CTIPC.PrescaleWriterServant;
import config.Configuration;
import dal.MasterTrigger;
import dal.RunControlApplicationBase;
import daq.rc.CommandSender;
import ipc.InvalidPartitionException;
import is.NamedInfo;
import is.Repository;
import java.io.*;
import java.sql.SQLException;
import java.util.*;

import java.util.concurrent.atomic.AtomicBoolean;

//import TrigConfL1PsKeyNamed
import org.apache.commons.cli.*;

import triggerdb.Entities.L1.L1Prescale;

@SuppressWarnings("serial")
    class TestISIssue extends ers.Issue {
	TestISIssue(final String message) { super(message); }
    }

@SuppressWarnings("serial")
class Issue extends ers.Issue {
    Issue(String message) { super(message); }
}

@SuppressWarnings("serial")
class LBIssue extends Issue {
    LBIssue(String message) { super(message); }
}

@SuppressWarnings("serial")
class ISIssue extends Issue {
    ISIssue(String message) { super(message); }
}


@SuppressWarnings("serial")
class PSIssue extends Issue {
    PSIssue(String message) { super(message); }
}

@SuppressWarnings("serial")
class ConfigIssue extends Issue {
    ConfigIssue(String message) { super(message); }
}

@SuppressWarnings("serial")
class ControlIssue extends Issue {
    ControlIssue(String message) { super(message); }
}

@SuppressWarnings("serial")
class SimIssue extends Issue {
    SimIssue(String message) { super(message); }
}

public class AutoPrescaleEditor {
    
    private enum READYSTATE
    { STANDBY, PHYSICS, UNKNOWN }
    
    // IS listener to CTP update requests
    private APEAbsISListener<AutoPrescaleCTP> listener = null;

    // IS name of the CTP update request object
    private String isname = null;

    // elog pw from file
    private ELog elog;

    // boolean variable indicating if confirmation dialogs are enabled
    private boolean confirm;
    
    // Helper to safe prescale sets to the TriggerDB
    private L1PrescaleSaver prescaleSaver = null;

    // partition name and object
    private String partitionName = null;
    private ipc.Partition partition = null;

    private TriggerCommander triggerCommander = null;
    private int base_psk = 0;

    // number of parallel requests
    private int update_request = 0;
    private int reset_request = 0;

    final IguiInterface iguiInterface;
    
    // test mode simulates the CTP response by updating the current l1 key in IS
    private boolean testMode = false;
    public void setTestMode(boolean testMode) {
        this.testMode = testMode;
        if(this.testMode) {
            warning(new Issue("Setting testmode to " + this.testMode + ". This is ONLY for TESTING, inform Trigger Expert if you see this !"));
        } else {
            info(new Issue("Setting testmode to " + this.testMode));
        }
    }

    private int uploadDelay = 0;
    public void setUploadDelay(int uploadDelay) {
        this.uploadDelay = uploadDelay;
        if(this.uploadDelay>0) {
            warning(new Issue("Setting upload delay to " + this.uploadDelay + ". This is ONLY for TESTING, inform Trigger Expert if you see this !"));
        } else {
            info(new Issue("Setting upload delay to " + this.uploadDelay));
        }
    }

    private String blockerName = "";
    private final AtomicBoolean isBlocked = new AtomicBoolean(false);
    private synchronized void blockMe(final String caller) { isBlocked.set(true); blockerName = caller; }
    private synchronized void unblockMe() { isBlocked.set(false); blockerName = ""; }
    private synchronized boolean isblocked() { return isBlocked.get(); }
    private synchronized String blocker() { return blockerName; }

    // counts requests
    private synchronized void new_update_request() { update_request++; }
    private synchronized void end_update_request() { update_request--; }
    private synchronized int number_update_requests() { return update_request; }

    private synchronized void new_reset_request() { reset_request++; }
    private synchronized void end_reset_request() { reset_request--; }
    private synchronized int number_reset_requests() { return reset_request; }

    private AutoPrescalerServant servant = null;

    private PrescaleWriterServant prescaleServant = null;
    
    /**
     * 
     * @param partitionName
     * @param isname
     * @param triggerdb
     * @param configfile
     * @param elogpwfile
     * @param smk
     * @param confirm
     */
    public AutoPrescaleEditor( final String servantName, 
                               final String partitionName,
                               final String isname,
                               final String triggerdb,
                               final String configfile,
                               final String elogpwfile,
                               final int smk,
                               final boolean confirm) throws InvalidPartitionException, SQLException, InterruptedException
    {
        this.partitionName = partitionName;
        this.partition     = new ipc.Partition(partitionName);
        this.isname        = isname;
        this.confirm       = confirm;

        info(new Issue("Trigger Level 1 Autoprescaler started in non-interactive mode\nPartition: " + partitionName 
                       + " \nServant: " + servantName
                       + "\nIS name: " + isname
                       + "\nItem config file: " + configfile
                       + "\nversion: " + AutoPrescaleEditor.class.getPackage().getImplementationVersion()
                       ));

        this.servant = new AutoPrescalerServant(partition, servantName, new AutoPrescalerServant.AutoPrescalerOperations() {
            @Override
            public void hold(String caller) throws Exception {
                info(new ControlIssue("Holding Autoprescaler on request from " + caller));
                blockMe(caller);
            }
            @Override
            public void resume(String caller) throws Exception {
                info(new ControlIssue( "Resuming Autoprescaler on request from " + caller));
                unblockMe();
            }
        });

	// Add the new PSKWriter servant:
	this.prescaleServant = new PrescaleWriterServant(partition, servantName+"Writer", new PrescaleWriterServant.PrescaleWriterOperations() {
            @Override
	    public L1CTIPC.PrescaleReturn requestPrescale(final L1CTIPC.PrescaleRequest request) throws L1CTIPC.InvalidName,L1CTIPC.ObsoletePsk {

		// Just a wrapper for an internal function (!!)
		try{
		    L1CTIPC.PrescaleReturn myReturn = requestNewPrescaleKey(request);
		    IguiLogger.info("Returning: no exceptions yet");
		    IguiLogger.info("Returning: no exceptions yet"+myReturn.success);
		    IguiLogger.info("Returning: no exceptions yet"+myReturn.psk);
		    return new L1CTIPC.PrescaleReturn(myReturn.success, myReturn.psk);
		}
		catch(SQLException ex)
		    {
			// Database problem: return a failure
			return new L1CTIPC.PrescaleReturn(false, -1);
		    }
		catch(InterruptedException ex)
		    {
			// SMK problem: return a failure
			return new L1CTIPC.PrescaleReturn(false, -1);
		    }

            }
	    });        

        final ipc.Partition p = new ipc.Partition(partitionName);

        // Look for the Igui in IPC
        this.iguiInterface = new IguiInterface(p);

	// Parameters: Database, Items allowed to change, whether a confirmation is required
        //prescaleSaver = new L1PrescaleSaver(triggerdb, extractItemNames(configfile), this.confirm );
        prescaleSaver = new L1PrescaleSaver(triggerdb, new ArrayList<String>(), this.confirm );

        prescaleSaver.setIguiInterface(this.iguiInterface);
	
	// Shouldn't we just retrieve the latest SMK here? oO
	//getSMKFromIS()
        prescaleSaver.configureWithSMK(getSMKFromIS());
        
        getMasterTrigger();

	// The listener is no longer needed

	/*
        listener = new APEAbsISListener<AutoPrescaleCTP>(this.partition,
                                                         isname,
                                                         AutoPrescaleCTP.class,
                                                         updater);
        listener.subscribe();
	*/

        this.elog = new ELog(elogpwfile);
        
    }


    /**
     * handle any request received by the AutoPrescaleCTP listener
     *
     */
    private UpdateInfoT<AutoPrescaleCTP> updater = new UpdateInfoT<AutoPrescaleCTP>()
    {
        @Override
        public void updateInfo(final AutoPrescaleCTP info) throws SQLException {
            if(info.L1_PSK_reset>0) {
                if(number_reset_requests()>0) { return; }
                new_reset_request();
                handleResetRequest();
                end_reset_request();
            } else {
                if(number_update_requests()>0) { return; }
                new_update_request();
                handlePrescaleRequest(info);
                end_update_request();
            }
        }
    };


    /**
     * handles received reset request
     *
     * Reset the key to the current L1 psk currently defined as 
     * default STANDBY or PHYSICS L1 psk
     */
    private void handleResetRequest()
    {

        int reset_l1_psk_target = get_l1_key_for_current_state_from_is();
        if( reset_l1_psk_target == 0) { return; }
        
        int current_l1psk = get_current_l1psk_from_is();
        if( reset_l1_psk_target == current_l1psk) {
            debug(new PSIssue( "Abort reset request since the reset target key already matches current prescale " + current_l1psk), 0);
            return;
        }

        if(this.confirm) {
            boolean reset_yes = true; // by defaut accept the request
            String windowTitle = "CTP prescale reset request";
            String question = "LAr noise burst over. Do you want to reset to prescale key "+reset_l1_psk_target+" ?";
            reset_yes = iguiInterface.userDialog(question, windowTitle);
            IguiLogger.info( "User " + (reset_yes ? "confirms" : "denies") + " reset to L1 psk " + reset_l1_psk_target + ".");
            if(!reset_yes) { return; }
        }

        String msg = "End of hot L1 trigger period, resetting L1 PSK to default " + get_current_state_from_is() + " L1 PSK " + reset_l1_psk_target;
        IguiLogger.info(msg);
        warning(new PSIssue( msg));
        applyToCTP(reset_l1_psk_target);
    }

    
    private class BooleanInfo extends NamedInfo
    {
        boolean Ready4Physics;
        public BooleanInfo( ipc.Partition partition, String name ) {
            super( partition, name, "Boolean" );
        }
        @Override
        public void publishGuts( is.Ostream out ){
            super.publishGuts( out );
            out.put( Ready4Physics );
        }
        @Override
        public void refreshGuts( is.Istream in ){
            super.refreshGuts( in );
            Ready4Physics = in.getBoolean();
        }
    }

    
    private READYSTATE get_current_state_from_is()
    {
        Repository isRepository = new Repository(partition);
        BooleanInfo bi = new BooleanInfo(this.partition, "RunParams.Ready4Physics");
        try {
            isRepository.getValue("RunParams.Ready4Physics", bi);
            Boolean physics = bi.Ready4Physics;
            return physics?READYSTATE.PHYSICS:READYSTATE.STANDBY;
        }
        catch(is.InfoNotFoundException|is.RepositoryNotFoundException|is.InfoNotCompatibleException ex) {
            String msg = "Did not find RunParams.Ready4Physics. Can't reset L1 PSK" + ex.toString();
            IguiLogger.error(msg);
            error(new ISIssue( msg));
        }
        return READYSTATE.UNKNOWN;
    }
    
    
    private int get_l1_key_for_current_state_from_is()
    {
        Repository isRepository = new Repository(partition);

        READYSTATE rs = get_current_state_from_is();
        if(rs==READYSTATE.UNKNOWN) { return 0; }

        final String _isname = "RunParams."+(rs==READYSTATE.PHYSICS?"Physics":"Standby")+".L1PsKey";
        TrigConfL1PsKeyNamed isl1key = new TrigConfL1PsKeyNamed(this.partition, _isname);
        try {
            isRepository.getValue(_isname, isl1key);
            int l1_key = isl1key.L1PrescaleKey;
            return l1_key;
        }
        catch(is.InfoNotFoundException|is.RepositoryNotFoundException|is.InfoNotCompatibleException ex) {
            String msg = "Did not find " + _isname + ". Can't reset L1 PSK." + ex.toString();
            IguiLogger.error(msg);
            error(new ISIssue( msg));
        }
        return 0;
    }


    // Needed for the new approach

    private int getSMKFromIS() throws InterruptedException
    {
	Repository isRepository = new Repository(partition);

	TrigConfSmKeyNamed issmk = new TrigConfSmKeyNamed(partition, "RunParams.TrigConfSmKey");
	while(true)
	    {
		try {
		    isRepository.getValue("RunParams.TrigConfSmKey", issmk);
		    return issmk.SuperMasterKey;
		}
		catch(is.InfoNotFoundException|is.RepositoryNotFoundException|is.InfoNotCompatibleException e) {
		    String msg = "Did not find RunParams.TrigConfSmKey." + e.toString();
		    System.out.println(msg);
		    error(new TestISIssue( msg));
		    Object o = new Object();
		    synchronized (o) {
			o.wait(10*1000);
		    }
		}
	    }
    }

    // Test function which we should disable if we don't want bad things to happen online
    public void singleItemPrescale(final String trigName, int validPSK, int prescaleValue) throws L1CTIPC.InvalidName,L1CTIPC.ObsoletePsk,SQLException
    {
	L1CTIPC.PrescaleRequest request = new L1CTIPC.PrescaleRequest();
	request.psk = validPSK;
	L1CTIPC.PrescaleValue value = new L1CTIPC.PrescaleValue();
	value.item = trigName;
	value.prescale = prescaleValue;
	L1CTIPC.PrescaleValue[] valueArray = {value};
	request.values = valueArray;
	
	IguiLogger.info("Requesting a new prescale...");
	try{
	requestNewPrescaleKey(request);
	} catch(InterruptedException ex)
	    {
		IguiLogger.info("Problem encountered...");
	    }
    }
    
    

    // New operation mode: this will receive the forwarded request
    // from CORBA

    private L1CTIPC.PrescaleReturn requestNewPrescaleKey(final L1CTIPC.PrescaleRequest request) throws L1CTIPC.InvalidName,L1CTIPC.ObsoletePsk,SQLException,InterruptedException
    {
	// Our requests do not come SMKs, so we need to
	// retrieve it from IS: This is probably overkill...

	this.base_psk  = request.psk;

	// Use the SMK set-up by hand as a test
	//int theSMK = getSMKFromIS();
        prescaleSaver.configureWithSMK(getSMKFromIS());

	// We will need to create a L1PSK with only
	// modifications to the last loaded one
	L1Prescale pss = new L1Prescale();
	pss = prescaleSaver.readL1PSS(base_psk);

	// And here we modify only the items we care about...
	// To prevent multiple loops, only loop once over all entries

	// TODO: Change to 512 later?

	for(L1CTIPC.PrescaleValue myValue : request.values)
	    {
		boolean nameFound = false;
		for(int ctp_id = 0; ctp_id < L1Prescale.LENGTH; ctp_id++) {
		    
		    String item_name = prescaleSaver.getItem_name_by_ctp_id().get(ctp_id);
		    //IguiLogger.info("CTPID: "+(ctp_id));
		    //IguiLogger.info("Name: "+item_name);
		    //IguiLogger.info("Target: "+myValue.item);
		    //IguiLogger.info("PS: "+myValue.prescale);
		    if(myValue.item.equals(item_name)){
			nameFound = true;
			IguiLogger.info("Old PS: "+pss.get_val(ctp_id+1));
			//pss.set_val(ctp_id+1, (int)myValue.prescale);
			// Actually, here we need to change the schema
			pss.set_val(ctp_id+1, CutToValueConverter.calculateCutFromPrescale((double)myValue.prescale));
			IguiLogger.info("Old PS: "+pss.get_val(ctp_id+1));
		    }
		}	    
		if(!nameFound) throw new L1CTIPC.InvalidName("Invalid Trigger Item Name", myValue.item);
	    }	    
	
	L1PrescaleSaver.RequestState requestStatus = isValidPrescaleSet(pss);
        if( ! (requestStatus == L1PrescaleSaver.RequestState.VALID ||
               requestStatus == L1PrescaleSaver.RequestState.VALID_HIGH_RATE) ) {
	    return new L1CTIPC.PrescaleReturn(false, -1);            
        }
	
	// IguiLogger.info("Skipping prescale writing as a test");
	// return new L1CTIPC.PrescaleReturn(false, -1);

	// This is where we comment out prescale writing
        int l1psk = l1pskFromSafeToTriggerDB();
	if( l1psk <= 0 ) { // no prescale key safed
	    IguiLogger.info("Was smaller than 1: "+l1psk);
	    throw new L1CTIPC.ObsoletePsk("Obsolete PSK value", request.psk);
	    //return new L1CTIPC.PrescaleReturn(false, -1);
	}

	IguiLogger.info("Returning success: "+l1psk);
	return new L1CTIPC.PrescaleReturn(true, l1psk);
    }
    

    /**
     * handle received prescale request
     *
     */

    private void handlePrescaleRequest(final AutoPrescaleCTP ctpinfo) throws SQLException
    {

        IguiLogger.info("A change in IS for AutoPrescaleCTP has been detected");

        // received prescale request
        synchronized(isBlocked) {
            if(isblocked()) {
                info(new ControlIssue( "Autoprescale request ignored, since auto prescaler is blocked by " + blocker() ));
                return;
            }
        }

        this.base_psk  = ctpinfo.L1_PSK_current;

        prescaleSaver.configureWithSMK(ctpinfo.SMK_current);
        
        L1Prescale pss = new L1Prescale();
        int ctpid=0;
        for(long ps : ctpinfo.L1_Prescales) {
            pss.set_val(ctpid++,(int)ps);
        }
        
        L1PrescaleSaver.RequestState request = isValidPrescaleSet(pss);
        if( ! (request == L1PrescaleSaver.RequestState.VALID ||
               request == L1PrescaleSaver.RequestState.VALID_HIGH_RATE) ) {
            return; // invalid request
        }
        
        int l1psk = l1pskFromSafeToTriggerDB();
        if( l1psk <= 0 ) { // no prescale key safed
            return;
        }

        // simulate slow trigger DB upload to see how a intermittend warm-start would affect this
        if(this.uploadDelay>0) {

            String msg = "Delaying submit of l1 psk " + l1psk + " to CTP by " + this.uploadDelay + " seconds !";
            IguiLogger.error(msg);
            warning(new PSIssue( msg));

            try {
                Thread.sleep(1000*this.uploadDelay);
            }
            catch(InterruptedException ex) {}
        }

        // apply to CTP        
        if(this.confirm) {
            // user confirmation dialog
            String windowTitle = "CTP auto prescale request";
            String question = "Change CTP to the new prescale key "+l1psk+" ?";
            boolean apply_yes = iguiInterface.userDialog(question, windowTitle);
            if(!apply_yes) { return; }
        }


        // check if L1 psk has changed since request was issued
        int current_l1psk = get_current_l1psk_from_is();
        if( current_l1psk != base_psk ) { 
            String msg = "Current L1 PS key " + current_l1psk + " has changed since request was issued ( " + this.base_psk + "). Cancel auto-prescale request!";
            IguiLogger.error(msg);
            warning(new PSIssue( msg));
            return;
        }

        synchronized(isBlocked) {
            if(isblocked()) {
                info(new ControlIssue( "Autoprescale request aborted, since auto prescaler has been blocked in the meantime by " + blocker() ));
                return;
            }
        }
        
        // send to CTP
        String msg = "Sending autoprescale key " + l1psk + " to CTP.";
        IguiLogger.info(msg);
        info(new PSIssue( msg));
        boolean success = applyToCTP(l1psk);
        if(success) {
            if(request == L1PrescaleSaver.RequestState.VALID_HIGH_RATE ) {
                msg = "L1 prescales automatically changed due to very high rate! Please inform trigger, LAr, and Tile shifters. PSK " + l1psk + "(" + prescaleSaver.getL1ps().get_name() + ") -> ";
                msg += prescaleSaver.getL1ps().get_comment();
                error(new PSIssue( msg));
            } else {
                msg = "L1 prescales automatically changed due to high rate! PSK " + l1psk + "(" + prescaleSaver.getL1ps().get_name() + ") -> ";
                msg += prescaleSaver.getL1ps().get_comment();
                warning(new PSIssue( msg));
            }
            //elog.sendElogMsg("Automatic L1 prescale set change", msg);
        }
    }

    
    private L1PrescaleSaver.RequestState isValidPrescaleSet(final L1Prescale l1_prescales) throws SQLException
    {
        L1PrescaleSaver.RequestState request = prescaleSaver.validateNewPrescales(this.base_psk, l1_prescales );
        if( request == L1PrescaleSaver.RequestState.VALID ||
            request == L1PrescaleSaver.RequestState.VALID_HIGH_RATE ) { return request; }
        if( request != L1PrescaleSaver.RequestState.CANCELED_REQUEST) {
            error(new PSIssue( "Auto prescale request from CTP not valid: " + request.toString()));
        }
        return request;
    }
    
    
    private int l1pskFromSafeToTriggerDB() throws SQLException
    {
        IguiLogger.info("Requested a new prescale set based on L1 PSK: " + this.base_psk);
        int current_l1psk = get_current_l1psk_from_is();
        if( current_l1psk != this.base_psk ) { 
            String msg = "Current L1 PS key " + current_l1psk + " has changed since request was issued ( " + this.base_psk + "). Cancel auto-prescale request!";
            IguiLogger.error(msg);
            warning(new PSIssue( msg));
            return 0;
        }
        int new_l1psk = prescaleSaver.saveNewL1PrescaleSet();
        IguiLogger.info("New L1 PSK: " + new_l1psk);
        return new_l1psk;
    }

    
    private boolean applyToCTP(final int new_l1psk)
    {
        if(this.testMode) {
            return simulateApplyToCTP(new_l1psk);
        }
        try{
            triggerCommander.setL1Prescales(new_l1psk);
        } catch (Exception ex) {
            String msg = "Exception when sending L1 PSK " + new_l1psk + " to CTP. ("+ex.toString()+")";
            IguiLogger.error(msg);
            error(new PSIssue( msg));
            return false;
        }

        final String controller = "RootController";
        final String command    = "LUMIBLOCKINCREASE";
        final String arguments  = "";
        CommandSender commander  = new CommandSender(this.partition.getName(), "IGUITriggerPanel");

        IguiLogger.info("Send run-control command \"" + command + "\" with arguments \"" + arguments + "\" to controller \"" + controller + "\"");
        try {
            commander.userCommand(controller, command, new String[]{});
            info(new LBIssue( "Issued LUMIBLOCK increase"));
        }
        catch(Exception e) {
            IguiLogger.error("Could not send command \"" + command + "\"." + e);
            error(new LBIssue( "Could not increase LUMIBLOCK"));
            return false;
        }

        return true;
    }

    private boolean simulateApplyToCTP(final int new_l1psk)
    {
        warning(new SimIssue( "AutoPrescaler in test mode, CTP not called!"));
        sendL1PSKToIS(new_l1psk);
        return true;
    }    
    

    private void sendL1PSKToIS(final int key)
    {
        Repository isRepository = new Repository(this.partition);
        info(new SimIssue( "Sending L1 PSK " + key +  " to IS"));
        final String infoname = "RunParams.TrigConfL1PsKey";
        TrigConfL1PsKeyNamed isl1key = new TrigConfL1PsKeyNamed(this.partition, infoname);
        isl1key.L1PrescaleKey = key;
        try {
            isRepository.checkin( infoname, isl1key);
        }
	//        catch(is.InfoNotFoundException e) {
	//            isRepository.insert( infoname, isl1key);
	//        }
        catch(is.RepositoryNotFoundException ex) {
            String msg = "Did not find is repository. Can't publish L1 PSK";
            IguiLogger.error(msg);
            error(new ISIssue( msg));
        }
	catch(is.InfoNotCompatibleException exe) {
	    String msg = "Trying to write wrong type to IS";
            IguiLogger.error(msg);
            error(new ISIssue( msg));
        }
        return;
    }
    

    public static List<String> extractItemNames(final String configfile)
    {
        List<String> items = new ArrayList<String>();
        try {
            File f = new File(configfile);
            BufferedReader reader = new BufferedReader(new FileReader(f));
            String line = null;
            while ((line=reader.readLine()) != null) {
                Scanner linescanner = new Scanner(line);
                try {
                    items.add(linescanner.next());
                }
                catch(Exception ex) {}
            }
            reader.close();
        }
        catch(FileNotFoundException ex) {
            String msg = "Could not find '"+configfile+"'. " + ex.toString();
            IguiLogger.error(msg);
            fatal(new ConfigIssue( msg));
            System.exit(1);
        }
        catch (IOException e) {
            String msg = "Could not read " + configfile+e.toString();
            IguiLogger.error(msg);
            fatal(new ConfigIssue( msg));
            System.exit(1);
        }
        IguiLogger.info("In '"+configfile+"' found these items that are allowed to change: "+items.toString());
        return items;
    }

    
    private void getMasterTrigger()
    {
        String mtControllerName = null;
        try {
            Configuration db = new Configuration("rdbconfig:RDB@" + this.partitionName);
            dal.Partition dalPartition = dal.Partition_Helper.get(db, partitionName);

            final MasterTrigger mt = dalPartition.get_MasterTrigger();
            if(mt == null) {
                IguiLogger.error("The Master Trigger is not defined in the partition; it will not be possible to hold or resume the trigger");
                return;
            }             
                
            final RunControlApplicationBase rc = mt.get_Controller();
            if(rc == null) {
                IguiLogger.error("The Master Trigger does not define a controller; it will not be possible to hold or resume the trigger");
                return;
            }

            mtControllerName = rc.UID();
            IguiLogger.info("The partition Master Trigger is " + mtControllerName);
            this.triggerCommander = new TriggerCommander(this.partition, mtControllerName);
        }
        catch(final Exception ex) {
            IguiLogger.error("Exception when looking for the Master Trigger: " + ex);
        }
    }

    
    private int get_current_l1psk_from_is()
    {
        try {
	    CtpControllerL1PSKey isl1key = new CtpControllerL1PSKey(); //this.partition, "L1CT.CtpControllerLoadedL1PSKey");
	    //TrigConfL1PsKeyNamed isl1key = new TrigConfL1PsKeyNamed(this.partition, "RunParams.TrigConfL1PsKey");
            Repository isRepository = new Repository(partition);
            //isRepository.getValue("RunParams.TrigConfL1PsKey", isl1key);
	    // New place to look for this value
	    // Loaded, or scheduled?
            isRepository.getValue("L1CT.CtpControllerLoadedL1PSKey", isl1key);
            return isl1key.L1PrescaleKey;
	}
        catch(is.InfoNotFoundException|is.RepositoryNotFoundException|is.InfoNotCompatibleException ex) {
            IguiLogger.error("Did not find RunParams.TrigConfL1PsKey. Assume it is missing.",ex);
        }
        return 0;
    }

        
    static void info(final ers.Issue i) {ers.Logger.info(i);}
    static void warning(final ers.Issue i) {ers.Logger.warning(i);}
    static void error(final ers.Issue i) {ers.Logger.error(i);}
    static void fatal(final ers.Issue i) {ers.Logger.fatal(i);}
    static void debug(final ers.Issue i, final int n) {ers.Logger.debug(n, i);}
    static void log(final ers.Issue i) {ers.Logger.error(i);}

    public static void main( String[] args ) throws InterruptedException, InvalidPartitionException, SQLException {

        String servant    = "L1AutoPrescaler";
        String partition  = "ATLAS";
        String isname     = "L1CT.AutoPrescaleCTP";
        String triggerdb  = "TRIGGERDBR2";
        String configfile = "/det/ctp/MaxItemRates.txt";
        String elogpwfile = "/atlas-home/1/stelzer/APE/.elogpw.txt";
        int smk = 0;
        int uploaddelay = 0;
        
        CommandLine cmd = null;
        Options options = new Options();

        try {
            CommandLineParser parser = new DefaultParser(); // PosixParser();
            options.addOption("h", false, "help");
            options.addOption("s", true, "servant name ["+servant+"]");
            options.addOption("p", true, "partition name ["+partition+"]");
            options.addOption("n", true, "is object name ["+isname+"]");
            options.addOption("db", true, "triggerdb ["+triggerdb+"]");
            options.addOption("smk", true, "smk ["+smk+"]");
            options.addOption("c", true, "config file ["+configfile+"]");
            options.addOption("elog", true, "elog pw file ["+elogpwfile+"]");
            options.addOption("confirm", false, "confirmation required");
            options.addOption("testmode", false, "testmode simulates CTP response");
            options.addOption("uploaddelay", true, "simulates slow upload (delay in seconds [" + uploaddelay + "])");
            cmd = parser.parse(options, args);
        } catch (ParseException ex) {
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("SendL1Pss2IS <options>", options);
            System.exit(1);
        }
        if (cmd.hasOption("h")) {
            HelpFormatter formatter = new HelpFormatter();
            formatter.printHelp("SendL1Pss2IS <options>", options);
            System.exit(0);
        }
        if (cmd.hasOption("s"))   { servant    = cmd.getOptionValue("s"); }
        if (cmd.hasOption("p"))   { partition  = cmd.getOptionValue("p"); }
        if (cmd.hasOption("n"))   { isname     = cmd.getOptionValue("n"); }
        if (cmd.hasOption("db"))  { triggerdb  = cmd.getOptionValue("db"); }
        if (cmd.hasOption("smk")) { smk        = Integer.valueOf(cmd.getOptionValue("smk")); }
        if (cmd.hasOption("c"))   { configfile = cmd.getOptionValue("c"); }
        if (cmd.hasOption("elog")){ elogpwfile = cmd.getOptionValue("elog"); }
        if (cmd.hasOption("uploaddelay")){ uploaddelay = Integer.valueOf(cmd.getOptionValue("uploaddelay")); }

        boolean confirm   = cmd.hasOption("confirm");
        boolean testmode  = cmd.hasOption("testmode");

        AutoPrescaleEditor ape = new AutoPrescaleEditor( servant, partition, isname, triggerdb, configfile, elogpwfile, smk, confirm);
        if(testmode) { ape.setTestMode(testmode); }
        if(uploaddelay>0) { ape.setUploadDelay(uploaddelay); }
        
	IguiLogger.info("Ready and listening!");
	
	// Hard-coded test...
	/*
	try{
	    ape.singleItemPrescale("L1_J30_EMPTY", 25, 125);
	} catch(L1CTIPC.InvalidName|L1CTIPC.ObsoletePsk|SQLException ex) { IguiLogger.info("Caught...");}
	
	IguiLogger.info("NewPSKMade?");
	*/	

	// Catch...
        Object o = new Object();
        synchronized (o) {
            while (true) {
                o.wait(500);
            }
        }
	

    }

}    
