package autoprescaleeditor;

import Igui.IguiLogger;

import java.net.InetAddress;

import java.io.*;

import ers.Logger;

import java.util.*;

public class ELog {
    // elog pw from file
    private boolean elogenabled=false;
    private String elogpw="";
    private List<String> systems = new ArrayList<String>();
    private boolean sendEmail=false;

    ELog(final String pwfile) {
        try {
            File f = new File(pwfile);
            BufferedReader reader = new BufferedReader(new FileReader(f));
            String line = null;
            while ( (line=reader.readLine()) != null) {
                String[] arr = line.split(" ");
                if(arr.length==0) continue;
                if(arr[0].toLowerCase().equals("pw")) {
                    this.elogpw = arr[1];
                } else if(arr[0].toLowerCase().equals("elog")) {
                    this.elogenabled = arr[1].toLowerCase().equals("yes");
                } else if(arr[0].toLowerCase().equals("systems")) {
                    for(int i=1; i<arr.length; i++) { this.systems.add(arr[i]); }
                } else if(arr[0].toLowerCase().equals("email")) {
                    this.sendEmail = arr[1].toLowerCase().equals("yes");
                }
            }
            if(this.elogenabled) {
                String msg = "ELog account settings initialized:\n  Systems affected: " + this.systems + "\n  Send Email: " + (this.sendEmail?"Yes":"No");
                AutoPrescaleEditor.info(new ConfigIssue( msg));
            }
            reader.close();
        }
        catch(FileNotFoundException ex) {
            String msg = "Could not find file '"+pwfile+"' with elog pw. No e-log entries from auto prescaler." + ex.toString();
            IguiLogger.error(msg);
            AutoPrescaleEditor.warning(new ConfigIssue( msg));
            return;
        }
        catch (IOException e) {
            String msg = "Could not read " + pwfile+"' with elog pw. No e-log entries from auto prescaler."+e.toString();
            IguiLogger.error(msg);
            AutoPrescaleEditor.warning(new ConfigIssue( msg));
            return;
        }
        catch (Exception e) {
            String msg = "Could not read pw from '" + pwfile+"'. No e-log entries from auto prescaler. "+e.toString();
            IguiLogger.error(msg);
            AutoPrescaleEditor.warning(new ConfigIssue( msg));
            return;
        }
        return;
    }

    /*
     * @param entrySubject The subject of the elog entry to be submitted (this cannot be null or empty)
     * @param messageText The text to be appended to the elog entry (it will appear in the dialog text area) 
     */
    void sendElogMsg(final String subject, final String msg) {
        if( ! this.elogenabled ) { return; }
        
        String hostAddress = "unknown";
        try {
            hostAddress = (InetAddress.getLocalHost()).getHostAddress();
        } catch (java.net.UnknownHostException exe) {
            System.out.println("Default host is not found by getHostAddress().  " +
                               "Reason: java.net.UnknownHostException");
        }
        
        List<String> args = new ArrayList<String>();
        args.add("elog");
        args.add("-h"); args.add("pc-atlas-www.cern.ch" );
        args.add("-p"); args.add("8100" );
        args.add("-l"); args.add("ATLAS");
        if( ! this.sendEmail) { args.add("-x"); } // suppress email
        args.add("-u"); args.add("attrgonl"); args.add(this.elogpw);
        args.add("-a"); args.add("valid=valid");
        args.add("-a"); args.add("Message Type=Trigger");
        args.add("-a"); args.add("User=attrgonl");
        args.add("-a"); args.add("Author=Lvl1HotItemAutoPrescaler");
        args.add("-a"); args.add("Rem_IP=" + hostAddress);
        args.add("-a"); args.add("Status=closed");
        for(String system: this.systems) {
            args.add("-a"); args.add("System Affected=" + system);
        }
        args.add("-a"); args.add("Subject=" + subject);
        args.add(msg);
        
        try {
            Runtime rt = Runtime.getRuntime();
            Process process = rt.exec((String[])args.toArray());
            
            InputStreamReader reader = new InputStreamReader(process.getInputStream());
            InputStreamReader err = new InputStreamReader(process.getErrorStream());

            BufferedReader buf_reader = new BufferedReader(reader);
            BufferedReader buf_reader_err = new BufferedReader(err);

            String line;
            while ((line = buf_reader.readLine()) != null) { 
                //AutoPrescaleEditor.info(new ConfigIssue(ersSystem.ERS_HERE(), "E-log submission " + line),0);
                System.out.println("OUTPUT      : " + line);
            }
            while ((line = buf_reader_err.readLine()) != null) {
                System.err.println("ERROR OUTPUT: " + line);
            }
        } catch (IOException e) {
            AutoPrescaleEditor.warning(new ConfigIssue( "Could not make e-log entry"));
        }            
            
            
        return;
    }
}
