package autoprescaleeditor;

import iguiCommander.MessageSeverity;
import java.util.Calendar;
import java.text.SimpleDateFormat;

// IGui commander for user dialogs
import iguiCommander.ClientPOA;
import iguiCommander.ClientRequest;
import iguiCommander.UserAnswer;

import java.util.concurrent.atomic.AtomicBoolean;

public class IguiInterface {
    
    final AtomicBoolean callbackReceived = new AtomicBoolean(false);
    final AtomicBoolean userAnswer = new AtomicBoolean(false);

    iguiCommander.Server s = null;

    public IguiInterface( final ipc.Partition p )
    {
        // Look for the Igui in IPC
        try {
            s = p.lookup(iguiCommander.Server.class, iguiCommander.servantName.value);
        }
        catch(ipc.InvalidPartitionException e) {
            System.err.println("Could not find IGuiCommander: " + iguiCommander.Server.class + ": " + iguiCommander.servantName.value + "[" + e + "]");
        }
        catch(ipc.InvalidObjectException e) {
            System.err.println("Could not find IGuiCommander: " + iguiCommander.Server.class + ": " + iguiCommander.servantName.value + "[" + e + "]");
        }
    }

    private String now() {
        Calendar cal = Calendar.getInstance();
        //SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        return sdf.format(cal.getTime());
    }


    public boolean userDialog(String question, String windowTitle) {
        final ClientPOA cl = new ClientPOA() {
                @Override
                public void done(final UserAnswer ua) {
                    // Remember that this call-back method is always executed in one of the jacorb threads
                    // Use proper synchronization if needed
                    userAnswer.set((ua == UserAnswer.YES));

                    //System.out.println("The user answer is \"" + ua.toString() + "\"");
                    //System.out.println("The understood answer is \"" + (userAnswer.get()?"YES":"NO") + "\"");

                    synchronized(this) {
                        callbackReceived.set(true);
                        this.notify();
                    }
                }
            };

        windowTitle = now()+": "+ windowTitle;

        // Create a client request
        final ClientRequest cr = new ClientRequest(windowTitle, question, cl._this(ipc.Core.getORB()));

        callbackReceived.set(false);

        // Send the request to the Igui
        s.askUser(cr);

        // Wait for the answer (needed to not let the test application exit before receiving the call-back)
        synchronized(cl) {
            try {
                while(callbackReceived.get() == false) {
                    cl.wait();
                }
            }
            catch(final InterruptedException ex) {
                ex.printStackTrace();
            }
           
        }

        return userAnswer.get();
    }
    
    void informUser(final String msg, final MessageSeverity severity) {
        if(s==null) {
            System.out.println(severity.toString() + ": " + msg);
        } else {
            s.informUser(msg, iguiCommander.MessageSeverity.INFORMATION);
        }
    }

}
