/*
 * author Joerg Stelzer
 *
 * Created on Jul 31, 2010, 9:38:59 AM
 */

package autoprescaleeditor;

import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JRadioButton;
import javax.swing.JFrame;

import java.awt.GridBagConstraints;

import java.util.ArrayList;
import java.util.List;
import triggerdb.Entities.L1.L1Prescale;

/**
 *
 * @author stelzer
 */
public class UploadConfirmationDialog extends javax.swing.JDialog {
    private static final long serialVersionUID = 1L;

    private final List<String> itemnames;
    private final L1Prescale prescales;
    private boolean confirmed = false;

    public boolean confirmed() { return this.confirmed; }

    /** Creates new form NewJDialog
     * @param parent
     * @param modal
     * @param itemnames
     * @param prescales */
    public UploadConfirmationDialog(JFrame parent, boolean modal, final List<String> itemnames, 
                                    final L1Prescale prescales) {
        super(parent, modal);
        this.items = new ArrayList<>();
        this.radiobuttons = new ArrayList<>();
        this.itemnames=itemnames;
        this.prescales=prescales;
        initComponents();
    }

    public L1Prescale selectedPrescales() {
        int bi = 0;
        for(JRadioButton rb : radiobuttons) {
            if(rb.isSelected()) {
                break;
            }
            bi++;
        }
        return this.prescales;
    }

    @SuppressWarnings("unchecked")
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        int nItems = this.itemnames.size();

        bg = new javax.swing.ButtonGroup();
        mainpanel = new javax.swing.JPanel();
        selectpanel = new javax.swing.JPanel();
        buttonpanel = new javax.swing.JPanel();
        cancelbutton = new javax.swing.JButton("CANCEL");
        okbutton = new javax.swing.JButton("OK");


        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Select Auto Prescales");
        setIconImage(null);
        setAlwaysOnTop(true);
        getContentPane().setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 20, 10));

        mainpanel.setLayout(new javax.swing.BoxLayout(mainpanel, javax.swing.BoxLayout.Y_AXIS));

        selectpanel.setLayout(new java.awt.GridBagLayout());

        for(int y=0; y<nItems; y++) {
            JLabel item = new JLabel(this.itemnames.get(y));
            gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = y;
            gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
            selectpanel.add(item, gridBagConstraints);

            for(int x=0; x<4; x++) {
                gridBagConstraints = new GridBagConstraints();
                gridBagConstraints.gridx = x+1;
                gridBagConstraints.gridy = y;
                try {
                   selectpanel.add(new JLabel(prescales.get_val(x).toString()), gridBagConstraints);
                }
                catch(Exception ex) {}
            }
        }

        String rbtitle[] = new String[] { "requested", "next", "equal", "disable" };
        
        for(int r=0;r<4;r++) {
            JRadioButton rb = new JRadioButton(rbtitle[r]);
            if(r==2) rb.setSelected(true);
            rb.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
            rb.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
            gridBagConstraints = new java.awt.GridBagConstraints();
            gridBagConstraints.gridx = r+1;
            gridBagConstraints.gridy = 3;
            selectpanel.add(rb, gridBagConstraints);
            bg.add(rb);
            radiobuttons.add(rb);
        }

        mainpanel.add(selectpanel);

        buttonpanel.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));
        cancelbutton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    confirmed = false;
                    dispose();
                    //closeDialog();
                }
            });
        buttonpanel.add(cancelbutton);

        okbutton.setText("OK");
        okbutton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(java.awt.event.ActionEvent evt) {
                    confirmed = true;
                    dispose();
                }
            });
        buttonpanel.add(okbutton);

        mainpanel.add(buttonpanel);

        getContentPane().add(mainpanel);

        pack();
    }
        
    


    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {

        final List<String> in = new ArrayList<>();
        in.add("L1_EM3_EMPTY");
        in.add("L1_TAU5_EMPTY");
        in.add("L1_J5_EMPTY");

        final List<L1Prescale> ps = new ArrayList<>(4);
        for(int i=0;i<4;i++) {
            ps.add(new L1Prescale());
        }
        ps.get(0).set_val(1,5);
        ps.get(0).set_val(2,13);
        ps.get(0).set_val(3,60);

        ps.get(1).set_val(1,5);
        ps.get(1).set_val(2,25);
        ps.get(1).set_val(3,125);

        ps.get(2).set_val(1,125);
        ps.get(2).set_val(2,125);
        ps.get(2).set_val(3,125);

        ps.get(3).set_val(1,-1);
        ps.get(3).set_val(2,-2);
        ps.get(3).set_val(3,-10);
        
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                UploadConfirmationDialog dialog = new UploadConfirmationDialog(null, true, in, ps.get(2));
                dialog.addWindowListener(new java.awt.event.WindowAdapter() {
                    @Override
                    public void windowClosing(java.awt.event.WindowEvent e) {
                        System.exit(0);
                    }
                });
                dialog.setVisible(true);
                System.out.println(dialog.confirmed()?"Confirmed":"Canceled");
                if(dialog.confirmed()) {
                    System.out.println(dialog.selectedPrescales().toString());
                }
            }
        });
    }

    /**
     *
     * @param parent
     * @param itemNames
     * @param itemPrescales
     * @return
     */
    public static UploadConfirmationDialog showDialog(final JFrame parent, final List<String> itemNames, final L1Prescale itemPrescales ) {
        UploadConfirmationDialog dialog = new UploadConfirmationDialog(parent, true, itemNames, itemPrescales);
        dialog.setVisible(true);
        return dialog;
    }

    private javax.swing.ButtonGroup bg;
    private javax.swing.JPanel buttonpanel;
    private javax.swing.JButton okbutton;
    private javax.swing.JButton cancelbutton;
    private final List<JTextField> items;
    private final List<JRadioButton> radiobuttons;
    private javax.swing.JPanel mainpanel;
    private javax.swing.JPanel selectpanel;
}
