//------------------------------------------------------------------------------
#include <iostream>
#include <vector>
#include "rc/RunParams.h"
#include "rc/RCStateInfo.h"
//------------------------------------------------------------------------------
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ipc/partition.h>
//------------------------------------------------------------------------------
#include <AutoPrescaler/ipc/ISServer.h>
#include <AutoPrescaler/prescaler/Prescaler.h>
#include <AutoPrescaler/ipc/PrescalerIpc.h>
//------------------------------------------------------------------------------
#include "AutoPrescaler/prescaler/PrescalerWrapper.h"
//------------------------------------------------------------------------------


// ------------------ IMPLEMENTATION -----------------

using namespace std;
using namespace L1CT;

/** \brief Constructor: initialise variables
 */
PrescalerWrapper::PrescalerWrapper() : daq::rc::Controllable()
{
  ERS_LOG(" *** Start of PrescalerWrapper::PrescalerWrapper() ***");
}

/** \brief Get application configuration from OKS.
 */
void PrescalerWrapper::configure(const daq::rc::TransitionCmd& )
{
  ERS_LOG(" *** Start of PrescalerWrapper::configure() ***");
  
  daq::rc::OnlineServices& services = daq::rc::OnlineServices::instance();
  //Configuration& config = services.getConfiguration();
  
  //const L1CTCooldal::L1CT2CoolPrescalerWrapperApplication* self = config.cast<L1CTCooldal::L1CT2CoolPrescalerWrapperApplication>(&daq::rc::OnlineServices::instance().getApplication());
  
  //if (self == NULL) {
  //  L1CT_FATAL_MESSAGE("Configuration not found! Will use default settings");
  //} else {
  //  m_useSQLite = self->get_UseSQLite();
  //  m_SQLiteFileName = self->get_SQLiteFileName();
  //  m_coolFolderToCheck = self->get_FolderToCheck();
  //  m_useLabMode = self->get_LabMode();
  //}
  // Write configuration to log file
  //ERS_LOG("Configuration Parameters as read from OKS:  useSQLite = "<<m_useSQLite<<" SQLiteFileName = "<<m_SQLiteFileName<<" CoolFolderToCheck = "<<m_coolFolderToCheck<<" useLabMode = "<<m_useLabMode);
  
  // Initialiase handle to IS
  services.getIPCPartition();
  const IPCPartition part(daq::rc::OnlineServices::instance().getIPCPartition());
  m_isDict = new ISInfoDictionary(part);
  
  // Create cool access string and create watchdog worker class
  //if(m_useSQLite) {
    // std::string sqlitefile="/tmp/l1ct_LBtoCool_sqlite.db";
    //m_cool_dbid = "sqlite://;schema=";
    //m_cool_dbid += m_SQLiteFileName;
    //m_cool_dbid += ";dbname=";
  //} else /* if(m_confObj->dalCtpcore->get_COOLDB() == "Oracle") */ {
  //m_cool_dbid = "COOLONL_TRIGGER/";
  //}
  
  // Create watchDogChecks class depending on configuration, and configure the class
  // also add the DB to look at (PROD = CONDBR2 or MON = MONP200
  //if (m_coolFolderToCheck == "LB") {
  //  m_cool_dbid += L1CTUtils::COOL_PROD_DB;
  //  m_watchdogChecks = new PrescalerWrapperChecks_LB2Cool(m_cool_dbid);
  //} else if (m_coolFolderToCheck ==  "BUSY") {
  //  m_cool_dbid += L1CTUtils::COOL_MON_DB;
  //  m_watchdogChecks = new PrescalerWrapperChecks_Busy2Cool(m_cool_dbid);
  //} else if (m_coolFolderToCheck == "COUNTERS"){
  //  m_cool_dbid += L1CTUtils::COOL_PROD_DB;
  //  m_watchdogChecks = new PrescalerWrapperChecks_Counters2Cool(m_cool_dbid);
  //} else {
  //  
  //}

  ERS_LOG(" *** Reading in configuration: ***");  
  
  const std::string partitionNameInp = static_cast<std::string>(m_partName);
  if(partitionNameInp != ISServer::ATLAS.PartitionName) {
    ISServer::ATLAS.redefine(partitionNameInp, ISServer::ATLAS.ServerName);
   }
  
  // Fix this later
  //const std::string partitionNameInp = static_cast<std::string>(ISServer::ATLAS.PartitionName);

  m_prescaler = new Prescaler();

  //const std::string ipcName = getenv("TDAQ_APPLICATION_NAME") == nullptr ? "CtpAutoPrescaler" : getenv("TDAQ_APPLICATION_NAME");
  const std::string ipcName = "CtpAutoPrescaler";
  m_prescalerIpc = new PrescalerIpc(ipcName, *m_prescaler);

  ERS_LOG("Setting log-level to " << int(m_verbose));

  // Trap it damn it
  // Normal operation
  
  //  L1CTLogger::INFO() << " Sleeping... " << std::endl;
  //while(1){
  //  sleep(8);
  //  prescaler->processRequests();
  //}

  ERS_LOG(" *** End of PrescalerWrapper::configure() ***");
  
}

/** \brief Run is about to start: start timer. Time interval should be set to max LB duration / 1.2
 */
void PrescalerWrapper::prepareForRun(const daq::rc::TransitionCmd& )
{
  // get runnumber
  //RunParams runParams;
  //m_isDict->getValue("RunParams.RunParams", runParams);
  //m_watchdogChecks->SetRunNumber(runParams.run_number);
  //L1CT_DEBUG_MESSAGE(1, "Set RunNumber to "<<runParams.run_number);
  //LBSettings lbSettings;
  //m_isDict->getValue("RunParams.LBSettings", lbSettings);
  //m_watchdogChecks->SetLBInterval(lbSettings.Interval);
  //L1CT_DEBUG_MESSAGE(1, "Set LB interval to "<<lbSettings.Interval);
  //m_watchdogChecks->ClearBuffers();
}

/** \brief Run was just stopped: stop timer and clear buffers. We're done.
 */
void PrescalerWrapper::stopArchiving(const daq::rc::TransitionCmd& )
{
}

/** \brief Publish() is called regularly by RC. PrescalerWrapper checks are issued from here.
 *  Additionally the frequency of the calls of Publish() is monitored and compared to a dedicated OKS 
 * parameter. In case publish is called to rarely WARNINGS are issued.
 */
void PrescalerWrapper::publish()
{
  // check if root controller is in running state
  RCStateInfo rcStateInfo;
  m_isDict->getValue("RunCtrl.RootController", rcStateInfo);
  if (rcStateInfo.state != "RUNNING") {
    //L1CT_DEBUG_MESSAGE(1, "RC state is not RUNNING but "<<rcStateInfo.state<<", nothing to be done yet");
    ERS_LOG(" *** PrescalerWrapper - Trying to publish, but RC state is not RUNNING ***");
    return;
  }
  
  //L1CT_DEBUG_MESSAGE(1, "Calling processRequests");
  //ERS_LOG(" *** RC state is RUNNING: Calling processRequests ***");
  m_prescaler->processRequests();
  
  //L1CT_DEBUG_MESSAGE(1, "Gathering COOL data");
  //m_watchdogChecks->GatherCoolData();
  //L1CT_DEBUG_MESSAGE(1, "running checks on gathered cool data");
  //m_watchdogChecks->CheckCoolData();
  //m_watchdogChecks->CheckISStatus(m_isDict);
}


