#include <AutoPrescaler/rulechecker/BeamModeCond.h>
#include <AutoPrescaler/rulechecker/RuleChecker.h>
#include <AutoPrescaler/rulechecker/DataAbstraction.h>

//------------------------------------------------------------------------------
using namespace L1CT;
//------------------------------------------------------------------------------

void BeamModeCond::reset()
{ 
  // Do nothing
}

bool BeamModeCond::truthValue()
{
  // Extremely simple logic

  // Check if the monitored item is present: probably want a smarter protection
  if(!rulechecker().dataModel().abstraction().checkItem(m_itemName))
    {
      ERS_DEBUG(3,"Item is not in the abstraction: ");
      return false;
    }      

  // If item is disabled, we just idle
  if(!rulechecker().dataModel().abstraction().isEnabled(m_itemName))
    {
      ERS_DEBUG(3,"Item is disabled: " << m_itemName);
      reset();
      return false;
    }
  
  std::string beamWord = rulechecker().dataModel().abstraction().getBeamMode();

  bool goodMode = false;
  
  for(unsigned int i=0; i< m_beamModes.size(); i++)
    {
      if(beamWord == m_beamModes.at(i))
	goodMode = true;
    }
  
  return goodMode;

  /*  
      if(beamWord == "FLAT TOP" || beamWord == "SQUEEZE") //|| beamWord == "CYCLING")
      return true;
      else
      return false;
  */
}

void BeamModeCond::print()
{
  // Debug print-outs
  ERS_DEBUG(3,"Inside an BeamModeCond: ");
  ERS_DEBUG(3,"Set-up: Item Monitored: "<< m_itemName);
  for(unsigned int i=0; i< m_beamModes.size(); i++)
    {
      ERS_DEBUG(3,"Checking for beamWords: " << m_beamModes.at(i));
    }
  ERS_DEBUG(3,"Beam Mode is: " << rulechecker().dataModel().abstraction().getBeamMode());
  ERS_DEBUG(3,"Condition is Satisfied: " << truthValue());
}

bool BeamModeCond::update()
{
  // Do nothing, once again.
  print();
  
  return true;
}
