//------------------------------------------------------------------------------
#include <iostream>
#include <vector>
#include <string>

#include "rc/RunParams.h"
#include "rc/RCStateInfo.h"
//------------------------------------------------------------------------------
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ipc/partition.h>
//------------------------------------------------------------------------------
#include <AutoPrescaler/ipc/ISServer.h>
#include <AutoPrescaler/rulechecker/RuleChecker.h>
#include <AutoPrescaler/rulechecker/PrescalerRule.h>
#include <AutoPrescaler/rulechecker/AvgRateCond.h>
#include <AutoPrescaler/rulechecker/AvgRateAction.h>
#include <AutoPrescaler/rulechecker/BeamModeCond.h>
#include <AutoPrescaler/rulechecker/BeamModeAction.h>
#include <AutoPrescaler/rulechecker/DataAbstraction.h>
//------------------------------------------------------------------------------
#include <APRules/AutoPrescalerRuleList.h>
//------------------------------------------------------------------------------
#ifndef ERS_ERROR
#define ERS_ERROR(X) do { std::cerr << X << std::endl; } while(false)
#endif
//------------------------------------------------------------------------------
#include <AutoPrescaler/rulechecker/RuleCheckerWrapper.h>
#include <AutoPrescaler/rulechecker/RuleCheckerWrapper.hpp>
//------------------------------------------------------------------------------


// ------------------ IMPLEMENTATION -----------------

using namespace std;
using namespace L1CT;

// brief Constructor: initialise variables  
RuleCheckerWrapper::RuleCheckerWrapper() : daq::rc::Controllable()
{
  ERS_LOG(" *** Start of RuleCheckerWrapper::RuleCheckerWrapper() ***");
}

// brief Get application configuration from OKS.
 
void RuleCheckerWrapper::configure(const daq::rc::TransitionCmd& )
{
  ERS_LOG(" *** Start of RuleCheckerWrapper::configure() ***");

  const std::string envVar = "TDAQ_DB_DATA";

  ERS_LOG(" *** Reading in configuration: ***");  
  
  std::unique_ptr<::Configuration> oksHandle = nullptr;
  if(getenv(envVar.c_str())) {

    const std::string xml = getenv(envVar.c_str());

    try {

      oksHandle.reset(new ::Configuration("oksconfig:" + xml));

    } catch (std::exception &e) {
      std::stringstream ss; ss << "Caught exception while reading XML: " << e.what();
      ERS_ERROR(ss.str());
      return;

    }

  } else {

    ERS_ERROR("Can't read XML directly: " << envVar << " not set.");
    return;

  }
  
  ERS_LOG(" Configuration read properly.  Starting to configure...");
  auto * const oksConf = oksHandle.get();

  std::vector<ConfigObject> objs;
  oksConf->get("AutoPrescalerRuleList", objs);

  ERS_LOG(" Got RuleList...");

  if(objs.size() != 1) {
    ERS_ERROR("Found " << char(objs.size()) << " instances of AutoPrescalerRuleList, expected one.");
    return;
  }

  
  auto * const ruleListConf = oksHandle->get<APRules::AutoPrescalerRuleList>(objs[0]);

  const auto &ruleList = ruleListConf->get_ListOfRules();
  if(ruleList.size() == 0) {
    ERS_INFO("Rule list for application contains no rules.");
      return;
  }

  const std::string partitionNameInp = static_cast<std::string>(m_partName);
  if(partitionNameInp != ISServer::ATLAS.PartitionName) {
    ERS_INFO("Re-defining partition name.");
    ISServer::ATLAS.redefine(partitionNameInp, ISServer::ATLAS.ServerName);
  }
  
  // use the initialL1CT to simply mean "initial" partition
  ISServer::initialL1CT.redefine("initial", ISServer::initialL1CT.ServerName);
    ERS_DEBUG(int(m_verbose)," Creating RuleChecker...");  


m_rulechecker = new RuleChecker();

 stringstream stringstream_rule;
 for(const auto * const rule : ruleList) {
   ERS_DEBUG(int(m_verbose)," Adding Rule:");
   //    rule->print(0, 1, L1CTLogger::INFO());                                                                                                                                                
   // rule->print(0, 1, ERS_DEBUG(int(m_verbose))); // does not work                                                                                                                           
   rule->print(0, 1, stringstream_rule);
   ERS_DEBUG(int(m_verbose),stringstream_rule.str());
   //clear stringstream_rule                                                                                                                                                                   
   stringstream_rule.str("");
   addRule(rule);
 }
  ERS_LOG(" *** End of RuleCheckerWrapper::configure() ***");

  return;
  
  
}

/** \brief Run is about to start: start timer. Time interval should be set to max LB duration / 1.2
 */
void RuleCheckerWrapper::prepareForRun(const daq::rc::TransitionCmd& )
{
  // get runnumber
  //RunParams runParams;
  //m_isDict->getValue("RunParams.RunParams", runParams);
  //m_watchdogChecks->SetRunNumber(runParams.run_number);
  //L1CT_DEBUG_MESSAGE(1, "Set RunNumber to "<<runParams.run_number);
  //LBSettings lbSettings;
  //m_isDict->getValue("RunParams.LBSettings", lbSettings);
  //m_watchdogChecks->SetLBInterval(lbSettings.Interval);
  //L1CT_DEBUG_MESSAGE(1, "Set LB interval to "<<lbSettings.Interval);
  //m_watchdogChecks->ClearBuffers();
}

/** \brief Run was just stopped: stop timer and clear buffers. We're done.
 */
void RuleCheckerWrapper::stopArchiving(const daq::rc::TransitionCmd& )
{
}

/** \brief Publish() is called regularly by RC. RuleCheckerWrapper checks are issued from here.
 *  Additionally the frequency of the calls of Publish() is monitored and compared to a dedicated OKS 
 * parameter. In case publish is called to rarely WARNINGS are issued.
 */
void RuleCheckerWrapper::publish()
{
  // check if root controller is in running state
  //RCStateInfo rcStateInfo;
  //m_isDict->getValue("RunCtrl.RootController", rcStateInfo);
  //if (rcStateInfo.state != "RUNNING") {
  //  L1CT_DEBUG_MESSAGE(1, "RC state is not RUNNING but "<<rcStateInfo.state<<", nothing to be done yet");
  //  return;
  //}
  
  //L1CT_DEBUG_MESSAGE(1, "Gathering COOL data");
  //m_watchdogChecks->GatherCoolData();
  //L1CT_DEBUG_MESSAGE(1, "running checks on gathered cool data");
  //m_watchdogChecks->CheckCoolData();
  //m_watchdogChecks->CheckISStatus(m_isDict);
}


//------------------------------------------------------------------------------
