'''Definition of Translator interface'''

from abc import ABCMeta, abstractmethod
from typing import Optional

from .. import model


class Translator(metaclass=ABCMeta):
    """Abstract base class for all translators.
    """

    @abstractmethod
    def write(self, rep: model.BeamSpotRepresentation):
        """Translate representation to external format.

        Parameters
        ----------
        rep : BeamSpotRepresentation
            In-memory beam spot representation.
        """
        pass

    @abstractmethod
    def read(self) -> Optional[model.BeamSpotRepresentation]:
        """Translate external format to in-memory representation.

        Returns
        -------
        Instance of BeamSpotRepresentation class, can return None
        on failure.

        Raises
        ------
        Exception can be raised for some critical failure modes.
        """
        pass
