'''Library of fly-weight subclasses for COOL DB translation'''

from .db_tran import DBTranslator, ConnParams


HLT_CONN = ConnParams(connStr='COOLONL_INDET/CONDBR2',
                      folder='/Indet/Onl/Beampos',
                      tag='IndetBeamposOnl-HLT-UPD1-001-00')

MON_CONN = ConnParams(connStr='COOLONL_INDET/CONDBR2',
                      folder='/Indet/Onl/Beampos',
                      tag='IndetBeamposOnl-LiveMon-001-00')

BUNCH_CONN = ConnParams(connStr='COOLONL_INDET/CONDBR2',
                        folder='/Indet/Onl/BeamposPerBunch',
                        tag='IndetBeamposOnlPerBunch-LiveMon-001-00')

OFFLINE_CONN = ConnParams(connStr='COOLOFL_INDET/CONDBR2',
                          folder='/Indet/Beampos',
                          tag='IndetBeampos-ES1-UPD2')


def HLTTranslator(**kwargs):
    """Make read-only translator instance for HLT COOL conditions.
    """
    return DBTranslator(HLT_CONN,
                        readOnly=True,
                        create=False,
                        openIOV=True,
                        zeroBCID=-1,
                        **kwargs)


def HLTUpdTranslator(**kwargs):
    """Make update-capable translator instance for HLT COOL conditions.

    Not for regular operations (updates should normally go through CTP).
    """
    return DBTranslator(HLT_CONN,
                        readOnly=False,
                        create=True,
                        openIOV=True,
                        zeroBCID=-1,
                        **kwargs)


def BunchTranslator(**kwargs):
    """Make read-write translator instance for per-bunch COOL conditions.
    """
    return DBTranslator(BUNCH_CONN,
                        readOnly=False,
                        create=False,
                        openIOV=False,
                        allowedBCIDs=set(range(0, 3564)),
                        **kwargs)


def MonitoringTranslator(**kwargs):
    """Make read-write translator instance for monitoring COOL conditions.
    """
    return DBTranslator(MON_CONN,
                        readOnly=False,
                        create=False,
                        openIOV=False,
                        zeroBCID=-1,
                        **kwargs)


def OfflineCOOLTranslator(readOnly=True, create=False, **kwargs):
    """Make read-only translator instance for off-line COOL conditions.
    """
    return DBTranslator(OFFLINE_CONN,
                        readOnly=readOnly,
                        create=create,
                        openIOV=False,
                        zeroBCID=-1,
                        **kwargs)
