//
// @author Rob Mina, robmina@stanford.edu
// @version $Id$
//

#include <string>
#include <vector>

#include "BeamSpotUtils/ISWriter.h"
#include "BeamSpotUtils/BeamSpotParameters.h"

#include <boost/python.hpp>

using namespace boost::python;

// take the unwrapped list of beamspot parameters from python
// produce the std::vector object needed by ISWriter
// finally, call ISWriter.write()
bool writePylist2IS(list& bspars, 
                    std::string isObjName, 
                    std::string isByBunchObjName)
{
  std::vector<daq::BeamSpotParameters> bsParsVec;
  const int n = len(bspars);
  int i = -1;
  unsigned temp_status = 4;
  float temp_posX = -999.     , temp_posY = -999.     , temp_posZ = -999.      ,
        temp_sigmaX = -999.   , temp_sigmaY = -999.   , temp_sigmaZ = -999.    ,
        temp_tiltX = -999.    , temp_tiltY = -999.    , temp_sigmaXY = -999.   ,
        temp_posXErr = -999.  , temp_posYErr = -999.  , temp_posZErr = -999.   ,
        temp_sigmaXErr = -999., temp_sigmaYErr = -999., temp_sigmaZErr = -999. ,
        temp_tiltXErr = -999. , temp_tiltYErr = -999. , temp_sigmaXYErr = -999.;

  do {
    if ( ++i < n ) temp_status     = extract<unsigned>(bspars[i]); else break;
    if ( ++i < n ) temp_posX       = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_posY       = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_posZ       = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaX     = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaY     = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaZ     = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_tiltX      = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_tiltY      = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaXY    = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_posXErr    = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_posYErr    = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_posZErr    = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaXErr  = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaYErr  = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaZErr  = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_tiltXErr   = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_tiltYErr   = extract<float> (bspars[i]); else break;
    if ( ++i < n ) temp_sigmaXYErr = extract<float> (bspars[i]); else break;
    daq::BeamSpotParameters bsp ( temp_status,
                                  temp_posX,
                                  temp_posY,
                                  temp_posZ,
                                  temp_sigmaX,
                                  temp_sigmaY,
                                  temp_sigmaZ,
                                  temp_tiltX,
                                  temp_tiltY,
                                  temp_sigmaXY,
                                  temp_posXErr,
                                  temp_posYErr,
                                  temp_posZErr,
                                  temp_sigmaXErr,
                                  temp_sigmaYErr,
                                  temp_sigmaZErr,
                                  temp_tiltXErr,
                                  temp_tiltYErr,
                                  temp_sigmaXYErr ) ;
    bsParsVec.push_back(bsp);
  } while (true);

  daq::beamspot::ISWriter writer( isObjName, isByBunchObjName );
  return writer.write( bsParsVec );
}

BOOST_PYTHON_MODULE(libISWriterPyWrapper)
{
  using namespace boost::python;
  def("writePylist2IS",writePylist2IS);
}
