
/**
 * This is an example application taking command-line arguments and using ROOT classes.
 */
 
#include <iostream>
#include <sstream>
#include <string>
#include <cstdlib>

#include "TSystem.h"
#include "TROOT.h"
#include "TStyle.h"
#include "TCanvas.h"
#include "TF1.h"
#include "TF2.h"
#include "TFile.h"
#include "TGraph.h"
#include "TH1.h"
#include "TH2.h"
#include "TString.h"
#include "TLatex.h"
#include "TColor.h"
#include "TGaxis.h"
#include "TMath.h"
#include "TText.h"
#include "TPaveText.h"
#include "TRandom3.h"

//#include "AtlasStyle.h"


namespace {
// DEFAULTS WILL BE OVERWRITTEN DURING EXECUTION!!!
TString plotTag   = "SiTrack";
TString plotExt   = ".png";
TString plotType  = TString("_") + plotTag + plotExt;
//TString fileDir   = "/u1/jcogan/";
//TString fileName  = "testMakePretty.root";
//TString plotDir   = "HistogrammingHLT/Top-LVL2-L2-Segment-1-Gatherer/EXPERT/T2VertexBeamSpot_SiTrack/";
TString plotDir   = "HistogrammingHLT/Top-LVL2-L2-Segment-1-Gatherer/EXPERT/T2VertexBeamSpot_SiTrack/";
TString outputDir = "hopefullyOneFile/";
Double_t factor   = 15;
Int_t lhcFill     = 1121;
Int_t runNum      = 155678;
TString date      = Form("Mon May 24 2010 (LHC Fill %d)", lhcFill);
TString com       = "#sqrt{s} = 7 TeV";
TString BigLabel;//  = Form("Run %d                ",runNum);
Int_t nTrackPerPV = 6;
Float_t pTCutMev  = 1000.0;
const bool atlasLabel   = true;
const bool preliminary  = true;
Float_t intLumi   = 1070; 
//Int_t NRebinZ     = 1;
Float_t NRMS      = 1.5;
//Float_t ibcid     = -10.;
//Float_t fbcid     = 2300.;

TFile *f;

void usage( const std::string& commandName, int exitCode );
int strToInt( const std::string& str);
float strToFloat( const std::string& str);
TH1* safeGet(TString name);
void titleEmptyHist( TNamed* redo );
//void ATLASLabel(Double_t x,Double_t y,bool preliminary=false, bool solidBackground=false);

void SetAtlasStyle();
TStyle* AtlasStyle();
void formatTPaveText( TPaveText *pt
                    , const bool solidBackground=false
                    );

TPaveText*  slapAFitLabelOnIt( const int    argc
                             , TF1 *fit
                             , const char * names[]
                             , const char * fitName
                             , const char * units="mm"
                             );

TPaveText** slapALabelOnIt  ( const float x1=.17
                            , const float y1=.6
                            , const float scale=1
                            , const bool externalLabel=atlasLabel
                            , const bool solidBackground=false
                            , const bool isVertices=false
                            );

#if 0
void myText                 ( Double_t x
                            , Double_t y
                            , const char *text
                            , Double_t tsize=0.05
                            , Color_t color=1
                            );
#endif

void setPrettyPalette(Int_t NCont = 255);
int passPlot(const std::string& saxis, const bool useZoom);
//int relPlot(const std::string& saxis, Float_t NRMS);
int eventStats();
int d0phiPassPlot();
int tilt(const std::string& saxis);
int vertexPosVsBCID(const std::string& saxis);
int numVertexVsBCID();
int xVsY();
int split(const std::string& saxis);

class CmdLineArgs {
public:
    CmdLineArgs( int argc, char *argv[] );
  
  
    std::string fileLocation;
    int runNumber;
    std::string commonStr;
    std::string date;
    std::string outputDir;
    std::string plotTag;
    std::string plotExt;
    std::string plotDir;
    float factor;
    float intLumi;
    int lhcFill;
    int nTrackPerPV;
    float ptCutMev;
    float NRMS;
 
    std::string command;
};

} // unnamed namespace


int myMain( int argc, char *argv[] )
{
    CmdLineArgs arg( argc, argv );

    plotTag   = arg.plotTag;
    plotExt   = arg.plotExt;
    plotType  = TString("_") + plotTag + plotExt;
    f  = new TFile(arg.fileLocation.c_str(), "READ");
    plotDir   = arg.plotDir;
    factor    = arg.factor;
    lhcFill   = arg.lhcFill;
    runNum    = arg.runNumber;
    date      = arg.date;
    outputDir = arg.outputDir;
    com       = arg.commonStr;
    nTrackPerPV = arg.nTrackPerPV;
    pTCutMev  = arg.ptCutMev;
    intLumi   = arg.intLumi;
    NRMS      = arg.NRMS;

    BigLabel  = Form("Run %d",runNum);
    //BigLabel  = Form("Run %d                ",runNum);

    SetAtlasStyle(); 


    gStyle->SetOptStat(0);
    gStyle->SetOptFit(0);
    //===============================
    // Setup
    //===============================
    bool doZPassPlot     = true;
    bool doYPassPlot     = true;
    bool doXPassPlot     = true;
  
    bool doXRelPlot      = false;
    bool doYRelPlot      = false;
    bool doZRelPlot      = false;

    bool doTiltX         = true;
    bool doTiltY         = true;

    bool doSplitXPlot    = true;
    bool doSplitYPlot    = true;
    bool doSplitZPlot    = true;
  
    bool doXvsYPlot      = true;
    bool doEventStats    = false;
    bool dod0phiPassPlot = true;
  
  
    bool doNVertexBCID   = false;
    bool doZVertexBCID   = false;
    bool doYVertexBCID   = false;
    bool doXVertexBCID   = false;
  

    //===============================
    // Object Initialization
    //===============================
  
    //Don't think this is even needed
    //Utils::style();
  
    //===============================
    // X
    //===============================
    if( doXPassPlot ) passPlot("X", false);
    if( doYPassPlot ) passPlot("Y", false);
    if( doZPassPlot ) passPlot("Z", false);

    //===============================
    // X - X_nominal
    //===============================
    if (doXRelPlot) passPlot("X", true);
    if (doYRelPlot) passPlot("Y", true);
    if (doZRelPlot) passPlot("Z", true);
    //if (doXRelPlot) relPlot("X", NRMS);
    //if (doYRelPlot) relPlot("Y", NRMS);

    //===============================
    // d0phi
    //===============================
    if (dod0phiPassPlot) d0phiPassPlot();
  
    //===============================
    // TILT: X
    //===============================
    if (doTiltX) tilt("X");
    if (doTiltY) tilt("Y");

    //===============================
    // Vertex vs BCID
    //===============================
    if (doXVertexBCID) vertexPosVsBCID("X");
    if (doYVertexBCID) vertexPosVsBCID("Y");
    if (doZVertexBCID) vertexPosVsBCID("Z");
    if (doNVertexBCID) numVertexVsBCID();

    //===============================
    // Event Statistics (cut flow)
    //===============================

    if (doEventStats) eventStats();
    //===============================
    // Y vs X
    //===============================
    if (doXvsYPlot) xVsY();

    //===============================
    // Split Vertex X
    //===============================
    if (doSplitXPlot) split("X");
    if (doSplitYPlot) split("Y");
    if (doSplitZPlot) split("Z");
    
    return 0;
}

int main( int argc, char *argv[] ) {
  return myMain(argc, argv);
}


// ****************************************************************************************
// Internally-linked Functions
// ****************************************************************************************

namespace {
    //cmt wins I just copied and pasted in
    void SetAtlasStyle()
    {
        static TStyle* atlasStyle = 0;
        std::cout << "\nApplying ATLAS style settings...\n" << std::endl ;
        if ( atlasStyle==0 ) atlasStyle = AtlasStyle();
        gROOT->SetStyle("ATLAS");
        gROOT->ForceStyle();
    }
    
    TStyle* AtlasStyle() 
    {
        TStyle *atlasStyle = new TStyle("ATLAS","Atlas style");
      
        // use plain black on white colors
        Int_t icol=0; // WHITE
        atlasStyle->SetFrameBorderMode(icol);
        atlasStyle->SetFrameFillColor(icol);
        atlasStyle->SetCanvasBorderMode(icol);
        atlasStyle->SetCanvasColor(icol);
        atlasStyle->SetPadBorderMode(icol);
        atlasStyle->SetPadColor(icol);
        atlasStyle->SetStatColor(icol);
        //atlasStyle->SetFillColor(icol); // don't use: white fill color for *all* objects
      
        // set the paper & margin sizes
        atlasStyle->SetPaperSize(20,26);
      
        // set margin sizes
        atlasStyle->SetPadTopMargin(0.05);
        atlasStyle->SetPadRightMargin(0.05);
        atlasStyle->SetPadBottomMargin(0.16);
        atlasStyle->SetPadLeftMargin(0.16);
      
        // set title offsets (for axis label)
        atlasStyle->SetTitleXOffset(1.4);
        atlasStyle->SetTitleYOffset(1.4);
      
        // use large fonts
        //Int_t font=72; // Helvetica italics
        Int_t font=42; // Helvetica
        Double_t tsize=0.05;
        atlasStyle->SetTextFont(font);
      
        atlasStyle->SetTextSize(tsize);
        atlasStyle->SetLabelFont(font,"x");
        atlasStyle->SetTitleFont(font,"x");
        atlasStyle->SetLabelFont(font,"y");
        atlasStyle->SetTitleFont(font,"y");
        atlasStyle->SetLabelFont(font,"z");
        atlasStyle->SetTitleFont(font,"z");
        
        atlasStyle->SetLabelSize(tsize,"x");
        atlasStyle->SetTitleSize(tsize,"x");
        atlasStyle->SetLabelSize(tsize,"y");
        atlasStyle->SetTitleSize(tsize,"y");
        atlasStyle->SetLabelSize(tsize,"z");
        atlasStyle->SetTitleSize(tsize,"z");
      
        // use bold lines and markers
        atlasStyle->SetMarkerStyle(20);
        atlasStyle->SetMarkerSize(1.2);
        atlasStyle->SetHistLineWidth(2.);
        atlasStyle->SetLineStyleString(2,"[12 12]"); // postscript dashes
      
        // get rid of X error bars 
        //atlasStyle->SetErrorX(0.001);
        // get rid of error bar caps
        atlasStyle->SetEndErrorSize(0.);
      
        // do not display any of the standard histogram decorations
        //atlasStyle->SetOptTitle(0);
      // JOSH FIX buwahahaha!
        atlasStyle->SetOptTitle(1);
        //atlasStyle->SetOptStat(1111);
        atlasStyle->SetOptStat(0);
        //atlasStyle->SetOptFit(1111);
        atlasStyle->SetOptFit(0);
      
        // put tick marks on top and RHS of plots
        atlasStyle->SetPadTickX(1);
        atlasStyle->SetPadTickY(1);
      
        return atlasStyle;
      
    }


int passPlot( const std::string& saxis
            , const bool useZoom
            ){

    TCanvas *intCanv = new TCanvas("intCanv");
    intCanv->SetTopMargin(0.06);
    const float nRMS = 0.7; //for fitting
    
    TString axis(saxis); 
    axis.ToUpper();
    if (axis.CompareTo("X") != 0  &&  
        axis.CompareTo("Y") != 0  &&
        axis.CompareTo("Z") != 0){

        std::cout << "handed bad value exiting!" << std::endl;
        delete intCanv;
        return 1;
    }

    if (axis == "Z")  TGaxis::SetMaxDigits(3);
    else              TGaxis::SetMaxDigits(4);

    TString name;
    if (useZoom)
        name = TString(plotDir + "Vertex" + axis + "ZoomPass");
    else
        name = TString(plotDir + "Vertex" + axis + "Pass");

    TH1F* h1 = (TH1F*) safeGet(name);

    axis.ToLower();
    h1->SetStats(kTRUE);
    //if (NRebin > 0)   h1->Rebin(NRebin);
    h1->SetFillColor(42);
    h1->GetXaxis()->SetLabelOffset(0.017);
    h1->GetYaxis()->SetTitleOffset(1.6);
    if (axis == "z")
        h1->GetYaxis()->SetDecimals(1);

    if (useZoom)
        h1->GetXaxis()->SetTitle("Vertex "+axis+"-"+axis+"_{nominal}"+" [mm]");
    else
        h1->GetXaxis()->SetTitle("Vertex " + axis +" [mm]");

    h1->GetYaxis()->SetTitle("Number of Vertices");

    h1->GetYaxis()->SetRangeUser(0,1.3*h1->GetMaximum());
    h1->Draw("");

    TPaveText *fText = 0;
    if (h1->GetEntries() < 1){
        titleEmptyHist(h1);
    } else {
        float m = h1->GetMean();
        float r = h1->GetRMS();

        h1->Fit("gaus", "QR","", m - nRMS*r, m + nRMS*r);
        TF1 *gFit = h1->GetFunction("gaus");
        gFit->SetLineColor(kRed);

	//Some edits to accomodate collaboration comments
	if (axis == "X" || axis == "x") {
	  h1->GetXaxis()->SetRangeUser(-0.5, 0.5);
	} else if (axis == "Y" || axis == "y") {
	  h1->GetXaxis()->SetRangeUser(0.5, 1.55);
	} else if (axis == "Z" || axis == "z") {
	  h1->GetXaxis()->SetRangeUser(-250, 250);
	  gFit->SetRange(-100, 100);
	} else {
	  h1->GetXaxis()->SetRangeUser( m - factor * r 
					, m + factor * r 
					);
	}
        
        
        const char *names[] = {"Mean", "Raw Width"};
        fText = slapAFitLabelOnIt( 2
                                 , gFit
                                 , names
                                 , "Gaussian"
                                 );
    }
    
    TPaveText **pts = slapALabelOnIt( .18
                                    , .65
                                    , 1
                                    , true
                                    , false
                                    , true
                                    );
    
    
    intCanv->Modified();
    if (useZoom)
        intCanv->SaveAs(outputDir + Form("Run%d_Vertex",runNum)  + axis + "Zoom" + plotType);
    else
        intCanv->SaveAs(outputDir + Form("Run%d_Vertex",runNum)  + axis + plotType);

    delete [] pts;
    if (fText) delete fText;
    delete intCanv;
    return 0;
}

void usage( const std::string& commandName, int exitCode )
{
    std::string message;
    message += "\n";
    message += "This program produces many online beamspot plots.l\n";
    message += "Use the python code to handle its inputs\n";
  
    std::string::size_type s = commandName.size();
    std::string::size_type p = commandName.rfind( '/', s );
    std::string shortName = commandName.substr( p+1, s );
  
    std::cout << "\n";
    std::cout << "Usage: " << shortName << " rootFile runNumber commonStr date outputDir plotTag plotExt plotDir factor intLumi lhcFill nTrackPerPV ptCutMev";
    std::cout << message << "\n";
    exit( exitCode );
}


CmdLineArgs::CmdLineArgs( int argc, char *argv[] )
{
    //std::cout << "parsing cmdline!\n";
    int expectedArgs = 15;
    command = argv[0];
    if( argc > expectedArgs )
    usage( command, 1 );
    else if( argc < expectedArgs )
    usage( command, 0 );
  
    fileLocation = argv[1];
    runNumber = strToInt(argv[2]);
    commonStr = argv[3];
    date      = argv[4];
    outputDir = argv[5];
    plotTag   = argv[6];
    plotExt   = argv[7];
    plotDir   = argv[8];
    factor    = strToFloat(argv[9]);
    intLumi   = strToFloat(argv[10]);
    lhcFill   = strToInt(argv[11]);
    nTrackPerPV = strToInt(argv[12]);
    ptCutMev  = strToFloat(argv[13]);
    NRMS      = strToFloat(argv[14]);

}

void titleEmptyHist(TNamed* redo){
    TString newTitle(redo->GetTitle());
    newTitle += "--Is Empty";
    redo->SetTitle(newTitle);
}

int strToInt(const std::string& str){
    std::istringstream n_stream(str);
    int intRep;
    n_stream >> intRep;
    return intRep;
}

float strToFloat(const std::string& str){
    std::istringstream n_stream(str);
    float floatRep;
    n_stream >> floatRep;
    return floatRep;
}

TH1* safeGet(TString name)
{
    TH1* h1 = (TH1*)f->Get(name); 
    if (!h1)
    {
        h1 = (TH1*)f->Get(name+"_runsummary");
        if (!h1)
        {
            std::cout << "Failed to find " << name <<"(_runsummary)";
            std::cout << std::endl;
            exit(-1);
        }

    }
    h1->SetTitle("");
    return h1;
}

TPaveText*  slapAFitLabelOnIt( const int    argc
                             , TF1 *fit
                             , const char * names[]
                             , const char * fitName
                             , const char * units
                             )
{

    const char * templateStr = "%s = %1.3f #pm%s %1.3f %s";
    float maxErr = .001;
    float error, param;
    const char * sMod;

    TPaveText *fText = new TPaveText(0.61, .75, .92, .75+.08*argc, "NDC");
    fText->AddText(Form("%s Fit:", fitName));

    if (strcmp(fitName, "Linear") == 0) {
        templateStr = "%s = %1.2f #pm%s %1.2f %s";
        maxErr = .010;
    }

    for (int i=0; i<argc; i++)
    {
        param = fit->GetParameter(i+1);
        error = fit->GetParError (i+1);

        std::cout << names[i] << " is " << param << std::endl;
        error = std::max(error, maxErr);        
        sMod = (error==maxErr) ? " <" : "";

        fText->AddText(Form(templateStr, names[i], param, sMod, error, units));
    }

    formatTPaveText(fText, true);
    fText->Draw();
    return fText;
}

void formatTPaveText( TPaveText *pt
                    , const bool solidBackground)
{
    pt->SetBorderSize(0);
    pt->SetFillColor(0);
    pt->SetFillStyle(solidBackground ? 1001 : 0);
    pt->SetTextFont(72);
    pt->SetTextAlign(12);
}

TPaveText** slapALabelOnIt( const float x1
                          , const float y1
                          , const float scale
                          , const bool externalLabel
                          , const bool solidBackground
                          , const bool isVertices
                          )
{
    const float nLines      = 3 + isVertices;
    const float lenLine     = .05 * scale;
    const float lenX        = .43 * scale;
    const float lenY        =  lenLine * (nLines) + lenLine*1.3;
    
    //const float fracYBottom = lenLine * nLines/(nLines + 1.0);
    const float middleDivide= y1 + lenLine * nLines;

    TPaveText *pt1 = new TPaveText( x1
                                  , middleDivide
                                  , x1 + lenX
                                  , y1 + lenY
                                  ,"NDC"
                                  );
    formatTPaveText(pt1, solidBackground);
    if (externalLabel)
    {
        if (preliminary)
        {
            pt1->AddText("ATLAS Operations");
        } else {
            pt1->AddText("ATLAS");
        }
    } else {
        pt1->AddText(BigLabel);

    }
    pt1->Draw();

    TPaveText *pt2 = new TPaveText( x1
                                  , y1
                                  , x1 + lenX
                                  , middleDivide 
                                  ,"NDC"
                                  );
    
    formatTPaveText(pt2, solidBackground);
    pt2->SetTextFont(42);
    pt2->AddText(date);
    pt2->AddText(Form("LHC Fill: %d", lhcFill));
    pt2->AddText(com);
    if (isVertices)
        pt2->AddText("Online Primary Vertex");
    //pt2->AddText(Form("N_{track} per vertex #geq %d", nTrackPerPV));
    //pt2->AddText(Form("p_{T} > %1.1lf GeV", (pTCutMev/1000.0)));
    //pt2->AddText(Form("|#eta| < 2.5"));
    //pt2->AddText(Form("Labeled Was Slapped"));
    pt2->Draw();

    TPaveText **pts = new TPaveText*[2];
    pts[0] = pt1;
    pts[1] = pt2;

    return pts;
}

#if 0
void myText(Double_t x,Double_t y,const char *text, Double_t tsize,Color_t color) {
    TLatex l; 
    l.SetTextSize(tsize); 
    l.SetNDC();
    l.SetTextColor(color);
    l.DrawLatex(x,y,text);
}
#endif

void setPrettyPalette(Int_t NCont) {
    const Int_t NRGBs = 5;
    Double_t stops[NRGBs] = { 0.00, 0.34, 0.61, 0.84, 1.00 };
    Double_t red[NRGBs]   = { 0.00, 0.00, 0.87, 1.00, 0.51 };
    Double_t green[NRGBs] = { 0.00, 0.81, 1.00, 0.20, 0.00 };
    Double_t blue[NRGBs]  = { 0.51, 1.00, 0.12, 0.00, 0.00 };
    TColor::CreateGradientColorTable(NRGBs, stops, red, green, blue, NCont);
    gStyle->SetNumberContours(NCont);
}

int eventStats(){
    TCanvas *intCanv = new TCanvas("intCanv");
    intCanv->SetLeftMargin(0.13);
    intCanv->SetRightMargin(0.13);
    intCanv->SetTopMargin(0.07);
    intCanv->SetBottomMargin(0.12);

    TString name(plotDir + "EventStatistics");
    TH1F* h1_stats = (TH1F*) safeGet(name);

    std::string statNames[10] = {"All Input","Has TE","Has ROI","Has tracks","Has seed track","Enough Tracks","Has Cluster","Has Vertex","Accepted Vtx"};
    //if (h1_stats) {
    for (int i = 1; i<h1_stats->GetNbinsX()+1; i++) 
    {
        h1_stats->GetXaxis()->SetBinLabel(i,statNames[i-1].c_str());
    }
    h1_stats->SetMinimum(0.0);
    h1_stats->SetStats(kFALSE);
    h1_stats->SetFillColor(kYellow);
    h1_stats->GetYaxis()->SetTitleOffset(1.4);
    h1_stats->GetYaxis()->SetTitle("Fraction Accepted By Selection");
    //h1_stats->SetMinimum(0);
    //}
    if (h1_stats->GetEntries() >= 1) h1_stats->Scale(1./h1_stats->GetBinContent(1));
    else titleEmptyHist(h1_stats);

//    TH1F* h_reject = new TH1F(*h1_stats);
//    for (int i=0; i <= h1_stats->GetNbinsX(); ++i)
//    {
//        h_reject->SetBinContent(i, 1-h1_stats->GetBinContent(i));
//    }
//    h1_stats = h_reject;
    h1_stats->GetYaxis()->SetRangeUser(1E-5, 1.0);
    h1_stats->GetYaxis()->SetRangeUser(.9, 1.01);
    h1_stats->Draw();
    h1_stats->Draw("same");
    //h1_stats->Draw("textsame");
    //intCanv->SetLogy(1);
    intCanv->Modified();

    TPaveText **pts = slapALabelOnIt(.17, .55);
       
    // Resolution vs ntrk
    intCanv->SaveAs(outputDir + Form("Run%d_EventStats",runNum) + plotType);
    delete [] pts;
    delete intCanv;
    return 0;
}

int d0phiPassPlot(){
    TCanvas *intCanv = new TCanvas("intCanv");

    TString name(plotDir + "TrackPhiPass_vs_TrackD0Pass");
    TH1F* h1_d0phi = (TH1F*) safeGet(name);

    h1_d0phi->SetStats(kTRUE);
    h1_d0phi->SetFillColor(42);
    h1_d0phi->GetXaxis()->SetLabelOffset(0.017);
    h1_d0phi->GetYaxis()->SetTitleOffset(1.5);
    h1_d0phi->GetXaxis()->SetTitle("Online Track #phi");
    h1_d0phi->GetYaxis()->SetTitle("Online Track d_{0} [mm]");
    h1_d0phi->GetYaxis()->SetRangeUser(-1.5,2.0);
    h1_d0phi->GetYaxis()->SetDecimals(1);
    h1_d0phi->SetMarkerSize(1.0);
    h1_d0phi->Draw("");
  
    gPad->SetGridx(kTRUE);
    gPad->SetGridy(kTRUE);
  
    TPaveText *fText = 0;

    if (h1_d0phi->GetEntries() < 1) {
        titleEmptyHist(h1_d0phi);
    } else {
        TF1 *sineFit = new TF1( "sineFit"
                              , "[2]*cos(x) - [1]*sin(x)"
                              , -1. * TMath::Pi()-0.1
                              ,       TMath::Pi()+0.1
                              );

        h1_d0phi->Fit(sineFit);
        h1_d0phi->GetFunction("sineFit")->SetLineColor(kRed);
        //sineFit->Draw("same");
        const char *names[] = {"x", "y"};
        fText = slapAFitLabelOnIt(2, sineFit, names, "Sinusoid");
    }
  
    TPaveText **pts = slapALabelOnIt( .2, .7);

    gPad->SetGridx(kFALSE);
    gPad->SetGridy(kFALSE);

    intCanv->Modified();
    intCanv->SaveAs(outputDir + Form("Run%d_TrackD0Phi",runNum) + plotType);
  
    delete [] pts;
    if (fText) delete fText;
    delete intCanv;
    return 0;
}

int tilt(const std::string& saxis){
    TCanvas *intCanv = new TCanvas("intCanv");
    intCanv->SetTopMargin(0.06);

    TString axis(saxis); 
    axis.ToUpper();
    if (axis.CompareTo("X") != 0  &&  
        axis.CompareTo("Y") != 0){
    std::cout << "handed bad value exiting!\n";
    delete intCanv;
    return 1;
    }

    TString name( plotDir + "VertexZPass_vs_Vertex" 
                + axis + "Pass"
                );
    //TProfile *h_axisVsZ = (TProfile*) safeGet(name);
    TH1D *h_axisVsZ = (TH1D*) safeGet(name);

    axis.ToLower();
    h_axisVsZ->SetXTitle("Vertex z [mm]");
    h_axisVsZ->SetYTitle("Vertex " + axis + " [mm]");
    h_axisVsZ->GetYaxis()->SetTitleOffset(1.6);
    h_axisVsZ->GetYaxis()->SetDecimals(3);
    h_axisVsZ->GetYaxis()->SetNoExponent(1);
  
    TPaveText *fText = 0;
    TF1 *tiltFit     = 0;
  
    if (h_axisVsZ->GetEntries() < 1)
    {
        titleEmptyHist(h_axisVsZ);
    } else {
        tiltFit = new TF1("tiltFit","[0] +  [1]*x/1000000", -20, 20);
        h_axisVsZ->Fit(tiltFit);
        h_axisVsZ->GetFunction("tiltFit")->SetLineColor(kRed);

    }

    float c   = h_axisVsZ->GetBinContent(h_axisVsZ->FindBin(0));
    float low  = c - 1.5 * 40*80.0/1E6;
    float high = c + 3.0 * 40*80.0/1E6;
    h_axisVsZ->GetYaxis()->SetRangeUser(low,high);
    h_axisVsZ->GetXaxis()->SetRangeUser(-85,85);
    h_axisVsZ->Draw(); 

    TPaveText **pts = slapALabelOnIt( .2
                                    , .68
                                    );
    if (tiltFit)
    {
        const char *names[] = {"Slope"};
        fText = slapAFitLabelOnIt(1, tiltFit, names, "Linear", "#murad");
    }
  
    intCanv->Modified();
    intCanv->SaveAs(outputDir + Form("Run%d_Tilt", runNum) + axis + plotType);
    delete [] pts;
    if (fText) delete fText;
    delete tiltFit;
    delete intCanv;
    return 0;
}

int vertexPosVsBCID(const std::string& saxis){
    TCanvas *intCanv = new TCanvas("intCanv");

    TString axis(saxis); 
    axis.ToUpper();
    if (axis.CompareTo("X") != 0 &&  
        axis.CompareTo("Y") != 0 && 
        axis.CompareTo("Z") != 0 && 
        axis.CompareTo("N") != 0){
    std::cout << "handed bad value exiting!\n";
    delete intCanv;
    return 1;
    }

    TString name;
    if (axis.CompareTo("N") == 0)
        name = TString(plotDir + "BCID_vs_NvtxPass");
    else
        name = TString(plotDir + "BCID_vs_Vertex" + axis + "Pass");


    axis.ToLower();
    TH1F* h1 = (TH1F*) safeGet(name);
    //TH1F* h1 = (TH1F*)f->Get(name);
    //if (!h1)  h1 = new TH1F("h1", "Histogram *not* found in file", 1, 0,1);

    std::vector<float> vy;
    std::vector<float> vx;
    float c;
    for (int i=0; i < h1->GetNbinsX(); i++){
    c = h1->GetBinContent(i);
    if (c == 0) continue;

    vy.push_back(c);
    vx.push_back(h1->GetBinCenter(i));
    } 

    TGraph *g1 = new TGraph( std::max(1,(int) vx.size()) );
    g1->SetTitle( h1->GetTitle() );

    for (unsigned int i=0; i < vx.size(); i++){
    g1->SetPoint(i, vx.at(i), vy.at(i));
    }
  
    g1->GetXaxis()->SetTitle("Bunch Crossing Identifier (BCID)");
    g1->GetYaxis()->SetTitle("Average " + axis + " Vertex Position [mm]");
    g1->GetYaxis()->SetTitleOffset(1.6);
    g1->SetMarkerSize(1.4);
    g1->SetMarkerStyle(20);

    if (vx.size() < 1)    titleEmptyHist(g1);
  
    g1->Draw("ap"); 

//  h1->SetStats(kFALSE);
//  h1->GetXaxis()->SetRangeUser(ibcid,fbcid);
//  h1->GetYaxis()->SetRangeUser(-2.95,-2.6);
  
    TPaveText **pts = slapALabelOnIt();
  
    intCanv->SaveAs(outputDir + Form("Run%d_Vertex",runNum) +axis+"_BCID" + plotType);
    delete [] pts;
    delete intCanv;
    return 0;
}

int numVertexVsBCID(){ return vertexPosVsBCID("N"); }

int xVsY(){
    TCanvas *intCanv = new TCanvas("intCanv");

    TString name(plotDir + "VertexXPass_vs_VertexYPass");
    TH2F* h2_XvsY = (TH2F*) safeGet(name);

    if (h2_XvsY->GetEntries() < 1.0){
    titleEmptyHist(h2_XvsY);
    }
    setPrettyPalette();
    intCanv->SetCanvasSize(652,579);
    intCanv->SetRightMargin(0.13);
    h2_XvsY->SetStats(kTRUE);
    h2_XvsY->GetXaxis()->SetLabelOffset(0.017);
    h2_XvsY->GetYaxis()->SetTitleOffset(1.5);
    h2_XvsY->GetXaxis()->SetTitle("Vertex x [mm]");
    h2_XvsY->GetYaxis()->SetTitle("Vertex y [mm]");
    h2_XvsY->GetYaxis()->SetDecimals(1);
    if (h2_XvsY->GetEntries() >= 1) gPad->SetLogz(kTRUE);
    h2_XvsY->Draw("colz");
  
    TPaveText **pts = slapALabelOnIt(.20, .20, .8, true, true, true);
  
    intCanv->Modified();
    intCanv->SaveAs(outputDir + Form("Run%d_Vertex_XvsY", runNum) + plotType);

    delete [] pts;
    delete intCanv;
    return 0;
}

int split(const std::string& saxis){
    TCanvas *intCanv = new TCanvas("intCanv");

    TString axis(saxis); 
    axis.ToUpper();
    if (axis.CompareTo("X") != 0 &&  
        axis.CompareTo("Y") != 0 && 
        axis.CompareTo("Z") != 0 ){

        std::cout << "handed bad value exiting!\n";
        delete intCanv;
        return 1;
    }

    TString name( plotDir + "SplitVertexDNTrksPass_vs_SplitVertexD"
                + axis +"Pass");
    axis.ToLower();
    TH2F* h2_XvsY = (TH2F*) safeGet(name);
    setPrettyPalette();
    intCanv->SetCanvasSize(652,579);
    intCanv->SetRightMargin(0.13);
    h2_XvsY->SetStats(kTRUE);
    h2_XvsY->GetXaxis()->SetLabelOffset(0.017);
    h2_XvsY->GetXaxis()->SetTitle("Average Number of Tracks per Half Vertex");

    h2_XvsY->GetYaxis()->SetTitleOffset(1.3);
    h2_XvsY->GetYaxis()->SetDecimals(1);
    h2_XvsY->GetYaxis()->SetTitle("#Delta "+axis+" Between Half Vertices [mm]");
    gPad->SetLogz(kTRUE);

    if (h2_XvsY->GetEntries() < 1)  {
        gPad->SetLogz(kFALSE);
        titleEmptyHist(h2_XvsY);
    }

    h2_XvsY->Draw("colz");
    TPaveText **pts = slapALabelOnIt(.50, .75, .75);

    delete [] pts;
    
    intCanv->SaveAs(outputDir + Form("Run%d_Vertex_SplitDelta"+axis, runNum) + plotType);
    delete intCanv;
    return 0;
}

#if 0
  void makePrettyBeamSpotPlots(char * fileLocation,
			       char * runNumber,
			       char * commonStr,
			       char * date,
			       char * outputDir,
			       char * plotTag,
			       char * plotExt,
			       char * plotDir,
			       char * factor,
			       char * intLumi,
			       char * lhcFill,
			       char * nTrackPerPV,
			       char * ptCutMev,
			       char * NRMS
			       ) {

    int argc = 15;
    char * argv[15] = {0,
		       fileLocation,
		       runNumber,
		       commonStr,
		       date,
		       outputDir,
		       plotTag,
		       plotExt,
		       plotDir,
		       factor,
		       intLumi,
		       lhcFill,
		       nTrackPerPV,
		       ptCutMev,
		       NRMS};
    
    myMain(argc, argv);
}
#endif

} // unnamed namespace
