#!/usr/bin/env tdaq_python

"""Unit test for BeamSpotUtils.proc_run module
"""

import logging
import unittest

from BeamSpotUtils.proc_run import SingleRun, SingleRunConfig
from BeamSpotUtils.model import BeamSpotRepresentation, BSSnapshot, STATUS_OK


class TestProcRun(unittest.TestCase):
    """Test case for BeamSpotUtils.proc_run module.
    """

    def setUp(self):
        logging.basicConfig(level=logging.DEBUG)
        self.logger = logging.getLogger("proc_run")

    def tearDown(self):
        pass

    def _makeProcRun(self, config=None, bscalc_vertex=None, bscalc_track=None,
                     run_number=None, emscan=None, ready4physics=None, output_dir=None):
        return SingleRun(
            config=config,
            bscalc_vertex=bscalc_vertex,
            bscalc_track=bscalc_track,
            run_number=run_number,
            emscan=emscan,
            ready4physics=ready4physics,
            output_dir=output_dir,
        )

    def test_choseBS(self):
        """Test for _choseBS method"""

        bsVtxEmpty = BeamSpotRepresentation()
        bsVtx = BeamSpotRepresentation([BSSnapshot(status=STATUS_OK)])
        bsTrkEmpty = BeamSpotRepresentation()
        bsTrk = BeamSpotRepresentation([BSSnapshot(status=STATUS_OK)])

        config = SingleRunConfig()
        procRun = self._makeProcRun(config=config)

        # if second is None or empty then first is returned
        bs = procRun._choseBS(bsVtxEmpty, bsTrkEmpty)
        self.assertIs(bs, bsVtxEmpty)
        bs = procRun._choseBS(None, bsTrkEmpty)
        self.assertIsNone(bs)
        bs = procRun._choseBS(bsVtxEmpty, None)
        self.assertIs(bs, bsVtxEmpty)
        bs = procRun._choseBS(bsVtx, None)
        self.assertIs(bs, bsVtx)
        bs = procRun._choseBS(None, None)
        self.assertIsNone(bs)

        # if first is non-empty then it is always returned
        bs = procRun._choseBS(bsVtx, None)
        self.assertIs(bs, bsVtx)
        bs = procRun._choseBS(bsVtx, bsTrkEmpty)
        self.assertIs(bs, bsVtx)
        bs = procRun._choseBS(bsVtx, bsTrk)
        self.assertIs(bs, bsVtx)

        # second is returned when it's not empty but first is
        bs = procRun._choseBS(bsVtxEmpty, bsTrk)
        self.assertIs(bs, bsTrk)
        bs = procRun._choseBS(None, bsTrk)
        self.assertIs(bs, bsTrk)


if __name__ == "__main__":
    unittest.main()
