//
// ROOT macro for running root batch jobs on proxyHist and print out
// the standard plots into an output psotscript file
// Will use the DBproxy log file name to derive output file names
// e.g. logfile = "[dir]/ARealName.out"
// will result in
//      output  = proxyHist_ARealName.ps 
//
#include "../DaqDbProxyUtils/proxyHist.h" 

void batchPlot( const char* fileName = 
                "data/EF-Segment-07-rack-Y04-06D2-dbproxy_pc-tdq-lfs-30.cern.ch_1215688464.out" ) 
{
  //***** Change the following input info for each job ******
  int version = 0;
  int maskIP  = 0xa920000; 
  //***** End of input info for each job ********************

  //
  //  Construct plot title label and file name from original log file name
  //
  string pname(120,' ');
  string Hfile(120,' ');
  proxyHist* uHist = new proxyHist();
  // uHist->SetTimeMax(3.);  // manually control time history time max
  uHist->OutputName(fileName,pname);
  Hfile = "proxyHist_" + pname + ".ps";
  cout << "Output plot file: " << Hfile << endl;
  //
  // Run log analysis to generate histograms 
  //
  uHist->execute(fileName,version,maskIP);
  //
  // Load the plotting macro 
  //
  gROOT->LoadMacro("./plotProxyHist.C");
  //
  // Executing the plotting macro for 4 standard pages
  // to save the plots in the output .ps file
  //
  int maxpage = 4;
  for(int i=0;i<maxpage;i++) {  
    char* pFile = &Hfile.c_str();
    plotProxyHist(i,pname.c_str());
    TCanvas* c0 = gROOT->GetListOfCanvases()->FindObject("c0");  
    string pfstr = Hfile; 
    if(i==0)              { pfstr = Hfile+"("; }
    else if(i==maxpage-1) { pfstr = Hfile+")"; }
    char* pfchar = &pfstr.c_str();
    c0->Print(pfchar);
  }
}
