//
// ROOT macro to plot proxyHist histograms  
//
//  Input argument
//  option = 0   2x3 summary plots
//         = 1   2x3 transaction delta-time by type  
//         = 2   1x2 Transaction density and proxy busy time history
//         = 3   1x1 Transaction bytes time history
//  title  = Global title (e.g. dataset name, test run name etc) 
//           [this is optional. Can also just call plotRackHist(option) ] 
//  
void plotProxyHist(int option, char* dtitle){ 

  gROOT->Reset();
  //
  // Check title argument decide if extra top banner needed
  // and setup Canvas/pad 
  // 
  string stitle=std::string(dtitle);

  TCanvas *c0;
  TPad    *c2;
  if(stitle.length()==0) { 
    c0 = new TCanvas("c0", "DBproxy Tests", 
                            200, 100, 750, 600 ) ; 
    c0->cd();
    c0->Range(0,0,1,1);
    c2 = new TPad("pad","pad",0.001,0.001,0.999,0.999);
    c2->Draw();
  } else { 
    c0 = new TCanvas("c0", "DBproxy Tests", 
                            200, 100, 750, 640 ) ;  
    c0->cd();
    c0->Range(0,0,1,1);
    TPad* c1 = new TPad("titlepad","title area",0.005,0.925,0.995,0.995);
    c1->Draw(); 
    c1->cd(); 
    TPaveText* pt = new TPaveText(0.15,0.1,0.85,0.9,"br");
    pt->SetFillColor(5);
    TText *t1 = new TText();
    t1 = pt->AddText(0.5,0.85,dtitle);
    t1->SetTextColor(28);
    t1->SetTextAlign(22);
    pt->Draw();
    c0->cd();
    TPad* c2 = new TPad("disppad","display area",0.005,0.005,0.995,0.925);
    c2->Draw();            
  }
  c2->cd();

  //  
  // 2x3 plots of summary plots  
  // 
  if(option==0) { 

    gStyle->SetOptStat(0);
    gStyle->SetStatW(0.3);
    gStyle->SetStatH(0.3);
    gStyle->SetOptTitle(0);
    gStyle->SetOptLogy(0);
        
    gStyle->SetPadBottomMargin(0.1);
    gStyle->SetPadLeftMargin(0.15);
    gStyle->SetPadRightMargin(0.03);

    c2->Divide(3,2,0.001,0.001);

    // connections per node 
    c2->cd(1);
    connx_vs_node->SetFillColor(4);  
    connx_vs_node->SetYTitle("Connections");
    connx_vs_node->SetXTitle("Node Index");
    dispnorm(connx_vs_node);
    connx_vs_node->Draw("box"); 
    gStyle->SetOptLogy(0);

    // Messages per node 
    c2->cd(4);
    nmsg_vs_node->SetFillColor(4);
    nmsg_vs_node->SetYTitle("Messages");
    nmsg_vs_node->SetXTitle("Node Index");
    dispnorm(nmsg_vs_node);
    nmsg_vs_node->Draw("box");

    // bytes per node
    c2->cd(2);
    Allbytes_vs_node->SetFillColor(48);
    Allbytes_vs_node->SetYTitle("Query Size (kbytes)");
    Allbytes_vs_node->SetXTitle("Node Index");
    dispnorm(Allbytes_vs_node);
    Cbytes_vs_node->SetFillColor(4);
    Qbytes_vs_node->SetFillColor(2);
    Allbytes_vs_node->SetTitleOffset(1.32,"Y");
    Allbytes_vs_node->Draw("box");
    Cbytes_vs_node->Draw("box,SAME");
    Qbytes_vs_node->Draw("box,SAME");
    TLegend* leg = new TLegend(0.65,0.85,0.97,0.99);
    leg->AddEntry(Allbytes_vs_node,"All bytes","f");
    leg->AddEntry(Cbytes_vs_node,"Cached Query","f");
    leg->AddEntry(Qbytes_vs_node,"NonCached Query","f");
    leg->Draw();

    // detla time per node 
    c2->cd(5);
    Alldtime_vs_node->SetLineColor(0);
    Alldtime_vs_node->SetFillColor(7);
    Cdtime_vs_node->SetYTitle("Delta Time Sum (msec)");
    Cdtime_vs_node->SetXTitle("Node Index");
    dispnorm(Cdtime_vs_node);
    Cdtime_vs_node->SetFillColor(4);
    Qdtime_vs_node->SetFillColor(2);
    Cdtime_vs_node->Draw("box");
    Qdtime_vs_node->Draw("box,SAME");

    // delta time vs bytes 
    // (Jul/08 - use the max byte transfer time reference hist) 
    c2->cd(3);
    dTime_vs_Cbytem->SetFillColor(4);
    dTime_vs_Cbytem->SetXTitle("Query Size (kbytes)");
    dTime_vs_Cbytem->SetYTitle("Delta Time (msec)"); 
    dispnorm(dTime_vs_Cbytem);
    // limit max entries to suppress common popular entries
    // make single transactions visible
    dTime_vs_Cbytem->SetMaximum(7.); 
    dTime_vs_Cbytem->Draw("box");
    dTime_vs_Qbytem->SetFillColor(2);
    dTime_vs_Qbytem->SetMaximum(7.);
    dTime_vs_Qbytem->Draw("box,SAME");

    // Messages vs time
    c2->cd(6);
    gPad->SetLogy(1);
    int indcolor[5] = { 8, 4, 2, 7, 1 };
    int indorder[5] = { 0, 1, 2, 3, 4 };
    TH1F* h_indx[5];
    h_indx[0] = Default_th;
    h_indx[1] = Cache_th;
    h_indx[2] = NonCache_th;
    h_indx[3] = Ctor_th;
    h_indx[4] = Dtor_th;
    // Look for the maximum entry hist to plot it first
    float valmax = 0;
    int   indmax = 0;
    for(int j=0;j<5;j++) { 
      float hvmax = h_indx[j]->GetMaximum(); 
      if(hvmax>valmax) {
        valmax = hvmax;
        indmax = j;
      }
    }  
    if(indmax!=0) {
      indorder[0] = indmax; 
      indorder[indmax] = 0;
    }       
    for(int j=0;j<5;j++) {        
      TH1F* h_pntr = h_indx[indorder[j]];
      h_pntr->SetMinimum(0.1);
      h_pntr->SetLineColor(indcolor[indorder[j]]);
      h_pntr->SetLineWidth(1);
      if(j==0) { 
        dispnorm1(h_pntr);
        h_pntr->SetXTitle("Wall time (sec)");
        h_pntr->SetYTitle("Messages");
        h_pntr->Draw();
      } else { 
        h_pntr->Draw("SAME");
      }
    }
    
    TLegend* legx = new TLegend(0.77,0.83,0.99,0.99);
    legx->AddEntry(Default_th,"Default","l");
    legx->AddEntry(NonCache_th,"NonCache","l");
    legx->AddEntry(Cache_th,"Cache","l");
    legx->AddEntry(Ctor_th,"Ctor","l");
    legx->AddEntry(Dtor_th,"Dtor","l");
    legx->Draw();
    
  //  
  // 2x2 plots of individual command type delta times  
  //
  } else if(option==1) { 

    gStyle->SetPadBottomMargin(0.15);
    gStyle->SetPadTopMargin(0.1);
    gStyle->SetPadLeftMargin(0.1);
    gStyle->SetPadRightMargin(0.1);

    gStyle->SetOptLogy(1);
    gStyle->SetTitleW(0.6);
    gStyle->SetTitleH(0.08);
    gStyle->SetOptTitle(1);

    gStyle->SetOptStat(1110);
    gStyle->SetStatW(0.35);
    gStyle->SetStatH(0.25);
         
    c2->Divide(2,2,0.001,0.001);

    int hgroup[3] = { 3, 4, 1 };
    for(int i=0;i<4;i++) {
      TH1F* h_pntr; 
      c2->cd(i+1);
      if(i<3) { 
        int idh = hgroup[i];
        char hname[9]; 
        sprintf(hname,"dt_comm%02d",idh);
        h_pntr = (TH1F*)gDirectory->Get(hname); 
        h_pntr->SetXTitle("Delta Time (msec)");
     } else {
        h_pntr = (TH1F*)gDirectory->Get("dt_loggap");       
        h_pntr->SetXTitle("log10(Delta Time in msec)");
      }   
      h_pntr->SetMinimum(0.1);
      h_pntr->SetFillColor(4);
      h_pntr->SetTitleSize(0.06,"X");
      h_pntr->Draw();   
    }
    gStyle->SetOptLogy(0);

  //  
  // 1x2 time history plots  
  // 
  } else if(option==2) { 

    gStyle->SetOptLogy(0);
    gStyle->SetOptStat(0);
    gStyle->SetOptTitle(0);
        
    gStyle->SetPadBottomMargin(0.10);
    gStyle->SetPadLeftMargin(0.07);
    gStyle->SetPadRightMargin(0.03);

    c2->Divide(1,2,0.001,0.001);
    c2->cd(1);
    c2->Range(0,0,1,1); 

    dispnorm(NodeDflt_vs_time);
    NodeDflt_vs_time->SetTitleOffset(0.5,"Y"); 
    NodeDflt_vs_time->SetXTitle("Wall time (sec)");
    NodeDflt_vs_time->SetYTitle("Node Number");
    NodeDflt_vs_time->SetFillColor(8);
    NodeCache_vs_time->SetFillColor(4);    
    NodeNonC_vs_time->SetFillColor(2);    
    NodeDflt_vs_time->Draw("box");  
    NodeCache_vs_time->Draw("box,SAME");   
    NodeNonC_vs_time->Draw("box,SAME");   

    double xt    = NodeDflt_vs_time->GetXaxis()->GetXmin();   
    double ytmax = NodeDflt_vs_time->GetYaxis()->GetXmax();   
    double ytmin = NodeDflt_vs_time->GetYaxis()->GetXmin();
    double yt    = ytmax + 0.03*(ytmax-ytmin);   
    TText* title = new TText(xt,yt,"Transaction Count Time History");
    title->SetTextColor(28);
    title->SetTextSize(0.07);
    title->Draw();
         
    c2->cd(2);
    THStack* hs = new THStack("Busy_Stack","Busy contributions");
    Busy_Cache->SetFillColor(4);
    Busy_NonCache->SetFillColor(2);
    Busy_Default->SetFillColor(8);
    Busy_Ctor->SetFillColor(7);
    Proxy_Idle->SetFillColor(5);
    hs->Add(Busy_Cache);
    hs->Add(Busy_NonCache);
    hs->Add(Busy_Default);
    hs->Add(Busy_Ctor);
    hs->Add(Proxy_Idle);
    hs->Draw();    
    hs->GetXaxis()->SetTitle("Wall Time (sec)");
    hs->GetYaxis()->SetTitle("Proxy Busy Fraction (%)");
    hs->GetXaxis()->SetTitleOffset(0.8);
    hs->GetYaxis()->SetTitleOffset(0.5);
    hs->GetXaxis()->SetTitleSize(0.06);
    hs->GetYaxis()->SetTitleSize(0.06);   
    
    Proxy_Busy->SetFillColor(0);
    Proxy_Busy->SetLineColor(6);
    Proxy_Busy->Draw("SAME");
    TLegend* leg = new TLegend(0.88,0.60,0.999,0.95);
    leg->AddEntry(Proxy_Idle,"Idle","f");
    leg->AddEntry(Busy_Ctor,"Ctor","f");
    leg->AddEntry(Busy_Default,"Default","f");
    leg->AddEntry(Busy_NonCache,"NonCache","f");
    leg->AddEntry(Busy_Cache,"Cache","f");
    leg->Draw();
  //  
  // Single Byte time history plot  
  // 
  } else if(option==3) { 

    gStyle->SetOptLogy(0);
    gStyle->SetOptStat(0);
    gStyle->SetOptTitle(0);
        
    gStyle->SetPadBottomMargin(0.10);
    gStyle->SetPadLeftMargin(0.09);
    gStyle->SetPadRightMargin(0.03);

    c2->Divide(1,1,0.001,0.001);
    c2->cd(1);
    c2->Range(0,0,1,1); 

    NodeCbytes_vs_time->SetTitleOffset(1.0,"X"); 
    NodeCbytes_vs_time->SetTitleOffset(1.0,"Y"); 
    NodeCbytes_vs_time->SetTitleSize(0.04,"X"); 
    NodeCbytes_vs_time->SetTitleSize(0.04,"Y"); 
    NodeCbytes_vs_time->SetXTitle("Wall time (sec)");
    NodeCbytes_vs_time->SetYTitle("Node Number");
    NodeCbytes_vs_time->SetFillColor(4);    
    NodeQbytes_vs_time->SetFillColor(2);    
    NodeDbytes_vs_time->SetFillColor(8);
    NodeCbytes_vs_time->Draw("box,SAME");  
    NodeQbytes_vs_time->Draw("box,SAME");   
    NodeDbytes_vs_time->Draw("box,SAME");   

    double xtmin = NodeDbytes_vs_time->GetXaxis()->GetXmin();   
    double xtmax = NodeDbytes_vs_time->GetXaxis()->GetXmax();   
    double ytmax = NodeDbytes_vs_time->GetYaxis()->GetXmax();   
    double ytmin = NodeDbytes_vs_time->GetYaxis()->GetXmin();
    double yt    = ytmax + 0.03*(ytmax-ytmin);   
    TText* title = new TText(xtmin,yt,"Transfer Bytes Time History");
    title->SetTextColor(28);
    title->SetTextSize(0.04);
    title->Draw();

    int maxnode = int(ytmax);
    for(int i=1;i<maxnode;i++) {
      double yline = i;
      TLine* NodeLine = new TLine(xtmin,yline,xtmax,yline);
      NodeLine->SetLineColor(28);
      NodeLine->SetY1(yline);
      NodeLine->SetY2(yline);
      NodeLine->Draw("SAME");
    }
         
    TLegend* leg = new TLegend(0.88,0.80,0.999,0.95);
    leg->AddEntry(NodeDbytes_vs_time,"Default","f");
    leg->AddEntry(NodeCbytes_vs_time,"Cached","f");
    leg->AddEntry(NodeQbytes_vs_time,"NonCached","f");
    leg->Draw();
  }
}

//
// Standard plot axis offset/size
//
void dispnorm(TH2F* h2) {
  h2->SetTitleSize(0.06,"X");
  h2->SetTitleOffset(0.7,"X");
  h2->SetTitleSize(0.06,"Y");
  h2->SetTitleOffset(1.25,"Y"); 
}
void dispnorm1(TH1F* h1) {
  h1->SetTitleSize(0.06,"X");
  h1->SetTitleOffset(0.7,"X");
  h1->SetTitleSize(0.06,"Y");
  h1->SetTitleOffset(1.25,"Y"); 
}
//
// Allow dropping title argument
//
void plotProxyHist(int option){
  plotProxyHist(option,"");
} 
