/*
 * ExpertSystemInterface.h
 *
 *  Created on: May 28, 2013
 *      Author: avolio
 */

#ifndef DAQ_ES_EXPERTSYSTEMINTERFACE_H_
#define DAQ_ES_EXPERTSYSTEMINTERFACE_H_

#include <dvs/action.h>
#include <tmgr/tmresult.h>

#include <string>
#include <vector>
#include <memory>
#include <map>


namespace daq { namespace es {

struct Transition {
    std::string m_uid;
    std::string m_command;
    std::string m_source_state;
    std::string m_destination_state;
};

struct Command {
    std::string m_uid;
    std::string m_name;
    std::vector<std::string> m_arguments;
};

struct ParentInformation {
    bool m_exiting;
    bool m_transitioning;
    bool m_initializing;
    std::string m_fsm_state;
    Transition m_current_transition;
    Transition m_last_transition;
};

struct ProcessInfo {
    std::string m_status;
    std::string m_running_host;
    bool m_membership;
    bool m_restarting;
    bool m_not_responding;
    long long m_exit_code;
    long long m_exit_signal;
    long long m_info_time;
    ParentInformation m_parent_info;
};

struct StateInfo {
    std::string m_fsm_state;
    Transition m_last_transition;
    Transition m_current_transition;
    long long m_last_transition_time;
    Command m_last_command;
    bool m_internal_error;
    bool m_busy;
    long long m_info_time;
    ParentInformation m_parent_info;
};

struct ComponentTestInfo {
    std::string m_component_name;
    daq::tmgr::TestResult m_test_result;
    std::vector<daq::dvs::FailureAction> m_actions;
};

struct TestInfo {
    daq::tmgr::TestResult m_global_result;
    std::vector<std::shared_ptr<ComponentTestInfo>> m_components;
};

class ExpertSystemInterface {
    public:
        virtual ~ExpertSystemInterface() noexcept {;}

        virtual void applicationUpdate(const std::string& appName, const ProcessInfo& info) = 0;
        virtual void applicationUpdate(const std::string& appName, const StateInfo& info) = 0;
        virtual void applicationUpdate(const std::string& appName, const TestInfo& info) = 0;
        virtual void applicationUpdate(const std::map<std::string, StateInfo>& infoList) = 0;
        virtual void applicationUpdate(const std::map<std::string, ProcessInfo>& infoList) = 0;
        virtual void dbReloadInit() = 0;
        virtual void dbReloadDone() = 0;
        virtual void reset() = 0;
        virtual void transitionDone(const std::string& transitionName, const std::string& newState) = 0;
};

}}


#endif /* DAQ_ES_EXPERTSYSTEMINTERFACE_H_ */
