/*
 * RemoteES.h
 *
 *  Created on: May 28, 2013
 *      Author: avolio
 */

#ifndef DAQ_ES_REMOTEES_H_
#define DAQ_ES_REMOTEES_H_

#include "ExpertSystemInterface/ExpertSystemInterface.h"

#include <es/es.hh>
#include <ipc/partition.h>

#include <string>
#include <map>


namespace daq { namespace es {

class RemoteES : public ExpertSystemInterface {
    public:
        RemoteES(const std::string& partitionName, const std::string& esName);

        // throws daq::es::ESInterface::CannotNotify
        void applicationUpdate(const std::string& appName, const ProcessInfo& info) override;

        // throws daq::es::ESInterface::CannotNotify
        void applicationUpdate(const std::string& appName, const StateInfo& info) override;

        // throws daq::es::ESInterface::CannotNotify
        void applicationUpdate(const std::string& appName, const TestInfo& info) override;

        // throws daq::es::ESInterface::CannotNotify
        void applicationUpdate(const std::map<std::string, StateInfo>& infoList) override;

        // throws daq::es::ESInterface::CannotNotify
        void applicationUpdate(const std::map<std::string, ProcessInfo>& infoList) override;

        // throws daq::es::ESInterface::CannotNotify
        void dbReloadInit() override;

        // throws daq::es::ESInterface::CannotNotify
        void dbReloadDone() override;

        // throws daq::es::ESInterface::CannotNotify
        void reset() override;

        // throws daq::es::ESInterface::CannotNotify
        void transitionDone(const std::string& transitionName, const std::string& newState) override;

        static std::string centralESName();

    protected:
        ::es::TestResult convert(const daq::tmgr::TestResult& ret) const;

        static void fillInfo(const ProcessInfo& from, ::es::ProcessInformation& to);
        static void fillInfo(const StateInfo& from, ::es::StateInformation& to);

    private:
        const IPCPartition m_partition;
        const std::string m_es_name;
};

}}


#endif /* DAQ_ES_REMOTEES_H_ */
