/** @file  Base interface defining a service which is used by ServiceLoader in StreamFactory to locate all implemented streams
 * @author Andrei Kazarov
 * @version 1.0
 * @brief ERS StreamRegistrator interface.
 */

package ers;

/** defines interface to be implemented by ERS Input and Output streams.
 * allows to register the streams as Service and then to look up all registered streams from ERS framework.
 * Classes shall be annotated with @ProviderFor(StreamRegistrator.class).
 * See InputStream, OutputStream base classes and realizations of different streams.
 * See <a href="http://code.google.com/p/spi/wiki/GettingStarted">Google SPI: getting started</a> and
 * <a href="http://code.google.com/p/spi/wiki/GeneralSPI">General SPI</a> documentation.
 * 
 * @warning This approach requires that implementer classes must have empty constructor, which does nothing -
 * instances are created only to get the Class object from them.<br>
 * TODO: consider using Google reflections: 
 * http://code.google.com/p/reflections/<br>
 * <code> 
 *   Reflections reflections = new Reflections("daq") ;
 *   Set<Class<? extends StreamRegistrator>> subTypes = reflections.getSubTypesOf(StreamRegistrator.class) ;
 * </code> 
*/
public interface StreamRegistrator
{
/** 
@return name of the stream for stream registration, e.g. 'file', 'mts' etc.
*/
String getStreamName() ; 	

/** to be called when program exits, place your destruction and resource deallocation */
void cleanUp() ; 		
}
