/*
 *  StandardStreamOutput.java
 *  
 *  Created by Claire Biau on 02.06.08
 *  Copyright 2008 CERN. All rights reserved.
 *
 */

/** \file  StandardStreamOutput.java This file defines standard stream output function for the ERS streams. It extends from OutputStream
 * \author Claire Biau
 *	This defines print function for output an issue to any standard .output stream.
 */

package ers.streams;

import java.io.FileOutputStream;
import java.util.Map;

import org.mangosdk.spi.ProviderFor ;

import ers.Logger;

@ProviderFor(ers.StreamRegistrator.class)
public class FileStream extends ers.AbstractOutputStream {
	
	@Override
	public void configure(final Map<String, String> config) {
		// TODO Auto-generated method stub
		
	}

	private java.io.FileOutputStream stream;
	private String filename;

	public String getStreamName()
		{
		return "file" ;
		}
	/**
	 * constructor
	 * @param filename
	 * @param append
	 */
	public FileStream( final String filename, final String append ) throws java.io.FileNotFoundException
		{
		super() ;
		this.filename = filename ;
		stream = new FileOutputStream(filename, Boolean.parseBoolean(append)) ;		
		}

	/**
	 * constructor
	 * @param filename
	 */
	public FileStream( final String filename ) throws java.io.FileNotFoundException
		{
		this(filename, "true") ;	
		}

	/**
	* Default constructor, not used
	*/
	public FileStream( ) { super() ; } 
	
	/**
	 * This function writes the issue given in attribute on the standartStreamOutput
	 * It writes is in a human readable way
	 * @param issue
	 */
	synchronized public void write(final Exception issue, final ers.Severity severity) {
		try 	{
			ers.IssuePrinter.println(stream, issue, severity, Logger.verbosityLevel()) ;
			chained().write( issue, severity );	
			}
		catch ( java.io.IOException ex )
			{
			System.err.println("Failed to write issue to file stream: " + ex) ;
			}
	}
	

	public void cleanUp()	{
		try 	{
			stream.flush() ;
			stream.close() ;
			}
		catch ( java.io.IOException ex )
			{
			System.err.println("Failed to close file " + filename + ": " + ex) ;
			}
	} 
}
