//
// Implementation of Annotations
//

//-----------------------
// This Class's Header --
//-----------------------
#include "Annotations.h"

//-----------------
// C/C++ Headers --
//-----------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "boost/lexical_cast.hpp"

using namespace MIG;

namespace {

const std::string level0 = "providers";    // annotations key for lowest level count
const std::string level_prefix = "mig.lvl";   // annotations key for other levels (followed by a number)

const int MAX_MIG_LEVEL = 16;  // Reasonable limit on number of merging levels

int annKeyLevel(const std::string& key)
{
    if (key == level0) {
        return 0;
    }
    if (key.compare(0, level_prefix.size(), level_prefix) == 0) {
        try {
            int lvl = boost::lexical_cast<unsigned>(key.substr(level_prefix.size()));
            // constrain the range
            if (lvl > MAX_MIG_LEVEL) {
                return -1;
            }
            return lvl;
        } catch (const boost::bad_lexical_cast& exc) {
            // just ignore it
            return -1;
        }
    }
    return -1;
}

}


void Annotations::reset()
{
    m_counters.clear();
}

void Annotations::merge(const ISAnnotations& ann)
{
    for (auto& pair: ann) {
        int lvl = ::annKeyLevel(pair.first);
        if (lvl >= 0) {
            try {
                unsigned counter = pair.second.getValue<unsigned>();
                if (m_counters.size() < unsigned(lvl + 1)) {
                    // extend it to fit current level
                    m_counters.resize(lvl + 1, 0);
                }
                m_counters[lvl] += counter;
            } catch (const daq::is::ValueConversionFailed& exc) {
                // malformed annotation, ignore
            }
        }
    }
}

Annotations& Annotations::operator+=(const Annotations& other)
{
    if (other.m_counters.size() > m_counters.size()) {
        m_counters.resize(other.m_counters.size());
    }
    for (unsigned i = 0; i != other.m_counters.size(); ++i) {
        m_counters[i] += other.m_counters[i];
    }
    return *this;
}

void Annotations::update(ISAnnotations& ann, unsigned currenLevelCount)
{
    // Assumption here is that levels that already exist in annotations
    // are a subset of levels collected in this object, so we do not need
    // to clear anyhting from a map.
    // NOTE: iteration is done one-past-end of the vector, on purpose
    for (unsigned lvl = 0; lvl <= m_counters.size(); ++lvl) {
        unsigned counter = lvl < m_counters.size() ? m_counters[lvl] : currenLevelCount;
        ISAnnotation value(std::to_string(counter));
        if (lvl == 0) {
            ann.insert_or_assign(::level0, std::move(value));
        } else {
            ann.insert_or_assign(::level_prefix+std::to_string(lvl), std::move(value));
        }
    }
}
