"""Mock classes for use in unit tests.
"""

from __future__ import absolute_import, print_function, division

import collections

import ers
from ispy import Reason


class MockOnlPrefLumi(object):
    def __init__(self, RunNumber, LBN, StartTime, EndTime, Channel, AlgorithmID,
                 LBAvInstLumi, LBAvEvtsPerBX, LumiType, Valid, BunchInstLumi):
        self.RunNumber = RunNumber
        self.LBN = LBN
        self.StartTime = StartTime
        self.EndTime = EndTime
        self.Channel = Channel
        self.AlgorithmID = AlgorithmID
        self.LBAvInstLumi = LBAvInstLumi
        self.LBAvEvtsPerBX = LBAvEvtsPerBX
        self.LumiType = LumiType
        self.Valid = Valid
        self.BunchInstLumi = BunchInstLumi


class MockISCallbackInfo(object):
    def __init__(self, infoName, reason, isinfo, istype, time, tag=0):
        self._infoName = infoName
        self._reason = reason
        self._isinfo = isinfo
        self._istype = istype
        self._time = time
        self._tag = tag

    def name(self):
        return self._infoName

    def reason(self):
        return self._reason

    def type(self):
        return self._istype

    def time(self):
        return self._time

    def tag(self):
        return self._tag

    def value(self, mocDynAny):
        mocDynAny.__dict__.update(self._isinfo.__dict__)


class MockISDynAny(object):
    pass


class MockISInfoReceiver(object):

    def __init__(self, part, serialize_callbacks):
        self.subscriptions = dict()

    def scheduleSubscription_info(self, infoName, callback, event_mask):
        self.subscriptions[infoName] = (callback, set(event_mask))

    def unsubscribe(self, infoName, wait):
        del self.subscriptions[infoName]

    def receive(self, infoName, isinfo, time, reason=Reason.Updated, istype="OnlPrefLumi", tag=0):
        callback, event_mask = self.subscriptions[infoName]
        if reason in event_mask:
            cinfo = MockISCallbackInfo(infoName, reason, isinfo, istype, time, tag)
            callback(cinfo)


class MockISObjectStableBeams(object):

    stableBeams = None

    def __init__(self, sbeamPart, sbeamInfo, infoType):
        self.value = None

    def checkout(self):
        if self.stableBeams is None:
            raise ers.Issue("mocking failure", {}, None)
        self.value = self.stableBeams
