#!/usr/bin/env tdaq_python

"""Unit test for StableBeamsFlag class.
"""

from __future__ import absolute_import, print_function, division

import six
if six.PY2:
    import Queue as queue
else:
    import queue
import unittest

from ispy import Reason
from pyolc2hlt import ISReceiver
from _mocks import MockISDynAny, MockISInfoReceiver, MockOnlPrefLumi


class TestCase(unittest.TestCase):

    def setUp(self):

        ISReceiver.ISInfoReceiverClass = MockISInfoReceiver
        ISReceiver.ISInfoDynAnyClass = MockISDynAny

        self.part = "part"
        self.infoName = "OLC.OnlPrefLumi"

        self.run = 222222
        self.lbn = 0
        self.time = 1500000000 * 1000000000

        self.queue = queue.Queue()

    def _makeISInfo(self, lumi, lb_len=60, channel=0, algoID=0, lumiType=0, valid=0):

        self.lbn += 1
        endTime = self.time + lb_len * 1000000000
        per_bunch_lumi = lumi / 5.
        info = MockOnlPrefLumi(
            RunNumber=self.run,
            LBN=self.lbn,
            StartTime=self.time,
            EndTime=endTime,
            Channel=channel,
            AlgorithmID=algoID,
            LBAvInstLumi=lumi,
            LBAvEvtsPerBX=per_bunch_lumi,
            LumiType=lumiType,
            Valid=valid,
            BunchInstLumi=[0., per_bunch_lumi, 0, per_bunch_lumi, 0., per_bunch_lumi,
                           0., per_bunch_lumi, 0., per_bunch_lumi],
        )
        self.time = endTime
        return info

    def _publish(self, data):

        for entry in data:
            self.queue.put(entry)
        self.queue.join()

    def test01(self):

        recv = ISReceiver(self.part, self.infoName, self.queue)
        recv.subscribe()

        mock_is_recv = recv.receiver

        isinfo = self._makeISInfo(1.)
        mock_is_recv.receive(self.infoName, isinfo, isinfo.EndTime + 1000000, reason=Reason.Subscribed)
        self.assertEqual(self.queue.qsize(), 1)
        reason, isinfo2 = self.queue.get()
        self.assertEqual(reason, Reason.Subscribed)
        self.assertEqual(isinfo2.RunNumber, self.run)
        self.assertEqual(isinfo2.LBN, self.lbn)
        self.assertEqual(isinfo2.LBAvInstLumi, 1.)

        isinfo = self._makeISInfo(100.)
        mock_is_recv.receive(self.infoName, isinfo, isinfo.EndTime + 1000000, reason=Reason.Updated)
        self.assertEqual(self.queue.qsize(), 1)
        reason, isinfo2 = self.queue.get()
        self.assertEqual(reason, Reason.Updated)

        recv.unsubscribe()


if __name__ == "__main__":
    unittest.main()
