#!/usr/bin/env tdaq_python

"""Unit test for Smoother class.
"""

from __future__ import absolute_import, print_function, division

#--------------------------------
#  Imports of standard modules --
#--------------------------------
import unittest

#---------------------------------
#  Imports of base class module --
#---------------------------------

#-----------------------------
# Imports for other modules --
#-----------------------------
from pyolc2hlt import Smoother

#---------------------
# Local definitions --
#---------------------

#-------------------------------
#  Unit test class definition --
#-------------------------------


class testSmoother(unittest.TestCase):

    def test01(self):
        '''
        Test for max 3 LBs, no spike removal
        '''
        smoother = Smoother(3, 10000000, False)

        smoother.addLB(0, 1, 10.)
        self.assertEqual(len(smoother.lbs), 1)
        self.assertEqual(smoother.value(), 10.)

        smoother.addLB(2, 3, 20.)
        self.assertEqual(len(smoother.lbs), 2)
        self.assertEqual(smoother.value(), 15.)

        smoother.addLB(3, 6, 20.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 18.)

        smoother.addLB(6, 7, 0.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 16.)

        smoother.addLB(7, 8, 0.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 12.)

        smoother.addLB(8, 9, 0.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 0.)

        smoother.reset()
        self.assertEqual(len(smoother.lbs), 0)
        self.assertEqual(smoother.value(), 0.)

    def test02(self):
        '''
        Test for time limits
        '''
        smoother = Smoother(1000, 10, False)

        smoother.addLB(0000000000, 2000000000, 10.)
        self.assertEqual(len(smoother.lbs), 1)
        self.assertEqual(smoother.value(), 10.)

        smoother.addLB(2000000000, 4000000000, 20.)
        self.assertEqual(len(smoother.lbs), 2)
        self.assertEqual(smoother.value(), 15.)

        smoother.addLB(5000000000, 7000000000, 30.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 20.)

        smoother.addLB(8000000000, 12000000000, 15.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 20.)

        smoother.addLB(12000000000, 16000000000, 5.)
        self.assertEqual(len(smoother.lbs), 2)
        self.assertEqual(smoother.value(), 10.)

    def test03(self):
        '''
        Test for spikes removal
        '''
        smoother = Smoother(5, 10, True)

        smoother.addLB(0, 1, 0.)
        self.assertEqual(len(smoother.lbs), 1)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 1.)
        self.assertEqual(len(smoother.lbs), 2)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 0.)
        self.assertEqual(len(smoother.lbs), 3)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 0.)
        self.assertEqual(len(smoother.lbs), 4)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 0.)
        self.assertEqual(len(smoother.lbs), 5)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 0.)
        self.assertEqual(len(smoother.lbs), 5)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 1.)
        self.assertEqual(len(smoother.lbs), 5)
        self.assertEqual(smoother.value(), 0.)

        smoother.addLB(0, 1, 4.)
        self.assertEqual(len(smoother.lbs), 5)
        self.assertEqual(smoother.value(), 1.)


if __name__ == "__main__":
    unittest.main()
