#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
# Created by Andre Anjos <andre.dos.anjos@cern.ch>
# Mon Sep 10 14:09:49 CEST 2007

"""This module contains some helpers for the generation of partitions.
"""

def parse_cmdline(doc, options):
  """Parses the commanline options for plugins in a uniform way.
  
  Keyword arguments:

  doc -- This is extra documentation that will be prefixed to the usage message
  that is automatically generated by the parser (EventApps.myopt.Parser)

  options -- This is the option dictionary that should be received as
  constructor parameter for the Parser (EventApps.myopt.Parser)

  Returns a dictionary that contains the arguments passed. Each key in this
  dictionary represents the long option name of the options in your input
  dictionary. The values are pythonic version of your input arguments.
  """

  from EventApps import myopt
  from sys import argv

  p = myopt.Parser(short_intro=doc, extra_args=False)
  for (k,v) in list(options.items()):
    p.add_option(k, v['short'], v['description'], v['arg'], 
        v['default'], v['group'])
  return p.parse(argv[1:])

def post_process(args, objects):
  """Post processes the objects in "objects" if -Z option given"""

  # if the user has set the environment variable TDAQ_PM_POSTPROCESSOR
  # we take that into account
  import os
  if 'TDAQ_PM_POSTPROCESSOR' in os.environ:
    for k in os.environ['TDAQ_PM_POSTPROCESSOR'].split():
      ns = {}
      exec('from %s import modify' % k, ns)
      modify = ns['modify']
      objects = modify(objects)

  # if the user has set post processing scripts manually, also take them
  if args['post-processor']:
    for k in args['post-processor']:
      ns = {}
      exec('from %s import modify' % k, ns)
      modify = ns['modify']
      objects = modify(objects)

  # else just return the objects you input
  return objects
