#!/usr/bin/env tdaq_python
# vim: set fileencoding=utf-8 :
# Created by Andre Anjos <andre.dos.anjos@cern.ch>
# Mon 17 Sep 2007 06:08:31 PM CEST 

from __future__ import print_function
help_mesg="""Generates an OKS database with common shared objects.

A list of common object that are generated by this script:
  RunControlTemplateApplication
  TriggerConfiguration (pudummy)
"""

def setup_parser():
  """Sets-up the command line parser."""
  from EventApps.myopt import Parser
  import pm.option.specific as specific

  options = {}
  options['hlt-implementation'] = specific.option['hlt-implementation']
  
  p = Parser(short_intro=help_mesg, extra_args=True)
  for (k,v) in list(options.items()):
    p.add_option(k, v['short'], v['description'], v['arg'], 
        v['default'], v['group'])

  return p

def main():
  import sys
  import pm
  from pm.dal import dal
  import pm.common

  parser = setup_parser()
 
  if len(sys.argv) == 1:
    print(parser.usage())
    sys.exit(1)

  opt, args = parser.parse(sys.argv[1:])

  if len(args) != 1:
    raise RuntimeError("The number of extra arguments should be == 1, got %s" % args)
  
  db = pm.project.Project(args[0], ['daq/schema/df.schema.xml',
			'daq/sw/repository.data.xml'])
  objs = []
  objs.append(dal.RunControlTemplateApplication('Controller-1'))
  objs.append(pm.common.create_trigger_config(opt['hlt-implementation']))
  db.addObjects(objs)
  return 0

if __name__ == "__main__":
  from sys import exit
  exit(main())

