package RCUtils;

import java.util.*;

import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.*;
import javax.swing.event.*;

import java.awt.*;
import java.io.*;

import org.jfree.chart.JFreeChart;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.ChartFactory;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.*;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.data.time.Second;
import org.jfree.data.time.TimeSeries;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;

public class RunEfficiencyPanel extends JPanel {

  // -----------------------------
  //  panel stuff
  // -----------------------------

  /** vertical SplitPane */
  JSplitPane verticalSplitPanel;

  /** Panel for Run info */
  JPanel topPanel;

  /** Labels for summary */
  JLabel partition;
  JLabel interval;
  JLabel empty;
  JLabel totalTime;
  JLabel beamTime;
  JLabel runningTime;
  JLabel runNumber;
  JLabel meanTime;
  JLabel fraction;
  JLabel fractionBeam;

  /** the chart for the time series */
  JFreeChart chart;

  /** the chart panel */
  ChartPanel chartPanel;

  /** The plot. */
  private XYPlot plot;

  private TimeSeriesCollection beamCollection;
  private TimeSeriesCollection runCollection;

  private String partitionName="";
  private String intervalName="";
  private String totalTimeName="";
  private String beamTimeName="";
  private String runningTimeName="";
  private String runNumberName="";
  private String meanTimeName="";
  private String fractionName="";
  private String fractionBeamName="";
  private String option = "";

  public RunEfficiencyPanel() {

    XYDataset dataset = createDataset();
    
    if (option.compareTo("option1") == 0) {
      chart = ChartFactory.createTimeSeriesChart(
              "", 
              "Time", 
              "Rate",
              dataset, 
              true, 
              true, 
              false
          );
      this.plot = chart.getXYPlot();
      this.plot.setDataset(1, beamCollection);
      this.plot.setRenderer(1, new StandardXYItemRenderer());
      this.plot.setDataset(2, runCollection);
      this.plot.setRenderer(2, new StandardXYItemRenderer());
      chart.setAntiAlias(false);
      chartPanel = new ChartPanel(chart);
    } else {
      chart = ChartFactory.createTimeSeriesChart(
              "",
              "Time",
              "Run",
              dataset,
              true,
              true,
              false
          );
      this.plot = chart.getXYPlot();
      this.plot.setDataset(1, beamCollection);
      this.plot.setRenderer(1, new StandardXYItemRenderer());
      chart.setAntiAlias(false);
      chartPanel = new ChartPanel(chart);
    }

//    chartPanel.setMouseZoomable(true, false);
    chartPanel.setPreferredSize(new Dimension(1000, 400));

    partition   = new JLabel(partitionName);
    partition.setBackground(new Color(255, 255, 168));
    partition.setOpaque(true);
    partition.setHorizontalAlignment(SwingConstants.CENTER);
    partition.setFont( new java.awt.Font("dialog.bold", 1, 16) );
    interval   = new JLabel(intervalName);
    interval.setBackground(new Color(255, 255, 168));
    interval.setOpaque(true);
    interval.setHorizontalAlignment(SwingConstants.CENTER);
    interval.setFont( new java.awt.Font("dialog.bold", 1, 14) );
    empty = new JLabel("");
    empty.setBackground(new Color(255, 255, 168));
    empty.setOpaque(true);
    empty.setHorizontalAlignment(SwingConstants.CENTER);
    totalTime   = new JLabel(totalTimeName);
    totalTime.setBackground(new Color(255, 255, 168));
    totalTime.setOpaque(true);
    totalTime.setHorizontalAlignment(SwingConstants.CENTER);
    beamTime   = new JLabel(beamTimeName);
    beamTime.setBackground(new Color(255, 255, 168));
    beamTime.setOpaque(true);
    beamTime.setHorizontalAlignment(SwingConstants.CENTER);
    runningTime = new JLabel(runningTimeName);
    runningTime.setBackground(new Color(255, 255, 168));
    runningTime.setOpaque(true);
    runningTime.setHorizontalAlignment(SwingConstants.CENTER);
    runNumber   = new JLabel(runNumberName);
    runNumber.setBackground(new Color(255, 255, 168));
    runNumber.setOpaque(true);
    runNumber.setHorizontalAlignment(SwingConstants.CENTER);
    meanTime    = new JLabel(meanTimeName);
    meanTime.setBackground(new Color(255, 255, 168));
    meanTime.setOpaque(true);
    meanTime.setHorizontalAlignment(SwingConstants.CENTER);
    fraction    = new JLabel(fractionName);
    fraction.setBackground(new Color(255, 255, 168));
    fraction.setOpaque(true);
    fraction.setHorizontalAlignment(SwingConstants.CENTER);
    fraction.setFont( new java.awt.Font("dialog.bold", 1, 14) );
    fractionBeam    = new JLabel(fractionBeamName);
    fractionBeam.setBackground(new Color(255, 255, 168));
    fractionBeam.setOpaque(true);
    fractionBeam.setHorizontalAlignment(SwingConstants.CENTER);
    fractionBeam.setFont( new java.awt.Font("dialog.bold", 1, 14) );

    topPanel = new JPanel();
    topPanel.setLayout(new java.awt.GridLayout(10, 1));

    topPanel.add(partition);
    topPanel.add(interval);
    topPanel.add(empty);
    topPanel.add(totalTime);
    topPanel.add(beamTime);
    topPanel.add(runningTime);
    topPanel.add(runNumber);
    topPanel.add(meanTime);
    topPanel.add(fraction);
    topPanel.add(fractionBeam);

    verticalSplitPanel = new JSplitPane();
    verticalSplitPanel.setOrientation(JSplitPane.VERTICAL_SPLIT);
    //verticalSplitPanel.setDividerSize(3);
    verticalSplitPanel.setOneTouchExpandable(true);
    verticalSplitPanel.setDividerLocation(200);
    verticalSplitPanel.add(topPanel, JSplitPane.TOP);
    verticalSplitPanel.add(chartPanel, JSplitPane.BOTTOM);
    this.add(verticalSplitPanel, BorderLayout.CENTER);
  }

    private XYDataset createDataset() {
      TimeSeries timeSeries = new TimeSeries("Run Data");
      final TimeSeries beamSeries = new TimeSeries("Beam Data");
      final TimeSeries runSeries = new TimeSeries("Run Data");

      double beamValue = 1.2;
      double runValue = 1.0;
      
      BufferedReader bufferedReader;
      try {
        bufferedReader = new BufferedReader(new FileReader("/tmp/runs.log"));
        String times = new String("1234");
//        String option = new String("");
        try {
          option = bufferedReader.readLine();
        } catch (java.io.IOException e) {
          System.out.println("read bean data IOException caught");
        }
        if (option.compareTo("option1") == 0) {
          timeSeries = new TimeSeries("Trigger Rate");
        }
        try {
          bufferedReader.mark(1200000);
        } catch (java.io.IOException e) {
          System.out.println("mark BufferedReader IOException caught");
        }
        while ( !(times.compareTo("endbeam") == 0)) {
          try {
            times = bufferedReader.readLine();
          } catch (java.io.IOException e) {
            System.out.println("read bean data IOException caught");
          }
        }
        if (option.compareTo("option0") == 0) {
          while ( !(times.compareTo("end") == 0)) {
            try {
              times = bufferedReader.readLine();
              if (!(times.compareTo("end") == 0)) {
                String startOfRun = times.substring(0, 13);
                Long sOR = new Long(startOfRun);
                long sor = sOR.longValue();
                timeSeries.addOrUpdate(new Second(new java.util.Date(sor)), 0.0);
                timeSeries.addOrUpdate(new Second(new java.util.Date(sor+1000)), 1.0);
                String endOfRun = times.substring(14, 27);
                Long eOR =  new Long(endOfRun);
                long eor = eOR.longValue();
                timeSeries.addOrUpdate(new Second(new java.util.Date(eor-1000)), 1.0);
                timeSeries.addOrUpdate(new Second(new java.util.Date(eor)), 0.0);
              }  
            } catch (java.io.IOException e) {
              System.out.println("read run data IOException caught");
            }
          }
        }
        if (option.compareTo("option1") == 0) {
          double maxValue = 0;
          while ( !(times.compareTo("end") == 0)) {
            try {
              times = bufferedReader.readLine();
              if (!(times.compareTo("end") == 0)) {
                String time = times.substring(0, 13);
                Long timeLong = new Long(time);
                long timelong = timeLong.longValue();
                int length = times.length();
                String value = times.substring(14, length);
//                System.out.println("value = " + value);
                Double valueDouble = new Double(value);
                double valuedouble = valueDouble.doubleValue();
                if (valuedouble > maxValue) {
                  maxValue = valuedouble;
                }
                timeSeries.addOrUpdate(new Second(new java.util.Date(timelong)), valuedouble);
                          }
            } catch (java.io.IOException e) {
              System.out.println("read run data IOException caught");
            }
          }
          if (maxValue > runValue) {
            runValue = maxValue*1.05;
          }
          while ( !(times.compareTo("finalend") == 0)) {
            try {
              times = bufferedReader.readLine();
              if (!(times.compareTo("finalend") == 0)) {
                String startOfRun = times.substring(0, 13);
                Long sOR = new Long(startOfRun);
                long sor = sOR.longValue();
                runSeries.addOrUpdate(new Second(new java.util.Date(sor)), 0.0);
                runSeries.addOrUpdate(new Second(new java.util.Date(sor+1000)), runValue);
                String endOfRun = times.substring(14, 27);
                Long eOR =  new Long(endOfRun);
                long eor = eOR.longValue();
                runSeries.addOrUpdate(new Second(new java.util.Date(eor-1000)), runValue);
                runSeries.addOrUpdate(new Second(new java.util.Date(eor)), 0.0);
              }
            } catch (java.io.IOException e) {
              System.out.println("read run data IOException caught");
            }
          }
          if (maxValue > beamValue) {
            beamValue = maxValue * 1.1;
          }
        }

        try {
          partitionName = bufferedReader.readLine();
          intervalName = bufferedReader.readLine();
          totalTimeName = bufferedReader.readLine();
          beamTimeName = bufferedReader.readLine();
          runningTimeName = bufferedReader.readLine();
          runNumberName = bufferedReader.readLine();
          meanTimeName = bufferedReader.readLine();
          fractionName = bufferedReader.readLine();
          fractionBeamName = bufferedReader.readLine();
        } catch (java.io.IOException e) {
           System.out.println("late IOException caught");
        }  
        try {
          bufferedReader.reset();
        } catch (java.io.IOException e) {
           System.out.println("reset IOException caught");
        }
        while ( !(times.compareTo("endbeam") == 0)) {
          try {
            times = bufferedReader.readLine();
            if (!(times.compareTo("endbeam") == 0)) {
              String startOfBeam = times.substring(0, 13);
              Long sOB = new Long(startOfBeam);
              long sob = sOB.longValue();
              beamSeries.addOrUpdate(new Second(new java.util.Date(sob)), 0.0);
              beamSeries.addOrUpdate(new Second(new java.util.Date(sob+1000)), beamValue);
              String endOfBeam = times.substring(14, 27);
              Long eOB =  new Long(endOfBeam);
              long eob = eOB.longValue();
              beamSeries.addOrUpdate(new Second(new java.util.Date(eob-1000)), beamValue);
              beamSeries.addOrUpdate(new Second(new java.util.Date(eob)), 0.0);
            }
          } catch (java.io.IOException e) {
            System.out.println("read bean data IOException caught");
          }
        }
      
      } catch (java.io.FileNotFoundException ex) {
        System.out.println("File /tmp/runs.log not found");
        System.exit(0);
      }
      final TimeSeriesCollection dataset = new TimeSeriesCollection();
      dataset.addSeries(timeSeries);
      beamCollection = new TimeSeriesCollection();
      beamCollection.addSeries(beamSeries);
      runCollection = new TimeSeriesCollection();
      runCollection.addSeries(runSeries);
      return dataset;

    }
  
}
