// eff_csv_splunk.cc
// 
// App to produce the splunk-friendly format of the output from eff_daq_eff
// The app is independent of the number of rows and columns in the input file
//
// Author A. Chitan
// 
// 29 April 2015
//
//	AC		17/06/2015		Added the inter-run stable beam periods calculation; needs dummies.data and beams.data files 
//								created by eff_daq_efficiency app in /tmp/;	saves output in overflows.data
//	AC		07/08/2015		Added information about fill no and run no for dummy LBs in overflows.data
//	AC		12/08/2015		Added missingFillTimes.data output for inter-run period dummy LBs
//	AC		20/01/2016		Modified to accomodate the extended HT and Stop maps (see eff_daq_efficiency.cc comments)

#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>

#include "owl/time.h"

std::string get_date(int seconds) {
	OWLTime owl_date = OWLTime(seconds);
	long year = owl_date.year();
	// unsigned short month = owl_date.month()+1;
	unsigned short month = owl_date.month();
	unsigned short day = owl_date.day();
	std::string date_string = std::string(owl_date.c_str());
	size_t lastIndex = date_string.size();
	std::string time_string = date_string.substr(lastIndex-8, lastIndex);
	std::ostringstream stm;
	stm << year;
	stm << "-";
	if (month < 10) {
		stm << "0";
	}
	stm << month << "-";
	if (day < 10) {
		stm << "0";
	}
	stm << day;
	stm << " ";
	stm << time_string;
	return stm.str();
}

int main() {
	
	std::vector<std::string> titleNames;
	std::vector<std::string> fillNo;
	std::vector<std::string> runNo;
	std::string bufferStr;
	std::string titleLine;
	std::string rowIn;
	std::size_t index = 0;
	std::size_t indexOld = -1;

	//FILE streams
	std::ifstream csvFile("run_eff_csv.data");
	std::ofstream outputFile;

	//get the first line of the csv file
	getline(csvFile,titleLine);
	
	//create the title items
	index = 0;
	indexOld = -1;
	while(titleLine.find(",",index+1) != std::string::npos) {
		index = titleLine.find(",",index+1);
		bufferStr = titleLine.substr(indexOld+1,index-1-indexOld);
		indexOld = index;
		bufferStr += "=";
		titleNames.push_back(bufferStr);
		bufferStr.clear();
	}
	
	//link the entries with the titles for every row and put them in the output.data file
	index = 0;
	indexOld = -1;
	int i = 0;
	outputFile.open("output.data");
	while(getline(csvFile,titleLine)) {
		rowIn.clear();
		index = 0;
		indexOld = -1;
		i = 0;
		while(titleLine.find(",",index+1) != std::string::npos) {
			index = titleLine.find(",",index+1);
			bufferStr = titleLine.substr(indexOld+1,index-1-indexOld);
			indexOld = index;
			bufferStr += ",";
			rowIn += titleNames.at(i);
			rowIn += bufferStr;
			i++;
		}
		outputFile << rowIn << std::endl;
	}

	csvFile.close();
	outputFile.close();

	//fill the pause times vectors
	std::vector<long long> pauseStarts;
	std::vector<long long> pauseEnds;
	std::vector<std::string> stopVector;
	std::ifstream pausesFile("dummies.data");
	std::string bufferRow;
	OWLTime *bufferOwl;
	while(getline(pausesFile,bufferRow) && bufferRow.find("\t") != std::string::npos) {
		index = bufferRow.find("\t");
		bufferStr = bufferRow.substr(0,index);
		bufferOwl = new OWLTime(bufferStr.c_str());
		pauseStarts.push_back(bufferOwl->total_mksec_utc());
		bufferStr = bufferRow.substr(index+1,19);
		bufferOwl = new OWLTime(bufferStr.c_str());
		pauseEnds.push_back(bufferOwl->total_mksec_utc());
	}
	while(getline(pausesFile,bufferRow) && bufferRow.compare("endrun")) {
		runNo.push_back(bufferRow);
	}
	while(getline(pausesFile,bufferRow)) {
		//index = bufferRow.find("\t");
		//bufferStr = bufferRow.substr(0,index);
		//bufferRow.pop_back();
		stopVector.push_back(bufferRow);
	}
	pausesFile.close();

	//fill the beam times vectors
	std::vector<long long> beamStarts;
	std::vector<long long> beamEnds;
	std::ifstream beamsFile("beams.data");
	while(getline(beamsFile,bufferRow) && bufferRow.find("\t") != std::string::npos) {
		index = bufferRow.find("\t");
		bufferStr = bufferRow.substr(0,index);
		bufferOwl = new OWLTime(bufferStr.c_str());
		beamStarts.push_back(bufferOwl->total_mksec_utc());
		bufferStr = bufferRow.substr(index+1,19);
		bufferOwl = new OWLTime(bufferStr.c_str());
		beamEnds.push_back(bufferOwl->total_mksec_utc());
	}
	while(getline(beamsFile,bufferRow)) {
		fillNo.push_back(bufferRow);
	}
	beamsFile.close();

	int rowsNo = 0;
	int contor = 0;
	std::ofstream overflows("overflows.data");
	for(int iPauses = 0; iPauses < pauseStarts.size(); iPauses++) {
		for(int iBeams = 0; iBeams < beamStarts.size(); iBeams++) {
			if(beamStarts.at(iBeams) >= pauseStarts.at(iPauses) && beamEnds.at(iBeams) <= pauseEnds.at(iPauses)) {
				overflows << get_date(beamStarts.at(iBeams)/1000000) << "\t" << get_date(beamEnds.at(iBeams)/1000000) << "\t" << (beamEnds.at(iBeams)-beamStarts.at(iBeams))/1000;
				overflows << "\t" << fillNo.at(iBeams); 
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
			else if(beamStarts.at(iBeams) >= pauseStarts.at(iPauses) && beamStarts.at(iBeams) < pauseEnds.at(iPauses)) {
				overflows << get_date(beamEnds.at(iBeams)/1000000) << "\t" << get_date(pauseEnds.at(iPauses)/1000000) << "\t" << (pauseEnds.at(iPauses)-beamEnds.at(iBeams))/1000;
				overflows << "\t" << fillNo.at(iBeams);
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
			else if(beamEnds.at(iBeams) > pauseStarts.at(iPauses) && beamEnds.at(iBeams) <= pauseEnds.at(iPauses)) {
				overflows << get_date(pauseStarts.at(iPauses)/1000000) << "\t" << get_date(beamEnds.at(iBeams)/1000000) << "\t" << (beamEnds.at(iBeams)-pauseStarts.at(iPauses))/1000;
				overflows << "\t" << fillNo.at(iBeams);
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
			else if(pauseStarts.at(iPauses) >= beamStarts.at(iBeams) && pauseEnds.at(iPauses) <= beamEnds.at(iBeams)) {
				overflows << get_date(pauseStarts.at(iPauses)/1000000) << "\t" << get_date(pauseEnds.at(iPauses)/1000000) << "\t" << (pauseEnds.at(iPauses)-pauseStarts.at(iPauses))/1000;
				overflows << "\t" << fillNo.at(iBeams);
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
			else if(pauseStarts.at(iPauses) >= beamStarts.at(iBeams) && pauseStarts.at(iPauses) < beamEnds.at(iBeams)) {
				overflows << get_date(pauseStarts.at(iPauses)/1000000) << "\t" << get_date(beamEnds.at(iBeams)/1000000) << "\t" << (beamEnds.at(iBeams)-pauseStarts.at(iPauses))/1000;
				overflows << "\t" << fillNo.at(iBeams);
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
			else if(pauseEnds.at(iPauses) > beamStarts.at(iBeams) && pauseEnds.at(iPauses) <= beamEnds.at(iBeams)) {
				overflows << get_date(beamStarts.at(iBeams)/1000000) << "\t" << get_date(pauseEnds.at(iPauses)/1000000) << "\t" << (pauseEnds.at(iPauses)-beamStarts.at(iBeams))/1000;
				overflows << "\t" << fillNo.at(iBeams);
				overflows << "\t" << runNo.at(iPauses) << "\t";
				for(int inputs = 0; inputs < 34; inputs++) {
					overflows << stopVector.at(contor*34+inputs) << "\t";
				}
				overflows << std::endl;
				rowsNo++;
			}
		}
		contor++;
	}
	overflows.close();

	std::ifstream overflowsInput("overflows.data");
	std::ofstream missingFillTimes("missingFillTimes.data");
	rowIn.clear();
	std::vector<std::string> valuesForDummies;
	while(getline(overflowsInput,bufferRow)) {
		index = 0;
		indexOld = -1;
		while(bufferRow.find("\t",index+1) != std::string::npos) {
			index = bufferRow.find("\t",index+1);
			valuesForDummies.push_back(bufferRow.substr(indexOld+1,index-1-indexOld));
			indexOld = index;
		}
	}

	titleNames.push_back("Other_S=");
	titleNames.push_back("PIX_S=");
	titleNames.push_back("SCT_S=");
	titleNames.push_back("TRT_S=");
	titleNames.push_back("LAR_S=");
	titleNames.push_back("TIL_S=");
	titleNames.push_back("RPC_S=");
	titleNames.push_back("MDT_S=");
	titleNames.push_back("TGC_S=");
	titleNames.push_back("CSC_S=");
	titleNames.push_back("BCM_S=");
	titleNames.push_back("LUCID_S=");
	titleNames.push_back("ZDC_S=");
	titleNames.push_back("ALFA_S=");
	titleNames.push_back("L1CALO_S=");
	titleNames.push_back("L1CT_S=");
	titleNames.push_back("HLT_S=");
	titleNames.push_back("DAQ_S=");
	titleNames.push_back("IBL_S=");
	titleNames.push_back("DBM_S=");
	titleNames.push_back("MMEGA_S=");
	titleNames.push_back("L1TOPO_S=");
	titleNames.push_back("Solenoid_S=");
	titleNames.push_back("Toroid_S=");
	titleNames.push_back("Cooling_S=");
	titleNames.push_back("LHC_S=");
	titleNames.push_back("FTK_S=");
	titleNames.push_back("Unknown=");
	titleNames.push_back("SystemBusy=");
	titleNames.push_back("SystemNeedsReconfiguration=");
	titleNames.push_back("HardwareFailure=");
	titleNames.push_back("BeamDump=");
	titleNames.push_back("HandShake=");
	titleNames.push_back("TestRun=");
	for(int j = 0; j < rowsNo; j++) {
		rowIn.clear();
		for(int ja = 0; ja < titleNames.size(); ja++) {
			rowIn += titleNames.at(ja);
			if(titleNames.at(ja).compare("StableBeams=") == 0)
				rowIn += "1";
			else if(titleNames.at(ja).compare("StartDate=") == 0)
				rowIn += valuesForDummies.at(39*j);
			else if(titleNames.at(ja).compare("EndDate=") == 0)
				rowIn += valuesForDummies.at(39*j+1);
			else if(titleNames.at(ja).compare("Fill=") == 0)
				rowIn += valuesForDummies.at(39*j+3);
			else if(titleNames.at(ja).compare("Duration=") == 0)
				rowIn += valuesForDummies.at(39*j+2);
			else if(titleNames.at(ja).compare("RunNumber=") == 0)
				rowIn += valuesForDummies.at(39*j+4);
			else if(titleNames.at(ja).compare("Other_S=") == 0)
				rowIn += valuesForDummies.at(39*j+5);
			else if(titleNames.at(ja).compare("PIX_S=") == 0)
				rowIn += valuesForDummies.at(39*j+6);
			else if(titleNames.at(ja).compare("SCT_S=") == 0)
				rowIn += valuesForDummies.at(39*j+7);
			else if(titleNames.at(ja).compare("TRT_S=") == 0)
				rowIn += valuesForDummies.at(39*j+8);
			else if(titleNames.at(ja).compare("LAR_S=") == 0)
				rowIn += valuesForDummies.at(39*j+9);
			else if(titleNames.at(ja).compare("TIL_S=") == 0)
				rowIn += valuesForDummies.at(39*j+10);
			else if(titleNames.at(ja).compare("RPC_S=") == 0)
				rowIn += valuesForDummies.at(39*j+11);
			else if(titleNames.at(ja).compare("MDT_S=") == 0)
				rowIn += valuesForDummies.at(39*j+12);
			else if(titleNames.at(ja).compare("TGC_S=") == 0)
				rowIn += valuesForDummies.at(39*j+13);
			else if(titleNames.at(ja).compare("CSC_S=") == 0)
				rowIn += valuesForDummies.at(39*j+14);
			else if(titleNames.at(ja).compare("BCM_S=") == 0)
				rowIn += valuesForDummies.at(39*j+15);
			else if(titleNames.at(ja).compare("LUCID_S=") == 0)
				rowIn += valuesForDummies.at(39*j+16);
			else if(titleNames.at(ja).compare("ZDC_S=") == 0)
				rowIn += valuesForDummies.at(39*j+17);
			else if(titleNames.at(ja).compare("ALFA_S=") == 0)
				rowIn += valuesForDummies.at(39*j+18);
			else if(titleNames.at(ja).compare("L1CALO_S=") == 0)
				rowIn += valuesForDummies.at(39*j+19);
			else if(titleNames.at(ja).compare("L1CT_S=") == 0)
				rowIn += valuesForDummies.at(39*j+20);
			else if(titleNames.at(ja).compare("HLT_S=") == 0)
				rowIn += valuesForDummies.at(39*j+21);
			else if(titleNames.at(ja).compare("DAQ_S=") == 0)
				rowIn += valuesForDummies.at(39*j+22);
			else if(titleNames.at(ja).compare("IBL_S=") == 0)
				rowIn += valuesForDummies.at(39*j+23);
			else if(titleNames.at(ja).compare("DBM_S=") == 0)
				rowIn += valuesForDummies.at(39*j+24);
			else if(titleNames.at(ja).compare("MMEGA_S=") == 0)
				rowIn += valuesForDummies.at(39*j+25);
			else if(titleNames.at(ja).compare("L1TOPO_S=") == 0)
				rowIn += valuesForDummies.at(39*j+26);
			else if(titleNames.at(ja).compare("Solenoid_S=") == 0)
				rowIn += valuesForDummies.at(39*j+27);
			else if(titleNames.at(ja).compare("Toroid_S=") == 0)
				rowIn += valuesForDummies.at(39*j+28);
			else if(titleNames.at(ja).compare("Cooling_S=") == 0)
				rowIn += valuesForDummies.at(39*j+29);
			else if(titleNames.at(ja).compare("LHC_S=") == 0)
				rowIn += valuesForDummies.at(39*j+30);
			else if(titleNames.at(ja).compare("FTK_S=") == 0)
				rowIn += valuesForDummies.at(39*j+31);
			else if(titleNames.at(ja).compare("Unknown=") == 0)
				rowIn += valuesForDummies.at(39*j+32);
			else if(titleNames.at(ja).compare("SystemBusy=") == 0)
				rowIn += valuesForDummies.at(39*j+33);
			else if(titleNames.at(ja).compare("SystemNeedsReconfiguration=") == 0)
				rowIn += valuesForDummies.at(39*j+34);
			else if(titleNames.at(ja).compare("HardwareFailure=") == 0)
				rowIn += valuesForDummies.at(39*j+35);
			else if(titleNames.at(ja).compare("BeamDump=") == 0)
				rowIn += valuesForDummies.at(39*j+36);
			else if(titleNames.at(ja).compare("HandShake=") == 0)
				rowIn += valuesForDummies.at(39*j+37);
			else if(titleNames.at(ja).compare("TestRun=") == 0)
				rowIn += valuesForDummies.at(39*j+38);
			else
				rowIn += "0";
			rowIn += ",";
		}
		missingFillTimes << rowIn << std::endl;
	}
	overflowsInput.close();
	missingFillTimes.close();

	return 0;
}