#!/usr/bin/env python

from itertools import cycle
from math import ceil


def setaffinity(irq, coreidx, ncores, force=False):

    digits = int(ceil(1. * ncores / 4))
    cpumask = '{0:x}'.format(1 << coreidx).zfill(digits)
    affinity = '/proc/irq/%s/smp_affinity' % irq

    with open(affinity, 'r') as affinityfile:
        current_affinity = affinityfile.readline().rstrip('\n')

    with open(affinity, 'w') as affinityfile:
        write = force or current_affinity != cpumask
        if write:
            print('Updating')
            affinityfile.write(cpumask)


if __name__ == '__main__':

    with open('/proc/interrupts', 'r') as f:
        irqmap = [l for l in f]

    robinirq = [l.split()[0].strip(':') for l in irqmap if 'robinnp' in l]
    robinirq.sort(key=int)

    mlxirqs = [l.split()[0].strip(':') for l in irqmap if 'mlx4-' in l]

    with open('/proc/cpuinfo', 'r') as cpuinfo:
        ncores = len([l for l in cpuinfo if 'processor	:' in l])

    # Distribute Robin IRQ uniformely on the available cores
    for idx, irq in enumerate(robinirq):
        coreidx = idx % ncores
        print(coreidx, irq)
        setaffinity(irq, coreidx, ncores)

    # For the Mellanox cards we expect 25 interrups for each cards
    # One is a common interrupt (the first one), followed by two sets
    # of twelve, one set for each port. Unfortunately we have to rely
    # on ordering
    netirqs = (mlxirqs[:13], mlxirqs[13:25], mlxirqs[25:38], mlxirqs[38:])

    # Group eth irqs into distinct subsets of the CPU cores
    # Simple approximation, does not work for all PCs
    # it works for the Gen3 PC model
    subsetsize = ncores // len(netirqs)

    for idx, irqs in enumerate(netirqs):
        startidx = idx * subsetsize
        cores = range(startidx, startidx + subsetsize)

        for irq, coreidx in zip(irqs, cycle(cores)):
            print(irq, coreidx)
            setaffinity(irq, coreidx, ncores)
