/*
 *  exceptions.h
 *  rmgr
 *
 */
#ifndef RM_EXCEPTIONS_H
#define RM_EXCEPTIONS_H

#include <ers/ers.h>
//#include <omniORB4/CORBA.h>
#include <ers/Issue.h>
#include <ers/Assertion.h>

/*! \file exceptions.h Defines RM issues. 
 * \author Igor Alexandrov
 * \version 1.0 
 */

/*! \namespace daq
 *  This is a wrapping namespace for all RM exceptions.
 */
 
namespace daq {
 /*! \namespace daq::rm
 *  This is a wrapping namespace for all RM exceptions.
 */

    /*! \class rmgr::Exception
     *  This is a base class for all RM exceptions.
     */
    ERS_DECLARE_ISSUE( rmgr, Exception, ERS_EMPTY , ERS_EMPTY )

    /*! \class rmgr::ConfigurationException
     *  This exception raised in case of any daq:config exception raised.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       ConfigurationException,
                       rmgr::Exception,
                       "ConfigException raised with reason: " << reason,
                       ERS_EMPTY,
                       ((std::string)reason )
                      )

    /*! \class rmgr::PartitionNotFound
     *	This exception raised when there is no required partition in the RM.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
		       PartitionNotFound,
                       rmgr::Exception,
		       "Can not find partition \"" << partition_name << "\" in the RM ",
                       ERS_EMPTY,
		       ((std::string)partition_name )
		      )

    /*! \class rmgr::PartitionAlreadyLoaded
     *	This exception raised if one tryes to load partition into the RM more than once.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       PartitionAlreadyLoaded,
                       rmgr::Exception,
                       "Can not load partition \"" << partition_name << "\". This partition has been already loaded.",
                       ERS_EMPTY,
                       ((std::string)partition_name )
		     )

    /*! \class rmgr::ApplicationNotFound
     *  This exception raised when there is no required application in the partition.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       ApplicationNotFound,
                       rmgr::Exception,
                       "Can not find application with name \"" << application_name << "\" in the partition  \"" << partition_name << "\".",
                       ERS_EMPTY,
                       ((std::string)application_name)
                       ((std::string)partition_name )
                     )

    /*! \class rmgr::ObjectNotFound
     *  This exception raised when there is no required object in the RM.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       ObjectNotFound,
                       rmgr::Exception,
                       "Can not find object \"" << type << "\".",
                       ERS_EMPTY,
                       ((std::string)type )
                     )

    /*! \class rmgr::UnavailableResources
     *  This exception raised if at least some of required resources are unavailable.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       UnavailableResources,
                       rmgr::Exception,
//                       "Can not grant resources. Unavailable in global scope:  \"" << global_resources << "\". Unavailable in partition scope:  \""  << partition_resources << '\"',
                       "Can not grant resources." << refuse_mess, //global_resources,
                       ERS_EMPTY,
                       ((std::string) refuse_mess )
                       //((std::string)global_resources )
                       //((std::string)partition_resources)
                     )
 
    /*! \class rmgr::HandleIdNotFound
     *  This exception raised when there is no required RM handle identifier in the RM.
     */
   ERS_DECLARE_ISSUE_BASE( rmgr,
                      HandleIdNotFound,
                       rmgr::Exception,
                      "Can not find RM handle identifier in the RM." ,
                      ERS_EMPTY, ERS_EMPTY
                    )

    /*! \class rmgr::ResourceNotFound
     *  This exception raised when there is no RM resource in the RM with required identifier.
     */
   ERS_DECLARE_ISSUE_BASE( rmgr,
                      ResourceNotFound,
                       rmgr::Exception,
                      "Can not find RM resource in the RM."
                      ,ERS_EMPTY
                      ,ERS_EMPTY
                    )

    /*! \class rmgr::ComputerNotFound
     *  This exception raised when there is no computer with the required identifier.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                      ComputerNotFound,
                       rmgr::Exception,
                      "Can not find computer with identifier \"" << computer_name << "\" in the RM.",
                      ERS_EMPTY,
                       ((std::string)computer_name )
                    )
    /*! \class rmgr::CommunicationException
     *  This exception raised when any communication exception was appear.
     * This exception usually thorwn when any communication exceptions catched in the  RM.
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       CommunicationException,
                       rmgr::Exception,
                       "CommunicationException raised with reason: " << reason,
                       ERS_EMPTY,
                       ((std::string)reason )
                      )
    /*! \class rmgr::ProcessNotFound
     *  This exception raised when for according computer there is no process for which any resource was granted).
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       ProcessNotFound,
                       rmgr::Exception,
                       "Can not find process with pid=" << pid << " on the computer \""<< computer << "\" those that for this process any resource in the partition \"" << partition << "\" was granted by the RM.",
                       ERS_EMPTY,
                       ((std::string)partition )
                       ((std::string)pid )
                       ((std::string)computer )
                     )
//sep 08
    /*! \class rmgr::AcceessDenied
     *  This exception raised if Access Manager did not allow to start action
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       AcceessDenied,
                       rmgr::Exception,
                       "Access Manager did not allow to free all resources in partition \"" << partition <<"\" for user " << myname << " from host " << myhost,
                       ERS_EMPTY,
                       ((std::string)partition )
                       ((std::string)myname )
                       ((std::string)myhost )
                     )

//oct 08
    /*! \class rmgr::CanNotStartServer
     *  This exception raised if RM_Server can not start by some reason
     */
    ERS_DECLARE_ISSUE_BASE( rmgr,
                       CanNotStartServer,
                       rmgr::Exception,
                       "RM_Server did not started. Reason:" << reason,
                       ERS_EMPTY,
                       ((std::string)reason )
                     )


/*
    /  *! \class rmgr::UniqueComputerNotFound
     *  This exception raised when more than one computer with required name found in the RM.
     *  /

    ERS_DECLARE_ISSUE( rmgr,
                      UniqueComputerNotFound,
                      "More than one computer with name \"" << computer_name << "\" found in the RM. Please check computer objects with indentifiers: " << 
computers_ids,
                       ((std::string)computer_name )
                       ((std::string)computers_ids )
                    )
*/
}

#endif 




