#include <iostream>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>
#include "ers/ers.h"

  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;

int main(int argc, char** argv)
{
 	// Declare arguments
        CmdArgStr  computer_name('c',"computer", "computer_name",
                         "computer id; parameter is used if someone needs to free all the resources in all the partitions for a particular machine.",  CmdArg::isREQ);
 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "Resource Manager server name; please use only for tests.");
	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &computer_name, 
                    &rm_server_name, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description( "Free all comuter Resource Manager  resources." );
       // Parse arguments
	cmd.parse(arg_iter);

  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found

  bool access = RMC->checkAccesPermission();
  std::cerr<<"ACCESS="<<access<<std::endl;
  if(!access) {
    ers::fatal( ers::Message( ERS_HERE, "No permission to free resources."));
    exit(-1);
  }

  const std::string my_computer = computer_name.isNULL() ? "*" : computer_name; 
  std::ostringstream txt ;

  try {
       RMC->freeComputer((const char*) computer_name);
       txt<<"All resources for computer \"" << computer_name << "\" were succesfully released.";
       ers::info( ers::Message( ERS_HERE, txt.str()));
  }
  catch (daq::rmgr::Exception & ex) {
           ers::error(ex);
  }
  delete RMC;
}
