#include <iostream>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>
#include "ers/ers.h"
#include <system/Host.h>
#include <system/User.h>

  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;

int main(int argc, char** argv)
{
 	// Declare arguments
 	CmdArgInt	handle_id('t', "handle", "handle_id", "RM handle which to be used to free resources(mandatory).", CmdArg::isREQ);
 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "resource manager server name; use only for tests.");
	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &handle_id, &rm_server_name, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description("Free Resource Manager resources that where granted with the RM handle.");
	handle_id = -1;
       // Parse arguments
	cmd.parse(arg_iter);
  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found

  bool access = RMC->checkAccesPermission();
  std::cerr<<"ACCESS="<<access<<std::endl;
  if(!access) {
    ers::fatal( ers::Message( ERS_HERE, "No permission to free resources."));
    exit(-1);
  }


  long   hndlId = handle_id;

   std::ostringstream txt ;
   try {
         RMC->freeResources( hndlId);
         txt<<"freeResources operation for handle ID = " << hndlId;
         ers::info( ers::Message( ERS_HERE, txt.str()));
   }
   catch (daq::rmgr::HandleIdNotFound & ex) {
           ers::error(ex);
   }     
   catch (daq::ipc::Exception & exipc) {
           ers::error(exipc);
   }
   catch(CORBA::SystemException & ex2) {
           txt<<"CORBA::SystemException " << ex2._name() << " received.";
           ers::info( ers::Message( ERS_HERE, txt.str()));
   }
   delete RMC; 
}
