#include <iostream>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>

#include "ers/ers.h"

  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;
int main(int argc, char** argv)
{
 	// Declare arguments
 	CmdArgStr	partition_name('p', "partition", "partition_id", "partition identifier (mandatory).", CmdArg::isREQ);
        CmdArgStr       binary_name('b', "binary", "binary_id", "Software  object (binary) identifier. Object for  which resources to be granted.", CmdArg::isREQ);
        CmdArgStr       computer_name('c', "computer", "computer_id", "identifier of the computer on which binary supposed to be started.", CmdArg::isREQ);
 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "resource manager server name; use only for tests.");
	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &partition_name, //&application_name, 
                   &binary_name, &computer_name, 
                    &rm_server_name, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description("Free application resources request.");
       // Parse arguments
	cmd.parse(arg_iter);
  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found

  std::string comp = computer_name.isNULL() ? "" : (const char*) computer_name;
  std::ostringstream txt ;
  if (partition_name.isNULL())
  {
        txt<<"rmgr_free_applicatioin_resources: No partition name in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
  else if (binary_name.isNULL())
  {
        txt<<"rmgr_free_applicatioin_resources: No binary ID in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
  else
  {
    try {
      	  RMC->freeApplicationResources( (const char*)partition_name, (const char*) binary_name, comp );
          txt<<"Application  resources were succesfully released.";
          ers::info( ers::Message( ERS_HERE, txt.str()));
    }     
    catch ( daq::rmgr::Exception & ex) {
          ers::error(ex);
    } 
  }
  delete RMC;
}
