#include <iostream>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>
#include "ers/ers.h"
  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;
int main(int argc, char** argv)
{
 	// Declare arguments
 	CmdArgStr	partition_name('p', "partition", "partition_name", "partition name (mandatory); partition to be checked if it has any resources.");
 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "resource manager server name; use only for tests.");

	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &partition_name, 
                    &rm_server_name, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description( "exit code 0 if there are no resources in partition and 1 - if at least one resource is allocated for this partition in Resource Manager server. Exit -1(255) if can not connect to server.");
       // Parse arguments
	cmd.parse(arg_iter);

	std::string   p;
  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found

  std::ostringstream txt ;
  std::string pname = "*";
  if( !partition_name.isNULL() )
    pname =(const char*) partition_name;
  int retv;
  try {
       daq::rmgr::Allocated_InfoHelper* res = RMC->getPartitionInfo(pname);
       if(res->allocInfo->length() < 1) {
          retv = 0;
          txt<< "No resources.";
       }
       else {
          retv = 1;
          txt << "Partition has resources.";
       }
       ers::info( ers::Message( ERS_HERE, txt.str()));
       delete res;
  }
  catch (daq::rmgr::PartitionNotFound & exp) {
           ers::error(exp);
           retv = 0;
  }
  catch (daq::ipc::Exception & exipc) {
           ers::error(exipc);
           retv = -1;
  }
  catch(CORBA::SystemException & ex) {
           txt<<"CORBA::SystemException " << ex._name() << " received.";
           ers::info( ers::Message( ERS_HERE, txt.str()));
           retv = -1;
  }
  catch (daq::rmgr::Exception & ex) {
           ers::error(ex);
           retv = -1;
  }

  delete RMC; 
  exit(retv);
}
