#include <iostream>
#include <system/User.h>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>

#include "ers/ers.h"

  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;
int main(int argc, char** argv)
{
 	// Declare arguments
 	CmdArgStr	partition_name('p', "partition", "partition_id", "partition identifier (mandatory).", CmdArg::isREQ);
        CmdArgStr       binary_name('b', "binary", "binary_id", "Software  object (binary) identifier. Object for  which resources to be granted.", CmdArg::isREQ);
        CmdArgStr       computer_name('c', "computer", "computer_id", "identifier of the computer on which binary supposed to be started.", CmdArg::isREQ);
        CmdArgStr       client_name('e', "client", "clientname", "Client which will be woner of hardare resoruces");
        CmdArgInt       process_id('t', "process", "processid", "Process id. Process for which resources to be allocated");
        CmdArgStr       hw_class_name('h', "hw_class", "hw_class_id", "Hardware class", CmdArg::isREQ);
        CmdArgStr       hw_obj_name('o', "hw_obj", "hw_obj_id", "Hardware object identifier", CmdArg::isREQ);

 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "resource manager server name; use only for tests.");
//        CmdArgStr  resource_id('r', "resource", "resource_id", "!!Used only for test with \'t\' parameter. UID of the RM resource to be requested for process.");
///        CmdArgBool test_process   ('t', "test_process",
///                                              "!!! Only for tests!! Request resources for process that will be started by this binary.");
//
	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &partition_name,  
                   &binary_name, &computer_name, &client_name,
                   &process_id, &hw_class_name, &hw_obj_name,
		   //  &resource_id, &test_process, //AI 051107 
                    &rm_server_name, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description("Request Resource Manager hardware resources for binary to be started on the computer. Only resource with the same as hw_class input parameter will be allocated if possible");
       process_id = 0;
       // Parse arguments
	cmd.parse(arg_iter);
  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found
///  int proc = test_process.flags() && CmdArg::GIVEN ? 1 : 0; 
  long handleId;
///  std::string comp = computer_name.isNULL() ? "" : (const char*) computer_name;
  std::ostringstream txt ;
  if (partition_name.isNULL())
  {
        txt<<"rm_allocate_resources: No partition name in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
  else if (binary_name.isNULL())
  {
        txt<<"rm_allocate_resources: No binary ID in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }

  ::System::User myname;
  std::string clnt= client_name.isNULL() ? myname.name() : (const char*) client_name;

  unsigned long procId = process_id;
  if(procId == 0)
     procId = getpid();

    try {
      	  handleId = RMC->requestHWResources( (const char*)partition_name, (const char*) binary_name, (const char*)computer_name, procId, (const char*)hw_class_name, (const char*) hw_obj_name, clnt );
          txt<<"Handle ID = " << handleId;
          ers::info( ers::Message( ERS_HERE, txt.str()));
    }     
    catch ( daq::rmgr::Exception & ex) {
          ers::error(ex);
    } 
  delete RMC;
}
