#include <signal.h>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/convenience.hpp>

#include "src/RM_Server.h"
#include <cmdl/cmdargs.h>
#include <ipc/alarm.h>
#include <system/File.h>

#define RMSERVERDEFAULT "RM_Server"  //tmp

 daq::rmgr::RM_Server* RMS = (daq::rmgr::RM_Server*)0;

/****
 bool autosaveBackup( void * server) {
   if( !(server == 0) )
      ( (daq::rmgr::RM_Server*) (server) )->saveBackupByCondition();
   return true;
 }
****/

void signal_handler( int sig )
{
  std::cout<<"Got signal "<<sig<<std::endl;
  std::cout.flush();

  if( RMS) {
    RMS->stop();
  }
}

int main(int argc,  char** argv)
{
      // Declare arguments
  CmdArgStr	server_name('n', "server", "server-name", "server to start.");
  CmdArgStr	log_file('l', "log", "backup-file", "Path to backup file. Backup file produced if this parameter added and parameter \'o\' is absent.");
  CmdArgBool     do_restore('r', "do_restore", "Restore data from backup during rm_server start in case this parameter added. rm_server starts without restore if backup file does not exists or unavailable.");
  CmdArgBool     no_backup('o', "no_backup", "Do not produce backup when rm_server running. Backup does not produced also in case of absence of backup-file prameter.");
  CmdArgInt       alarm_time('t', "alarm_t", "alarm_time", "Period in minutes to check if autosave backup should be done. Use for tests only");
  CmdArgInt       silence_time('k', "silence_t", "silence_time", "Time in seconds.Autosave backup may be produced only if there were no requests to the RM during this time. Use for tests only.");
  CmdArgInt       max_journallines('j', "max_journallines", "max_journallines", "Number of operations to keep in journal. Autosave backup will be produced when  this limit will be achived. Use for tests only.");
       // Declare command object and its argument-iterator
  CmdLine  cmd(*argv, &server_name, &log_file, &do_restore, &no_backup, &alarm_time, &silence_time, 
                &max_journallines, NULL);
  alarm_time = -1;
  silence_time = -1;
  max_journallines = -1;

  CmdArgvIter  arg_iter(--argc, ++argv);
  cmd.description("Starts Resource Manager server."); 

       // Parse arguments
  cmd.parse(arg_iter);

  signal( SIGINT , signal_handler );
  signal( SIGTERM, signal_handler );

  std::string lgf(log_file.isNULL() ?  "" : log_file);
  std::string serv(server_name.isNULL() ? RMSERVERDEFAULT : server_name);
  bool doresto = do_restore.flags() && CmdArg::GIVEN && (lgf.length() > 0) ;
  bool nobck = ( no_backup.flags() && CmdArg::GIVEN ) || lgf.length() == 0;
 /// std::string rf;
 /// rf = doresto ? lgf :  "";
  if(nobck && !doresto)
    lgf = "";
  try {
        std::list< std::pair< std::string, std::string > > opt = IPCCore::extractOptions(argc,argv);
        opt.push_front( std::make_pair( std::string("threadPerConnectionPolicy"), std::string("0") ) );
        opt.push_front( std::make_pair( std::string("maxServerThreadPoolSize"), std::string("8") ) );
        opt.push_front( std::make_pair( std::string("threadPoolWatchConnection"), std::string("0") ) );
        opt.push_front( std::make_pair( std::string("supportPerThreadTimeOut"), std::string("1") ) );

     IPCCore::init( opt );
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

/***
//move here from RM_Server
    if( doresto ) { // && !rf.empty() ) {
      std::ostringstream txt;
      try{
        ::System::File tst_restore( lgf.c_str() );
        if( !tst_restore.exists()) {
          throw std::runtime_error("file does not exist");
        }
///        if(tst_restore.size() < 1) {
///          throw std::runtime_error("file is empty");
///        }
        std::ifstream f(lgf.c_str());
///        if(!f) throw std::runtime_error("cannot read file");
        if(!f && tst_restore.size() > 0 ) throw std::runtime_error("cannot read file");
      }
      catch( std::exception& ex) {
        txt<<" restore backup file \""<<lgf<<"\" failed: " << ex.what();
        ers::fatal( ers::Message( ERS_HERE, txt.str()) );
       /// rf.clear(); //forget about wrong restore path
      }
    }

//check if journal file can be written (create directory if needed etc)
   if( !lgf.empty() ) {
     try {
        boost::filesystem::path myparent( boost::filesystem::path( lgf ).branch_path() );
        if ( !boost::filesystem::exists( myparent ) )  {
            boost::filesystem::create_directories( myparent );
        }
        std::ofstream logostrm;
        logostrm.open ( lgf.c_str(), std::ios::app );
        logostrm.close();
    }
    catch (std::ofstream::failure ex1) {
        ers::fatal( ers::Message( ERS_HERE, "Can not create backup \"" + lgf + "\":", ex1 ) );
        lgf.clear();
    }
    catch( std::exception & ex ) {
        ers::fatal( ers::Message( ERS_HERE, "Can not create directory for backup \"" + lgf + "\":", ex ) );
        lgf.clear();
    }
    if( lgf.empty() )
       ers::error(ers::Message( ERS_HERE, "RM_Server will be started without backup" ) );
  }
*****/

  RMS = new daq::rmgr::RM_Server(serv.c_str(), lgf.c_str(),!nobck, doresto );

/****
  if( silence_time > 0) {
//    std::cerr<<"MAX_OP_JOURNAL="<<silence_time<<std::endl;
    long valtime( silence_time );
    RMS->setNoActivityMin( valtime );
  }

//  IPCAlarm * alarm = new IPCAlarm( 60 * 2 , autosaveBackup, RMS ); // every 2 minutes
  IPCAlarm * alarm = 0;
  if( alarm_time > 0 ) {
// std::cerr<<" alarm_time="<<alarm_time<<std::endl;
    alarm = new IPCAlarm( 60 * alarm_time, autosaveBackup, RMS );
  }
  else
    alarm = new IPCAlarm( 60 * 10 , autosaveBackup, RMS ); // every 10 minutes

  if( max_journallines > 0 ) {
    long maxlines(max_journallines);
    RMS->setMaxOpJournal( maxlines );
  }
****/


  try{
    RMS->run();
  }
  catch (daq::ipc::InvalidPartition & ex_p1) {
      ers::fatal(ex_p1);
  }
  catch (daq::ipc::InvalidObjectName & ex_p2) {
      ers::fatal(ex_p2);
  }
  catch ( daq::rmgr::CanNotStartServerE & srvex ) {
     std::ostringstream txt ;
     txt << "RM_Server did not started. Reason:" << srvex.reason.in();
      ers::fatal( ers::Message( ERS_HERE, txt.str())  );
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }
  
  if(RMS)
  	delete RMS;
//////  delete alarm;
}
