package daq.rmgr;

/**
 * This class implements all public API functionality for
 * the Resource manager client.
 * Creation date: (07.03.2005 17:54:58)
 * @author: Igor Aleksandrov
 */
 import java.util.Vector;

import org.omg.CORBA.Object;
import config.Configuration;
import config.Query;
import dal.*;
import ipc.*;
import rmgr.*;
 
 
public class RMClientImpl {
   static final String  RMSERVERDEFAULT = "RM_Server";
   ResMgr rmdb = null;
   boolean debug = false;
   String serverName = "";
   int verbose = 0;

   public String _last_granted;
   public String _last_released;   
/**
 * RMClientImpl default constructor.
 */
public RMClientImpl() {
   super();
   serverName = RMSERVERDEFAULT;
   try {
     rmdb = getDDB();
   }
   catch (RuntimeException ex) {
     rmdb = null;
   }
}
/**
 * Sets verbosity value
 * @param int val
*/
 public void setVerbose(int val) { verbose = val; }
/**
 * Gets verbosity value
 * @return int
*/
 public int getVerbose() { return verbose; }

/** Constructor to create object to inteact with RM server with name ser_name 
 * @param :serv_name Name of the RM server
 */
public RMClientImpl(String serv_name) {
   super();
   if(serv_name != null && serv_name.length() > 0)
       serverName = serv_name;
   else
       serverName = RMSERVERDEFAULT;
   rmdb = getDDB();
}

/**
     * Returns the Resource Manager server handle.
     * If the handle is not found, a message is printed at standard output
     * and RuntimeException raised.
*/
 public ResMgr getDDB() throws RuntimeException 
 {
      ipc.Partition p = new ipc.Partition();
      try {
	  rmgr.ResMgr rmdb_ = (rmgr.ResMgr) p.lookup(rmgr.ResMgr.class, serverName);
	  return rmdb_;
      }
      catch(Exception e) {
	  throw  new RuntimeException("Can not lookup RM server object");
      }
 }

public config.Configuration getConfiguration(String dbname) throws config.ConfigException {
  config.Configuration conf = new config.Configuration(dbname);
  return conf;
}

private int getMyPid() {
    int process_id = 0;

    try {
      process_id = Integer.parseInt( ( new java.io.File("/proc/self")).getCanonicalFile().getName() );
    }
    catch (NumberFormatException e) {
      e.printStackTrace();
    }
    catch (java.io.IOException e) {
      e.printStackTrace();
    }
    return process_id;
}

/**
 * Requests resource for process. This method got process id 
 * automatically from the system and this value is used in the 
 * RM server in resource request and stored in the RM in case
 * of resource was granted. This value can be used to free 
 * resource or get information about it.
 * @return int  RM handle identifier. If this value is equled to zero - means there are no needs of resources and can run. If handle less than zero - process shoudl be stopped (no available resources for it). 
 * @param partition java.lang.String Name of the partition
 * @param resource  java.lang.String Resource configuration UID
 * @param computerID  java.lang.String Computer object UID.
 * 
 * @exception daq.rmgr.ObjectNotFoundE  Some accociated with the applciation object was not found in the RM server. It could be computer or RM resource etc.
 * @exception daq.rmgr.UnavailableResourcesE   There are no enough resources for the application.
 * @exception daq.rmgr.ResourceNotFoundE Requested resource was not found in configuration. 
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
*/
public int requestResourceForMyProcess( String partition, String resource,  String computerID )  throws  
              daq.rmgr.ObjectNotFoundE 
              ,daq.rmgr.UnavailableResourcesE
 //             ,daq.rmgr.ComputerNotFoundE
              ,daq.rmgr.ResourceNotFoundE              
              ,RuntimeException 
 {
     getDDB();
     String user = System.getProperty("user.name");
     int process_id = getMyPid();
     ResID[] res_lst = new ResID[1];
     res_lst[0] = new ResID();
     res_lst[0].id = resource;
       long tm = System.currentTimeMillis();
       tm = (tm << 5) >> 5;
       int time = (int) tm;
       int hndl = rmdb.requestResourcesForProcess( partition, res_lst, computerID, process_id, user, time, true);
     return hndl;
}

/**
 *  Free resources of the process. Resource that was granted for process 
 * with the same partition, resource and computer UID will be released 
 * in the RM and can be used for other clients.
 * @param partition java.lang.String Name of the partition
 * @param computerID  java.lang.String Computer object UID.
 * @param pid int prosecc id
 * @exception daq.rmgr.PartitionNotFoundE No partition with allocated resources in the RM.
 * @exception daq::rm::ProcessNotFoundE No allcoated resources for requested process.
 */
public void freeProcessResources( String partition, String computerID, int pid) throws
          daq.rmgr.PartitionNotFoundE
         ,daq.rmgr.ProcessNotFoundE
         ,daq.rmgr.ObjectNotFoundE
{
     getDDB();
     rmdb.freeProcessResources( partition, computerID, pid,
                                true, getCallerInfo() );
} 


/**
 * Requests resources for software object.
 * Creation date: (15.10.2006 18:40:54)
 * @return int  RM handle identifier. If this value is equled to zero - means there are no needs of resources for the application.
 * @param partition java.lang.String Name of the partition
 * @param swobjID java.lang.String Software object UID.
 * @param computerID  java.lang.String Computer object UID.
 * @param client java.lang.String Client name. This name is stored in the RM server as the owner of the application.
 * @exception daq.rmgr.ObjectNotFoundE  Some accociated with the applciation object was not found in the RM server. It could be computer or RM resource etc.
 * @exception daq.rmgr.UnavailableResourcesE   There are no enough resources for the application.
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
 */
public int requestResources(String partition, String swobjID, String computerID, String client) throws 
              daq.rmgr.ObjectNotFoundE, daq.rmgr.UnavailableResourcesE
//	      ,daq.rmgr.ComputerNotFoundE
              ,RuntimeException {
       getDDB();
       String user = client == null || client.length() == 0 ? System.getProperty("user.name") : client;
       int hndl;
        _last_granted = "";
    int process_id = getMyPid();
    long tm = System.currentTimeMillis();
    tm = (tm << 5) >> 5;
    int time = (int) tm;

       hndl = rmdb.requestResources (partition, swobjID, computerID, process_id,  user,
                 time, true);
       if(verbose > 0)
         if(hndl <= 0) {
              System.out.println("***RMClientImpl.requestResources called with pname = "
                       + partition + ", SW object ID  = " + swobjID + " , Computer ID = " + computerID + " and client name " + client);
         }
      return (int) hndl;
}

/**
 * Requests resources for software object. The same as requestResources(String partition, String swobjID, String computerID, String client) but for client user name is used.
 */
public int requestResources(String partition, String swobjID, String computerID ) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ObjectNotFoundE, daq.rmgr.UnavailableResourcesE
//, daq.rmgr.ComputerNotFoundE 
              , RuntimeException
{
 return requestResources(partition, swobjID, computerID, System.getProperty("user.name"));
}

/**
 * Requests resources for software object. The same as requestResources(String partition, String swobjID, String computerID) but for computerID just " " as value used. Can be used for binaries when only resources of software resource type linked to binary.
 */
public int requestResources(String partition, String swobjID) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ObjectNotFoundE, daq.rmgr.UnavailableResourcesE
//, daq.rmgr.ComputerNotFoundE
              , RuntimeException
{
/// return requestResources(partition, swobjID, " " , System.getProperty("user.name"));
 return requestResources(partition, swobjID, getMyHost() , System.getProperty("user.name"));
}


/**
 * Frees resources that were granted with the RM handle.
 * @param int handle  RM handle identifier. 
 * @param boolean test
 * @exception daq.rmgr.HandleIdNotFoundE  There is no handle with requested identifier in the RM server.
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
 */
public void freeResources(int handle) throws daq.rmgr.HandleIdNotFoundE
              ,RuntimeException
{
    getDDB();
    if(rmdb == null) {
	if(verbose > 0) 
	    System.out.println("rmdb == null");
    }
    rmdb.freeResources(handle, true, getCallerInfo());
}

//forIGUI

/** Checks if resource exists in the RM server.
 * @return true if resource exists and false if no. 
 * @param java.lang.String partitionName Name of the partition.
 * @param java.lang.String rname Name of the resource.
 */
public boolean isResourceExists(String partitionName, String rname) {
    try{
        getDDB();
       Allocated_Resource[] test = getResourceInfo(partitionName, rname); 
       if (test == null || test.length == 0)
	   return false;
       else
           return true;
    }
    catch(daq.rmgr.ResourceNotFoundE exr) {
          return false;
    }
    catch(Exception ex) { 
       return false;
    }
}

/**
 * Gets information about resources that were granted with requested handle indentifier.
 * @return java.lang.String  Information about resources that were granted with requested handle indentifier.
 * @param long handle   Handle identifier.
 * @exception daq.rmgr.HandleIdNotFoundE There is no handle with requested identifier in the RM server.
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
 */
public Handle_Info getHandleInfo(long handle) throws daq.rmgr.HandleIdNotFoundE, RuntimeException {
	getDDB();
        return rmdb.getHandleInfo((int)  handle);
}

/**
 * Gets information about granted resources in the partition.
 * @return daq.rmgr.Allocated_Info  List of granted resources.
 * @param partition java.lang.String partition  Name of the partition. Value "*" means in all partitions.
 */
public daq.rmgr.Allocated_Resource[] getPartitionInfo(String partition) throws daq.rmgr.PartitionNotFoundE, RuntimeException
{
	    getDDB();
            if(rmdb == null) {
		if(verbose > 0) 
		    System.out.println("RMDB is null: incorrect connection with RM_SEVER" );
	//	throws new  daq.rmgr.PartitionNotFoundE(" No server found");
	    }
            return rmdb.getPartitionInfo(partition);
}

/**
 *  * Gets information about granted resources in the partition.
 *   * @return  daq.rmgr.Allocated_Info  List of  granted resources.
 *    * @param partition java.lang.String partition  Name of the partition. Value "*" means in all partitions.
 *     * @param rname java.lang.String  rname Name of the resource. Value "*" means for all resources.
 *       */
public daq.rmgr.Allocated_Resource[] getResourceInfo(String partition, String rname) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ResourceNotFoundE, RuntimeException
{
            getDDB();
            if(rmdb == null) {
                if(verbose > 0)
                    System.out.println("RMDB is null: incorrect connection with RM_SEVER" );
                //throws new  daq.rmgr.PartitionNotFoundE(" No server found");
            }
            return rmdb.getResourceInfo(partition, rname);
}

/**
 *  Gets information about granted resources in the partition.
 *  @return  daq.rmgr.Allocated_Info  List of  granted resources.
 *  @param partition java.lang.String partition  Name of the partition. Value "*" means in all partitions.
 *  @param rname java.lang.String  rname Name of the resource. Value "*" means for all resources.
 **/
public Allocated_Resource[] getComputerResourceInfo(String partition, String rname, String computer) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ResourceNotFoundE, RuntimeException
{
            getDDB();
            if(rmdb == null) {
                if(verbose > 0)
                    System.out.println("RMDB is null: incorrect connection with RM_SEVER" );
             //   throws new  daq.rmgr.PartitionNotFoundE(" No server found");
            }
            return rmdb.getComputerResourceInfo(computer,partition, rname);
}


/**
 * Gets list of the partitions.
 * @return java.util.Vector partitions that were registered in the RM server.
 * @exception RntimeException  Runtime exception. May be RM server object did not found.
 */
public RM_str[] getPartitionsList() throws RuntimeException {
//	Vector partitions = new Vector();
	RM_str[] plist = null;
	getDDB();
	if(rmdb == null){
            plist = new  RM_str[0];
	    return plist;
        }
	plist =  rmdb.getPartitionsList();
	return plist;
}

/**
 *  Gets information about configuration
 *  @return  daq.rmgr.
**/
public Config_update getCinfigInfo() {
        getDDB();
        Config_update[] cinfo = null;
        if(rmdb == null) {
           return null;   
        }
        return rmdb. getConfigInfo();
} 


/**
 * Gets list of running RDBs.
 * @return java.util.Vector list of strings with running RDBs.
 */
public Vector getRDBList() {
   Vector rdbList = new Vector();

   try {
   PartitionEnumeration pit = new PartitionEnumeration();
   while ( pit.hasMoreElements() ){
       ipc.Partition p = pit.nextElement();
       TypeEnumeration tit = new TypeEnumeration( p );
       while ( tit.hasMoreElements() ){
	   String type = tit.nextElement();
	   ObjectEnumeration oit = new ObjectEnumeration( p, type );
	   if(type.equals("rdb/cursor")){
	       while( oit.hasMoreElements() ){
                   ipc.Element obj = oit.nextElement();
                   String st = "rdbconfig:"+p.getName()+"::"+obj.name;
                   rdbList.add(st);
	       }
	   }
       }
   }
   }
   catch(InvalidPartitionException ex) {
       System.err.println(ex);
   }
   return rdbList;
}
    

public void rmConfigurationUpdate ( String dbname ) throws config.ConfigException , daq.rmgr.ConfigException,  RuntimeException
{
      //create  configuration and fill according data 
      daq.rmgr.Config_update idl_conf = new daq.rmgr.Config_update();
      config.Configuration db = new config.Configuration(dbname);

      int i;
      dal.RM_SW_Resource[] swrs = dal.RM_SW_Resource_Helper.get(db, (config.Query) null);
      if(swrs != null) {
        idl_conf.swr_lst = new daq.rmgr.SW_Resource[swrs.length];
        for( i=0; i<swrs.length; i++) {
          idl_conf.swr_lst[i] = new daq.rmgr.SW_Resource();
           idl_conf.swr_lst[i].rid = swrs[i].UID();
           idl_conf.swr_lst[i].max_total = swrs[i].get_MaxCopyTotal();
           idl_conf.swr_lst[i].max_pp = swrs[i].get_MaxCopyPerPartition();
        }
      }
      dal.RM_HW_Resource[] hwrs = dal.RM_HW_Resource_Helper.get(db, (config.Query) null);
      if(hwrs != null) {
         idl_conf.hwcr_lst = new daq.rmgr.HWC_Resource[hwrs.length];
         for( i=0; i<hwrs.length; i++) {
           idl_conf.hwcr_lst[i] = new daq.rmgr.HWC_Resource();
           idl_conf.hwcr_lst[i].rid = hwrs[i].UID();
           idl_conf.hwcr_lst[i].max_total = hwrs[i].get_MaxCopyTotal();
           idl_conf.hwcr_lst[i].max_pp = hwrs[i].get_MaxCopyPerPartition();
         }
      }

      dal.SW_Object[] swobjs = dal.SW_Object_Helper.get(db, (config.Query) null);
      if(swobjs != null) {
         int numsw = 0;
         idl_conf.swobj_lst = new daq.rmgr.SW_Object[swobjs.length];
         //only SW obj with resources should be taken into account
         Vector indexes = new Vector();
         for( i=0; i<swobjs.length; i++) { //AI 2017 comment if statment
//           if(swobjs[i].get_Needs() != null) {
             indexes.add(new Integer(i));
             numsw++;
//           }
         }
         idl_conf.swobj_lst = new daq.rmgr.SW_Object[numsw];
         for( i=0; i<numsw; i++) {
           idl_conf.swobj_lst[i] = new daq.rmgr.SW_Object();
           int idal = ((Integer) indexes.get(i)).intValue();
           RM_Resource[] rlst = swobjs[idal].get_Needs();
           idl_conf.swobj_lst[i].swid = swobjs[idal].UID();
           idl_conf.swobj_lst[i].res_list = new ResID[rlst.length];
           for (int j=0; j<rlst.length; j++) {
             idl_conf.swobj_lst[i].res_list[j] = new ResID();
             idl_conf.swobj_lst[i].res_list[j].id = rlst[j].UID();
           }
         }
      }
      db.unload();

      // configuratioin is created. Now call to server
      getDDB();
      rmdb.rmConfigUpdate( idl_conf );
 
}

public String getMyHost() {
   String host;

    try {
        java.net.InetAddress addr = java.net.InetAddress.getLocalHost();
        host = addr.getHostName();
    }
    catch (java.net.UnknownHostException e) {
      host = " ";
    }
  return host;  
}

/**
 * Free all resources in the partition.
 * @return java.lang.String String representation of the action status.
 * @param p java.lang.String Name of the partition.
 * @exception  daq.rmgr.PartitionNotFoundE  Partition was not found in the RM server that means was not registered.
 * @exception   daq.rmgr.ApplicationNotFoundE Application with requested name was not found in the partition.
 * @exception   daq.rmgr.AcceessDeniedE Access Manager denied for user to free resources
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
 */
///////public void freeAllResources(String p) 
public void freePartition( String p ) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ObjectNotFoundE,daq.rmgr.AcceessDeniedE, RuntimeException //// , daq.rmgr.ApplicationNotFoundE
{
   String user_id = System.getProperty("user.name");
   String host = getMyHost();

   getDDB();
   rmdb.freePartition( host, user_id, p );
}

/**
 * Gets those client names to which according resources were granted.
 * @return java.lang.String Client names separated by semicolomns.
 * @param rname java.lang.String  Name of the resource.
 * @param p java.lang.String Name of the partition.
 * @exception  daq.rmgr.PartitionNotFoundE  Partition was not found in the RM server that means was not registered.
 * @exception RuntimeException  Runtime exception. May be RM server object did not found.
 */
public RM_str[]  getResourceOwners(String rname, String pname) throws daq.rmgr.PartitionNotFoundE, daq.rmgr.ResourceNotFoundE, RuntimeException
{
       getDDB();
       return rmdb.getResourceOwners(rname, pname);
}

/**
 * Checks if RM server running.
 * @return boolean true if RM server is running or false otherwise. 
 */
public boolean rmServerExists() {
   try {
     getDDB();
   }
   catch (RuntimeException ex) {
      if( verbose > 0 )
         ex.printStackTrace();
      rmdb = null;
   }
   catch (Throwable thr) {
      if(  verbose > 0 )
         thr.printStackTrace();
      rmdb = null;
   }
   return rmdb != null;
}

//AI oct 2015
public CallerInfo getCallerInfo() {
     daq.rmgr.CallerInfo caller = new daq.rmgr.CallerInfo();
     caller.host = getMyHost();
     caller.user = System.getProperty("user.name");
     caller.isJava = true;
     return caller;
}

}
