#include "ers/ers.h"
#include "ResourceManager/RM_Client.h"
#include <dal/Partition.h>
#include <dal/SW_Object.h>
#include <dal/RM_Resource.h>
#include <dal/RM_SW_Resource.h>
#include <dal/RM_HW_Resource.h>

#include <system/Host.h>
#include <system/User.h>

#include <time.h>

namespace daq   {
 namespace rmgr  {

static boost::mutex cltstamp_mutex;
static const long sleepTime = 6; // 4;

static const long maxDelay = 12; 

static const std::string envPar = "TDAQ_RM_CLIENT_RETRY_TIMEOUT";
static uint16_t stampID = 0;
static const uint16_t ip_shifts[4] = {40,32,24,16};
static const  uint64_t shift = 0xFFFF;
static const  uint64_t pid_mask = (shift << 48); //pid mask

//static const uint64_t pidMax_mask = ((uint64_t)((int64_t)(-1))) >> 16; //to be used to find higher key value for pid
static const uint16_t pid_shift = 48;
//static const uint16_t clid_mask = 0xFFFF;
 }
} //end namespace daq

#define RMSERVERDEFAULT "RM_Server"  //tmp

daq::rmgr::RM_ClientImpl::RM_ClientImpl():m_defpart(),m_rmDbName(RMSERVERDEFAULT)
{
}

daq::rmgr::RM_ClientImpl::RM_ClientImpl(const char *sid):m_defpart(),m_rmDbName(sid)
{
}


daq::rmgr::Rmstr_lstHelper*
daq::rmgr::RM_ClientImpl::getPartitionsList(){
  lookupRMDDB();
  try {
    daq::rmgr::Rmstr_lst* lst =  RMDB->getPartitionsList();
    daq::rmgr::Rmstr_lstHelper* retv = new daq::rmgr::Rmstr_lstHelper(lst);
    return retv;
  }
  catch(CORBA::SystemException & ex) {
  }
  daq::rmgr::Rmstr_lst* lstNull;
  lstNull = 0;
  return new daq::rmgr::Rmstr_lstHelper(lstNull); //empty
}


uint16_t daq::rmgr::RM_ClientImpl::getStampId() { 
  boost::mutex::scoped_lock scoped_lock( cltstamp_mutex );
  return ++stampID; 
}

uint64_t daq::rmgr::RM_ClientImpl::generateCallStamp() {
  uint64_t my_key = ( ((uint64_t) getpid()) << pid_shift);
  try{
     ::System::LocalHost myhost;
     std::string mhost;
     mhost = myhost.full_name();
     std::string ip = myhost.ip_string();
     std::string delim = ".";
     int ind1;
     int ind2( -1 );
     std::string* ipP = new std::string[4]; 
     for(int i=0; i<4; i++) {
       ind1 = ind2 + 1;
       ind2 = ip.find_first_of(delim,ind1);
       ipP[i] = std::string(ip,ind1, ind2-ind1);
     }

// call stamp structure: in uint64_t (8 bytes total, to be unique for any client)
// pid - 2 bytes
// ip4 - 1 byte
// ip3 - 1 byte
// ip2 - 1 byte
// ip1 - 1 byte
//generate call stamp key
    for(int ii=3; ii>=0; ii--) {
      my_key |= ( (uint64_t) atoi( ipP[ii].c_str() ) ) << ip_shifts[ii];
    }
    my_key |= getStampId();
//   uint64_t keyMax = my_key | pidMax_mask;
    delete[] ipP;
  }
  catch( ... ) {
   //test
   std::cerr<<"Can not create my Host"<<std::endl;
   ////throw daq::rmgr::AcceessDenied( ERS_HERE, partition.c_str(), "unknown host", "unknown user");
   my_key = 0;
  }
  return my_key;
}

unsigned long mypid =  getpid();

 long daq::rmgr::RM_ClientImpl::requestResources (const std::string& partition, const std::string& swobjectid, const std::string& computerName, const std::string& clientName) {
     unsigned long mypid =  getpid();
     std::string no_client("NO");
     std::string client(clientName);
     if( client == no_client) {
       try{
        ::System::User myname;
        client = myname.name();
       }
       catch( ... ) {}
     }
     return requestResources ( partition, swobjectid, computerName, mypid, client.c_str() );
}

 long daq::rmgr::RM_ClientImpl::requestResources (const std::string& partition, const std::string& swobjectid, const std::string& computerName, unsigned long mypid, const std::string& clientName) {
  std::string tmpstr;
  std::ostringstream text;
  int attemps = 3;
  int itray = 0;
  uint64_t tstamp = generateCallStamp();
  long time0 = (long) time(NULL);
  long retv( 0 );  

  while(itray < attemps) { //if unusual exception - try once more
   try {
        if( itray > 0 ) 
          sleep( getSleepTime( time0 ) ); // if troubles in connecting to server - sleep sleepTime sec before next call

        lookupRMDDB(); 
        retv = RMDB->requestResources ( partition.c_str(), swobjectid.c_str(), computerName.c_str(), mypid, clientName.c_str(),
              tstamp , itray == 0 );
        if( itray > 0 ) {
           std::ostringstream txt;
           txt<<retv;
           ers::warning (ers::Message( ERS_HERE, "got RM handle=" + txt.str() + " after several attempts" ) );
        }

        return retv;
   }
   catch (daq::rmgr::PartitionNotFoundE & exp) {
           throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
   }
   catch (daq::rmgr::ObjectNotFoundE & exo) {
           throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
   }
   catch (daq::rmgr::UnavailableResourcesE  & exr) {
           throw daq::rmgr::UnavailableResources ( ERS_HERE, exr.refused_mess.in() );
   }
   catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
   }
   catch (daq::ipc::Exception & exipc) {
           itray++;
           if(itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
   }
   catch(CORBA::SystemException & excorb) {
           itray++;
           if(itray >= attemps ) {
              //text << excorb;
              tmpstr = excorb._name();
              throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
           }
   }
   catch( ...) {
           itray++;
           if(itray >= attemps ) {
              throw daq::rmgr::CommunicationException( ERS_HERE, " Unkown exception in RM" );
           }
   }
  }
  return retv;
}

 long daq::rmgr::RM_ClientImpl::requestHWResources (const std::string& partition, const std::string& swobjectid, const std::string& computerName, unsigned long pid, const std::string& hwClass, const std::string& hwId, const std::string& client) {
  std::string tmpstr;
  std::ostringstream text;
  int attemps = 3;
  int itray = 0;
  uint64_t tstamp = generateCallStamp();
  long time0 = (long) time(NULL);
  long retv( 0 );

  while(itray < attemps) { //if unusual exception - try once more
   try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) ); // if troubles in connecting to server - sleep sleepTime sec before next call

        lookupRMDDB();
        retv = RMDB->requestHWResources ( partition.c_str(), swobjectid.c_str(), computerName.c_str(),  hwClass.c_str(), hwId.c_str(), pid, client.c_str(),
              tstamp , itray == 0 );
        if( itray > 0 ) {
           std::ostringstream txt;
           txt<<retv;
           ers::warning (ers::Message( ERS_HERE, "got RM handle=" + txt.str() + " after several attempts" ) );
        }

        return retv;
   }
   catch (daq::rmgr::ObjectNotFoundE & exo) {
           throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
   }
   catch (daq::rmgr::UnavailableResourcesE  & exr) {
           throw daq::rmgr::UnavailableResources ( ERS_HERE, exr.refused_mess.in() );
   }
   catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
   }
   catch (daq::ipc::Exception & exipc) {
           itray++;
           if(itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
   }
   catch(CORBA::SystemException & excorb) {
           itray++;
           if(itray >= attemps ) {
              //text << excorb;
              tmpstr = excorb._name();
              throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
           }
   }
   catch( ...) {
           itray++;
           if(itray >= attemps ) {
              throw daq::rmgr::CommunicationException( ERS_HERE, " Unkown exception in RM" );
           }
   }
  }
  return retv;
}

void daq::rmgr::RM_ClientImpl::freeResources (long handleId, bool log_this_call ) {
  std::string tmpstr;
  std::ostringstream text;
  std::string retstr;
  int attemps = 3;
  int itray = 0;
  long time0 = (long) time(NULL);
  ::CORBA::Boolean add2log(log_this_call);
  const daq::rmgr::CallerInfo& caller = getCallerInfo();

 while(itray < attemps) { //RM server restart case into account
   try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) ); // if troubles in connecting to server - sleep sleepTime sec before next call
        lookupRMDDB();
        RMDB->freeResources( handleId, add2log, caller);

        return;
   }
   catch (daq::rmgr::HandleIdNotFoundE & exh) {
           throw daq::rmgr::HandleIdNotFound( ERS_HERE );
   }
   catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
   }
    catch (daq::ipc::Exception & exipc) {
           itray++;
           if(itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
    }
    catch(CORBA::TRANSIENT& ex) {
    }
    catch(CORBA::SystemException & excorb) {
           itray++;
           if(itray >= attemps ) {
              //text << excorb;
              tmpstr = excorb._name();
              throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
           }
    }
    catch( ...) {
           itray++;
           if(itray >= attemps ) {
              throw daq::rmgr::CommunicationException( ERS_HERE, " Unkown exception in RM" );
           }
    }
 }

}

//enum RequestType {FreePartition, FreeComputer, FreeResource, FreePartitionResource, FreeComputerResource, FreeApplicationResources};

void daq::rmgr::RM_ClientImpl::freePartition( const std::string&  p ) {
  std::string mhost;
  std::string mname;
  try{
   ::System::LocalHost myhost;
   mhost = myhost.full_name();
   ::System::User myname;
   mname = myname.name();
  }
  catch( ... ) {
   throw daq::rmgr::AcceessDenied( ERS_HERE, p.c_str(), "unknown host", "unknown user");
  }

       daq::rmgr::ClientRequest request(daq::rmgr::RequestType::FreePartition, mhost.c_str(), mname.c_str(), p.c_str(), true); 
       performRequest(request); 
}

void  daq::rmgr::RM_ClientImpl::freePartitionResource(const std::string&  p, const std::string& resource, bool log_this_call) {
      daq::rmgr::ClientRequest request( daq::rmgr::RequestType::FreePartitionResource, p.c_str(), resource.c_str(), log_this_call );
      performRequest(request);
}

void  daq::rmgr::RM_ClientImpl::freeComputer(const std::string& computer, bool log_this_call) {
       daq::rmgr::ClientRequest request(daq::rmgr::RequestType::FreeComputer, computer.c_str(), log_this_call);
       performRequest(request);
}

void  daq::rmgr::RM_ClientImpl::freeResource(const std::string& resource, bool log_this_call) {
       daq::rmgr::ClientRequest request(daq::rmgr::RequestType::FreeResource, resource.c_str(), log_this_call);
       performRequest(request);
}

void  daq::rmgr::RM_ClientImpl::freeComputerResource(const std::string& resource, const std::string& computer, const std::string&  p, bool log_this_call) {
       daq::rmgr::ClientRequest request(daq::rmgr::RequestType::FreeComputerResource, resource.c_str(), computer.c_str(), p.c_str(), log_this_call);
       performRequest(request);
}

void daq::rmgr::RM_ClientImpl::freeApplicationResources(const std::string& p, const std::string& swObj, const std::string& computer, bool log_this_call) {
       daq::rmgr::ClientRequest request(daq::rmgr::RequestType::FreeApplicationResources, p.c_str(), swObj.c_str(), computer.c_str(), log_this_call);
       performRequest(request);
}

//std::vector<std::string> 
daq::rmgr::Rmstr_lstHelper*
daq::rmgr::RM_ClientImpl::getResourceOwners (const std::string& rname, const std::string& p){
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;
    std::vector<std::string> retv;
    try {
            daq::rmgr::Rmstr_lst* lst =
              RMDB->getResourceOwners(rname.c_str(), p.c_str());
            daq::rmgr::Rmstr_lstHelper* retv = new daq::rmgr::Rmstr_lstHelper(lst);
            return retv;
    }
    catch (daq::rmgr::PartitionNotFoundE & exp) {
           throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
    }
    catch (daq::rmgr::ResourceNotFoundE & exr) {
           throw daq::rmgr::ResourceNotFound( ERS_HERE );
    }
    catch( daq::rmgr::CommunicationException&  excom) {
           text << excom;
           throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
    }
    catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
    }
    catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
    }
//    return retv;	
  daq::rmgr::Rmstr_lst* lstNull;
  lstNull = 0;
  return new daq::rmgr::Rmstr_lstHelper(lstNull); //empty
}

// throws daq::rmgr::CommunicationException
void daq::rmgr::RM_ClientImpl::lookupRMDDB() {
  std::ostringstream text;
  std::string tmpstr;
  try {
    RMDB = m_defpart.lookup< ::rmgr::ResMgr>(m_rmDbName.c_str());
  }
  catch (daq::ipc::Exception & exipc) {
       text << exipc;
       throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
       //text << excorb;
       tmpstr = excorb._name();
       throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

long daq::rmgr::RM_ClientImpl::requestResourceForMyProcess(
                        const std::string& partition,
                        const std::string& resource,
                        const std::string& computer,
                        const std::string& clientName) {
   std::ostringstream text;
   std::string tmpstr;
   unsigned long mypid =  getpid();
   daq::rmgr::Res_ID_list res_lst;
   res_lst.length(1);
   res_lst[0].id = CORBA::string_dup(resource.c_str());
   int attemps = 3;
   int itray =0;
   uint64_t tstamp = generateCallStamp();
   long time0 = (long) time(NULL);
   long retv( 0 );

  while(itray < attemps) { //if unusual exception - try once more
   
   try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) ); // if troubles in connecting to server - sleep sleepTime sec before next call
        lookupRMDDB();
        retv = RMDB->requestResourcesForProcess(partition.c_str(), res_lst, computer.c_str(), mypid, clientName.c_str(),
              tstamp , itray == 0 );
        if( itray > 0 ) {
           std::ostringstream txt;
           txt<<retv;
           ers::warning (ers::Message( ERS_HERE, "got RM handle=" + txt.str() + " after several attempts" ) );
        }
        return retv;
   }
   catch (daq::rmgr::PartitionNotFoundE & exp) {
           throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
   }
   catch (daq::rmgr::ObjectNotFoundE & exo) {
           throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
   }
   catch (daq::rmgr::UnavailableResourcesE  & exr) {
           throw daq::rmgr::UnavailableResources ( ERS_HERE,  exr.refused_mess.in() );
   }
   catch (daq::rmgr::ResourceNotFoundE & excres) {
           throw daq::rmgr::ResourceNotFound ( ERS_HERE );
   }
    catch (daq::ipc::Exception & exipc) {
           itray++;
           if(itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
    }
    catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
    }
    catch(CORBA::SystemException & excorb) {
           itray++;
           if(itray >= attemps ) {
            //text << excorb;
             tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
           }
    }
    catch( ...) {
           itray++;
           if(itray >= attemps ) {
              throw daq::rmgr::CommunicationException( ERS_HERE, " Unkown exception in RM" );
           }
   }

  }
  return retv;
}

 long daq::rmgr::RM_ClientImpl::requestHWResourceForProcess(
                        const std::string& partition,
                        const std::string& resource,
                        const std::string& computer,
                        const std::string& hwId,
                        unsigned long pid,
                        const std::string& client) {
   std::ostringstream text;
   std::string tmpstr;
   int attemps = 3;
   int itray =0;
   uint64_t tstamp = generateCallStamp();
   long time0 = (long) time(NULL);
   long retv( 0 );

  while(itray < attemps) { //if unusual exception - try once more

   try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) ); // if troubles in connecting to server - sleep sleepTime sec before next call
        lookupRMDDB();
        retv = RMDB->requestHWResourceForProcess(partition.c_str(), resource.c_str(), computer.c_str(), hwId.c_str(), pid, client.c_str(), tstamp , itray == 0 );
        if( itray > 0 ) {
           std::ostringstream txt;
           txt<<retv;
           ers::warning (ers::Message( ERS_HERE, "got RM handle=" + txt.str() + " after several attempts" ) );
        }
        return retv;
   }
   catch (daq::rmgr::ObjectNotFoundE & exo) {
           throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
   }
   catch (daq::rmgr::UnavailableResourcesE  & exr) {
           throw daq::rmgr::UnavailableResources ( ERS_HERE,  exr.refused_mess.in() );
   }
   catch (daq::rmgr::ResourceNotFoundE & excres) {
           throw daq::rmgr::ResourceNotFound ( ERS_HERE );
   }
   catch (daq::ipc::Exception & exipc) {
           itray++;
           if(itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
    }
    catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
    }
    catch(CORBA::SystemException & excorb) {
           itray++;
           if(itray >= attemps ) {
            //text << excorb;
             tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
           }
    }
    catch( ...) {
           itray++;
           if(itray >= attemps ) {
              throw daq::rmgr::CommunicationException( ERS_HERE, " Unkown exception in RM" );
           }
   }

  }
  return retv;
}




void daq::rmgr::RM_ClientImpl::freeProcessResources(
                        const std::string& partition,
                        const std::string& computer,
                        unsigned long pid,
                        bool log_this_call
) {
   std::ostringstream text;
   std::string tmpstr;
   CORBA::String_var retv = "";
   std::string retstr;
  int attemps = 3;
  int itray = 0;
  long time0 = (long) time(NULL);
//oct 2015
  ::CORBA::Boolean add2log(log_this_call);
  const daq::rmgr::CallerInfo& caller = getCallerInfo();

 while(itray < attemps) { //if unusual exception - try once more

  try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) );
        lookupRMDDB();
        return RMDB->freeProcessResources(partition.c_str(), computer.c_str(), pid, add2log, caller);
  }
  catch (daq::rmgr::PartitionNotFoundE & exp) {
        throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
  }
  catch (daq::rmgr::ProcessNotFoundE & excproc) {
        throw daq::rmgr::ProcessNotFound (ERS_HERE, excproc.partition.in(), excproc.pid.in(), excproc.computer.in() );
  }
  catch (daq::rmgr::HandleIdNotFoundE & exchndl) {
        throw daq::rmgr::HandleIdNotFound( ERS_HERE);
  }
  catch (daq::rmgr::ObjectNotFoundE & exo) {
        throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
  }
  catch (daq::ipc::Exception & exipc) {
        itray++;
        if( itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
        }
  }
  catch(CORBA::SystemException & excorb) {
        itray++;
        if( itray >= attemps ) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
        }
  }
 }
}

void daq::rmgr::RM_ClientImpl::freeAllProcessResources( long handleId,
                        const std::string& partition,
                        const std::string& computer,
                        unsigned long pid,
                        bool log_this_call
) {
   std::ostringstream text;
   std::string tmpstr;
   std::string retstr;
   int attemps = 3;
   int itray = 0;
   long time0 = (long) time(NULL);
//oct 2015
   ::CORBA::Boolean add2log(log_this_call);
   const daq::rmgr::CallerInfo& caller = getCallerInfo();
    
 while(itray < attemps) { //if unusual exception - try once more

  try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) );
        lookupRMDDB();
        return RMDB->freeAllProcessResources( handleId, partition.c_str(), computer.c_str(), pid, add2log, caller );
  }
  catch (daq::rmgr::PartitionNotFoundE & exp) {
        throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
  }
  catch (daq::rmgr::ProcessNotFoundE & excproc) {
        throw daq::rmgr::ProcessNotFound (ERS_HERE, excproc.partition.in(), excproc.pid.in(), excproc.computer.in() );
  }
  catch (daq::rmgr::HandleIdNotFoundE & exchndl) {
        throw daq::rmgr::HandleIdNotFound( ERS_HERE);
  }
  catch (daq::rmgr::ObjectNotFoundE & exo) {
        throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
  }
  catch (daq::ipc::Exception & exipc) {
        itray++;
        if ( itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
        }
  }
  catch(CORBA::SystemException & excorb) {
        itray++;
        if ( itray >= attemps ) {
            tmpstr = excorb._name();
            ///text << excorb;
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
        }
  }
 }
}

long daq::rmgr::RM_ClientImpl::getSleepTime( long startTime ) {
 long retv1 = (long) time(NULL) - startTime; 
 if( retv1 > maxDelay ) {
    return 0;
 }
 char * envSleep = getenv( envPar.c_str() );
 long retv = envSleep == 0? sleepTime : atol( envSleep );
 long retv2 = (retv+retv1)< maxDelay ? retv : maxDelay - retv - retv1;
 if( retv2 < 0 )
    retv2 = 0;
 std::cerr<<"  sleepTime="<<sleepTime<<"   maxDelay="<<maxDelay <<" retv="<<retv<<"  retv1="<<retv1<<"    finaly retv2="<<retv2<<std::endl;

 return retv2; //(retv+retv1)< maxDelay ? retv : maxDelay - retv - retv1;
 
}


//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Configuration* daq::rmgr::RM_ClientImpl::getConfiguration( const std::string& dbname ) {
   std::ostringstream text;
   Configuration * sw_conf = 0;
   try{
        sw_conf = new Configuration(dbname);
   }
   catch (daq::config::Exception & ex) {
      if(!(sw_conf == 0))
          delete sw_conf;
      text << ex;
      throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str()  );
   }
   return sw_conf;
}

//create and fill daq::rmgr::Config_update structure
void daq::rmgr::RM_ClientImpl::setRMData(::Configuration * conf, daq::rmgr::Config_update & rm_conf) {
  const std::string compHW("Computer");
  const std::string emptyStr("");
  //get all resources
  std::vector<const daq::core::RM_Resource *> resources;
  conf->get(resources);
  int iswr = 0, ihwr = 0; //, icr = 0;
  //calculate number of each type
  for(std::vector<const daq::core::RM_Resource *>::iterator ir0 = resources.begin(); ir0 != resources.end(); ++ir0) {
    if((*ir0)->class_name() == "RM_SW_Resource")
      iswr++;
    else if((*ir0)->class_name() == "RM_HW_Resource")
      ihwr++;
  }
  //now add resources
   rm_conf.swr_lst.length( iswr); //if 0?
   rm_conf.hwcr_lst.length( ihwr );
   iswr = ihwr = 0;
  for(std::vector<const daq::core::RM_Resource *>::iterator ir = resources.begin(); ir != resources.end(); ++ir) {
     if((*ir)->class_name() == "RM_SW_Resource") {
        const daq::core::RM_SW_Resource* sw_r = conf->cast<daq::core::RM_SW_Resource>(*ir);
        rm_conf.swr_lst[iswr].rid = CORBA::string_dup( sw_r->UID().c_str() );
        rm_conf.swr_lst[iswr].max_total = sw_r->get_MaxCopyTotal();
        rm_conf.swr_lst[iswr++].max_pp = sw_r->get_MaxCopyPerPartition();
     }
     else if ((*ir)->class_name() == "RM_HW_Resource") {
         const daq::core::RM_HW_Resource* hw_r = conf->cast<daq::core::RM_HW_Resource>(*ir);
         rm_conf.hwcr_lst[ihwr].rid = CORBA::string_dup(hw_r->UID().c_str());
         rm_conf.hwcr_lst[ihwr].max_total = hw_r->get_MaxCopyTotal();
         rm_conf.hwcr_lst[ihwr].max_pp = hw_r->get_MaxCopyPerPartition();
         //if Computer - set as empty
         if( compHW != hw_r->get_HardwareClass() )
             rm_conf.hwcr_lst[ihwr].hwClass = CORBA::string_dup( hw_r->get_HardwareClass().c_str()); 
         else
            rm_conf.hwcr_lst[ihwr].hwClass = CORBA::string_dup( emptyStr.c_str() );
         ihwr++;
     }
  }
 //sw objects
  std::vector<const daq::core::SW_Object *> sw_objects;
  conf->get(sw_objects);
  //calculate number of sw objects with resources
  int nr = 0;
  for(std::vector<const daq::core::SW_Object *>::iterator j = sw_objects.begin(); j != sw_objects.end(); ++j) 
      //AI 2017 send all sw objects: comment 1 line
      //if(!((*j)->get_Needs().empty()))
        nr++;
 int  inum = 0;
  rm_conf.swobj_lst.length( nr );
  for(std::vector<const daq::core::SW_Object *>::iterator j = sw_objects.begin(); j != sw_objects.end(); ++j) {
     //AI 2017 send all SW objects: 2 lines commented
    // if((*j)->get_Needs().empty())
    //    continue;
     rm_conf.swobj_lst[inum].swid = CORBA::string_dup((*j)->UID().c_str() );
     int len = (*j)->get_Needs().size();
     rm_conf.swobj_lst[inum].res_list.length(len );
     //add res references
     for ( size_t ii=0; ii < (*j)->get_Needs().size(); ii++ )
        rm_conf.swobj_lst[inum].res_list[ii].id = CORBA::string_dup( (*j)->get_Needs()[ii]->UID().c_str());
     inum++;
  }

}

void daq::rmgr::RM_ClientImpl::rmConfigurationUpdate ( const std::string& dbname ) {
  Configuration * conf = getConfiguration( dbname );
  rmConfigurationUpdate( conf );
  if(!(conf == 0))
    delete conf;
}

void daq::rmgr::RM_ClientImpl::rmConfigurationUpdate ( Configuration * conf ) {
  daq::rmgr::Config_update rm_conf;
  std::string tmpstr;
  std::ostringstream text;
  try {
    setRMData( conf , rm_conf);
    lookupRMDDB();
    RMDB->rmConfigUpdate( rm_conf );
  }
  catch (daq::config::Exception & ex) {
       text << ex;
       throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str()  );
  }
  catch (daq::ipc::Exception & exipc) {
       text << exipc;
       throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
 }
 catch(CORBA::SystemException & excorb) {
       tmpstr = excorb._name();

       // TODO: temporary hack because IDL cannot be modified (see ATDAQCCRM-11)
       // "UNKNOWN" usually means that the server throws an user exception that is
       // not been declared. The only current case is "rmConfigUpdate"
       if(tmpstr == "UNKNOWN") {
           daq::rmgr::ConfigurationException ex(ERS_HERE,
                                                std::string("The RM server configuration could not be updated because inconsistent and/or corrupted; ") +
                                                    std::string("please, look at the RM server log for additional details"),
                                                "bad configuration");
           throw ex;
       }

       throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Handle_InfoHelper*
daq::rmgr::RM_ClientImpl::getHandleInfo ( long handleId ){
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

   try {
       daq::rmgr::Handle_Info* info =  RMDB->getHandleInfo(handleId);
       daq::rmgr::Handle_InfoHelper* retv = new daq::rmgr::Handle_InfoHelper(info);
       return retv;
   }
   catch (daq::rmgr::HandleIdNotFoundE & exh) {
           throw daq::rmgr::HandleIdNotFound( ERS_HERE );
   }
   catch( daq::rmgr::CommunicationException&  excom) {
           text << excom;
           throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
   }
   catch (daq::ipc::Exception & exipc) {
           text << exipc;
           throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
   }
   catch(CORBA::SystemException & excorb) {
           //text << excorb;
           tmpstr = excorb._name();
           throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
   }
}


///daq::rmgr::Resource_Info*
daq::rmgr::Allocated_InfoHelper*
daq::rmgr::RM_ClientImpl::getPartitionInfo (const std::string& partition) {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Allocated_Info* info = RMDB->getPartitionInfo( partition.c_str());
    daq::rmgr::Allocated_InfoHelper* helper = new daq::rmgr::Allocated_InfoHelper( info );
    return helper;
  }
  catch (daq::rmgr::PartitionNotFoundE & exp) {
           throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Allocated_InfoHelper*
daq::rmgr::RM_ClientImpl::getResourceInfo(const std::string& partition,
                                   const std::string& rname) {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Allocated_Info* info = RMDB->getResourceInfo( partition.c_str(), rname.c_str());
    daq::rmgr::Allocated_InfoHelper* helper = new daq::rmgr::Allocated_InfoHelper( info );
    return helper;
  }
  catch (daq::rmgr::PartitionNotFoundE & exp) {
           throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
  }
  catch (daq::rmgr::ResourceNotFoundE & exr) {
           throw daq::rmgr::ResourceNotFound( ERS_HERE );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_ClientImpl::getFullResInfo(const std::string& rname) {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Allocated_Info* info = RMDB->getFullResInfo(rname.c_str());
    daq::rmgr::Allocated_InfoHelper* helper = new daq::rmgr::Allocated_InfoHelper( info );
    return helper;
  }
  catch (daq::rmgr::ResourceNotFoundE & exr) {
           throw daq::rmgr::ResourceNotFound( ERS_HERE );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_ClientImpl::getComputerInfo(const std::string& computer) {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Allocated_Info* info = RMDB->getComputerInfo(computer.c_str());
    daq::rmgr::Allocated_InfoHelper* helper = new daq::rmgr::Allocated_InfoHelper( info );
    return helper;
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_ClientImpl::getComputerResourceInfo(const std::string& computer, const std::string& partition, const std::string& resource) {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Allocated_Info* info = RMDB->getComputerResourceInfo(computer.c_str(), partition.c_str(), resource.c_str());
    daq::rmgr::Allocated_InfoHelper* helper = new daq::rmgr::Allocated_InfoHelper( info );
    return helper;
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

long daq::rmgr::RM_ClientImpl::performRequest(daq::rmgr::ClientRequest& req) {
   std::ostringstream text;
   std::string tmpstr;
   std::string retstr;
   int attemps = 3;
   int itray = 0;
   long time0 = (long) time(NULL);
//oct 2015
  ::CORBA::Boolean add2log(req._log_this_call);
  const daq::rmgr::CallerInfo& caller = getCallerInfo();   

   daq::rmgr::RequestType type(req.type);
 while(itray < attemps) { //if unusual exception - try once more

  try {
        if( itray > 0 )
          sleep( getSleepTime( time0 ) );
        lookupRMDDB();
//enum RequestType {FreePartition, FreeComputer, FreeResource, FreePartitionResource, FreeComputerResource};
        switch (type) {
          case daq::rmgr::RequestType::FreePartition:
              RMDB->freePartition( req.param1, req.param2, req.param3 );
              return 0;
          case daq::rmgr::RequestType::FreeComputer:
              RMDB->freeComputerAll( req.param1, add2log, caller );
              return 0;
          case daq::rmgr::RequestType::FreeResource:
              RMDB->freeResource( req.param1, add2log, caller );
              return 0;
          case daq::rmgr::RequestType::FreePartitionResource:
              RMDB->freePartitionResource( req.param1, req.param2, add2log, caller );
              return 0;
          case daq::rmgr::RequestType::FreeComputerResource:
              RMDB->freeComputerResource(  req.param1, req.param2, req.param3, add2log, caller );
              return 0;
         case daq::rmgr::RequestType::FreeApplicationResources: 
              RMDB->freeApplicationResources(  req.param1, req.param2, req.param3, add2log, caller );
              return 0;
          default:
              return 0;
        }
  }
  catch (daq::rmgr::PartitionNotFoundE & exp) {
        throw daq::rmgr::PartitionNotFound( ERS_HERE, exp.partition.in() );
  }
  catch (daq::rmgr::ProcessNotFoundE & excproc) {
        throw daq::rmgr::ProcessNotFound (ERS_HERE, excproc.partition.in(), excproc.pid.in(), excproc.computer.in() );
  }
  catch (daq::rmgr::HandleIdNotFoundE & exchndl) {
        throw daq::rmgr::HandleIdNotFound( ERS_HERE);
  }
  catch (daq::rmgr::ObjectNotFoundE & exo) {
        throw daq::rmgr::ObjectNotFound( ERS_HERE, exo.type.in() );
  }
  catch( daq::rmgr::CommunicationException&  excom) {
           itray++;
           if(itray >= attemps ) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
           }
  }
  catch (daq::ipc::Exception & exipc) {
        itray++;
        if ( itray >= attemps ) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
        }
  }
  catch(CORBA::SystemException & excorb) {
        itray++;
        if ( itray >= attemps ) {
            tmpstr = excorb._name();
            //text << excorb;
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
        }
  }
 }
  return 0; 
}

///ONLY FOR TESTS
void daq::rmgr::RM_ClientImpl::rmConfigurationUpdate4MyTest ( daq::rmgr::Config_update & idl_conf ) {
  std::string tmpstr;
  std::ostringstream text;
  try {
    lookupRMDDB();
    RMDB->rmConfigUpdate( idl_conf );
  }
  catch (daq::config::Exception & ex) {
       text << ex;
       throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str()  );
  }
  catch (daq::ipc::Exception & exipc) {
       text << exipc;
       throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
       tmpstr = excorb._name();
       throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

daq::rmgr::Config_updateHelper* daq::rmgr::RM_ClientImpl::getConfigInfo() {
  lookupRMDDB();
  std::string tmpstr;
  std::ostringstream text;

  try {
    daq::rmgr::Config_update* info = RMDB->getConfigInfo();
    daq::rmgr::Config_updateHelper* helper = new daq::rmgr::Config_updateHelper( info );
    return helper;
  }
  catch( daq::rmgr::CommunicationException&  excom) {
               text << excom;
               throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch (daq::ipc::Exception & exipc) {
              text << exipc;
              throw daq::rmgr::CommunicationException( ERS_HERE, text.str().c_str() );
  }
  catch(CORBA::SystemException & excorb) {
            //text << excorb;
            tmpstr = excorb._name();
            throw daq::rmgr::CommunicationException( ERS_HERE, tmpstr );
  }
}

bool daq::rmgr::RM_ClientImpl::checkAccesPermission() {
  std::string mhost;
  std::string mname;
  try{
   ::System::LocalHost myhost;
   mhost = myhost.full_name();
   ::System::User myname;
   mname = myname.name();
  }
  catch( ... ) {
    std::cerr<<"Can not create User. No permission to free resources."<<std::endl;
    return false;
  }
  try {
    daq::am::ServerInterrogator si;
    const daq::am::RMResource *rmRes  = daq::am::RMResource::getInstanceForActionANY( "any" );
    daq::am::RequestorInfo *pAccessSubject = new daq::am::RequestorInfo( mname, mhost );
    return  si.isAuthorizationGranted(*rmRes, *pAccessSubject);
  }
  catch(daq::am::Exception &ex) {
     ers::error(ex);
     return false;
  }
  catch( ... ) {
     std::cerr<<"Unknown exception during check access permission"<<std::endl;
     return false;
 }
}

daq::rmgr::CallerInfo daq::rmgr::RM_ClientImpl::getCallerInfo() {
  daq::rmgr::CallerInfo caller;
  std::string mhost;
  std::string mname;
 try{
   ::System::LocalHost myhost;
   mhost = myhost.full_name();
   ::System::User myname;
   mname = myname.name();
  }
  catch( ... ) {
  }
  caller.user = CORBA::string_dup(mname.c_str());
  caller.host = CORBA::string_dup(mhost.c_str());
  caller.isJava = false;
  return caller;
}

 daq::rmgr::RM_Client::RM_Client() {
  impl = new RM_ClientImpl();
 }
 daq::rmgr::RM_Client::RM_Client(const char *sid) {
  impl = new RM_ClientImpl(sid);
 }
 daq::rmgr::RM_Client::~RM_Client() {
  delete impl;
 }
 void daq::rmgr::RM_Client::rmConfigurationUpdate ( const std::string& dbname ) {
   impl->rmConfigurationUpdate( dbname );
 }
 void daq::rmgr::RM_Client::rmConfigurationUpdate ( Configuration * conf ) {
   impl->rmConfigurationUpdate( conf );
 }
 long daq::rmgr::RM_Client::requestResources (
                        const std::string& partition,
                        const std::string& swobjectid, //id in confdb
                        const std::string& computerName, // name in confdb
                        const std::string& clientName) {
   return impl->requestResources( partition, swobjectid, computerName, clientName );
 }
 long daq::rmgr::RM_Client::requestResources (
                        const std::string& partition,
                        const std::string& swobjectid, //id in confdb
                        const std::string& computerName, // name in confdb
                        unsigned long mypid,
                        const std::string& clientName) {
   return impl->requestResources( partition, swobjectid, computerName, mypid, clientName );
 }
 long daq::rmgr::RM_Client::requestResourceForMyProcess(
                        const std::string& partition,
                        const std::string& resource,
                        const std::string& computer,
                       const std::string& clientName) {
  return impl->requestResourceForMyProcess( partition, resource, computer, clientName );
 }
 long daq::rmgr::RM_Client::requestHWResources(
                        const std::string& partition,
                        const std::string& swId,
                        const std::string& computer,
                        unsigned long pid,
                        const std::string& hwClass,
                        const std::string& hwId,
                        const std::string& client) {
  return impl->requestHWResources( partition, swId, computer, pid, hwClass, hwId, client);
 }
 long daq::rmgr::RM_Client::requestHWResourceForProcess(
                        const std::string& partition,
                        const std::string& resource,
                        const std::string& computer,
                        const std::string& hwId,
                        unsigned long pid,
                        const std::string& client) {
   return impl->requestHWResourceForProcess(partition, resource, computer, hwId, pid, client);
 }

 void daq::rmgr::RM_Client::freeResources (long handleId, bool log_this_call) {
  impl->freeResources( handleId, log_this_call );
 }
 void daq::rmgr::RM_Client::freePartition(const std::string&  p ) {
  impl->freePartition( p );
 }
 void daq::rmgr::RM_Client::freeComputer(const std::string& computer, bool log_this_call) {
  impl->freeComputer( computer, log_this_call );
 }
 void daq::rmgr::RM_Client::freeResource(const std::string& resource, bool log_this_call) {
  impl->freeResource( resource, log_this_call );
 }
 void daq::rmgr::RM_Client::freePartitionResource(const std::string&  p, const std::string& resource, bool log_this_call) {
  impl->freePartitionResource( p, resource, log_this_call );
 }
 void daq::rmgr::RM_Client::freeComputerResource(const std::string& resource, const std::string& computer, const std::string&  p, bool log_this_call) {
  impl->freeComputerResource( resource, computer, p, log_this_call );
 }
 void daq::rmgr::RM_Client::freeApplicationResources(const std::string& p, const std::string& swObj, const std::string& computer, bool log_this_call) {
  impl->freeApplicationResources( p, swObj, computer, log_this_call );
 }
 void daq::rmgr::RM_Client::freeAllProcessResources( long handleId, const std::string& partition,
                           const std::string& computer, unsigned long pid, bool log_this_call ) {
  impl->freeAllProcessResources(handleId, partition,  computer, pid, log_this_call );
 }
 void daq::rmgr::RM_Client::freeProcessResources(
                        const std::string& partition,
                        const std::string& computer,
                        unsigned long pid,
                        bool log_this_call) {
  impl->freeProcessResources( partition, computer, pid, log_this_call );
 }
 daq::rmgr::Rmstr_lstHelper*  daq::rmgr::RM_Client::getPartitionsList() {
  return impl->getPartitionsList();
 }
 daq::rmgr::Rmstr_lstHelper* daq::rmgr::RM_Client::getResourceOwners(const std::string& rname, const std::string& p) {
  return impl->getResourceOwners( rname, p );
 }
 daq::rmgr::Handle_InfoHelper* daq::rmgr::RM_Client::getHandleInfo(long handleId) {
  return impl->getHandleInfo( handleId );
 }
 daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_Client::getPartitionInfo( const std::string& partition ) {
  return impl->getPartitionInfo( partition );
 }
 daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_Client::getResourceInfo( const std::string& partition, const std::string& rname) {
  return impl->getResourceInfo( partition, rname );
 }
 daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_Client::getFullResInfo(const std::string& resource) {
  return impl->getFullResInfo( resource );
 }
 daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_Client::getComputerInfo(const std::string& computer) {
  return impl->getComputerInfo( computer );
 }
 daq::rmgr::Allocated_InfoHelper* daq::rmgr::RM_Client::getComputerResourceInfo(const std::string& computer, const std::string& partition, const std::string& resource) {
  return impl->getComputerResourceInfo( computer, partition, resource );
 }
 bool daq::rmgr::RM_Client::checkAccesPermission() {
  return impl->checkAccesPermission();
 }
 void daq::rmgr::RM_Client::rmConfigurationUpdate4MyTest ( daq::rmgr::Config_update & idl_conf ) {
  return impl->rmConfigurationUpdate4MyTest ( idl_conf );
 }
 daq::rmgr::Config_updateHelper* daq::rmgr::RM_Client::getConfigInfo() {
  return impl->getConfigInfo();
 }
 uint64_t daq::rmgr::RM_Client::generateCallStamp() {
  return impl->generateCallStamp();
 }
 uint16_t daq::rmgr::RM_Client::getStampId() {
  return impl->getStampId();
 }
 long daq::rmgr::RM_Client::getSleepTime( long startTime ) {
  return impl->getSleepTime( startTime );
 }

 Configuration* daq::rmgr::RM_Client::getConfiguration( const std::string& dbname ) {
  return impl->getConfiguration( dbname );
 }
void daq::rmgr::RM_Client::setRMData(::Configuration * conf, daq::rmgr::Config_update & rm_conf) {
  impl->setRMData( conf, rm_conf );
 }
long daq::rmgr::RM_Client::performRequest(ClientRequest& req) {
 return impl->performRequest( req ); 
}





