#include "src/RMimpl.h"
#include "src/RM_Server.h"
#include <ResourceManager/idlhelper.h>
#include <system/File.h>

daq::rmgr::RM_Server::RM_Server() : destroyInProgress(false)
{
     const std::string backup((std::string)RMSERVERDEFAULT + (std::string)RMLOGNAME);
     RM_Backup* bck = new RM_Backup( backup.c_str(), true, true );
     rm_impl = new  RM_Impl(bck);
     //AI 2018 check config
     if(!rm_impl->isConfigAvailable())
       exit(14);

     bck->initialize( rm_impl );
     idlRapper = new RmgrImpl(RMSERVERDEFAULT, rm_impl);
}

daq::rmgr::RM_Server::RM_Server( const char * sid, const char* backup, bool doBck, bool doresto ) : destroyInProgress(false)
{
      RM_Backup* bck = new RM_Backup( backup, doBck, doresto );
      rm_impl = new  RM_Impl(bck);
      //AI 2018 check config
      if(!rm_impl->isConfigAvailable())
         exit(14);

      bck->initialize( rm_impl );
      idlRapper = new RmgrImpl(sid, rm_impl);
}

daq::rmgr::RM_Server::~RM_Server() {
   //close backup manager
  ///// delete rm_impl;
}

void daq::rmgr::RM_Server::run() {
  std::ostringstream txt ;
  IPCPartition defpart;
//  if ( defpart.isObjectValid< ::rmgr::ResMgr> ( rmdb_->getServerName() ) == false ) {
  if ( defpart.isObjectValid< ::rmgr::ResMgr> ( getServerName() ) == false ) {
     std::ostringstream txt1;
     txt1<<"Resource Manager with name=" << getServerName() << " is starting" ;
     if( rm_impl ) {
        if(rm_impl->isRestoreBackup()) {
            txt1<<" with restore from " << rm_impl->getBckPath();
            if(rm_impl->isDoBackup())
                txt1<<" Save backup with the same path.";
        }
        else if(rm_impl->isDoBackup())
            txt1<<"  backup path=" << rm_impl->getBckPath();
     }
     ers::debug(ers::Message( ERS_HERE, txt1.str()), 0 );
     idlRapper->publish();
std::cout << "idlRapper->publish() done" << std::endl;
     std::ostringstream txt2;
     txt2<<"Resource Manager server with name " << getServerName() << " was started in default partition."<<std::endl;
     ers::info( ers::Message( ERS_HERE, txt2.str()));
//    stopped = false;
    destroyInProgress = false;
    idlRapper->wait();
    //idlRapper->run();

    idlRapper->withdraw();
    destroyInProgress = true;
    delete rm_impl;
    idlRapper->_destroy(true);
    //TODO save backup and delete backup manager and rm_impl
    ers::info( ers::Message( ERS_HERE, "Resource manager server stopped."));
  }
  else {
        std::ostringstream txt ;
        txt<<"Resource manager server with name " << getServerName() << " already running in default partition.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
}

void daq::rmgr::RM_Server::removeServ() {
  try {
   idlRapper->withdraw();
  }
  catch( ... ){
    ers::error( ers::Message( ERS_HERE, " removeServ error" ));
  }
}

daq::rmgr::RmgrImpl::RmgrImpl() : RMIPCNamedObject(RMSERVERDEFAULT) {
  rm_impl = 0;
}

daq::rmgr::RmgrImpl::~RmgrImpl() {
  rm_impl = 0;
}

daq::rmgr::RmgrImpl::RmgrImpl(std::string serverName, RM_Impl* rmimpl)  : RMIPCNamedObject(serverName), _serverName(serverName) {
   rm_impl = rmimpl;
}

void daq::rmgr::RmgrImpl::rmConfigUpdate ( const daq::rmgr::Config_update& conf) {
 //oct 2015
 //save config for any case in order users can see content if any needs
 std::ostringstream text; //for info log
 text << "Configuration content before rmConfigUpdate:" << std::endl;
 addMesConfig( text, rm_impl->getConfigInfo() );
 text << "Configuration content after rmConfigUpdate:" << std::endl;
 
 ////boost::mutex::scoped_lock scoped_lock( m_mutex );
 //Aug 2014
//// boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex); //first lock rw_mutex as the same in other requests (to avoid problem)
 boost::upgrade_lock< boost::shared_mutex > lock1(so_mutex);
 boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock1(lock1);
//// boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);
{
 boost::mutex::scoped_lock scoped_lock( m_mutex ); //use single
     rm_impl->rmConfigurationUpdate( conf );
     //Aug 2014
     if(rm_impl->isDoBackup())
        rm_impl->getBck()->saveBackup();
}
//oct 2015
 addMesConfig( text, rm_impl->getConfigInfo() );
 ers::info( ers::Message( ERS_HERE, text.str()));
}


//request resources actions
 ::CORBA::Long  daq::rmgr::RmgrImpl::requestResources (
                                const char*  partitionName,
                                const char* softwareUID,
                                const char* computerName,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry) {
//for scal. tests
   timeval time1;
   gettimeofday( &time1, 0 );
long hndle;
{
/// boost::mutex::scoped_lock scoped_lock( m_mutex ); 
   if( !isFirstTry ) {
      long handle = rm_impl-> getHandleByStamp(callStamp);
      if(handle  > 0) //already handle was produced but did not returned to client
         return handle;
    }
    //check if no resources
  {
    boost::shared_lock<boost::shared_mutex> lock(so_mutex); //Aug 2014
    if( ! rm_impl->hasResources(softwareUID) )
         return 0;
  }
    const std::string p(partitionName);
    const std::string sw(softwareUID);
    const std::string computer(computerName);
    const std::string client(clientName);
    //Aug 2014
///    boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex);
///    boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);
 boost::mutex::scoped_lock scoped_lock( m_mutex ); //use just single

    hndle = rm_impl->allocateResources( p, sw, computer, pid, client,callStamp);
    //logout temporary here
    if(rm_impl->isDoBackup())
      rm_impl->getBck()->logRequestResources( p, sw, computer, pid, client, callStamp, hndle);
}
//tests
   timeval time2;
   gettimeofday( &time2, 0 );
   ///setVal(time2, time1);

    return hndle;
}

 ::CORBA::Long daq::rmgr::RmgrImpl::requestHWResources (
                                const char*  partitionName,
                                const char* softwareUID,
                                const char* computerName,
                                const char* hwClass,
                                 const char* hwId,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry) {
//// boost::mutex::scoped_lock scoped_lock( m_mutex );
/// boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex); //Aug 2014
   if( !isFirstTry ) {
      long handle = rm_impl-> getHandleByStamp(callStamp);
      if( handle > 0) //already handle was produced but did not returned to client
         return handle;
    }
    //check if no resources
   {
    boost::shared_lock<boost::shared_mutex> lock(so_mutex); //Aug 2014
    if( ! rm_impl->hasResources(softwareUID) )
         return 0;
   }
    const std::string p(partitionName);
    const std::string sw(softwareUID);
    const std::string computer(computerName);
    const std::string client(clientName);
///    boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock); //Aug 2014
boost::mutex::scoped_lock scoped_lock( m_mutex );
    
    long hndle = rm_impl->allocateResources(partitionName, softwareUID, computerName, pid, clientName, callStamp, hwClass, hwId);
    //logout
    if(rm_impl->isDoBackup())
      rm_impl->getBck()->logRequestHWResources( partitionName, softwareUID, computerName, hwClass, hwId, pid, clientName, callStamp, hndle);
    return hndle;
    
}

 ::CORBA::Long daq::rmgr::RmgrImpl::requestResourcesForProcess(const char* partitionName,
                                const daq::rmgr::Res_ID_list&  res_lst,
                                const char* computerName,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry) {
//// boost::mutex::scoped_lock scoped_lock( m_mutex );
/// boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex); //Aug 2014
   if( !isFirstTry ) {
      long handle = rm_impl-> getHandleByStamp(callStamp);
      if( handle > 0) //already handle was produced but did not returned to client
         return handle;
    }
    const std::string p(partitionName);
    const std::string computer(computerName);
    const std::string client(clientName);
////    boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock); //Aug 2014
boost::mutex::scoped_lock scoped_lock( m_mutex );

    long hndle = rm_impl->allocateResources4Process( p, res_lst, computer, pid, client,callStamp );
    //logout temporary here
    if(rm_impl->isDoBackup())
      rm_impl->getBck()->logRequestResourcesForProcess( p, res_lst, computer, pid, client, callStamp, hndle);
    return hndle;
}


 ::CORBA::Long daq::rmgr::RmgrImpl::requestHWResourceForProcess (
                                const char* partitionName,
                                const char* resource,
                                const char* computerName,
                     ////            const char* hwClass,
                                const char* hwId,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry) {
//// boost::mutex::scoped_lock scoped_lock( m_mutex );
/// boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex); //Aug 2014
   if( !isFirstTry ) {
      long handle = rm_impl-> getHandleByStamp(callStamp);
      if( handle > 0) //already handle was produced but did not returned to client
         return handle;
    }
    const std::string p(partitionName);
    const std::string res( resource ); 
    const std::string computer(computerName);
    const std::string client(clientName);
   //// const std::string hwClassStr( hwClass );
    const std::string hwIdStr( hwId );
///    boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock); //Aug 2014
boost::mutex::scoped_lock scoped_lock( m_mutex );

    long hndle = rm_impl->allocateHWResource4Process( p, res, computer, pid, client,
                    /*hwClassStr,*/ hwIdStr, callStamp);
    //logout
    if(rm_impl->isDoBackup())
      rm_impl->getBck()->logRequestHWResourceForProcess( partitionName, resource, computerName, hwId, pid, clientName, callStamp, hndle );
    return hndle;
}

void daq::rmgr::RmgrImpl::addMesRes( std::ostringstream& str, daq::rmgr::Allocated_Info* info ) {
  try{
   const std::string types[] = { "SW", "HW" };
   str << "Free resources info= "; 
   for( unsigned int i=0; i<info->length(); i++) {
     str << "{";
     str << " handle=" << (*info)[i].handle;
     str << " resource=" << (*info)[i].resource;
     str << " type=" << types[(*info)[i].type];
     str << " maxPP=" << (*info)[i].maxPP;
     str << " maxTotal=" << (*info)[i].maxTotal;
    // str << " hwClass=" << (*info)[i].hwClass << std::endl;
     str << std::endl;
     str << " swObj=" << (*info)[i].swObj;
     str << " partition=" << (*info)[i].partition;
     str << " client=" << (*info)[i].client;
     str << " computer=" << (*info)[i].computer;
     str << " pid=" << (*info)[i].pid;
     str << " hwId=" << (*info)[i].hwId;
     str << " allocPP=" << (*info)[i].allocPP;
     str << " allocTotal=" << (*info)[i].allocTotal;
     str << "}" << std::endl;
   }
  }
  catch( ... ) {}
}

void daq::rmgr::RmgrImpl::addMesRes( std::ostringstream& str, daq::rmgr::Handle_Info* info ) {
  try{
   str << "Free resources info= {";// << std::endl;
   str << " handle=" << info->handle;// << std::endl;
   str << " swObj=" << info->swObj;
   str << " computer=" << info->computer;
   str << " partition=" << info->partition;
   str << " client=" << info->client;
   str << " pid=" << info->pid << std::endl;
   str << "  Resources:" << std::endl;
   const std::string types[] = { "SW", "HW" };
   for(unsigned int i=0; i<info->resDetails.length(); i++) {
     str << "  {";
     str << "   resource=" << info->resDetails[i].resource;
     str << "   type=" << types[info->resDetails[i].type];
     str << "   maxPP=" << info->resDetails[i].maxPP;
     str << "   maxTotal=" << info->resDetails[i].maxTotal;
     str << "   allocPP=" << info->resDetails[i].allocPP;
     str << "   allocTotal=" << info->resDetails[i].allocTotal;
     str << "  }"<< std::endl;
   }
  }
  catch( ... ) {}
}

void daq::rmgr::RmgrImpl::addMesConfig( std::ostringstream& str, daq::rmgr::Config_update* inf ) {
  try{
    daq::rmgr::Config_updateHelper helper( inf );
    str<< helper<<std::endl;
    delete inf;
    helper.cinfo = 0;
  }
  catch( ... ) {}
}

//free resources actions
void daq::rmgr::RmgrImpl::freeProcessResources( const char* p, const char* computer, CORBA::ULong pid, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller) {
 //oct 2015 keep text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeProcessResources(\'" << p << "\',\'" << computer << "\'," << pid << ")\" called by " 
          << caller.user << "@" << caller.host << std::endl;
     daq::rmgr::Allocated_Info* inf = rm_impl->getPartitionCompProcInfo( p, computer, pid);
     addMesRes( text, inf);
     delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );
     //Aug 2014
///     boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex);
///     boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);

     rm_impl->freeResource( p, computer, pid);
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeProcessResources( p, computer, pid );
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}

void  daq::rmgr::RmgrImpl::freeAllProcessResources( ::CORBA::Long handleId, const char* partition, const char* computer, CORBA::ULong pid, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller) {
 //oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeAllProcessResources(" << handleId << ",\'"<< partition << "\',\'" << computer << "\'," << pid << ")\" called by "
          << caller.user << "@" << caller.host << std::endl;
     if(handleId > 0) {
        daq::rmgr::Handle_Info* inf = rm_impl->getHandleInfo( handleId );
        addMesRes( text, inf);
        delete inf;
     }
     else {
        daq::rmgr::Allocated_Info* inf2 = rm_impl->getPartitionCompProcInfo( partition, computer, pid);
        addMesRes( text, inf2);
        delete inf2;
     }
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );
     //Aug 2014
///     boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex);
///     boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);
                                       
     if( handleId > 0 ) 
        rm_impl->freeResource( handleId );
     rm_impl->freeResource( partition, computer, pid);
     if(rm_impl->isDoBackup()) {
       if( handleId > 0 )
          rm_impl->getBck()->logFreeResources( handleId );
       else
          rm_impl->getBck()->logFreeProcessResources( partition, computer, pid );
     }
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));     
}

void daq::rmgr::RmgrImpl::freeResources (::CORBA::Long handle, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeResources(" << handle <<  ")\" called by "
          << caller.user << "@" << caller.host << std::endl;
     daq::rmgr::Handle_Info* inf = rm_impl->getHandleInfo( handle );
     addMesRes( text, inf);
     delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );
     //Aug 2014
///     boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex);
///     boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);

     rm_impl->freeResource( handle );
     //logout temporary here
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeResources( handle );
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}

void daq::rmgr::RmgrImpl::freePartition( const char* myHost,  const char* myName,const char* p ) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 text << "freePartition(\'" << myHost << "\',\'" << myName << "\',\'" << p << "\')\" called by "
      << myName << "@" << myHost << std::endl;
 daq::rmgr::Allocated_Info* inf = rm_impl->getPartitionInfo(p);
 addMesRes( text, inf);
 delete inf;

 boost::mutex::scoped_lock scoped_lock( m_mutex );

     rm_impl->freePartition( myHost, myName, p );  
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreePartition( myHost, myName, p);
     //always add message for this method
     ers::info( ers::Message( ERS_HERE, text.str()));
}

void daq::rmgr::RmgrImpl::freePartitionResource( const char* p, const char* resource, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller ) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freePartitionResource(\'" << p << "\',\'" << resource << "\')\" called by "
          << caller.user << "@" << caller.host << std::endl;
     daq::rmgr::Allocated_Info* inf = rm_impl->getResourceInfo( p, resource );
     addMesRes( text, inf);
     delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );

     rm_impl->freePartitionResource( p, resource );
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreePartitionResource( p, resource );
     //keep info     
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));     
}
void daq::rmgr::RmgrImpl::freeComputerAll( const char* computer, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller ) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeComputerAll(\'" << computer << "\')\" called by "
         << caller.user << "@" << caller.host << std::endl;
     daq::rmgr::Allocated_Info* inf = rm_impl->getComputerInfo( computer );
     addMesRes( text, inf);
     delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );

     rm_impl->freeComputerAll( computer );
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeComputerAll( computer );
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}
void daq::rmgr::RmgrImpl::freeResource( const char* resource, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller ) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeResource(\'" << resource << "\')\" called by "
         << caller.user << "@" << caller.host << std::endl;
    daq::rmgr::Allocated_Info* inf = rm_impl->getResourceInfo( resource );
    addMesRes( text, inf);
    delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );

     rm_impl->freeResource( resource );
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeResource( resource );
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}
void daq::rmgr::RmgrImpl::freeComputerResource( const char* resource, const char* computer, const char* p, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller ) {
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeComputerResource(\'" << resource << "\',\'" << computer << "\',\'" << p << "\')\" called by "
          << caller.user << "@" << caller.host << std::endl;
    daq::rmgr::Allocated_Info* inf = rm_impl->getComputerResourceInfo( computer, p, resource );
    addMesRes( text, inf);
    delete inf;
 }

 boost::mutex::scoped_lock scoped_lock( m_mutex );

     rm_impl->freeComputerResource( resource, computer, p );
     if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeComputerResource( resource, computer, p );
     //keep info
     if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}

void daq::rmgr::RmgrImpl::freeApplicationResources( const char* p, const char* swObj, const char* computer, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller ) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
     //Aug 2014
///     boost::upgrade_lock< boost::shared_mutex > lock(rw_mutex);
///     boost::upgrade_to_unique_lock<boost::shared_mutex> uniq_lock(lock);
//oct 2015 create text for info message before mutex...
 std::ostringstream text;
 if(log_this_call) {
     text << "\"freeApplicationResources(\'" <<  p << "\',\'" << swObj << "\',\'" << computer << "\')\" called by "
          << caller.user << "@" << caller.host << std::endl;
    daq::rmgr::Allocated_Info* inf = rm_impl->getApplicationInfo( p, swObj, computer );
    addMesRes( text, inf);
    delete inf;
 }

    rm_impl->freeApplicationResources( p, swObj, computer );
    if(rm_impl->isDoBackup())
       rm_impl->getBck()->logFreeApplicationResources( p, swObj, computer );
    //keep info
    if(log_this_call)
        ers::info( ers::Message( ERS_HERE, text.str()));
}

//TODO: use express (lokie *) or add freeComputerResources, freeSWObjResources, FreePartitionResources  ??
////void daq::rmgr::RmgrImpl::freeAllResources(const char* myhost, const char* myname, const char* p ,const char* swobjid, const char* compname) {
////} 


//getting different info

daq::rmgr::Rmstr_lst* daq::rmgr::RmgrImpl::getPartitionsList() {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
//    boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
    return rm_impl->getPartitionsList();
}

daq::rmgr::Rmstr_lst* daq::rmgr::RmgrImpl::getResourceOwners(const char* rname, const char* pname) {
///    boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
 boost::mutex::scoped_lock scoped_lock( m_mutex );
    return rm_impl->getResourceOwners(rname, pname);
}


daq::rmgr::Handle_Info* daq::rmgr::RmgrImpl::getHandleInfo(::CORBA::Long handle) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getHandleInfo(handle);
}

daq::rmgr::Allocated_Info*  daq::rmgr::RmgrImpl::getResourceInfo(const char* p, const char* rn) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getResourceInfo( p, rn);
}

daq::rmgr::Allocated_Info* daq::rmgr::RmgrImpl::getFullResInfo(const char*  rname) { // raises( daq::rmgr::ResourceNotFoundE );
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getResourceInfo( rname );
}
daq::rmgr::Allocated_Info* daq::rmgr::RmgrImpl::getPartitionInfo(const char* partition) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getPartitionInfo( partition );
}
daq::rmgr::Allocated_Info* daq::rmgr::RmgrImpl::getComputerInfo(const char* computer) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getComputerInfo( computer );
}
daq::rmgr::Allocated_Info* daq::rmgr::RmgrImpl::getComputerResourceInfo(const char* computer, const char* partition, const char* resource) {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///     boost::shared_lock<boost::shared_mutex> lock(rw_mutex); //Aug 2014
     return rm_impl->getComputerResourceInfo( computer, partition, resource );
}

daq::rmgr::Config_update* daq::rmgr::RmgrImpl::getConfigInfo() {
 boost::mutex::scoped_lock scoped_lock( m_mutex );
///   boost::shared_lock<boost::shared_mutex> lock(so_mutex); //Aug 2014
   return rm_impl->getConfigInfo();
}

bool daq::rmgr::RM_Backup::createLogoutStream(const char* log_file) {
 if ( !_doBackup )
    return true;
  logostrm.exceptions ( std::ofstream::eofbit | std::ofstream::failbit
             | std::ofstream::badbit );
 try{
    if( logostrm.is_open() )
       logostrm.close();
 }
  catch (const std::ofstream::failure& ex0) {
     std::ostringstream txt0;
     txt0 << "Exception when close journal stream. Exception: " << ex0.what();
     ers::error(ers::Message( ERS_HERE, txt0.str()));
     //return false;
  }
  try {
    logostrm.open ( log_file, std::ios::app );
  }
  catch (const std::ofstream::failure& ex1) {
    std::ostringstream txt;
    txt<< "Exception opening journal file \"" << log_file << "\"."
       << "Exception" << ex1.what();
    ers::error(ers::Message( ERS_HERE, txt.str()));
    return false;
  }
   if(logostrm.tellp() == 0) //was not created before //jan 2011
   try{
///     logostrm << "RMDDB logout. Log file name is " << log_file <<std::endl;
     mytime();
     logostrm << rmStamp << " " << logTime << std::endl;
     _numOkOperations = 0;
   }
   catch (const std::ofstream::failure& ex2) {
     std::ostringstream txt1;
     txt1 << "Can not write to journal. Exception during write with the reason:"
          << ex2.what();
     ers::error(ers::Message( ERS_HERE, txt1.str()));
     return false;
   }
   return true;
}

void daq::rmgr::RM_Backup::initialize( daq::rmgr::RM_Impl*& rm_impl) {
   _rm_impl = rm_impl;
   if( _restoreFrBck ) {
        bool keep_saveBackup = _doBackup;
        _doBackup = false;
        ///TODO loadBackup( getBackupPath(), getBckConfigPath() );
        restoreArchive();
       _rm_impl->resetBackup( rm_impl->getBck() );
        rm_impl = _rm_impl; // because of serialization
        loadFromLog(  getJournalPath().c_str(), rm_impl );
        _doBackup = keep_saveBackup;
//AI 101014 commented
        //open journal in output mode if need
///        if( _doBackup )
///           createLogoutStream( getJournalPath().c_str() ); //here now append if exists
    }
//AI 101014
///    if(_doBackup) {
///        rename( getJournalPath().c_str(), ( getJournalPath() + OLD ).c_str() );
///        rename( getBckArchivePath().c_str(), ( getBckArchivePath() + OLD ).c_str() );
///        rename( getBackupPath().c_str(), ( getBackupPath() + OLD ).c_str() );
///    }
///    if( _doBackup && !_restoreFrBck) {
///        createLogoutStream( getJournalPath().c_str() );
///    }
   if( _doBackup )
     createLogoutStream( getJournalPath().c_str() ); //here now append if exists
}

bool daq::rmgr::RM_Backup::checkState() {
   mytime();
   if ( !_doBackup )
      return false;
   if(!logostrm.is_open()) {
      if( _doBackup ) {
         std::ostringstream txt00;
         txt00 << " !!!! Backup should be produced but journal file did not open !!!";
         ers::error(ers::Message( ERS_HERE, txt00.str())); 
      }
      return false;
   }
   if( logostrm )
      return true;
   else {
      std::ostringstream txt0;
      txt0 << " !!! ATTENTION!!! Bad state of journal: state=" << logostrm.rdstate();
      ers::error(ers::Message( ERS_HERE, txt0.str()));
      return false;
   }
}

void daq::rmgr::RM_Backup::doErrMess( std::ofstream::failure& ex ) {
   std::ostringstream txt;
   txt << bckMessErr << ex.what();
   ers::error(ers::Message( ERS_HERE, txt.str()));
}

/*
void daq::rmgr::RM_Backup::readNamedStrFromLog(std::ifstream &is, CORBA::String_member& val) {
  std::string tmp;
  is >> tmp;
  readStrFromLog(is,tmp);
  val = CORBA::string_dup(tmp.c_str());
}
*/

void daq::rmgr::RM_Backup::readStrFromLog(std::ifstream& logstream, std::string& dest) {
  unsigned int w(0);
  logstream >> w;
  if(w > 0)
    logstream.get(logBuf[0]); //skip " "
  for(unsigned int i=0;i<w+1;i++)
    logstream.get(logBuf[i]);
  const std::string tmp(logBuf,w);
  if(!w)
    dest.clear();
  else
    dest = tmp;
}

std::string& daq::rmgr::RM_Backup::uint2str( unsigned int inp, std::string& outstr ) {
    static char* ptr;
    ptr = bufUint;
    boost::spirit::karma::generate(ptr, boost::spirit::uint_, inp);
    outstr.assign(bufUint, ptr - bufUint);
    return outstr;
}

std::string& daq::rmgr::RM_Backup::ulong2str( unsigned long inp, std::string& outstr ) {
    static char* ptr;
    ptr = bufUint;
    boost::spirit::karma::generate(ptr, boost::spirit::ulong_, inp);
    outstr.assign(bufUint, ptr - bufUint);
    return outstr;
}

std::string& daq::rmgr::RM_Backup::long2str( long inp, std::string& outstr ) {
    static char* ptr;
    ptr = bufUint;
    boost::spirit::karma::generate(ptr, boost::spirit::long_, inp);
    outstr.assign(bufUint, ptr - bufUint);
    return outstr;
}

void daq::rmgr::RM_Backup::logStr( const std::string& valStr) {
   logostrm << prb << uint2str( valStr.length(), str4conv ) << prb <<  valStr;
}

void daq::rmgr::RM_Backup::logChr( const char* valChr) {
   logostrm << prb << uint2str( strlen(valChr), str4conv ) << prb <<  valChr;
}

void daq::rmgr::RM_Backup::logULong( CORBA::ULong val) {
  logostrm << prb <<  ulong2str( val, str4conv ); 
}

void daq::rmgr::RM_Backup::logRequestResourcesForProcess( const std::string& p, const daq::rmgr::Res_ID_list& res_lst, const std::string& computer, CORBA::ULong pid, const std::string& client, ::CORBA::ULongLong callStamp, long handle) {
   //mytime();
   if( checkState() )
     try {
         std::string res0(res_lst.length()>0 ? res_lst[0].id : "");
         if(handle > 0)
           logostrm << okey;
         else
           logostrm << nok;
         logostrm << long2str( logTime, str4conv ) << prb
                 << daq::rmgr::opRequestResourcesForProcess;
         logStr( p );
         logStr( res0 );
         logStr( computer );
         logULong( pid );
         logStr( client );
         logULong( callStamp );
         //logostrm << prb << uint2str( p.length(), str4conv ) << prb << p;
         //logostrm << prb << uint2str( res0.length(), str4conv ) << prb << res0;
         //logostrm << prb << uint2str( computer.length(), str4conv ) << prb << computer;
         //logostrm << prb <<  ulong2str( pid, str4conv );
         //logostrm << prb <<  uint2str( client.length(), str4conv )<< prb << client;
         //logostrm << prb <<  callStamp
         logostrm << idStr << long2str( handle, str4conv )
                  << std::endl << std::flush;
         ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logRequestResources( const std::string& p, const std::string& sw, const std::string& computer, CORBA::ULong pid, const std::string& client, ::CORBA::ULongLong callStamp, long handle) {
   //mytime();
   if( checkState() )
     try {
         if(handle > 0)
           logostrm << okey;
         else
           logostrm << nok;
         logostrm << long2str( logTime, str4conv ) << prb
                 << daq::rmgr::opRequestResources;
         logStr( p );
         logStr( sw );
         logStr( computer );
         logULong( pid );
         logStr( client );
         logULong( callStamp );
         //logostrm << prb << uint2str( p.length(), str4conv ) << prb << p;
         //logostrm << prb << uint2str( sw.length(), str4conv ) << prb << sw;
         //logostrm << prb << uint2str( computer.length(), str4conv ) << prb << computer;
         //logostrm << prb <<  ulong2str( pid, str4conv );
         //logostrm << prb <<  uint2str( client.length(), str4conv )<< prb << client;
         //logostrm << prb <<  callStamp
         logostrm << idStr << long2str( handle, str4conv )
                  << std::endl << std::flush;
         ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logRequestHWResources( const char* partition, const char* softwareUID, const char* computer, const char* hwClass, const char* hwId, CORBA::ULong pid, const char* client, ::CORBA::ULongLong callStamp, long handle) {
   if( checkState() )
     try {
         if(handle > 0)
           logostrm << okey;
         else
           logostrm << nok;
         logostrm << long2str( logTime, str4conv ) << prb
                 << daq::rmgr::opRequestHWResources;
         logChr( partition );
         logChr( softwareUID );
         logChr( computer );
         logChr( hwClass );
         logChr( hwId );
         logULong( pid );
         logChr( client );
         logULong( callStamp );
         logostrm << idStr << long2str( handle, str4conv )
                  << std::endl << std::flush;
         ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logRequestHWResourceForProcess( const char* partition, const char* resource, const char* computer, const char* hwId, CORBA::ULong pid, const char* clientName, ::CORBA::ULongLong callStamp, long hndle ) {
   if( checkState() )
     try {
         if(hndle > 0)
           logostrm << okey;
         else
           logostrm << nok;
         logostrm << long2str( logTime, str4conv ) << prb
                 << daq::rmgr::opRequestHWResourceForProcess;
         logChr( partition );
         logChr( resource );
         logChr( computer );
         logChr( hwId );
         logULong( pid );
         logChr( clientName );
         logULong( callStamp );
         logostrm << idStr << long2str( hndle, str4conv )
                  << std::endl << std::flush;
         ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeResources( long handle ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << daq::rmgr::opFreeResources;
        logostrm << prb << long2str( handle, str4conv )
                 << std::endl << std::flush;
           ++_numOkOperations;
        }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeApplicationResources( const std::string& p, const std::string& swObj, const std::string& computer ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreeApplicationResources;
        logStr( p );
        logStr( swObj );
        logStr( computer );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeProcessResources( const std::string& p, const std::string& computer, CORBA::ULong pid ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreeProcessResources;
        logStr( p );
        logStr( computer );
        logULong( pid );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreePartition( const char* myHost,  const char* myName, const char* p ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreePartition;
        logChr( myHost );
        logChr( myName );
        logChr( p );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreePartitionResource( const char* p, const char* resource ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreePartitionResource;
        logChr( p );
        logChr( resource );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeComputerAll( const char* computer ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreeComputerAll;
        logChr( computer );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeResource( const char* resource ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreeResource;
        logChr( resource );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::logFreeComputerResource( const char* resource, const char* computer, const char* p ) {
   if( checkState() )
     try {
        logostrm << okey << long2str( logTime, str4conv ) << prb
                 << opFreeComputerResource;
        logChr( resource );
        logChr( computer );
        logChr( p );
        logostrm << std::endl << std::flush;
           ++_numOkOperations;
     }
     catch (std::ofstream::failure& ex1) {
            doErrMess( ex1 );
     }
}

void daq::rmgr::RM_Backup::exitLogLoadWithError(std::string operation, int inum) {
   std::ostringstream txt;
   txt << "loadFromLog read error in operation " << operation
    << " in line number " << inum;
   ers::error( ers::Message( ERS_HERE, txt.str()));
   exit(14);
}

void daq::rmgr::RM_Backup::readStr(std::ifstream& logstream, std::string& dest) {
  unsigned int w(0);
  logstream >> w;
  if(w > 0)
    logstream.get(logBuf[0]); //skip " "
  for(unsigned int i=0;i<w+1;i++)
    logstream.get(logBuf[i]);
  const std::string tmp(logBuf,w);
  if(!w)
    dest.clear();
  else
    dest = tmp;
}

void daq::rmgr::RM_Backup::loadFromLog( const char* logPath, daq::rmgr::RM_Impl* rm_impl) {
   std::string p;
   std::string computer;
   std::string client;
   std::string sw;
   std::string hwClass;
   std::string hwId;
   std::string host;
   std::string user;
   std::string res0;
   long handle;
   unsigned long pid;
   int  action;
   std::ifstream fin(logPath);
   daq::rmgr::Res_ID_list res_lst;
   uint64_t callStamp;
//std::cerr<<" @@@@ daq::rmgr::RM_Backup::loadFromLog started"<<std::endl;
   int iline = 1;
   std::string _okey;

//first line 
//const std::string rmStamp = "Resource Manager time "; //todo write when start log with time

   std::string test;
   std::ostringstream txt;

 try{
   if (!fin.is_open()){
     return; // UNSUCCESS;
   }
   std::ifstream& pis = fin;
//adjust to 1st action read
   int ij=0;
   std::string strtmp ="";
   pis >> test;
   while(strtmp != "time") {
     ij++;
     pis >> strtmp;
     test +=" " + strtmp;
     if (ij >10) break;
   }
   
   if (!(test  == rmStamp)){ //log crashed
     txt<<"Can not start to load log with path=\"" << logPath << "\".";
     ers::info( ers::Message( ERS_HERE, txt.str()));
     exitLogLoadWithError("Log did not detected as Resource Manager log.", iline);
   }

   pis >> test; //time 
   pis >> _okey;

   while(!pis.eof()){
     pis  >> ij >> action;
     iline++;
//std::cerr<<" iline="<<iline<<" ij="<<ij<<" action="<<action<<std::endl;
     switch ( action) {
       case opRequestResources:
         try{
            readStr(pis, p);
            readStr(pis, sw);
            readStr(pis, computer);
            pis >> pid;
            readStr(pis, client);
            pis >> callStamp;
            pis >> test;
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opRequestResources: call  rm_impl->allocateResources( "
//<< p <<", "<<sw<<", "<<computer<<", "<< pid<<", "<<client<<", "<<callStamp<<")"<<std::endl;
                 rm_impl->allocateResources( p, sw, computer, pid, client,callStamp);
                 _numOkOperations++;
              } 
              catch ( daq::rmgr::Exception & ex) {
                ers::debug(ex, 1);
              }
            }   
         }
         catch( ...) {
            exitLogLoadWithError("RequestResources", iline);
         }

         break;

       case opRequestResourcesForProcess:
        try{
            readStr(pis, p);
            readStr(pis, res0);
            readStr(pis, computer);
            pis >> pid;
            readStr(pis, client);
            pis >> callStamp;
            pis >> test;
            res_lst.length(1);
            res_lst[0].id = CORBA::string_dup(res0.c_str());
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opRequestResourcesForProcess: call  rm_impl->allocateResources4Process( "
//<< p <<", "<<res_lst[0].id<<", "<<computer<<", "<< pid<<", "<<client<<", "<<callStamp<<")"<<std::endl;

                 rm_impl->allocateResources4Process( p, res_lst, computer, pid, client,callStamp );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("RequestResourcesForProcess", iline);
         }
         break;
       case opRequestHWResources:
        try{
            readStr(pis, p);
            readStr(pis, sw);
            readStr(pis, computer);
            readStr(pis, hwClass);
            readStr(pis, hwId);
            pis >> pid;
            readStr(pis, client);
            pis >> callStamp;
            pis >> test;
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opRequestHWResources: call  rm_impl->allocateResources( "
//<< p <<", "<<sw<<", "<<computer<<", "<< pid<<", "<<client<<", "<<callStamp<<", "<<hwClass<<", "<<hwId<< ")"<<std::endl;

                 rm_impl->allocateResources( p, sw, computer, pid, client, callStamp, hwClass, hwId);
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("RequestHWResources", iline);
         }
         break;
       case opRequestHWResourceForProcess:
        try{
            readStr(pis, p);
            readStr(pis, res0);
            readStr(pis, computer);
            readStr(pis, hwId);
            pis >> pid;
            readStr(pis, client);
            pis >> callStamp;
            pis >> test;
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opRequestHWResourceForProcess: call  rm_impl->allocateHWResource4Process( "
//<< p <<", "<<res0<<", "<<computer<<", "<< pid<<", "<<client<<", "<<hwId<<", "<<callStamp<< ")"<<std::endl;

                 rm_impl->allocateHWResource4Process( p, res0, computer, pid, client,hwId, callStamp);
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("RequestHWResourceForProcess", iline);
         }
         break;

       case opFreeResources:
         try{
           pis >> handle;
           if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreeResources: call  rm_impl->freeResource( "<<handle<<")"<<std::endl;
                 rm_impl->freeResource( handle );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                ers::debug(ex, 1);
              }
           }
         }
         catch( ...) {
            exitLogLoadWithError("FreeResources", iline);
         }
         break;
       case opFreeApplicationResources:
         try{
//std::cerr<<"  opFreeApplicationResources: call  rm_impl->freeApplicationResources( "
//<< p <<", "<<sw<<", "<<computer<< ")"<<std::endl;
            readStr(pis, p);
            readStr(pis, sw);
            readStr(pis, computer);
            if((const std::string) _okey == okey1) {
              try {
                 rm_impl->freeApplicationResources( p, sw, computer );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreeApplicationResources", iline);
         }
         break;
       case opFreeProcessResources:
         try{
            readStr(pis, p);
            readStr(pis, computer);
            pis >> pid;
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreeProcessResources: call  rm_impl->freeResource( "
//<< p <<", "<<computer<<", "<< pid<< ")"<<std::endl;
                 rm_impl->freeResource( p, computer, pid);
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreeProcessResources", iline);
         }
         break;

       case opFreePartition:
         try{
            readStr(pis, host);
            readStr(pis, user);
            readStr(pis, p);
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreePartition: call  rm_impl->freePartition( "
//<< host <<", "<<user<<", "<< p<< ")"<<std::endl;
                 rm_impl->freePartition( host.c_str(), user.c_str(), p );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreePartition", iline);
         }
         break;
       case opFreePartitionResource:
         try{
            readStr(pis, p);
            readStr(pis, res0);
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreePartitionResource: call  rm_impl->freePartitionResource( "
//<< p <<", "<<res0<< ")"<<std::endl;
                 rm_impl->freePartitionResource( p, res0 );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreePartitionResource", iline);
         }
         break;

       case opFreeComputerAll:
         try{
            readStr(pis, computer);
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreeComputerAll: call  rm_impl->freeComputerAll( "<<computer<< ")"<<std::endl;
                 rm_impl->freeComputerAll( computer );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreeComputerAll", iline);
         }
         break;

       case opFreeResource:
         try{
            readStr(pis, res0);
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreeResource call  rm_impl->freeResource( "<< res0<< ")"<<std::endl;
                 rm_impl->freeResource( res0 );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreeResource", iline);
         }
         break;

       case opFreeComputerResource:
         try{
            readStr(pis, res0);
            readStr(pis, computer);
            readStr(pis, p);
            if((const std::string) _okey == okey1) {
              try {
//std::cerr<<"  opFreeComputerResource: call  rm_impl->freeComputerResource( "
//<< res0<<", "<<computer<<", "<< p << ")"<<std::endl;
                 rm_impl->freeComputerResource( res0, computer, p );
                 _numOkOperations++;
              }
              catch ( daq::rmgr::Exception & ex) {
                 ers::debug(ex, 1);
              }
            }
         }
         catch( ...) {
            exitLogLoadWithError("FreeComputerResource", iline);
         }
         break;

       default:
          break; 
     }
     pis >> _okey;
   }
   fin.close();
 }
 catch( ... ) {
  exitLogLoadWithError("Uknown execption in load journal", iline);
 }
}

bool daq::rmgr::RM_Backup::saveArchive() {
//   std::ostringstream text;
  try {
     std::ofstream ofs( getBckArchivePath(true) ); //save always into new file
     boost::archive::text_oarchive oa(ofs);
     oa & _rm_impl;
  }
  catch( ... ) {
     std::ostringstream text;
     text<<"daq::rmgr::RM_Backup::saveArchive excpetion when save into \""
         <<  getBckArchivePath(true) << "\""<<std::endl;
     ers::error( ers::Message( ERS_HERE, text.str()));
     return false;
  }
  return true;
}

void daq::rmgr::RM_Backup::restoreArchive() {
  //check if file exists
/*   
::System::File tst_restore( getBckArchivePath() );
   if( !tst_restore.exists()) {
      std::ostringstream text;
      std::cerr<<"daq::rmgr::RM_Backup: restore file " << getBckArchivePath()<<" does not exists or corrupted" << std::endl;
     text<<"daq::rmgr::RM_Backup::restoreArchive: restore file " << getBckArchivePath()<<" does not exists or corrupted" << std::endl;
      ers::error( ers::Message( ERS_HERE, text.str()));
     return;
   }
*/
  std::ifstream ifs( getBckArchivePath() );
std::cerr<<"daq::rmgr::RM_Backup:   restoreArchive from \""<<getBckArchivePath()<<"\"" <<std::endl;
  if(!ifs) {
    std::ostringstream text;
    std::cerr<<"daq::rmgr::RM_Backup: restore file " << getBckArchivePath()<<" does not exists or corrupted" << std::endl;
    text<<"daq::rmgr::RM_Backup::restoreArchive: restore file " << getBckArchivePath()<<" does not exists" << std::endl;
    ers::error( ers::Message( ERS_HERE, text.str()));
    return;
  }
  boost::archive::text_iarchive ia(ifs);
  ia & _rm_impl;
}

bool daq::rmgr::RM_Backup::saveBackup() {
//saveArchive();
 std::ostringstream text;
 bool retv = saveArchive();
 if( !retv ) {
     text<<" problem during save archive. Keep journal as it was/"<<std::endl;
     ers::error( ers::Message( ERS_HERE, text.str()));
     return false;
 }
 // keep current journal info in OLD and reopen it.
 try {
    if( logostrm.is_open() )
       logostrm.close();  
    //now replace archive and journal names (keep old stuff as OLD)
    rename( getBckArchivePath().c_str(), ( getBckArchivePath() + OLD ).c_str() );
    rename( getJournalPath().c_str(), ( getJournalPath() + OLD ).c_str() );
    //archive saved always into NEW. Now remove NEW from name
    rename( getBckArchivePath( true ).c_str(), getBckArchivePath().c_str());
   //now reopen journal stream
   createLogoutStream( getJournalPath().c_str() );
 }
 catch (std::ofstream::failure ex1) {
    text<<"saveBackup: copy journal and open new journal failed with exception:"<<ex1.what()<<std::endl;
    ers::error( ers::Message( ERS_HERE, text.str()));
    return false;
 }
 return true;
}

