#ifndef RM_Server_H_INCLUDED
#define RM_Server_H_INCLUDED
#include "src/RMimpl.h"

#include "ipc/object.h"
#include "ipc/core.h"
#include "owl/semaphore.h"
#include <boost/thread/locks.hpp>
#include <boost/thread/shared_mutex.hpp>

#include <boost/spirit/include/karma.hpp>
#include <fstream>
#include <locale.h>
#include <time.h>
#include <map>
#include <vector>

#define RMIPCNamedObject ::IPCNamedObject<POA_rmgr::ResMgr>

#define TDAQ_RM_SERVER "TDAQ_RM_SERVER"
#define TDAQ_RM_LOGPATH_ENV "TDAQ_RM_LOGPATH_ENV"

#define RMLOGNAME "_rmgrbackup.data.xml" 
#define RMSERVERDEFAULT "RM_Server"

namespace daq
{
namespace rmgr
{

//backup constants
//static const std::string bckMessErr = "Exception when writing to backup";
static const std::string JOURNAL( ".rmgrjournal" );
static const std::string ARCH( ".rmgrarch" );
static const std::string OLD( ".old" );
static const std::string NEW( ".new" );
enum logAction { opRequestResources, opRequestResourcesForProcess, opRequestHWResources, opRequestHWResourceForProcess, opFreeResources, opFreeApplicationResources, opFreeProcessResources, opFreePartition, opFreePartitionResource, opFreeComputerAll, opFreeResource, opFreeComputerResource };
static const std::string logStrAction [12] = { "requestResources(",
 "requestResourcesForProcess(", "requestHWResources(", "requestHWResourceForProcess(",
 "freeResources(", "freeApplicationResources(",
 "freeProcessResources(", "freePartition(", "freePartitionResource(", "freeComputerAll(", "freeResource(", "freeComputerResource(" };

/**
*  This class implement the IDL interface and delegate
*  any action to the RM_Impl class
* **/
class RmgrImpl :  public RMIPCNamedObject, public OWLSemaphore
{
  public:
        RmgrImpl();
        RmgrImpl(std::string serverName, RM_Impl* rmimpl);
        ~RmgrImpl();
//config update
void rmConfigUpdate ( const daq::rmgr::Config_update& conf);

//request resources actions
 ::CORBA::Long requestResources (
                                const char*  partitionName,
                                const char* softwareUID,
                                const char* computerName,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry);
 ::CORBA::Long requestHWResources (
                                const char*  partitionName,
                                const char* softwareUID,
                                const char* computerName,
                                const char* hwClass,
                                 const char* hwId,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry);
 ::CORBA::Long requestResourcesForProcess(const char* partitionName,
                                const daq::rmgr::Res_ID_list&,
                                const char* computerName,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry);
 ::CORBA::Long requestHWResourceForProcess (
                                const char* partitionName,
                                const char* resource,
                                const char* computerName,
               ///////                 const char* hwClass,
                                const char* hwId,
                                CORBA::ULong pid,
                                const char* clientName,
                                ::CORBA::ULongLong callStamp,
                                ::CORBA::Boolean isFirstTry);
//free resources actions
void freeProcessResources( const char* p, const char* computer, CORBA::ULong pid, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller);
void  freeAllProcessResources( ::CORBA::Long handleId, const char* partition, const char* computer, CORBA::ULong pid, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller);
void freeResources (::CORBA::Long handle, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller);
void freePartition( const char* myhost, const char* myname, const char* p );
void freePartitionResource( const char* p, const char* resource, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller );
void freeComputerAll( const char* computer, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller );
void freeResource( const char* resource, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller );
void freeComputerResource( const char* resource, const char* computer, const char* p, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller );
void freeApplicationResources( const char* p, const char* swObj, const char* computer, CORBA::Boolean log_this_call, const daq::rmgr::CallerInfo& caller );

////void freeAllResources(const char* myhost, const char* myname, const char*  p = "*",const char* swobjid = "*", const char* compname = "*");

//getting different info
daq::rmgr::Rmstr_lst* getPartitionsList();
daq::rmgr::Rmstr_lst* getResourceOwners(const char* rname, const char* pname);

daq::rmgr::Handle_Info* getHandleInfo(::CORBA::Long handle);
daq::rmgr::Allocated_Info*  getResourceInfo(const char* p, const char* rn);
///                        /*bool*/ ::CORBA::Long addhandleinfo);

daq::rmgr::Allocated_Info* getFullResInfo(const char*  rname); // raises( daq::rmgr::ResourceNotFoundE );
daq::rmgr::Allocated_Info* getPartitionInfo(const char* partition);
daq::rmgr::Allocated_Info* getComputerInfo(const char* computer);
daq::rmgr::Allocated_Info* getComputerResourceInfo(const char* computer, const char* partition, const char* resource);
daq::rmgr::Config_update* getConfigInfo();
const std::string getServerName() const {
    return _serverName;
}
void shutdown() {
  post();
//  stop();
}
//for tests
///long getReqAverage() { return allocTimes.getReqAverage(); }
///bool setVal( timeval& t2, timeval& t1 ) { return allocTimes.setVal( t2, t1 ); };

    private:
        std::string _serverName;
        RM_Impl* rm_impl;
       /// RM_Backup* bck;
       
        boost::mutex m_mutex; //temprary
        boost::shared_mutex rw_mutex;
        boost::shared_mutex so_mutex;
    //for tests
   /// RM_Statistics allocTimes;
void addMesRes( std::ostringstream& str, daq::rmgr::Allocated_Info* info );
void addMesRes( std::ostringstream& str, daq::rmgr::Handle_Info* info );
void addMesConfig( std::ostringstream& str, daq::rmgr::Config_update* inf );
};

/**
*  Support all back-up functionality including journaling, store/restore data into/from backup file
*
*  **/
class RM_Backup {
    public:
      RM_Backup( const char * backupPath, bool doBackup, bool restoreFrBck = false)
         :  _backupPath(backupPath), _doBackup(doBackup), _restoreFrBck(restoreFrBck),
            _numOkOperations(0) {
      }
      ~RM_Backup() {
          //TODO
          if( _doBackup )
            saveBackup();
          _doBackup = false;
          try{
            if( logostrm.is_open() )
               logostrm.close();
          }
          catch( ... ) {}
      }
      const std::string& getBckPath() {
         return _backupPath;
      }
      bool isDoBackup() {
          return _doBackup;
      }
      bool isRestoreBackup() {
         return _restoreFrBck;
      }
//TODO
//      bool restoreFrBck() {
//      bool restore();

  bool saveBackup();

//all print to journal methods
  void logRequestResourcesForProcess( const std::string& p, const daq::rmgr::Res_ID_list& res_lst, const std::string& computer, CORBA::ULong pid, const std::string& client, ::CORBA::ULongLong callStamp, long handle);
  void logRequestResources( const std::string& p, const std::string& sw, const std::string& computer, CORBA::ULong pid, const std::string& client, ::CORBA::ULongLong callStamp, long handle);
void logRequestHWResources( const char* partition, const char* softwareUID, const char* computer, const char* hwClass, const char* hwId, CORBA::ULong pid, const char* client, ::CORBA::ULongLong callStamp, long hndle);
void logRequestHWResourceForProcess( const char* partition, const char* resource, const char* computer, const char* hwId, CORBA::ULong pid, const char* clientName, ::CORBA::ULongLong callStamp, long hndle );
void logFreeResources( long handle );
void logFreeApplicationResources( const std::string& p, const std::string& swObj, const std::string& computer );
void logFreeProcessResources( const std::string& p, const std::string& computer, CORBA::ULong pid );
void logFreePartition( const char* myHost,  const char* myName,const char* p );
void logFreePartitionResource( const char* p, const char* resource );
void logFreeComputerAll( const char* computer );
void logFreeResource( const char* resource );
void logFreeComputerResource( const char* resource, const char* computer, const char* p );

void loadFromLog( const char* logPath, daq::rmgr::RM_Impl* rm_impl);
void initialize( daq::rmgr::RM_Impl*& rm_impl );

  protected:
      bool createLogoutStream(const char* log_file);
      bool checkState();
      void doErrMess( std::ofstream::failure& ex );
      void readStrFromLog(std::ifstream& logstream, std::string& dest);
      ///void readNamedStrFromLog(std::ifstream &is, CORBA::String_member& val);
      //void loadFromBackup( const std::string& dbName);
      
  private:
      std::string _backupPath;
      bool _doBackup;
      bool _restoreFrBck;
      long _numOkOperations; //number of succeed requests
      std::ofstream logostrm; //stream for logout sysprints
      ///boost::mutex m_bck_mutex; // mutex to save backup safely
      long logTime;
      std::string str4conv;
      char bufUint[32];
      char logBuf[512];
      
      RM_Impl* _rm_impl; //only pointer, do not delete here

      //void daq::rm::RMrapper::saveBackup();
      //working constants
      const std::string rmStamp = "Resource Manager time"; //first line
      const std::string bckMessErr = "Exception when writing to backup";
      const std::string prb = " ";
      const std::string okey = ">>Ok ";
      const std::string okey1 = ">>Ok";
      const std::string nok = ">>No ";
      const std::string idStr = " id=";

      void mytime() { logTime = (long) time(NULL); }
      std::string& uint2str( unsigned int inp, std::string& outstr );
      std::string& ulong2str( unsigned long inp, std::string& outstr );
      std::string& long2str( long inp, std::string& outstr );
      void logStr( const std::string& valStr);
      void logChr( const char* valChr);
      void logULong( CORBA::ULong val);
      void exitLogLoadWithError(std::string operation, int inum);
      void readStr(std::ifstream& logstream, std::string& dest);
      std::string getJournalPath( bool isNew = false ) {
         return !isNew ? (_backupPath + JOURNAL) : _backupPath + JOURNAL + NEW;
      }
      std::string getBckArchivePath( bool isNew = false ) {
         return !isNew ? (_backupPath + ARCH) : _backupPath + ARCH + NEW;
      }
      std::string getBackupPath( bool isNew = false ) {
         return !isNew ? _backupPath : _backupPath + NEW; 
      }

      friend class boost::serialization::access;
      template<class Archive>
      void serialize(Archive & ar, const unsigned int version) {
         ar & _rm_impl;
      }

      bool saveArchive();
      void restoreArchive();
};

/*! \brief This class provides initialisation, start and stop of the RM server.
    \ingroup public
    \sa
 */
class RM_Server
{
  public:
//constructors
/*! \brief Create server object with server name RMSERVERDEFAULT name (hardcoded "RM_Server") and backup path RMSERVERDEFAULT + "_" + RMLOGNAME (hardcoded "RM_Server_rm.rmgrbackup.data.xml"
 *  */
        RM_Server();
/*! \brief Create server obejct with according parameters. If logfilename does not keep server name in its name then server name added at the end of logfilename.
 *   *   \param sid         RM server name
 *     *   \param backup  path where backup should be stored. Journal has the same as backup path but with additional extension ".journal". It keeps information about successful request/free resources Resource Manager operations. Both these files are needed for restore from backup but only backup path is needed as input parameter.
 *       * \param doBck         create backup file if parameter value is true
 *         * \param doresto       restore from backup file if parameter value is true and backup exists.
 *          */
        RM_Server(const char * sid, const char* backup = "RM_Server_rm.rmgrbackup.data.xml", bool doBck=true, bool doresto=true);
        ~RM_Server();
const std::string getServerName() const {
    return idlRapper->getServerName();
}

//server func
 void run();
 void stop() {
    if(idlRapper && !destroyInProgress) {
        idlRapper->post();
    }
 }
void removeServ();
    private:
        bool destroyInProgress; //stopped;
        RmgrImpl* idlRapper;
        RM_Impl* rm_impl;
///        RM_Backup* bck;
};


  } //end rm
} //end daq




#endif /* RM_Server_H_INCLUDED  */
