#include "src/RMimpl.h"
#include "src/RM_Server.h" //move backup to RMimpl file?
#include <ResourceManager/idlhelper.h>
#include <AccessManager/xacml/impl/RMResource.h>
#include <AccessManager/client/RequestorInfo.h>
#include <AccessManager/client/ServerInterrogator.h>
#include <AccessManager/util/ErsIssues.h>

namespace daq
{
namespace rmgr
{
const std::string eMess[] = {"\nResource ",  " unavailable in ", "partition", "global", " scope for requestor:\n", "* process ", " of binary ", " on host ", " run by ", " in partition ", "* handle ", " for process ",  " on computer ", "Resource holders:\n",
  " for hardware object ", " of class " };
}
const std::string mesMaxBusy[] = { " Max: ", " Max (per partition): ",  " granted:  " };
}
//const std::string compClass = "Computer";

// RMimpl class methods
daq::rmgr::RM_Impl::~RM_Impl() {
//std::cerr<<"@@@@@@@@@@@@@@ :~RM_Impl"<<std::endl;
   if( !( _bck == 0 ) ) {
     delete _bck;
//std::cerr<<"@@@@@@@@@@@@@@RM_Impl::~RM_Impl _bck delete done"<<std::endl;
   }
}

//AI 2017
Configuration* daq::rmgr::RM_Impl::loadConfiguration() {
   char * dbname = getenv( TDAQ_RM_CONFIG );
   Configuration * sw_conf = 0;
   std::ostringstream text;
   if( dbname == 0 ) {
      text << "No environment value for variable TDAQ_RM_CONFIG";
      //ers::error( ers::Message( ERS_HERE, text.str()));
      //AI 2018
      throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str()  );
      //throw daq::rmgr::ConfigException(  text.str().c_str() );
   }
   std::cerr<<"loadConfiguration() for dbname="<<dbname<<std::endl;
   try{
        sw_conf = new Configuration(dbname);
   }
   catch (daq::config::Exception & ex) {
      if(!(sw_conf == 0))
          delete sw_conf;
      //text << ex;
      //ers::error( ers::Message( ERS_HERE, text.str()));
      //AI 2018
      throw daq::rmgr::ConfigurationException( ERS_HERE, ex.what(), ex );
      //throw daq::rmgr::ConfigException(  text.str().c_str() );
   }
   catch ( ... ) {
      text << "Can not load configuration for DB name="<< dbname;
      //ers::error( ers::Message( ERS_HERE, text.str()));
      //AI 2018
      throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str() );
      //throw daq::rmgr::ConfigException(  text.str().c_str() );
   }

   return sw_conf;
}

bool daq::rmgr::RM_Impl::isConfigAvailable() { //AI 2018
  std::ostringstream text;
  try{
     Configuration* cf = loadConfiguration();
     if(!(cf == 0))
          delete cf;
     return true;
  }
  catch (daq::rmgr::ConfigurationException& ex) {
    //text << ex.reason;
    //ers::error( ers::Message( ERS_HERE, text.str()));
    ers::error(ex);
    return false;
  }
}
void daq::rmgr::RM_Impl::rmConfigurationUpdate(::Configuration * conf) {
    //get all config resources
    std::vector<const daq::core::RM_Resource *> resources;
    conf->get(resources);
    ResourceType type;
    //update resources
    bool firstUpdate = _conf_res_lst.empty();
    for(std::vector<const daq::core::RM_Resource *>::iterator ir = resources.begin(); ir != resources.end(); ++ir) {
      if((*ir)->class_name() == "RM_SW_Resource")
         type = daq::rmgr::ResourceType::SWR;
      else if((*ir)->class_name() == "RM_HW_Resource")
         type = daq::rmgr::ResourceType::HWR;
      else
         continue;
      RM_Resource r( (*ir)->UID().c_str(), (*ir)->get_MaxCopyTotal(), (*ir)->get_MaxCopyPerPartition(), type);
      //erase old value with the same key
      const auto& it =  _conf_res_lst.find(r.uid());
      if(it != _conf_res_lst.end()) {
          configWarningMess( &(it->second), &r );
         _conf_res_lst.erase(it);
      }
      else if(!firstUpdate)
          configWarningMess( 0, &r );
      //insert new value
      _conf_res_lst.insert( std::make_pair(r.uid(), r ) );
    }
//sw objects
 std::string strNull("");
    firstUpdate = _swobj_map.empty();
    std::vector<const daq::core::SW_Object *> sw_objects;
    conf->get(sw_objects);
    for(std::vector<const daq::core::SW_Object *>::iterator j = sw_objects.begin(); j != sw_objects.end(); ++j) {
       std::string keysw( (*j)->UID().c_str() );
       const auto& itsw = _swobj_map.find(keysw);
       unsigned int nr = (*j)->get_Needs().size();
       if( itsw != _swobj_map.end()) {
//          if( nr == 0)
              configWarningMess( *itsw, keysw, (*j)->get_Needs() );
//          else
//              configWarningMess( &itsw, *j );
          _swobj_map.erase(keysw);
       }
       else {
          if(nr > 0 && !firstUpdate)
             configWarningMess( keysw );
       }
       if( nr == 0 )
          continue;
       std::vector<RM_Resource> rList;
       //add res references
       for ( size_t ii=0; ii < (*j)->get_Needs().size(); ii++ ) {
          std::string res( (*j)->get_Needs()[ii]->UID().c_str() );
          const auto& lstptr =  _conf_res_lst.find(res);
          if(lstptr != _conf_res_lst.end())
              rList.push_back(lstptr->second);
       }
       if(rList.size() > 0)
         _swobj_map.insert( std::make_pair(keysw, rList) );
    } 
    
}
void daq::rmgr::RM_Impl::configWarningMess(  std::string& str ) {
  std::ostringstream txt;
  txt << "New software object \'" << str << "\' added"<<std::endl;
  ers::warning( ers::Message( ERS_HERE, txt.str()));
}
bool daq::rmgr::RM_Impl::listResourcesDiffer( std::vector<RM_Resource> oldV, const std::vector<const daq::core::RM_Resource*>& newVal ) {
//std::cerr<<"        daq::rmgr::RM_Impl::listResourcesDiffer started"<<std::endl;

   if(oldV.size() != newVal.size()) {
     std::cerr<<"        oldV.size() != newVal.size()"<<std::endl;
     return true;
  }
   bool found;// = false;
   for( unsigned int i=0; i<oldV.size(); i++) {
     found = false;
     for( unsigned int j=0; j<newVal.size(); j++ ) {
       if( oldV[i].uid() == newVal[j]->UID() ) {
          found = true;
          continue;
       }
     }
     if( !found )
       return true;
   }
   return false;
}
void daq::rmgr::RM_Impl::configWarningMess( const std::pair< std::string, std::vector<RM_Resource> > oldV, std::string& str,  const std::vector<const daq::core::RM_Resource*>& newVal ) {
   // std::cerr<<"        daq::rmgr::RM_Impl::configWarningMess started"<<std::endl;
   // std::cerr<<"        newVal.size() ="<<newVal.size()<<" oldV.second.size()="<<oldV.second.size()<<std::endl;
    std::ostringstream txt;
    if( newVal.size() == 0 ) {
       txt << " \n Resources removed from software object \'" << str << "\'"<<std::endl;
    }
    else if( oldV.second.size() == 0 ) {
      txt << " \n Resources added to software object \'" << str << "\'"<<std::endl;
    }
    else {
        if( listResourcesDiffer(oldV.second, newVal) )
      txt << " \n Resource list was changed in software object \'" << str << "\'"<<std::endl;
    }
    if( txt.tellp() > 10 )
       ers::warning( ers::Message( ERS_HERE, txt.str()));
}

void daq::rmgr::RM_Impl::configWarningMess( RM_Resource* oldVal, RM_Resource* newVal) {
    std::ostringstream txt;
    if( oldVal == 0 ) {
      txt << "\n New resource \'"<<newVal->uid()<<"\' added"<<std::endl;
      ers::warning( ers::Message( ERS_HERE, txt.str()));
      return;
    }
    if( oldVal->maxTot() == newVal->maxTot() 
         && oldVal->maxPP() == newVal->maxPP()
         && oldVal->type() == newVal->type() )
       return; //same resources
    txt<<"\n Resource \'"<<oldVal->uid()<<"\' changed:";
    if( oldVal->maxTot() != newVal->maxTot() )
       txt<<" [maxTotal: old=" << oldVal->maxTot() << " new=" << newVal->maxTot()<<"]";
    if( oldVal->maxPP() != newVal->maxPP() )
       txt<<" [maxPP: old=" << oldVal->maxPP() << " new=" << newVal->maxPP()<<"]";
    if( oldVal->type() != newVal->type() ) {
       txt<<" [Resource type:  old =\'";
       if(oldVal->type() == daq::rmgr::SWR)
         txt<<"SWR\'";
       else
          txt<<"HWR\'";
       txt<<" new=\'";
       if(newVal->type() == daq::rmgr::SWR)
         txt<<"SWR\'";
       else
          txt<<"HWR\'";
     }
     ers::warning( ers::Message( ERS_HERE, txt.str()));
}

void daq::rmgr::RM_Impl::rmConfigurationUpdate ( const daq::rmgr::Config_update& idl_conf ) {
   //AI 2017
   std::ostringstream text;
   try{
     Configuration* conf = loadConfiguration();
     rmConfigurationUpdate( conf );
     if(!(conf == 0))
       delete conf;
     return;
   }
   catch(daq::rmgr::ConfigurationException& ex) {
       ers::error(ex);

       text << ex;
       throw daq::rmgr::ConfigException(  text.str().c_str() );
   }
   catch (daq::config::Exception & ex) {
       ers::error(ex);

       text << ex;
       char * dbname = getenv( TDAQ_RM_CONFIG );
       text << std::endl << "The value of TDAQ_RM_CONFIG is ";
       if(dbname == 0)
          text << " NULL";
       else
          text << dbname;
       //ers::error( ers::Message( ERS_HERE, text.str()));
       //AI 2018
       //throw daq::rmgr::ConfigurationException( ERS_HERE, text.str().c_str()  );
       throw daq::rmgr::ConfigException(  text.str().c_str() );
       
  }

     
  // clean all and fill again with new data
  //  boost::mutex::scoped_lock scoped_lock( dbrep.sw_mutex );

// do not clear as some resources can absent 
// in new config but used in allocated
  // _conf_res_lst.clear(); 
  //AI 2017 commented 1 line below: should update instead of remove
  //_swobj_map.clear();
   unsigned int i;
  //add SW resources
  for( i=0; i<idl_conf.swr_lst.length(); i++) {
      std::string keyr(idl_conf.swr_lst[i].rid);
      //long maxTot(idl_conf.swr_lst[i].max_total);
      //long maxPP(idl_conf.swr_lst[i].max_pp);
      RM_Resource r( keyr,  idl_conf.swr_lst[i].max_total,  idl_conf.swr_lst[i].max_pp, daq::rmgr::ResourceType::SWR);
///      RM_Resource r2( keyr, maxTot, maxPP,  daq::rmgr::ResourceType::SWR);
      //erase old value with the same key
      const auto& it =  _conf_res_lst.find(r.uid());
      if(it != _conf_res_lst.end()) {
         _conf_res_lst.erase(it);
      }
      //now insert new value
      _conf_res_lst.insert( std::make_pair(r.uid(), r ) );
  }
   //add HW resources 
   for( i=0; i<idl_conf.hwcr_lst.length(); i++) {
      std::string keyhr(idl_conf.hwcr_lst[i].rid);
      const std::string hwcls( idl_conf.hwcr_lst[i].hwClass );
      RM_Resource rh( keyhr,  idl_conf.hwcr_lst[i].max_total,  idl_conf.hwcr_lst[i].max_pp, hwcls, daq::rmgr::ResourceType::HWR);
        //erase old value with the same key
       const auto& it2 =  _conf_res_lst.find(rh.uid());
       if(it2 != _conf_res_lst.end()) {
          _conf_res_lst.erase(it2);
       }
       //now insert new value
       _conf_res_lst.insert( std::make_pair(rh.uid(), rh ) );
    }
    //update SW objects
    for( unsigned int i=0; i<idl_conf.swobj_lst.length();  i++) {
        std::string keysw(idl_conf.swobj_lst[i].swid);
       //AI 2017 [
        const auto& itsw = _swobj_map.find(keysw);
        if( itsw != _swobj_map.end()) {
            _swobj_map.erase(keysw);
        }
        if(idl_conf.swobj_lst[i].res_list.length() == 0) {
          continue;
        }
        //AI ]
         std::vector<RM_Resource> rList;
        for(unsigned int j0=0; j0<idl_conf.swobj_lst[i].res_list.length(); j0++) {
            std::string res( idl_conf.swobj_lst[i].res_list[j0].id );
            const auto& lstptr =  _conf_res_lst.find(res);
            if(lstptr != _conf_res_lst.end()) {
               rList.push_back(lstptr->second);
            }
        }
        if(rList.size() > 0) {
            _swobj_map.insert( std::make_pair(keysw, rList) );
        }
    }
}

daq::rmgr::Config_update* daq::rmgr::RM_Impl::getConfigInfo() {
  daq::rmgr::Config_update* rm_conf = new daq::rmgr::Config_update();
  int iswr = 0, ihwr = 0;
  for (std::map<std::string, daq::rmgr::RM_Resource>::iterator it0=_conf_res_lst.begin(); it0 != _conf_res_lst.end(); ++it0)
       if( (it0->second).type() == daq::rmgr::SWR )
          iswr++;
       else
          ihwr++;
  rm_conf->swr_lst.length( iswr);
  rm_conf->hwcr_lst.length( ihwr );
  //set resources
  iswr = 0;
  ihwr = 0;
  for (std::map<std::string, RM_Resource>::iterator it=_conf_res_lst.begin(); it!=_conf_res_lst.end(); ++it) {
      RM_Resource r =  it->second;
      if(r.type() == daq::rmgr::SWR) {
         rm_conf->swr_lst[iswr].rid = CORBA::string_dup( r.uid().c_str() );
         rm_conf->swr_lst[iswr].max_total = r.maxTot();
         rm_conf->swr_lst[iswr++].max_pp = r.maxPP();
      }
      else { //HW
         rm_conf->hwcr_lst[ihwr].rid = CORBA::string_dup(r.uid().c_str());
         rm_conf->hwcr_lst[ihwr].max_total = r.maxTot();
         rm_conf->hwcr_lst[ihwr].max_pp = r.maxPP();
         rm_conf->hwcr_lst[ihwr].hwClass = CORBA::string_dup( r.hwCls().c_str());
         ihwr++;
      }
  }
  //set SWObj
  rm_conf->swobj_lst.length( _swobj_map.size() );
  int inum = 0, jnum = 0;
  for ( std::map< std::string, std::vector<RM_Resource> >::iterator it2 = _swobj_map.begin(); it2 != _swobj_map.end(); ++it2) {
      std::vector<RM_Resource> list = it2->second;
      const std::string key = it2->first;
      rm_conf->swobj_lst[inum].swid = CORBA::string_dup( key.c_str() );
      rm_conf->swobj_lst[inum].res_list.length( list.size() );
      jnum = 0;
      for( std::vector<RM_Resource>::iterator j = list.begin(); j != list.end(); j++ )
              rm_conf->swobj_lst[inum].res_list[jnum++].id = CORBA::string_dup( (*j).uid().c_str() );
      inum++;
  }
  return rm_conf;
}

//oct 2015 keep for human config content before and after updateConfig
void printConfigContent( daq::rmgr::Config_update* inf1, daq::rmgr::Config_update* inf2) {


 if(inf1)
   delete inf1;
 if(inf2)
   delete inf2;
}

//for unavailable info in exception
void daq::rmgr::RM_Control::putNoResourceMess( std::ostringstream& mess, const std::string& resource, const std::string& computer, bool isTot ) {
 mess << eMess[0] << resource;
 if(!computer.empty())
    mess << eMess[11] << computer;
 mess << eMess[1];
 if( isTot )
   mess << eMess[3];
 else
    mess << eMess[2];
}

void daq::rmgr::RM_Control::putResStatMess( std::ostringstream& mess, const daq::rmgr::RM_Resource& res, const long allocated, bool isTot) {
  if( isTot )
    mess << mesMaxBusy[0] << res.maxTot();
  else
     mess << mesMaxBusy[1] << res.maxPP();
  mess << mesMaxBusy[2] << allocated;
  mess << std::endl;
}
void daq::rmgr::RM_Control::putHandleMess( std::ostringstream& mess,  const std::string& computer, const std::string& binary, const std::string& client, const std::string& p, unsigned long pid, long hndl,  const std::string& hwObj,  const std::string& hwCls ) {
  if(hndl > 0) // otherwise - requestor, no handle
    mess << eMess[10] << hndl; //handle
  mess << eMess[11] << pid; //process
  mess << eMess[6] << binary; //of binary
  if(!computer.empty())
    mess << eMess[7] << computer; //on host
  mess << eMess[8] << client; //run by   client
  mess << eMess[9] << p; //in partition 
  //211014
  if( !hwObj.empty() ) {
     mess << eMess[14] << hwObj; //for hw obj
     mess << eMess[15] << hwCls; //in class
  }
  mess << std::endl;
}

void daq::rmgr::RM_Control::putHandleMess(  std::ostringstream& mess, std::vector<RM_AllocatedResource>& resLst) {
   std::vector<RM_AllocatedResource>::iterator i = resLst.begin();
   for( ; i != resLst.end(); ++i) {
     putHandleMess( mess, (*i).getComputer(), (*i).getSwObj(),(*i).getClient(),
                    (*i).getPartition(), (*i).getPid(), (*i).getHandle(),
                    (*i).getHwObj(), (*i).getHwClass()  );
   }
}

bool daq::rmgr::RM_Control::isResourceAvailable(const daq::rmgr::RM_Resource& res, const std::string& partition, const std::string& computer,
       const std::string& binary, const std::string& client, unsigned long pid // for exception
) {

            bool available = false;
//            int allocPP, allocT;
    std::ostringstream mess; //for exception
            const daq::rmgr::ResourceType resType = res.type();
            const std::string& resID = res.uid();
            
            if(resType == daq::rmgr::ResourceType::HWR) { 
              if(res.hwCls().length() > 0 && res.hwCls() != compClass) //for any case, should be checked before
                  return true; 
             // && res.hwCls().length() == 0) {  //or "Computer" ??
                // Check number of copies per partition
                const long allocPerPart = getComputerAllocatedPP(resID, partition, computer);
                if( allocPerPart < res.maxPP() ) {
                    // Check total number
                    const long allocTOT = getTotalAllocatedPerComputer(resID, computer);
                    if( allocTOT < res.maxTot() ) {
                        available = true;
                    }
                    else { // tot not avail.
                        putNoResourceMess( mess, resID, computer, true);
                        putHandleMess( mess, computer, binary, client, partition, pid, 0,"","");// requestor info
                        putResStatMess( mess, res, allocTOT, true);
                        //mess << eMess[0] << resID << eMess[11] << computer
                        //     << eMess[1] << eMess[3] << eMess[4];
//...
                    }
                }
                else {  //pp no avail.
                  putNoResourceMess( mess, resID, computer, false);
                  putHandleMess( mess, computer, binary, client, partition, pid, 0,"","");
                  putResStatMess( mess, res, allocPerPart, false);
                 // mess << eMess[0] << resID << eMess[11] << computer
                 //      << eMess[1] << eMess[2] << eMess[4];
                  //owner of some allocated resource info
                  
                }
            } else if(resType == ResourceType::SWR) {
                // Check number of copies per partition
                const long allocPerPart = getAllocatedPP(resID, partition);
                if( allocPerPart < res.maxPP() ) {
                    // Check total number
                    const long allocTOT = getTotalAllocated(resID);
                    if( allocTOT < res.maxTot() ) {
                        available = true;
                    }
                    else { // tot not avail.
                        putNoResourceMess( mess, resID, "", true);
                        putHandleMess( mess, computer, binary, client, partition, pid, 0,"","");
                        putResStatMess( mess, res, allocTOT, true);
                   }
                }
                else {  //pp no avail.
                  putNoResourceMess( mess, resID, "",  false);
                  putHandleMess( mess, computer, binary, client, partition, pid, 0,"","");
                  putResStatMess( mess, res, allocPerPart, false);
                }
            }
            if( !available) {
              mess << eMess[13] << std::endl;
              //get info about holders of allocated resources and put to message for exception
              if(resType == ResourceType::HWR) {
                  std::vector<RM_AllocatedResource> hInf = computerResourceInfo( computer,resID );
                  putHandleMess( mess, hInf );
              }
              else {
                std::vector<RM_AllocatedResource> hInf1 = resourceInfo( resID );
                putHandleMess( mess, hInf1 );
              }
              throw daq::rmgr::UnavailableResourcesE( mess.str().c_str() );
            }
            return available;
}

 bool  daq::rmgr::RM_Control::isHWClsResourceAvailable(const RM_Resource& res, const std::string& partition, const std::string& hwcls,  const std::string& hwid,
 const std::string& computer,
       const std::string& binary, const std::string& client, unsigned long pid
)  {
     bool available = false;
      const std::string& resID = res.uid();
    std::ostringstream mess; //for exception
///    if(resType == daq::rmgr::RESOURCE_TYPE::HW
///               && res.hwCls() == hwcls) {
                const long allocPerPart = getHWObjAllocatedPP( resID, partition, hwcls, hwid);
                if( allocPerPart < res.maxPP() ) {
                    const long allocTOT = getHWObjAllocated( resID, hwcls, hwid);
                    if( allocTOT < res.maxTot() ) {
                        available = true;
                    }
                    else {  // tot not avail.
                        putNoResourceMess( mess, resID, "",  false);
                        putHandleMess( mess, computer, binary, client, partition, pid, 0,hwid, hwcls);
                        putResStatMess( mess, res, allocTOT, true);
                    }
                }
                else {  ////pp no avail
                    putNoResourceMess( mess, resID, "",  false );
                    putHandleMess( mess, computer, binary, client, partition, pid, 0,hwid,hwcls);
                    putResStatMess( mess, res, allocPerPart, false);        
                }
                //AI 211014
                if( !available) {
                    mess << eMess[13] << std::endl;
                    std::vector<RM_AllocatedResource> hInf = hwObjResourceInfo( resID, hwcls, hwid );
                    putHandleMess( mess, hInf );
                    throw daq::rmgr::UnavailableResourcesE( mess.str().c_str() ); 
                }
///    }
    return available;
}

void daq::rmgr::RM_Impl::freeResource(long handle) {
////            boost::lock_guard<boost::upgrade_mutex> lk(m_write_mutex);
           _rm_ctrl.removeResource(handle);
}

void daq::rmgr::RM_Impl::freeResource(const std::string& partition, const std::string& computer, long pid) {
////            boost::lock_guard<boost::upgrade_mutex> lk(m_write_mutex);
            _rm_ctrl.removeResource(partition, computer, pid);
}

//freeAllResources or create freePartitionResources, freeComputerResources, freeResource (by ID)
//void freeAllResources(const std::string myhost, const std::string myname, const std::string p = "*",const std::string swobjid = "*", const std::string compname = "*") {
//TODO  take into account "*". Real use cases? all in partition, all for computer, all resource
//}

std::vector<daq::rmgr::RM_AllocatedResource> daq::rmgr::RM_Impl::allResources() const {
////            boost::shared_lock<boost::upgrade_mutex> lk(m_write_mutex);
            return _rm_ctrl.allResources();
}

bool daq::rmgr::RM_Impl::hasResources(const std::string swobjId) {
    const auto& it = _swobj_map.find(swobjId);
    if( it != _swobj_map.end() )
         return true;
    else
         return false;
}
  
long daq::rmgr::RM_Impl::allocateResources(const std::string& partition,
                              const std::string& swObject,
                              const std::string& computer,
                              unsigned long pid,
                              const std::string& client,
                              uint64_t callstamp,
                              const std::string& hwClass,
                              const std::string& hwId )
        {
//TODO keep info maxPP, maxTot, allocPP, allocTot for exception info in case of resource is not available
            long toReturn = 0;
            bool exactHW = hwId.length() > 1;
//            const std::string hardwrID = exactHW ? hwId : computer; 
            const auto& it = _swobj_map.find(swObject);
            if(it == _swobj_map.end())
                return 0; //no resources
            std::vector<RM_Resource>& rlst = it->second;
            bool isAvail = true;
            bool onlyNotHWcompCls = true; //!exactHW;
            //check if all resources are available
            //TODO locks
            for( unsigned int i=0; i<rlst.size(); i++) {
                //take into account only exact HW class if hwClass has real value (>0).
                //AI 20.10.14
////                if(exactHW && rlst[i].hwCls() != hwClass)
////                      continue;
                if(exactHW) {
                   if( rlst[i].hwCls() != hwClass )
                      continue;
                }
                else {
                   if( rlst[i].hwCls().length() > 0 &&  rlst[i].hwCls() != compClass )
                      continue;
                }

                RM_Resource& r = rlst[i];
                if( !isNotComputerHWcls( r ) )
                    onlyNotHWcompCls = false;

                isAvail = !exactHW ? _rm_ctrl.isResourceAvailable( r, partition, computer,swObject, client, pid) : _rm_ctrl.isHWClsResourceAvailable( r, partition, hwClass, hwId,   computer, swObject, client, pid);
                if(!isAvail ) {
                    //return for any case. Exception thrown in isResourceAvailable if !isAvail
                    return -1;
                 }
            }
            
            if(!exactHW && onlyNotHWcompCls) //only not Computer HW class resources in SW
                return toReturn; // return 0

            toReturn = nextHandle();
            for(  unsigned int i1=0; i1<rlst.size(); i1++) {
                //AI 20.10.14
                if(exactHW) {
                   if( rlst[i1].hwCls() != hwClass )
                      continue;
                }
                else {
                   if( rlst[i1].hwCls().length() > 0 &&  rlst[i1].hwCls() != compClass )
                      continue;
                }
                RM_Resource& r1 = rlst[i1];
                // Here we can lock on a resource-basis
                // Then checks on different resources may go in parallel while
                // checks for the same resource are serialized
                ///auto& res_mutex = *(m_mutex_map[resource->uid()]);
                ///boost::lock_guard<boost::mutex> lk_1(res_mutex);

                // Always acquire the shared lock on the resource list
                ///boost::upgrade_lock<boost::upgrade_mutex> lk_up(m_write_mutex);

                const RM_AllocatedResource allocRes( toReturn, r1.uid(), swObject, computer,
                                                     partition, pid, client,
                                                     callstamp, hwClass, 
                                                     /*hardwrID*/ hwId);
                    // Unique lock to protect the update of the resources
////                   boost::upgrade_to_unique_lock<boost::upgrade_mutex> lk_utu(lk_up);
                _rm_ctrl.addResource(allocRes);
              ////  } else {
              ////      toReturn = -1;
              ////  }
            }

            return toReturn;
}

std::string generateVirtSWobjId(const std::string& computer, unsigned long pid) {
    std::string retv = computer + "@";
    std::ostringstream txt;
    txt << pid;
    retv += txt.str();
    return retv;
}

long daq::rmgr::RM_Impl::allocateResources4Process( const std::string& partition, 
                                                  const daq::rmgr::Res_ID_list& res_lst,
                                                  const std::string& computer, unsigned long pid, 
                                                  const std::string& client,
                                                  uint64_t callstamp ) {
      if( res_lst.length() < 1 )
           throw daq::rmgr::ResourceNotFound( ERS_HERE );
      bool isAvail = true;
////      bool onlyNotHWcompCls = true; 
      if( res_lst.length() > 1) {
          //several resources: check if all available first
          for(unsigned int j = 0; j<res_lst.length(); j++) {
              std::string res_j( res_lst[j].id) ;
              const auto& it = _conf_res_lst.find(res_j);
              if(it == _conf_res_lst.end())
                    throw daq::rmgr::ResourceNotFound( ERS_HERE );
              RM_Resource& r = it->second;
////              if( !isNotComputerHWcls( r ) )
////                    onlyNotHWcompCls = false;
 
              isAvail =  _rm_ctrl.isResourceAvailable( r, partition, computer, "", client, pid);
              if( ! isAvail ) {
//                    throw daq::rmgr::UnavailableResourcesE( messTot.c_str() , messPP.c_str() );
                   return -1;                    
              } 
          }
      }
      long retv = 0;
////      if(onlyNotHWcompCls)  //only not Computer HW class resources in list
////          return retv;

      std::string swVirtId = generateVirtSWobjId(computer, pid);
      if(res_lst.length() > 1)
         retv =  nextHandle();

      for(unsigned int i = 0; i<res_lst.length(); i++) {
          std::string res_i( res_lst[i].id );
          const auto& rit = _conf_res_lst.find(res_i);
          if(rit == _conf_res_lst.end())
                 throw daq::rmgr::ResourceNotFound( ERS_HERE );
          RM_Resource& rr = rit->second;

          if(res_lst.length() == 1) {
//              isAvail =  _rm_ctrl->isHWClsResourceAvailable( rr, partition, hwcls, hwid);
              //isAvail =  _rm_ctrl.isResourceAvailable( rr, partition, computer);
              isAvail =  _rm_ctrl.isResourceAvailable( rr, partition, computer, swVirtId, client, pid); 
              if( !isAvail) {
//                    throw daq::rmgr::UnavailableResourcesE( messTot.c_str() , messPP.c_str() );
                 return -1;
              }
              retv =  nextHandle();
          }
          //now allocate
          const RM_AllocatedResource allocRes( retv, rr.uid(),
                                          swVirtId,
                                          computer, partition, pid, client, callstamp, "", ""); // hwcls, hwid);
         _rm_ctrl.addResource(allocRes);
      }
     return retv;

}

long daq::rmgr::RM_Impl::allocateHWResource4Process( const std::string& partition,
                                                  const std::string& resource,
                                                  const std::string& computer, unsigned long pid,
                                                  const std::string& client, 
                                            ////      const std::string& hwcls, 
                                                  const std::string& hwid,
                                                  uint64_t callstamp ) {
     const auto& it = _conf_res_lst.find(resource);
     if(it == _conf_res_lst.end())
          throw daq::rmgr::ResourceNotFound( ERS_HERE ); //no such resource; some info to be added?
     RM_Resource& r = it->second;
     std::string hwcls(r. hwCls());
     bool isAvail = _rm_ctrl.isHWClsResourceAvailable( r, partition, hwcls, hwid,
   computer, "", client, pid ) ;
     if(!isAvail) {
//                    throw daq::rmgr::UnavailableResourcesE( messTot.c_str() , messPP.c_str() );
         return -1;
     }
     long retv = nextHandle();
//     std::string swVirtId = generateVirtSWobjId(computer, pid);
     const RM_AllocatedResource allocRes( retv, r.uid(), 
                                          // swVirtId,
                                          generateVirtSWobjId(computer, pid), 
                                          computer, partition, pid, client, callstamp, hwcls, hwid);     
     _rm_ctrl.addResource(allocRes);
     return retv;
}

//void setMemStr( CORBA::String_member& toVal, const std::string frVal ) {
//     toVal = CORBA::string_dup( frVal.c_str() );
//}

daq::rmgr::Handle_Info* daq::rmgr::RM_Impl::getHandleInfo(long handle) {
    daq::rmgr::Handle_Info* info = new daq::rmgr::Handle_Info();
    
    std::vector<RM_AllocatedResource> resLst = _rm_ctrl.handleResources( handle );
    const std::string empty("");
    //check if empty
    if(resLst.size() == 0) {
          //throw exception ?
       info->handle = 0;
       //setMemStr(info->appName , empty);
       setMemStr(info->swObj, empty);
        setMemStr(info->computer, empty);
       setMemStr(info->partition, empty);
       setMemStr(info->client, empty);
       info->pid = 0;
       info->resDetails.length(0);
       return info;
    }
    int ii = 0;
   // const std::string amp("@");
    //const auto& i = resLst.begin();
    std::vector<RM_AllocatedResource>::iterator i = resLst.begin();
    for( ; i != resLst.end(); ++i) {
//    for( const std::vector<RM_AllocatedResource>::iterator i = resLst.begin(); i != resLst.end(); ++i) {
       if(i == resLst.begin()) {
          //std::string app ( (*i).getSwObj() + amp + (*i).getComputer() );
          info->handle = (*i).getHandle();
         // setMemStr(info->appName, app);
          setMemStr(info->swObj,(*i).getSwObj());
          setMemStr(info->computer,(*i).getComputer());
          setMemStr(info->partition, (*i).getPartition());
          setMemStr(info->client, (*i).getClient());
          info->pid =  i->getPid();
          info->resDetails.length( resLst.size() );
       }
       const RM_Resource r = getResource( (*i).getID() );
       setMemStr(info->resDetails[ii].resource, r.uid());
       info->resDetails[ii].type = r.type();
       info->resDetails[ii].maxPP = r.maxPP();
       info->resDetails[ii].maxTotal = r.maxTot();
       info->resDetails[ii].allocPP = _rm_ctrl.allocPP( r, *i );
       info->resDetails[ii].allocTotal = _rm_ctrl.allocTot( r, *i );
   }
   return info;
}

daq::rmgr::Rmstr_lst*  daq::rmgr::RM_Impl::getPartitionsList() {
  daq::rmgr::Rmstr_lst* list = new daq::rmgr::Rmstr_lst();
  std::vector<std::string> lst =  _rm_ctrl.getPartitions();
  list->length( lst.size() );
int j = 0;
  for( std::vector<std::string>::iterator i=lst.begin(); i< lst.end(); i++)
     setMemStr( (*list)[j++].val, *i);
  return list;
}

daq::rmgr::Rmstr_lst*  daq::rmgr::RM_Impl::getResourceOwners(const char* rname, const char* pname) {
   daq::rmgr::Rmstr_lst* list = new daq::rmgr::Rmstr_lst();
   std::string p(pname);
   std::string r(rname);
   std::vector<std::string> lst =  _rm_ctrl.getReosurceOwners( r, p );
   list->length( lst.size() );
   int j = 0;
   for( std::vector<std::string>::iterator i=lst.begin(); i< lst.end(); i++)
      setMemStr( (*list)[j++].val, *i);
  return list;
}

daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getResourceInfo(const char* p, const char* rn) {
   daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
   std::string pn(p);
   std::string r(rn);
   std::vector<RM_AllocatedResource> lst = _rm_ctrl.resourceInfo( pn, r );
   info->length( lst.size() );
   std::vector<RM_AllocatedResource>::iterator i = lst.begin();
   int j(0);
    for( ; i != lst.end(); ++i) {
          const RM_Resource r = getResource( (*i).getID() );
          (*info)[j].handle = (*i).getHandle();
          setMemStr((*info)[j].resource, (*i).getID());
          (*info)[j].type = r.type();
          (*info)[j].maxPP = r.maxPP();
          (*info)[j].maxTotal = r.maxTot();
          setMemStr((*info)[j].hwClass, r.hwCls());
          setMemStr((*info)[j].swObj, (*i).getSwObj());
          setMemStr((*info)[j].partition, (*i).getPartition());
          setMemStr((*info)[j].client, (*i).getClient());
          setMemStr((*info)[j].computer, (*i).getComputer());
          (*info)[j].pid = (*i).getPid();
          setMemStr((*info)[j].hwId, (*i).getHwObj());
          (*info)[j].allocPP = _rm_ctrl.allocPP( r, *i );
          (*info)[j].allocTotal = _rm_ctrl.allocTot( r, *i );
       j++;
    }
    return info;
}

daq::rmgr::Allocated_Info* daq::rmgr::RM_Impl::createAllocatedInfo(std::vector<RM_AllocatedResource>& lst, daq::rmgr::Allocated_Info* info) {
   info->length( lst.size() );
   std::vector<RM_AllocatedResource>::iterator i = lst.begin();
   int j(0);
    for( ; i != lst.end(); ++i) {
          const RM_Resource r = getResource( (*i).getID() );

          (*info)[j].handle = (*i).getHandle();
          setMemStr((*info)[j].resource, (*i).getID());
          (*info)[j].type = r.type();
          (*info)[j].maxPP = r.maxPP();
          (*info)[j].maxTotal = r.maxTot();
          setMemStr((*info)[j].hwClass, r.hwCls());
          setMemStr((*info)[j].swObj, (*i).getSwObj());
          setMemStr((*info)[j].partition, (*i).getPartition());
          setMemStr((*info)[j].client, (*i).getClient());
          setMemStr((*info)[j].computer, (*i).getComputer());
          (*info)[j].pid = (*i).getPid();
          setMemStr((*info)[j].hwId, (*i).getHwObj());
          (*info)[j].allocPP = _rm_ctrl.allocPP( r, *i );
          (*info)[j].allocTotal = _rm_ctrl.allocTot( r, *i );
       j++;
    }
    return info;
}

daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getResourceInfo( const std::string& res) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string r(res);
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.resourceInfo( r );
    return createAllocatedInfo( lst, info );
}
daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getPartitionInfo( const std::string& p ) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string partit( p );
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.partitionInfo( partit );


    return createAllocatedInfo( lst, info );
}
daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getComputerInfo( const std::string& comp ) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string c( comp );
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.computerInfo( c );
    return createAllocatedInfo( lst, info );
}
daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getComputerResourceInfo( const std::string& comp,  const std::string& partit, const std::string& res) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string p( partit );
    std::string c( comp );
    std::string r( res );
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.computerResourceInfo( c, p, r);
    return createAllocatedInfo( lst, info );
}
//oct 2015
daq::rmgr::Allocated_Info*  daq::rmgr::RM_Impl::getApplicationInfo( const std::string& partit, const std::string& swObj, const std::string& comp) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string p( partit );
    std::string c( comp );
    std::string bin( swObj);
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.applicationInfo( p, bin, c );
    return createAllocatedInfo( lst, info );
}
daq::rmgr::Allocated_Info* daq::rmgr::RM_Impl::getPartitionCompProcInfo(const std::string& partition, const std::string& computer, long pid) {
    daq::rmgr::Allocated_Info* info = new daq::rmgr::Allocated_Info();
    std::string p( partition );
    std::string c( computer );
    std::vector<RM_AllocatedResource> lst = _rm_ctrl.partitionCompProcInfo(p,c,pid);
    return createAllocatedInfo( lst, info );
}


long daq::rmgr::RM_Control::allocTot( const RM_Resource& r, const RM_AllocatedResource& allocR ) {
    switch (r.type()) {
       case daq::rmgr::SWR :
           return getTotalAllocated( r.uid() );
       case daq::rmgr::HWR :
          if( allocR.getHwClass().length() == 0 )
           return getTotalAllocatedPerComputer( r.uid(), allocR.getComputer() );
          return getHWObjAllocated( r.uid(), allocR.getHwClass(), allocR.getHwObj() );
    }
    return -1;
} 

long daq::rmgr::RM_Control::allocPP( const RM_Resource& r, const RM_AllocatedResource& allocR ) {
    switch (r.type()) {
       case daq::rmgr::SWR :
           return getAllocatedPP( r.uid(), allocR.getPartition() );
       case daq::rmgr::HWR :
          if( allocR.getHwClass().length() == 0 )
           return getComputerAllocatedPP( r.uid(), allocR.getPartition(), allocR.getComputer() );
          return getHWObjAllocatedPP( r.uid(), allocR.getPartition(), allocR.getHwClass(), allocR.getHwObj() );
    }
    return -1;
}
//Access Manager related
void daq::rmgr::RM_Impl::setUseAM() {
 char * envAM = getenv( "TDAQ_AM_AUTHORIZATION" );
//test
 if(!(envAM == 0) ) {
     std::string tmp(envAM);
     _use_am_auth = tmp == "on";
 }
 else
     _use_am_auth = 0;
std::cerr<<"daq::rmgr::RM_Impl::setUseAM() _use_am_auth="<<_use_am_auth<<std::endl;
}

bool daq::rmgr::RM_Impl::accessPermitied( const char* myHost, const char* myName, const char* pName ) {
  std::string      mhost(myHost);
  std::string      mname(myName);
  std::string      p(pName);
  bool allowed(false);
  if( isUseAM() ) {
    try {
//check permission
       daq::am::ServerInterrogator si;
       const daq::am::RMResource *rmRes  =
            daq::am::RMResource::getInstanceForActionANY(p);
       daq::am::RequestorInfo pAccessSubject( mname, mhost );
       allowed = si.isAuthorizationGranted(*rmRes, pAccessSubject);
       delete rmRes;
    }
    catch(am::Exception &ex) {
       ers::error(ex);
       allowed = false;
    }
  }
  else
    allowed = true;

  if(!allowed) {
       std::ostringstream txt1 ;
       txt1<<"Access Manager does not allow to free resources using host " << mhost <<  " for user " << mname;
       ers::error( ers::Message( ERS_HERE, txt1.str()));
       throw  daq::rmgr::AcceessDeniedE( pName, mhost.c_str(), mname.c_str() );
  }
  return allowed;
}

//backup manager related
bool daq::rmgr::RM_Impl::isDoBackup() {
   return _bck && _bck->isDoBackup();
}
const std::string daq::rmgr::RM_Impl::getBckPath() const {
   if(_bck)
     return _bck->getBckPath();
   return (const std::string)"";
}
bool daq::rmgr::RM_Impl::isRestoreBackup() {
   return _bck && _bck->isRestoreBackup();
}

std::ostream& daq::rmgr::operator<<(std::ostream& str, daq::rmgr::RM_AllocatedResource& alloc) {
  str << "RM_AllocatedResourc= { handle="<<alloc._handle<<"  res="<<alloc._id<<"  swObj="<<alloc._swObj<<"  computer="<<alloc._computer<<"  partition="<<alloc._partition<<"  pid="<<alloc._pid<<"  client="<<alloc._client<<"  hwClass="<<alloc._hwClass<<"  hwObj="<<alloc._hwObj <<" }";
 return str;  
}

//tests method
  bool daq::rmgr::RM_Statistics::setVal( timeval& t2, timeval& t1) {
     long i1,  i2;
     i2 = t2.tv_sec  * 100000 + t2.tv_usec;
     i1 = t1.tv_sec  * 100000 + t1.tv_usec;
     i2 -= i1;
     int cur;
     cur  = ++_curIndex;
     if(cur >= maxInd)
       return false;
     t0[cur] = i1;
     delta[cur] = i2;
     if( outStrm.is_open() )
       outStrm << i2 << " " << i1 << std::endl; 
     return true;
  }
  long daq::rmgr::RM_Statistics::getReqAverage() {
    std::cerr<<" _curIndex=" << _curIndex<< std::endl;
    if( _curIndex <= 0)
       return 0;
    long av = 0;
    for( int i=0; i<_curIndex; i++)
       av += delta[i];
    //double res =  av;
    long res = av / _curIndex;
     std::cerr<<" sum=" << av << " average=" << res <<std::endl;
    return res; 
  }
  bool daq::rmgr::RM_Statistics::openFile() {
       std::cerr<< " RM_Statistics::openFile in use. Open " << path << std::endl;
       return createOutStream(path.c_str());
  }
  bool daq::rmgr::RM_Statistics::createOutStream(const char* log_file) {
//create dir if does not exist
    try {
      boost::filesystem::path myparent( boost::filesystem::path(path).branch_path() );
      if ( !boost::filesystem::exists( myparent ) )  {
            boost::filesystem::create_directories( myparent );
      }
      const std::string add1 = "1_";
///      if( boost::filesystem::exists( path ) )
        rename( path.c_str(), (add1 + path).c_str() );
//      else { //check write permission
//        std::ofstream strm;
//        logostrm.open ( paht.c_str(), std::ios::app );
//        strm.close();
//      }
    }
    catch (const std::ofstream::failure& ex1) {
        ers::fatal( ers::Message( ERS_HERE, "Can not create reqtest.txt:", ex1 ) );
    }
    catch( std::exception & ex ) {
        ers::fatal( ers::Message( ERS_HERE, "Can not create reqtest.txt:", ex ) );
    }

    try{
      if( outStrm.is_open() )
         outStrm.close();
    }
    catch (const std::ofstream::failure& ex0) {
    }
    try {
      outStrm.open ( log_file, std::ios::app );
    }
    catch (const std::ofstream::failure& ex1) {
      return false;
    }
    return true;
  }
  void daq::rmgr::RM_Statistics::closeStream() {
    try{
      if( outStrm.is_open() )
         outStrm.close();
    }
    catch (const std::ofstream::failure& ex0) {
    }
  }

