#ifndef RMimpl_H_INCLUDED
#define RMimpl_H_INCLUDED

#define TDAQ_RM_CONFIG "TDAQ_RM_CONFIG"

#include <iostream>
#include <rmgr/rmgr.hh>
#include "ResourceManager/exceptions.h"
//for tests
#include <time.h>
#include <locale.h>
//2017
#include "config/Configuration.h"
#include "config/ConfigObject.h"
#include <dal/SW_Object.h>
#include <dal/RM_Resource.h>

#include <boost/config.hpp>
#include <algorithm>

#include <boost/multi_index_container.hpp>
#include <boost/multi_index/hashed_index.hpp>
#include <boost/multi_index/ordered_index.hpp>
#include <boost/multi_index/tag.hpp>
#include <boost/multi_index/composite_key.hpp>
#include <boost/multi_index/member.hpp>
#include <boost/thread.hpp>
#include <boost/thread/mutex.hpp>
//#include <boost/multi_index_container.hpp>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/convenience.hpp>

#include <fstream>
#include <iostream>  //
#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
//#include <boost/serialization/version.hpp>
#include <boost/serialization/list.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/map.hpp>
#include <boost/serialization/string.hpp>
#include <boost/serialization/split_member.hpp>

#include <string>
#include <map>
#include <vector>
#include <memory>
#include <utility>
#include <functional>
#include <chrono>
#include <atomic>

namespace mi = boost::multi_index;

namespace daq
{
namespace rmgr
{
const std::string compClass = "Computer";
/**
 * Config DB resources
 *
 * **/
class RM_Resource {
 public:
RM_Resource()
 : _id(""), _type(SWR), _maxTot(0), _maxPP(0)
 {}

    RM_Resource(const std::string& id, long maxTot,long maxPP, daq::rmgr::ResourceType type)
            : _id(id), _type(type), _maxTot(maxTot), _maxPP(maxPP)
        {
        }
   RM_Resource(const std::string& id, long maxTot,long maxPP, const std::string& hwCls, daq::rmgr::ResourceType type)
            : _id(id), _type(type), _maxTot(maxTot), _maxPP(maxPP), _hwcls(hwCls)
        {
        }
  
     daq::rmgr::ResourceType type() const {
      return _type;
    }

    const std::string& uid() const {
      return _id;
    }

    long maxPP() const {
      return _maxPP;
    }

    long maxTot() const {
      return _maxTot;
    }

    const std::string& hwCls() const {
      return _hwcls;
    }

    //below - should be during mute
    void update(long maxTot, long maxPP ) {
      _maxTot = maxTot;
      _maxPP = maxPP;
    }

 private:
  friend class boost::serialization::access;
  template<class Archive>
  void save(Archive & ar, const unsigned int )  const {
//    if( version ==0) { //just remove warnings
      ar & _id;
      ar & _maxTot;
      ar & _maxPP;
      unsigned int tp; //temporary solution
      tp = (unsigned int) _type;
      ar & tp;
      ar & _hwcls;
//    }
  }
  template<class Archive>
  void load(Archive & ar, const unsigned int ) {
//   if( version ==0 ) { //just remove warnings
     ar & _id;
     ar & _maxTot;
     ar & _maxPP;
     unsigned int tp; //temporary solution
     ar & tp;
     _type = (ResourceType) tp;
     ar & _hwcls;
//   }
  }
  BOOST_SERIALIZATION_SPLIT_MEMBER()

  std::string _id;
  ResourceType _type;
  long _maxTot;
  long _maxPP; 
  //for HW but not computer
  std::string _hwcls;
};
//BOOST_CLASS_VERSION(RM_Resource, 1)

/** 
 * This class represent an allocated resource data
 * **/
class RM_AllocatedResource {
friend class RM_Control;
  public:
 RM_AllocatedResource()
 : _handle(0), _id(""), _swObj(""), _computer(""),
        _partition(""), _pid(0), _client(""),
        _callstamp(0),
        _hwClass(""),
        _hwObj("")
 {}

    RM_AllocatedResource( long handle, std::string id,  std::string swObj,
         std::string computer, std::string partition,  unsigned  long pid,
         std::string client, uint64_t callstamp, 
         std::string hwClass ="",  std::string hwObj="")
      : _handle(handle), _id(id), _swObj(swObj), _computer(computer),
        _partition(partition), _pid(pid), _client(client), 
        _callstamp(callstamp),
        _hwClass(hwClass),
        _hwObj(hwObj) {
      }
    long getHandle() const {
      return _handle;
    }
    const std::string& getID() const {
      return _id;
    }
    const std::string& getSwObj() const {
       return _swObj;
    }
    const std::string& getComputer() const {
       return _computer;
    }
    const std::string& getPartition() const {
       return _partition;
    }
    unsigned  long getPid() {
       return _pid;
    }
    const std::string& getClient() const {
       return _client;
    }
    const std::string& getHwClass() const {
       return _hwClass;
    }
    const std::string& getHwObj() const {
       return _hwObj;
    }

  private:

    friend class boost::serialization::access;
    template<class Archive>
    void serialize(Archive & ar, const unsigned int) {
//     if( version ==0) { //just remove warnings
      ar & _handle;
      ar & _id;
      ar & _swObj;
      ar & _computer;
      ar & _partition;
      ar & _pid;
      ar & _client;
      ar & _callstamp;
      ar & _hwClass;
      ar & _hwObj; 
//     }
    }

    friend std::ostream& operator<<(std::ostream& str, RM_AllocatedResource& alloc);     
    static ::std::atomic_long  _lastHandle;
    long nextHandle() {
      ++_lastHandle;
      return _lastHandle;
    }
    long _handle;
    std::string _id;
    std::string _swObj;
    std::string _computer;
    std::string _partition;
    unsigned  long _pid;
    std::string _client;
    uint64_t _callstamp;
    std::string _hwClass;
    std::string _hwObj;
    // ::CORBA::ULongLong
};
//BOOST_CLASS_VERSION(RM_AllocatedResource, 1)

/**
 * Support all real RM functionality like allocate/free resources, get 
 * different kinds of info. Multy-map with all data about allocated resources is inside.
 * **/ 
class RM_Control {
   public:
        RM_Control() {
           /// _rmap.get<handle_key>().max_load_factor(0.9f);
        }

        void addResource(const RM_AllocatedResource& res) {
            _rmap.insert(res);
        }
        void removeResource(long handle) {
            auto& multi_map = _rmap.get<handle_key>();
            const auto& pair = multi_map.equal_range(handle);
            multi_map.erase(pair.first, pair.second);

        }
        void removeResource(const std::string& partition, const std::string& computer, long pid) {
            auto& multi_map = _rmap.get<part_comp_pid_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(partition, computer, pid));
            multi_map.erase(pair.first, pair.second);
       }
       void removePartition(const std::string& partition) {
            auto& multi_map = _rmap.get<partit_key>();
            const auto& pair = multi_map.equal_range(partition);
            multi_map.erase(pair.first, pair.second);
        }       
        void removePartitionResource(const std::string& resource, const std::string& partition) {
            auto& multi_map = _rmap.get<uid_part_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(resource, partition));
            multi_map.erase(pair.first, pair.second);
       }
       void removeComputerAll(const std::string& computer) {
            auto& multi_map = _rmap.get<comp_key>();
            const auto& pair = multi_map.equal_range(computer);
            multi_map.erase(pair.first, pair.second);
       }
       void removeResource(const std::string& resource) {
            auto& multi_map = _rmap.get<uid_key>();
            const auto& pair = multi_map.equal_range(resource);
            multi_map.erase(pair.first, pair.second);
       }
       void removeComputerResource(const std::string& resource, const std::string& computer, const std::string& partition) {
            auto& multi_map = _rmap.get<uid_part_comp_key>(); //uid_comp_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(resource, computer, partition));
            multi_map.erase(pair.first, pair.second);
       }
       void removeApplication(const std::string& partition, const std::string& swObj, const std::string& computer) {
            auto& multi_map = _rmap.get<part_comp_sw_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(partition, computer, swObj));
            multi_map.erase(pair.first, pair.second);
       }
//void removeResourceHWObj(const std::string& class, const std::string& hwid)


        long getHandle( const uint64_t callstamp ) {
            const auto& multi_map = _rmap.get<callstamp_key>();
            const auto& pair = multi_map.equal_range( callstamp ) ; //boost::make_tuple(callstamp));
            long retv = 0;
            if ( pair.first != multi_map.end())
                   retv = (pair.first)->_handle;
//            }
            return retv;
        }
 
        //use in check if available
        // sw type
        long getTotalAllocated(const std::string& resourceID) const {
            const auto& multi_map = _rmap.get<uid_key>();
            return multi_map.count(resourceID);
        }
        long getAllocatedPP(const std::string& resourceID,  const std::string& partition) const {
            const auto& multi_map = _rmap.get<uid_part_key>();
            return multi_map.count(boost::make_tuple(resourceID, partition));
        }
        //hw (Computer class)
        long getTotalAllocatedPerComputer(const std::string& resourceID, const std::string& computer) const {
            const auto& multi_map = _rmap.get<uid_comp_key>();
            return multi_map.count(boost::make_tuple(resourceID, computer));
        }
        long getComputerAllocatedPP(const std::string& resourceID, const std::string& partition, const std::string& computer) const {
            const auto& multi_map = _rmap.get<uid_part_comp_key>();
            return multi_map.count(boost::make_tuple(resourceID, partition, computer));
        }
        //hw
 long getHWObjAllocatedPP(const std::string& resourceID, const std::string& partition, const std::string& hwClass, const std::string& hwid) const {

            const auto& multi_map = _rmap.get<part_uid_hwcls_hwid_key>();
            return multi_map.count(boost::make_tuple( partition, resourceID, hwClass,hwid ));
        }
 long getHWObjAllocated(const std::string& resourceID, const std::string& hwClass, const std::string& hwid) const {
            const auto& multi_map = _rmap.get<uid_hwcls_hwid_key>();
            return multi_map.count(boost::make_tuple(resourceID, hwClass, hwid ));
        }
long allocTot( const RM_Resource& r, const RM_AllocatedResource& allocR );
long allocPP( const RM_Resource& r, const RM_AllocatedResource& allocR );

        bool isResourceAvailable(const RM_Resource& res, const std::string& partition, const std::string& computer,
const std::string& binary, const std::string& client, unsigned long pid //for exception info
); 
        bool isHWClsResourceAvailable(const RM_Resource& res, const std::string& partition, const std::string& hwcls,  const std::string& hwid,
 const std::string& computer, 
const std::string& binary, const std::string& client, unsigned long pid //for exception info
);

        // Example on how to get items from the multimap
        // Be careful to the order of arguments

        std::vector<RM_AllocatedResource> allResources() const {
            return std::vector<RM_AllocatedResource>(_rmap.begin(), _rmap.end());
        }

        std::vector<RM_AllocatedResource> handleResources ( const long handle) const {
            const auto& multi_map = _rmap.get<handle_key>();
            const auto& pair = multi_map.equal_range( handle );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
        }  

        std::vector<std::string> getPartitions() {
             std::map<std::string, bool> p_lst;
             for ( const auto& all: _rmap )
               p_lst.insert( std::pair<std::string,bool>( all.getPartition(),true));
             std::vector<std::string> retv;
             std::map<std::string, bool>::iterator it3 = p_lst.begin();
             for( ; it3!=p_lst.end(); ++it3) {
                std::string addme( it3->first );
                retv.push_back( addme );
             }
             return retv;
        }

        const std::vector<std::string>  getReosurceOwners(  const std::string res,  const std::string p ) {
            std::map<std::string, bool> o_lst;
            const auto& multi_map = _rmap.get<uid_part_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(res,p));
            const std::vector<RM_AllocatedResource> myList = 
                     std::vector<RM_AllocatedResource>(pair.first, pair.second);
            for( unsigned int j=0; j<myList.size(); j++) {
               o_lst.insert( std::pair<std::string,bool>( myList[j].getClient(), true));
            }

            std::vector<std::string> vec;
            std::map<std::string, bool>::iterator it3 = o_lst.begin();
            for( ; it3!=o_lst.end(); ++it3) {
                std::string addme( it3->first );
                vec.push_back( addme );
            }
            return vec; 
        }

        std::vector<RM_AllocatedResource> resourceInfo( const std::string p, const std::string res) {
            const auto& multi_map = _rmap.get<uid_part_key>();
            const auto& pair = multi_map.equal_range( boost::make_tuple(res,p) );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
        }
        std::vector<RM_AllocatedResource> resourceInfo( const std::string res) {
            const auto& multi_map = _rmap.get<uid_key>();
            const auto& pair = multi_map.equal_range( res );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
       }
       std::vector<RM_AllocatedResource> partitionInfo( const std::string p ) {
            const auto& multi_map = _rmap.get<partit_key>();
            const auto& pair = multi_map.equal_range( p );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
       }
       std::vector<RM_AllocatedResource> computerInfo( const std::string comp ) {
            const auto& multi_map = _rmap.get<comp_key>();
            const auto& pair = multi_map.equal_range( comp );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
       }
       std::vector<RM_AllocatedResource> computerResourceInfo( const std::string comp,  const std::string p, const std::string res ) {
            const auto& multi_map = _rmap.get<uid_part_comp_key>();
            const auto& pair = multi_map.equal_range( boost::make_tuple(res,p,comp) );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
      }
      std::vector<RM_AllocatedResource> computerResourceInfo( const std::string comp, const std::string res ) {
            const auto& multi_map = _rmap.get<uid_comp_key>();
            const auto& pair = multi_map.equal_range( boost::make_tuple(res,comp) );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
      }
      //AI 211014
       std::vector<RM_AllocatedResource> hwObjResourceInfo( const std::string res, const std::string hwClass, const std::string hwObj ) {
            const auto& multi_map = _rmap.get<uid_hwcls_hwid_key>();
            const auto& pair = multi_map.equal_range( boost::make_tuple(res,hwClass,hwObj) );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
       }
      //oct 2015
      std::vector<RM_AllocatedResource> applicationInfo( const std::string p, const std::string swObj, const std::string comp) {
            const auto& multi_map = _rmap.get<part_comp_sw_key>();
            const auto& pair = multi_map.equal_range( boost::make_tuple(p,comp,swObj) );
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
      } 
      std::vector<RM_AllocatedResource> partitionCompProcInfo(const std::string& partition, const std::string& computer, long pid) {
            auto& multi_map = _rmap.get<part_comp_pid_key>();
            const auto& pair = multi_map.equal_range(boost::make_tuple(partition, computer, pid));
            return std::vector<RM_AllocatedResource>(pair.first, pair.second);
      }
 
   private:
      friend class boost::serialization::access;
      template<class Archive>
      void serialize(Archive & ar, const unsigned int) {
 //      if( version ==0)  //just remove warnings
         ar & _rmap;
      }
      void putNoResourceMess( std::ostringstream& mess, const std::string& resource, const std::string& computer, bool isTot ); 
      void putResStatMess( std::ostringstream& mess, const daq::rmgr::RM_Resource& res, const long allocated, bool isTot);   
      void putHandleMess( std::ostringstream& mess,  const std::string& computer, const std::string& binary, const std::string& client,  const std::string& p, unsigned long pid, long hndl, const std::string& hwObj, const std::string& hwCl );
      void putHandleMess(  std::ostringstream& mess,std::vector<RM_AllocatedResource>& resLst);

     //Gueseppe
     // Simple keys
     struct uid_key {};
     struct handle_key {};
//june06
     struct partit_key {};
     struct comp_key {};
     

     struct callstamp_key {};
//_swObj
     // Composite keys
     struct uid_part_key {};
     struct uid_comp_key {};
     struct uid_part_comp_key {};
     struct part_comp_pid_key {};
     struct part_comp_sw_key {};
///     struct swid_hwcls_hwid_key {};  
     struct part_swid_hwcls_hwid_key {};

     struct uid_hwcls_hwid_key {};
     struct part_uid_hwcls_hwid_key {};

typedef ::boost::multi_index::multi_index_container<
                   daq::rmgr::RM_AllocatedResource,
                ::boost::multi_index::indexed_by<
                   ::boost::multi_index::hashed_non_unique<  ::boost::multi_index::tag<handle_key>,::boost::multi_index::member< daq::rmgr::RM_AllocatedResource, const long, &daq::rmgr::RM_AllocatedResource::_handle>>
                                                >
                                                   > RM_Resourcetest;


    typedef ::mi::multi_index_container<
          daq::rmgr::RM_AllocatedResource,
                    mi::indexed_by<
//                        mi::hashed_unique<mi::tag<handle_key>, mi::member< RM_AllocatedResource, const long, &RM_AllocatedResource::_handle>>,
                        mi::hashed_non_unique<mi::tag<handle_key>, mi::member< daq::rmgr::RM_AllocatedResource, const long, &daq::rmgr::RM_AllocatedResource::_handle>>,
                        mi::ordered_non_unique<mi::tag<uid_key>, mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>>,
//AI jun06
                        mi::ordered_non_unique<mi::tag<partit_key>, mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>>,
                        mi::ordered_non_unique<mi::tag<comp_key>, mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_computer>>,

                        mi::ordered_non_unique<mi::tag<callstamp_key>, mi::member<RM_AllocatedResource, const uint64_t /*const long long unsigned int*/, &RM_AllocatedResource::_callstamp>>,

                        mi::ordered_non_unique<mi::tag<uid_part_key>,
                                                        mi::composite_key<RM_AllocatedResource,
                                                            mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>,
                                                            mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>
                                                        >
                        >,
                        mi::ordered_non_unique<mi::tag<uid_comp_key>,
                                                        mi::composite_key<RM_AllocatedResource,
                                                            mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>,
                                                            mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_computer>
                                                        >
                        >,
                        mi::ordered_non_unique<mi::tag<uid_part_comp_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_computer>
                                                                      >
                        >,
                        mi::ordered_non_unique<mi::tag<part_comp_pid_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_computer>,
                                                                          mi::member<RM_AllocatedResource, const unsigned long, &RM_AllocatedResource::_pid>
                                                                      >
                        >,

                        mi::ordered_non_unique<mi::tag<part_comp_sw_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_computer>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_swObj>
                                                                      >
                        >,

                        mi::ordered_non_unique<mi::tag<uid_hwcls_hwid_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwClass>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwObj>
                                                                      >
                        >,
                        mi::ordered_non_unique<mi::tag<part_uid_hwcls_hwid_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_id>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwClass>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwObj>
                                                                      >
                        >
/*
,

                        mi::ordered_non_unique<mi::tag<swid_hwcls_hwid_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_swObj>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwClass>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwObj>
                                                                      >
                        >,
                        mi::ordered_non_unique<mi::tag<part_swid_hwcls_hwid_key>,
                                                                      mi::composite_key<RM_AllocatedResource,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_partition>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_swObj>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwClass>,
                                                                          mi::member<RM_AllocatedResource, const std::string, &RM_AllocatedResource::_hwObj>
                                                                      >
                        >
*/
                     >
        > RM_ResourceContainer;

        RM_ResourceContainer _rmap;
};


class RM_Backup;

/**
 * This is the class where the bulk of the RM functionality is implemented
 *
 * */
class RM_Impl {
public:
   typedef std::vector<RM_Resource> ResList; // Configuration resources (only SW and HW resource types there) key=resID
   typedef std::map<std::string, std::vector<std::string>> SWObjResources; //key=swpbjId, value - list of resource id's
RM_Impl() : _use_am_auth(0),_lastHandle(0) {
  _bck = 0;
}
RM_Impl(RM_Backup* bck) : _use_am_auth(0),_lastHandle(0)
//             , _rm_ctrl( new RM_Control())
 {
        _bck = bck;
        //  handleInit();
 }

///RM_Impl(const ResList& resList, const SWObjResources& swObjList);

~RM_Impl();

RM_Backup* getBck() { return _bck; }

long getHandleByStamp( uint64_t callStamp ) {
         return _rm_ctrl.getHandle(callStamp);
}

void rmConfigurationUpdate( const daq::rmgr::Config_update& conf);
bool isConfigAvailable(); //AI 2018
bool hasResources(const std::string swobjId);

long allocateResources( const std::string& partitionName, 
                        const std::string& softwareUID, 
                        const std::string& computer, 
                        unsigned long pid, 
                        const std::string& client,
                        uint64_t callstamp,
                        const std::string& hwClass = "",
                        const std::string& hwId = "" );
long allocateResources4Process( const std::string& partitionName,
                                const daq::rmgr::Res_ID_list& res_lst, //no  softwareUID
                                const std::string& computer, 
                                unsigned long pid, 
                                const std::string& client,
                                uint64_t callstamp);
long allocateHWResource4Process( const std::string& partition,
                                 const std::string& resource,
                                 const std::string& computer, 
                                 unsigned long pid,
                                 const std::string& client,
                          ////       const std::string& hwcls, 
                                 const std::string& hwid,
                                 uint64_t callstamp );
void freeResource(long handle);
void freeResource(const std::string& partition, const std::string& computer, long pid);
//june06
void freePartition( const char* myHost,  const char* myName, const std::string& partition) {
   if( accessPermitied( myHost, myName, partition.c_str()) )
      _rm_ctrl.removePartition( partition );
}
void freePartitionResource( const std::string& resource, const std::string& partition ) {
    _rm_ctrl.removePartitionResource( resource, partition);
}
void freeComputerAll(const std::string& computer) {
    _rm_ctrl.removeComputerAll( computer );
}
void freeResource( const std::string& resource ) {
   _rm_ctrl.removeComputerAll( resource );
}
void freeComputerResource(const std::string& resource, const std::string& computer, const std::string& partition) {
  _rm_ctrl.removeComputerResource( resource, computer, partition );
}

void freeApplicationResources(  const std::string& p,  const std::string& swObj,  const std::string& computer) {
  _rm_ctrl.removeApplication( p, swObj, computer );
}

  
// different info
daq::rmgr::Handle_Info* getHandleInfo(long handle);
daq::rmgr::Rmstr_lst* getPartitionsList();
daq::rmgr::Rmstr_lst* getResourceOwners(const char* rname, const char* pname);
daq::rmgr::Allocated_Info* getResourceInfo(const char* p, const char* rn);
daq::rmgr::Allocated_Info* getResourceInfo( const std::string& res);
daq::rmgr::Allocated_Info* getPartitionInfo( const std::string& p );
daq::rmgr::Allocated_Info* getComputerInfo( const std::string& comp );
daq::rmgr::Allocated_Info* getComputerResourceInfo( const std::string& comp,  const std::string& p, const std::string& res);
//oct 2015d
daq::rmgr::Allocated_Info* getApplicationInfo( const std::string& partit, const std::string& swObj, const std::string& comp);
daq::rmgr::Allocated_Info* getPartitionCompProcInfo(const std::string& partition, const std::string& computer, long pid);

const RM_Resource& getResource( const std::string id) {
   const auto& iter = _conf_res_lst.find( id );
//   if( iter == _conf_res_lst.end())
//       throw new Exception ( "resource absent") );
   //const RM_Resource retv 
   return iter->second; //retv;
} //throw no resource
void setMemStr( CORBA::String_member& toVal, const std::string& frVal ) {
     toVal = CORBA::string_dup( frVal.c_str() );
}
//2017
daq::rmgr::Config_update* getConfigInfo();
void rmConfigurationUpdate(::Configuration * conf);
void configWarningMess( RM_Resource* oldVal, RM_Resource* newVal);
void configWarningMess( const std::pair< std::string, std::vector<RM_Resource> > oldV, std::string& str, const std::vector<const daq::core::RM_Resource*>& newVal  );
bool listResourcesDiffer( std::vector<RM_Resource> oldV, const std::vector<const daq::core::RM_Resource*>& newVal );
void configWarningMess(  std::string& str );
//backup manager related
bool isDoBackup();
const std::string getBckPath() const ;
bool isRestoreBackup();
void resetBackup(RM_Backup* nBck) { if(nBck) _bck = nBck; }

//    long lastHndl; //for seril. only

private:

std::vector<daq::rmgr::RM_AllocatedResource> allResources() const ;
//long allocateResource(const std::string& partition, const std::string& swObject, const std::string& computer, const std::string& client, long pid);

    void handleInit( long val = 0) { _lastHandle = val; } // 0; }
    long nextHandle() {
      ++_lastHandle;
      return _lastHandle;
    }

//Access Manager in use
 bool _use_am_auth;
 void setUseAM();
 bool isUseAM() { return _use_am_auth; }
 bool accessPermitied( const char* myHost, const char* myName, const char* pName ); // throw daq::rmgr::AcceessDeniedE;
 bool isNotComputerHWcls( RM_Resource& r ) {
    if(r.type() == ResourceType::HWR
       && r.hwCls().length() > 0
       && r.hwCls() != compClass)
          return true;
     return false; 
 }

  friend class boost::serialization::access;
/*
  template<class Archive>
  void serialize(Archive & ar, const unsigned int version) {
   if( version ==0) { //just remove warnings
     //ar & _lastHandle;
     ar & lastHndl;
//test
//    ar & test1;
     ar & _conf_res_lst;
     ar & _swobj_map;
     ar & _rm_ctrl;
   }
  }
*/
  template<class Archive>
  void save(Archive & ar, const unsigned int) const {
//   if( version ==0) { //just remove warnings
    long lastHnd;
    lastHnd = (long) _lastHandle;
    ar & lastHnd;
    ar & _conf_res_lst;
    ar & _swobj_map;
     ar & _rm_ctrl;    
//   }
  }
  template<class Archive>
  void load(Archive & ar, const unsigned int) {
  std::cerr<<" RM_Impl load start"<<std::endl;
//   if (version ==0) { //just remove warnings
     long lastHnd;
     ar & lastHnd;
     handleInit( lastHnd );
     ar & _conf_res_lst;
     ar & _swobj_map;
     ar & _rm_ctrl;
//   }
  }
  BOOST_SERIALIZATION_SPLIT_MEMBER()

    std::atomic_long  _lastHandle; //keep last value of generated handle

    // all ConfDb resources that are passed by the clients
    // in requests
    std::map<std::string, RM_Resource> _conf_res_lst;
    // std::map<std::string, std::shared_ptr<Resource>>  _conf_res_lst;

   // This map establishes a relationship between the SWObj and the associated resources
   //from Config DB (passed in client requests)
   std::map< std::string, std::vector<RM_Resource> > _swobj_map;
   // std::multimap<std::string, std::shared_ptr<Resource>>  _swobj_map;

   //control map with allocated resources
   RM_Control _rm_ctrl;

        // When a command is received to reload the information, it should
        // be enough to update the two previous containers
        // NOTE: after a reload we should check if some inconsistency is created
        // with respect to the already allocated resources. Example: the total number of
        // allowed resources is reduced from 10 to 1 - some message should be sent to
        // make evident that now there are more allocated resources than allowed by the
        // new configuration

   RM_Backup* _bck; //_backup manager
daq::rmgr::Allocated_Info* createAllocatedInfo(std::vector<RM_AllocatedResource>& lst, daq::rmgr::Allocated_Info* info);
    //Gueseppe
        std::map<std::string, std::shared_ptr<boost::mutex>> _mutex_map;
        mutable boost::upgrade_mutex _write_mutex;

//oct 2015 keep for human config content before and after updateConfig
void printConfigContent( daq::rmgr::Config_update* inf1, daq::rmgr::Config_update* inf2); //delete both info in method at the end
//AI 2017
Configuration* loadConfiguration();
};


//for tests
class RM_Statistics {
private:
 int maxInd;
 std::atomic_long  _curIndex;
 int printed;

 std::ofstream outStrm;
 bool isOpened;
 const std::string path = "/tmp/rmgrtmp/reqtime.txt";
 
 long t0[200000];
 long delta[200000];
 void initArr() {
 maxInd = 200000;
      for( int i=0; i< maxInd; i++) {
         t0[i] = 0;
         delta[i] = 0;
      }
 }
public:
  RM_Statistics() : maxInd(200000), _curIndex(-1), printed(0), isOpened(false) {
std::cerr<<" RM_Statistics constructor" <<std::endl;
      initArr();
      openFile();
  }
  ~RM_Statistics() {
     outStrm << " Average=" << getReqAverage(); 
     closeStream();
  }
//  RM_Statistics(int max) : maxInd(max), _curIndex(-1) {
//      initArr();
//  }
//long diffMsecs(timeval& t2, timeval& t1) {
  //difference in microsecs * 10
  //  long tts = (t2.tv_sec - t1.tv_sec) * 100000;
  //    return tts + (t2.tv_usec - t1.tv_usec) /10;
  //    }
  
  bool setVal( timeval& t2, timeval& t1);
  long getReqAverage();

bool createOutStream(const char* log_file);
void writeValue();
void closeStream();
bool openFile();
};

  } //end rm
} //end daq


#endif  /* RMimpl_H_INCLUDED */                              


