#include <iostream>
#include <cmdl/cmdargs.h>
#include <ipc/core.h>
#include <ResourceManager/RM_Client.h>

#include "ers/ers.h"
#include <unistd.h>
#include <time.h>

  daq::rmgr::RM_Client* RMC = (daq::rmgr::RM_Client*)0;
int main(int argc, char** argv)
{
 	// Declare arguments
 	CmdArgStr	partition_name('p', "partition", "partition_id", "partition identifier (mandatory).", CmdArg::isREQ);
        CmdArgStr       binary_name('b', "binary", "binary_id", "Software  object (binary) identifier. Object for  which resources to be granted.", CmdArg::isREQ);
        CmdArgStr       computer_name('c', "computer", "computer_id", "identifier of the computer on which binary supposed to be started.", CmdArg::isREQ);
 	CmdArgStr  rm_server_name('n', "rm_server", "rm_server_name", 
			  "resource manager server name; use only for tests.");
//AI 051107 only for tests: temporary
        CmdArgStr  resource_id('r', "resource", "resource_id", "!!Used only for test with \'t\' parameter. UID of the RM resource to be requested for process.");
        CmdArgBool test_process   ('t', "test_process",
                                              "!!! Only for tests!! Request resources for process that will be started by this binary.");
 CmdArgInt       startime_v('T', "startime", "startime_v", "Time when to start request to RM_Server.");
        
	// Declare command object and its argument-iterator
 	CmdLine cmd(*argv, &partition_name, //&application_name, 
                   &binary_name, &computer_name, 
		     &resource_id, &test_process, //AI 051107 
                    &rm_server_name, &startime_v, NULL);
 	CmdArgvIter arg_iter(--argc, ++argv);
        cmd.description("Request Resource Manager resources for binary to be started on the computer on time");

        startime_v = 0;

       // Parse arguments
	cmd.parse(arg_iter);
  try {
     IPCCore::init(argc, argv);
  }
  catch( daq::ipc::Exception & ex ) {
     ers::fatal( ex );
  }

  if (rm_server_name.isNULL())
    RMC = new daq::rmgr::RM_Client();
  else
    RMC = new daq::rmgr::RM_Client(rm_server_name);
  if (RMC == (daq::rmgr::RM_Client*)0)
     exit(-1);  // server do not found
  int proc = test_process.flags() && CmdArg::GIVEN ? 1 : 0; 
  long handleId;
  std::string comp = computer_name.isNULL() ? "" : (const char*) computer_name;
  std::ostringstream txt ;
  if (partition_name.isNULL())
  {
        txt<<"rm_allocate_resources: No partition name in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
  else if (binary_name.isNULL() && !proc)
  {
        txt<<"rm_allocate_resources: No binary ID in input.";
        ers::warning (ers::Message( ERS_HERE, txt.str()));
  }
  else if (!resource_id.isNULL() && proc) {
//only for tests for some time
    try {
      handleId = RMC->requestResourceForMyProcess( (const char*)partition_name, (const char*)resource_id, comp, "test");
      txt<<"Handle ID = " <<  handleId;
      ers::info( ers::Message( ERS_HERE, txt.str()));
    }
    catch ( daq::rmgr::Exception & ex) {
          ers::error(ex);
    }
  }
  else
  {
    try {
        if( !(startime_v == 0) ) {
           long sec0 = startime_v - (long)time(NULL);
           unsigned int sec = sec0 > 0 ? (unsigned int) sec0 : 0 ;
           std::cerr<<"                       sleep="<<sec<<std::endl;
           sleep(sec);
        }
        if( !(startime_v == 0) )
	   std::cerr<<"     ###  RMC->requestResources start ... time="<<time(NULL)<<std::endl;

      	handleId = RMC->requestResources( (const char*)partition_name, (const char*) binary_name, comp );

        if( !(startime_v == 0)  ) {
            std::cerr<<"     @@@@@ RMC->requestResources finished. Time="<<time(NULL)<<std::endl;
            txt<<"Handle ID = " << handleId;
            ers::info( ers::Message( ERS_HERE, txt.str()));
            ers::info( ers::Message( ERS_HERE, txt.str()));
        }
    }     
    catch ( daq::rmgr::Exception & ex) {
          ers::error(ex);
    } 
  }
  delete RMC;

  if( !(startime_v == 0)  )
     std::cerr<<"     $$$$$$$$$ exit requestResources time="<<time(NULL)<<std::endl;
}
