#ifndef TM_HANDLE_H
#define TM_HANDLE_H

#include <future>
#include <memory>

#include <TM/TestResult.h>
//#include <TM/ComponentTester.h> // do not expose the implementation to clients, allowing easy patching

/**
 * \class daq::tm::TestHandle
 *
 * \brief Weak pointer of TestHandle object returned to the user, allowing to control (stop) the ongoing test and to get test output
 * 
 * Keeps a owning reference to a ComponentTester which is handling all the tests.
 * Constrcuted in Client::testComponent, a shared_ptr is kept in Client and destroyed in ~Client.
 * weak_ptr to it is returned to user from Client:test(), allowing to stop the tests and to get the result.
 * Upon destruction, stops all the tests. Also keeps a future<ComponentResult> which is eventually fulfilled in the Tester
 * 
 * This class is exposed to clients
 */
namespace daq {
namespace tm {

class ComponentTester ;

class TestHandle {
friend class Client ; // created there

public:
    ~TestHandle() ; 
    void stop()  ;   // stop all ongoing tests, wait for PMG callbacks, fullfill result

    ComponentResult getResult() { return m_result.get() ; } ; // blocks until result is fulfilled (from ComponentTester), throw ers::Issue
// TODO get Runtime output

private:
    TestHandle( std::shared_ptr<ComponentTester> tester ) ;
 
    std::shared_ptr<ComponentTester> m_tester ; // keeps ref to a ComponentTester in users' scope
    std::future<ComponentResult> m_result ;     // future to be fulfilled in ComponentTester
} ;

}}

#endif
