#include <stdlib.h>
#include <string>
#include <exception>
#include <functional> 
#include <mutex>  
	
#include <ers/ers.h>
#include "TM/ThreadPool.h"

const char TDAQ_TMGR_NTHREADS_ENVVAR[] = "TDAQ_TMGR_NTHREADS" ; // redefine N of threads in the pool, default 24

namespace daq {
ERS_DECLARE_ISSUE(tm,
                       BadConfiguration,
                       message,
                       ((std::string)message) )
}

using namespace daq::tm ;

void WorkerThread::operator()()
{
    std::function<void()> atask;
    while(true)
    {
    if ( pool.stop ) { return ; }

    if ( pool.tasks_queue.try_pop(atask) ) // if there is something in the queue
			{
			ERS_DEBUG(3, "Job is being executed") ;
			atask() ; 			// execute a task, continue
			ERS_DEBUG(3, "Job finished") ;
			}
    else // wait if nothing to do
			{
			std::unique_lock<std::mutex> lock(pool.queue_mutex) ;
			if ( pool.tasks_queue.empty() ) // check again within a mutex so to avoid waiting on non-empty queue
				{ 
				ERS_DEBUG(5, "No jobs, sleeping") ;
				pool.condition.wait(lock) ;
				}
			}
    }   
}

// the constructor just launches some amount of workers
ThreadPool::ThreadPool(size_t nthreads)
    :   stop(false)
{
try {
	if ( const char * var = getenv(TDAQ_TMGR_NTHREADS_ENVVAR) )
			{ nthreads = std::stoi(std::string(var));  }
	}
catch ( const std::exception& ex )
	{
	throw daq::tm::BadConfiguration(ERS_HERE, "Failed to parse TDAQ_TMGR_NTHREADS environment to integer", ex) ; 
	}
		
for(size_t i = 0; i < nthreads; ++i)
	{ worker_threads.push_back(std::thread(WorkerThread(*this))) ; }
	
ERS_DEBUG(0, "ThreadPool constructed: " << worker_threads.size() << " worker threads created" ) ;
}
 
// the destructor joins all threads
ThreadPool::~ThreadPool()
{
  if ( !tasks_queue.empty() )
	{
	ERS_LOG("There are still " << tasks_queue.unsafe_size() << " tasks scheduled, removing them from the queue") ;
	tasks_queue.clear() ;
	}

	// stop all threads
  stop = true;
  condition.notify_all();
 
	// join them
  for(size_t i = 0; i < worker_threads.size() ; ++i)
		{
	ERS_DEBUG(4, "Waiting for worker " << worker_threads[i].get_id() ) ;
		worker_threads[i].join() ;
		}
ERS_DEBUG(0, "ThreadPool destructred, no more working threads" ) ;
}
