#ifndef CTPCORETRIGGERRATEINFO_H
#define CTPCORETRIGGERRATEINFO_H

#include <is/info.h>

#include <TRP/CtpcoreItemRateInfo.h>
#include <TRP/CtpcorePITRateInfo.h>
#include <string>
#include <ostream>
#include <vector>


// <<BeginUserCode>>

// <<EndUserCode>>

namespace L1CT
{
/**
 * 
 * @author  generated by the IS tool
 * @version 01/03/17
 */

class CtpcoreTriggerRateInfo : public ISInfo {
public:

    /**
     * ATLAS run number
     */
    unsigned int                       run_number;

    /**
     * Current luminosity number
     */
    unsigned int                       lumiblock_number;

    /**
     * Number of Orbits used for pit rate calculation.
     */
    uint64_t                           orbit_pit;

    /**
     * Number of Orbits used for tbp rate calculation.
     */
    uint64_t                           orbit_tbp;

    /**
     * Number of Orbits used for tap rate calculation.
     */
    uint64_t                           orbit_tap;

    /**
     * Number of Orbits used for tav rate calculation.
     */
    uint64_t                           orbit_tav;

    /**
     * The number of the partition that is currently used.
     */
    uint64_t                           partitionNumber;

    /**
     * The L1A rate for the partition with partitionNumber.
     */
    float                              L1ARate;

    /**
     * TAV/TAP*100 [%] for the selected item (default is L1_EM12)
     */
    float                              live_fraction;

    /**
     */
    std::vector<CtpcoreItemRateInfo>   items;

    /**
     */
    std::vector<CtpcorePITRateInfo>    PITs;


    static const ISType & type() {
	static const ISType type_ = CtpcoreTriggerRateInfo( ).ISInfo::type();
	return type_;
    }

    virtual std::ostream & print( std::ostream & out ) const {
	ISInfo::print( out );
	out << std::endl;
	out << "run_number: " << run_number << "\t// ATLAS run number" << std::endl;
	out << "lumiblock_number: " << lumiblock_number << "\t// Current luminosity number" << std::endl;
	out << "orbit_pit: " << orbit_pit << "\t// Number of Orbits used for pit rate calculation." << std::endl;
	out << "orbit_tbp: " << orbit_tbp << "\t// Number of Orbits used for tbp rate calculation." << std::endl;
	out << "orbit_tap: " << orbit_tap << "\t// Number of Orbits used for tap rate calculation." << std::endl;
	out << "orbit_tav: " << orbit_tav << "\t// Number of Orbits used for tav rate calculation." << std::endl;
	out << "partitionNumber: " << partitionNumber << "\t// The number of the partition that is currently used." << std::endl;
	out << "L1ARate: " << L1ARate << "\t// The L1A rate for the partition with partitionNumber." << std::endl;
	out << "live_fraction: " << live_fraction << "\t// TAV/TAP*100 [%] for the selected item (default is L1_EM12)" << std::endl;
	out << "items[" << items.size() << "]:\t// " << std::endl;
	for ( size_t i = 0; i < items.size(); ++i )
	    out << i << " : " << items[i] << std::endl;
	out << "PITs[" << PITs.size() << "]:\t// " << std::endl;
	for ( size_t i = 0; i < PITs.size(); ++i )
	    out << i << " : " << PITs[i] << std::endl;
	return out;
    }

    CtpcoreTriggerRateInfo( )
      : ISInfo( "CtpcoreTriggerRateInfo" )
    {
	initialize();
    }

    ~CtpcoreTriggerRateInfo(){

// <<BeginUserCode>>

// <<EndUserCode>>
    }

protected:
    CtpcoreTriggerRateInfo( const std::string & type )
      : ISInfo( type )
    {
	initialize();
    }

    void publishGuts( ISostream & out ){
	out << run_number << lumiblock_number << orbit_pit << orbit_tbp << orbit_tap << orbit_tav;
	out << partitionNumber << L1ARate << live_fraction << items << PITs;
    }

    void refreshGuts( ISistream & in ){
	in >> run_number >> lumiblock_number >> orbit_pit >> orbit_tbp >> orbit_tap >> orbit_tav;
	in >> partitionNumber >> L1ARate >> live_fraction >> items >> PITs;
    }

private:
    void initialize()
    {
	orbit_pit = 0;
	orbit_tbp = 0;
	orbit_tap = 0;
	orbit_tav = 0;
	partitionNumber = 0;

// <<BeginUserCode>>

// <<EndUserCode>>
    }


// <<BeginUserCode>>

// <<EndUserCode>>
};

// <<BeginUserCode>>

// <<EndUserCode>>
inline std::ostream & operator<<( std::ostream & out, const CtpcoreTriggerRateInfo & info ) {
    info.print( out );
    return out;
}

}

#endif // CTPCORETRIGGERRATEINFO_H
