#ifndef EFD_H
#define EFD_H

#include <is/info.h>

#include <string>
#include <ostream>
#include <vector>


// <<BeginUserCode>>

// <<EndUserCode>>
/**
 * 
 * @author  generated by the IS tool
 * @version 19/11/09
 */

class EFD : public ISInfo {
public:

    /**
     * SharedHeap file fullpath
     */
    std::string                   SharedHeap;

    /**
     * Number of connected SFIs (sum over InputTasks)
     */
    unsigned short                ConnNbrSFIs;

    /**
     * Number of connected SFOs (sum over OutputTasks)
     */
    unsigned short                ConnNbrSFOs;

    /**
     * Number of connected PTs (sum over ExtPTsTasks)
     */
    unsigned short                ConnNbrPTs;

    /**
     * SharedHeap occupancy (%)
     */
    double                        HeapOcc;

    /**
     * Number of received events
     */
    int                           EventsRcv;

    /**
     * Number of events sent to SFO (ie: Dismissed-Deleted)
     */
    int                           EventsSent;

    /**
     * Number of events inside
     */
    int                           EventsInside;

    /**
     * Number of events waiting for processing
     */
    unsigned int                  EventsWaitingForProcessing;

    /**
     * Number of events waiting to be sent to SFO
     */
    unsigned int                  EventsWaitingForDelivery;

    /**
     * Number of events recovered from a previous EFD crash
     */
    unsigned int                  EventsRecovered;

    /**
     * Latest GID
     */
    unsigned int                  latestGID;

    /**
     * Current rate of incoming events (Hz)
     */
    double                        RateIn;

    /**
     * Current rate of events sent to SFO (Hz)
     */
    double                        RateOut;

    /**
     * Current rate of space allocation in SH; >~ data flux from SFI (MB/s)
     */
    double                        FluxIn;

    /**
     * Current data flux to SFO (MB/s)
     */
    double                        FluxOut;

    /**
     * Guess of the stop transition time (s)
     */
    double                        FlowCtrlStopTime;

    /**
     * Current flow control sleep time (ms)
     */
    unsigned int                  FlowCtrlSleepTime;

    /**
     * Number of times the input barrier has been locked
     */
    unsigned int                  FlowCtrlBarrierLocks;

    /**
     * Number of processing timeouts
     */
    int                           ptioNbrProcTimeouts;

    /**
     * Number of PT socket hung-ups
     */
    int                           ptioNbrSocketHungUps;

    /**
     * Number of force accepted events
     */
    int                           ptioNbrForceAccept;

    /**
     * Number of broken connections to SFI
     */
    int                           efioNbrSfiBrokenConn;

    /**
     * Number of broken connections to SFO
     */
    int                           efioNbrSfoBrokenConn;

    /**
     * Number of event duplication warnings (ie: missing event ack from SFO)
     */
    int                           EventDuplicWarnings;

    /**
     * Event sampled by emon service
     */
    int                           EventsSampled;

    /**
     * Type counters: phys, calib, reserved, debug, unknown, express, missing
     */
    std::vector<int>              EventTagTypesIn;

    /**
     * Type counters: phys, calib, reserved, debug, unknown, express, missing
     */
    std::vector<int>              EventTagTypesOut;

    /**
     * Load average of the node: 1,5,15 min
     */
    std::vector<double>           NodeLoadAverage;


    static const ISType & type() {
	static const ISType type_ = EFD( ).ISInfo::type();
	return type_;
    }

    virtual std::ostream & print( std::ostream & out ) const {
	ISInfo::print( out );
	out << "SharedHeap: " << SharedHeap << "	//SharedHeap file fullpath" << std::endl;
	out << "ConnNbrSFIs: " << ConnNbrSFIs << "	//Number of connected SFIs (sum over InputTasks)" << std::endl;
	out << "ConnNbrSFOs: " << ConnNbrSFOs << "	//Number of connected SFOs (sum over OutputTasks)" << std::endl;
	out << "ConnNbrPTs: " << ConnNbrPTs << "	//Number of connected PTs (sum over ExtPTsTasks)" << std::endl;
	out << "HeapOcc: " << HeapOcc << "	//SharedHeap occupancy (%)" << std::endl;
	out << "EventsRcv: " << EventsRcv << "	//Number of received events" << std::endl;
	out << "EventsSent: " << EventsSent << "	//Number of events sent to SFO (ie: Dismissed-Deleted)" << std::endl;
	out << "EventsInside: " << EventsInside << "	//Number of events inside" << std::endl;
	out << "EventsWaitingForProcessing: " << EventsWaitingForProcessing << "	//Number of events waiting for processing" << std::endl;
	out << "EventsWaitingForDelivery: " << EventsWaitingForDelivery << "	//Number of events waiting to be sent to SFO" << std::endl;
	out << "EventsRecovered: " << EventsRecovered << "	//Number of events recovered from a previous EFD crash" << std::endl;
	out << "latestGID: " << latestGID << "	//Latest GID" << std::endl;
	out << "RateIn: " << RateIn << "	//Current rate of incoming events (Hz)" << std::endl;
	out << "RateOut: " << RateOut << "	//Current rate of events sent to SFO (Hz)" << std::endl;
	out << "FluxIn: " << FluxIn << "	//Current rate of space allocation in SH; >~ data flux from SFI (MB/s)" << std::endl;
	out << "FluxOut: " << FluxOut << "	//Current data flux to SFO (MB/s)" << std::endl;
	out << "FlowCtrlStopTime: " << FlowCtrlStopTime << "	//Guess of the stop transition time (s)" << std::endl;
	out << "FlowCtrlSleepTime: " << FlowCtrlSleepTime << "	//Current flow control sleep time (ms)" << std::endl;
	out << "FlowCtrlBarrierLocks: " << FlowCtrlBarrierLocks << "	//Number of times the input barrier has been locked" << std::endl;
	out << "ptioNbrProcTimeouts: " << ptioNbrProcTimeouts << "	//Number of processing timeouts" << std::endl;
	out << "ptioNbrSocketHungUps: " << ptioNbrSocketHungUps << "	//Number of PT socket hung-ups" << std::endl;
	out << "ptioNbrForceAccept: " << ptioNbrForceAccept << "	//Number of force accepted events" << std::endl;
	out << "efioNbrSfiBrokenConn: " << efioNbrSfiBrokenConn << "	//Number of broken connections to SFI" << std::endl;
	out << "efioNbrSfoBrokenConn: " << efioNbrSfoBrokenConn << "	//Number of broken connections to SFO" << std::endl;
	out << "EventDuplicWarnings: " << EventDuplicWarnings << "	//Number of event duplication warnings (ie: missing event ack from SFO)" << std::endl;
	out << "EventsSampled: " << EventsSampled << "	//Event sampled by emon service" << std::endl;
	out << "EventTagTypesIn[" << EventTagTypesIn.size() << "]:	//Type counters: phys, calib, reserved, debug, unknown, express, missing" << std::endl;
	for ( size_t i = 0; i < EventTagTypesIn.size(); ++i )
	    out << i << ": " << EventTagTypesIn[i] << std::endl;
	out << "EventTagTypesOut[" << EventTagTypesOut.size() << "]:	//Type counters: phys, calib, reserved, debug, unknown, express, missing" << std::endl;
	for ( size_t i = 0; i < EventTagTypesOut.size(); ++i )
	    out << i << ": " << EventTagTypesOut[i] << std::endl;
	out << "NodeLoadAverage[" << NodeLoadAverage.size() << "]:	//Load average of the node: 1,5,15 min" << std::endl;
	for ( size_t i = 0; i < NodeLoadAverage.size(); ++i )
	    out << i << ": " << NodeLoadAverage[i] << std::endl;
	return out;
    }

    EFD( )
      : ISInfo( "EFD" )
    {
	initialize();
    }

    ~EFD(){

// <<BeginUserCode>>

// <<EndUserCode>>
    }

protected:
    EFD( const std::string & type )
      : ISInfo( type )
    {
	initialize();
    }

    void publishGuts( ISostream & out ){
	out << SharedHeap << ConnNbrSFIs << ConnNbrSFOs << ConnNbrPTs << HeapOcc << EventsRcv;
	out << EventsSent << EventsInside << EventsWaitingForProcessing << EventsWaitingForDelivery;
	out << EventsRecovered << latestGID << RateIn << RateOut << FluxIn << FluxOut << FlowCtrlStopTime;
	out << FlowCtrlSleepTime << FlowCtrlBarrierLocks << ptioNbrProcTimeouts << ptioNbrSocketHungUps;
	out << ptioNbrForceAccept << efioNbrSfiBrokenConn << efioNbrSfoBrokenConn << EventDuplicWarnings;
	out << EventsSampled << EventTagTypesIn << EventTagTypesOut << NodeLoadAverage;
    }

    void refreshGuts( ISistream & in ){
	in >> SharedHeap >> ConnNbrSFIs >> ConnNbrSFOs >> ConnNbrPTs >> HeapOcc >> EventsRcv;
	in >> EventsSent >> EventsInside >> EventsWaitingForProcessing >> EventsWaitingForDelivery;
	in >> EventsRecovered >> latestGID >> RateIn >> RateOut >> FluxIn >> FluxOut >> FlowCtrlStopTime;
	in >> FlowCtrlSleepTime >> FlowCtrlBarrierLocks >> ptioNbrProcTimeouts >> ptioNbrSocketHungUps;
	in >> ptioNbrForceAccept >> efioNbrSfiBrokenConn >> efioNbrSfoBrokenConn >> EventDuplicWarnings;
	in >> EventsSampled >> EventTagTypesIn >> EventTagTypesOut >> NodeLoadAverage;
    }

private:
    void initialize()
    {

// <<BeginUserCode>>

// <<EndUserCode>>
    }


// <<BeginUserCode>>

// <<EndUserCode>>
};

// <<BeginUserCode>>

// <<EndUserCode>>
inline std::ostream & operator<<( std::ostream & out, const EFD & info ) {
    info.print( out );
    return out;
}

#endif // EFD_H
