#ifndef IS_LB_H
#define IS_LB_H

#include <is/info.h>

#include <string>
#include <vector>
#include <iostream>

#include <ctime>

using std::cout;
using std::endl;

/**
 * IS class to contain luminosity block information from the CTPCORE
 * 
 * @author  generated by the IS tool
 * @version 07/08/07
 */

class IS_LB : public ISInfo {
public:

    /**
     * Run number
     */
    unsigned int                 runnumber;

    /**
     * Luminosity block number
     */
    unsigned short                LBN;

    /**
     * 0: IS entry not yet completed: still waiting for final update 1: IS entry complete for this luminosity block
     */
    bool                          Completed;

    /**
     * Starttime (second) human readable
     */
    std::string                   Starttime;
    
	/**
     * Starttime (second)
     */
    unsigned int                 Starttime_sec;

    /**
     * Starttime (nanosecond)
     */
    unsigned int                 Starttime_ns;

    /**
     * Stoptime (second) human readable
     */
    std::string                   Stoptime;
    
	/**
     * Stoptime (second)
     */
    unsigned int                 Stoptime_sec;

    /**
     * Stoptime (nanosecond)
     */
    unsigned int                 Stoptime_ns;

    /**
     * First L1ID seen by CTP
     */
    unsigned int                 Start_L1ID;

    /**
     * Last L1ID seen by CTP
     */
    unsigned int                 Stop_L1ID;

    /**
     * Number of L1A
     */
    uint64_t                 L1A_counter;

    /**
     * Number of LHC turns corresponding to the L1A counter
     */
    unsigned int                 L1A_turns;

    /**
     * Overflow bit for the L1A counter. True means: an overflow has occurred at least once for this counter False means: ok
     */
    bool                          L1A_overflow;

    /**
     * Mask of the 256 trigger items: true=enabled, false=disabled
     */
    std::vector<bool>             TriggerMask;

    /**
     * Values of the 256 pre-scale division factors (24-bit)
     */
    std::vector<unsigned int>    Prescale_dividers;

    /**
     * Array of 176 pattern-in-time counter values (31-bit)
     */
    std::vector<uint64_t>    PIT_counters;

    /**
     * Array of 256 trigger-before-prescale counter values (31-bit)
     */
    std::vector<uint64_t>    TBP_counters;

    /**
     * Array of 256 trigger-after-prescale counter values (31-bit)
     */
    std::vector<uint64_t>    TAP_counters;

    /**
     * Array of 256 trigger-after-veto counter values (31-bit)
     */
    std::vector<uint64_t>    TAV_counters;

    /**
     * 176 overflow bits for the PIT counters. True means: an overflow has occurred at least once for this counter False means: ok
     */
    std::vector<bool>             PIT_overflows;

    /**
     * 256 overflow bits for the TBP counters. True means: an overflow has occurred at least once for this counter False means: ok
     */
    std::vector<bool>             TBP_overflows;

    /**
     * 256 overflow bits for the TAP counters. True means: an overflow has occurred at least once for this counter False means: ok
     */
    std::vector<bool>             TAP_overflows;

    /**
     * 256 overflow bits for the TAV counters. True means: an overflow has occurred at least once for this counter False means: ok
     */
    std::vector<bool>             TAV_overflows;

    /**
     * Number of LHC turns corresponding to the set of PIT_counter values
     */
    unsigned int                 PIT_turns;

    /**
     * Number of LHC turns corresponding to the set of TBP_counter values
     */
    unsigned int                 TBP_turns;

    /**
     * Number of LHC turns corresponding to the set of TAP_counter values
     */
    unsigned int                 TAP_turns;

    /**
     * Number of LHC turns corresponding to the set of TAV_counter values
     */
    unsigned int                 TAV_turns;


    static const ISType & type() {
	static const ISType type_ = IS_LB( ).ISInfo::type();
	return type_;
    }

    virtual std::ostream & print( std::ostream & out ) const {
	ISInfo::print( out );
	out << "runnumber: " << runnumber << "	//Run number" << std::endl;
	out << "LBN: " << LBN << "	//Luminosity block number" << std::endl;
	out << "Completed: " << Completed << "	//0: IS entry not yet completed: still waiting for final update 1: IS entry complete for this luminosity block" << std::endl;
	out << "Starttime: " << Starttime << "	//Starttime (second) human readable" << std::endl;
	out << "Starttime_sec: " << Starttime_sec << "	//Starttime (second)" << std::endl;
	out << "Starttime_ns: " << Starttime_ns << "	//Starttime (nanosecond)" << std::endl;
	out << "Stoptime: " << Stoptime << "	//Stoptime (second) human readable" << std::endl;
	out << "Stoptime_sec: " << Stoptime_sec << "	//Stoptime (second)" << std::endl;
	out << "Stoptime_ns: " << Stoptime_ns << "	//Stoptime (nanosecond)" << std::endl;
	out << "Start_L1ID: " << Start_L1ID << "	//First L1ID seen by CTP" << std::endl;
	out << "Stop_L1ID: " << Stop_L1ID << "	//Last L1ID seen by CTP" << std::endl;
	out << "L1A_counter: " << L1A_counter << "	//Number of L1A" << std::endl;
	out << "L1A_turns: " << L1A_turns << "	//Number of LHC turns corresponding to the L1A counter" << std::endl;
	out << "L1A_overflow: " << L1A_overflow << "	//Overflow bit for the L1A counter. True means: an overflow has occurred at least once for this counter False means: ok" << std::endl;
	out << "TriggerMask[" << TriggerMask.size() << "]:	//Mask of the 256 trigger items: true=enabled, false=disabled" << std::endl;
	for ( size_t i = 0; i < TriggerMask.size(); ++i )
	    out << i << ": " << TriggerMask[i] << std::endl;
	out << "Prescale_dividers[" << Prescale_dividers.size() << "]:	//Values of the 256 pre-scale division factors (24-bit)" << std::endl;
	for ( size_t i = 0; i < Prescale_dividers.size(); ++i )
	    out << i << ": " << Prescale_dividers[i] << std::endl;
	out << "PIT_counters[" << PIT_counters.size() << "]:	//Array of 176 pattern-in-time counter values" << std::endl;
	for ( size_t i = 0; i < PIT_counters.size(); ++i )
	    out << i << ": " << PIT_counters[i] << std::endl;
	out << "TBP_counters[" << TBP_counters.size() << "]:	//Array of 256 trigger-before-prescale counter values" << std::endl;
	for ( size_t i = 0; i < TBP_counters.size(); ++i )
	    out << i << ": " << TBP_counters[i] << std::endl;
	out << "TAP_counters[" << TAP_counters.size() << "]:	//Array of 256 trigger-after-prescale counter values" << std::endl;
	for ( size_t i = 0; i < TAP_counters.size(); ++i )
	    out << i << ": " << TAP_counters[i] << std::endl;
	out << "TAV_counters[" << TAV_counters.size() << "]:	//Array of 256 trigger-after-veto counter values" << std::endl;
	for ( size_t i = 0; i < TAV_counters.size(); ++i )
	    out << i << ": " << TAV_counters[i] << std::endl;
	out << "PIT_overflows[" << PIT_overflows.size() << "]:	//176 overflow bits for the PIT counters. True means: an overflow has occurred at least once for this counter False means: ok" << std::endl;
	for ( size_t i = 0; i < PIT_overflows.size(); ++i )
	    out << i << ": " << PIT_overflows[i] << std::endl;
	out << "TBP_overflows[" << TBP_overflows.size() << "]:	//256 overflow bits for the TBP counters. True means: an overflow has occurred at least once for this counter False means: ok" << std::endl;
	for ( size_t i = 0; i < TBP_overflows.size(); ++i )
	    out << i << ": " << TBP_overflows[i] << std::endl;
	out << "TAP_overflows[" << TAP_overflows.size() << "]:	//256 overflow bits for the TAP counters. True means: an overflow has occurred at least once for this counter False means: ok" << std::endl;
	for ( size_t i = 0; i < TAP_overflows.size(); ++i )
	    out << i << ": " << TAP_overflows[i] << std::endl;
	out << "TAV_overflows[" << TAV_overflows.size() << "]:	//256 overflow bits for the TAV counters. True means: an overflow has occurred at least once for this counter False means: ok" << std::endl;
	for ( size_t i = 0; i < TAV_overflows.size(); ++i )
	    out << i << ": " << TAV_overflows[i] << std::endl;
	out << "PIT_turns: " << PIT_turns << "	//Number of LHC turns corresponding to the set of PIT_counter values" << std::endl;
	out << "TBP_turns: " << TBP_turns << "	//Number of LHC turns corresponding to the set of TBP_counter values" << std::endl;
	out << "TAP_turns: " << TAP_turns << "	//Number of LHC turns corresponding to the set of TAP_counter values" << std::endl;
	out << "TAV_turns: " << TAV_turns << "	//Number of LHC turns corresponding to the set of TAV_counter values" << std::endl;
	return out;
    }

    IS_LB()
    : ISInfo( "IS_LB" )
    {
		initialize();
    }
    
	IS_LB( const std::string & type )
    : ISInfo( type )
    {
		initialize();
    }

    ~IS_LB()
	{
// <<BeginUserCode>>

// <<EndUserCode>>
    }

// <<BeginUserCode>>

    void increase_counters(int factor)
    {
      cout << "LB: " << LBN << endl;
      
      cout<<" for testing increase TBP, TAP, TAV counters and turns  by adding "<<factor<<endl;
      
      for (int ii=0;ii<256;ii++)
	{
	  TBP_counters[ii] += factor + rand()%40;
	  TAP_counters[ii] += factor/2 + rand()%40;
	  TAV_counters[ii] += factor/4 + rand()%40;
      	}
      
      L1A_counter = rand()%75000;
      
      Starttime_sec = std::time(0);
      Stoptime_sec = std::time(0) + 2;
      cout << " TIME: start - stop: " << Starttime_sec << " - " << Stoptime_sec << endl;
    }
    
    
    void increase_counters_add(int factor)
    {
      cout << "LB: " << LBN << endl;
      
      cout<<" for testing increase TBP, TAP, TAV counters and turns  by adding "<<factor<<endl;
      
      for (int ii=0;ii<256;ii++)
	{
	  TBP_counters[ii] += factor;
	  TAP_counters[ii] += factor;
	  TAV_counters[ii] += factor;
      	}
      
      L1A_counter += factor;
    }
    
    void setcounters(int ct)
    {
      cout << "LB: " << LBN << endl;
      
      cout<<" for testing set TBP, TAP, TAV counter to "<<ct<<endl;
      
      for (int ii=0;ii<256;ii++)
	{
	  TBP_counters[ii] =ct;
	  TAP_counters[ii] =ct;
	  TAV_counters[ii] =ct;
      	}
      
      L1A_counter =ct;
    }
    
    void setStartTime(time_t tt){
      Starttime_sec = tt;
    }
    void setStopTime(time_t tt){
      Stoptime_sec=tt;
    }
    void setTurns(unsigned long turns){
      L1A_turns =turns;
      PIT_turns =turns;
      TBP_turns =turns;
      TAP_turns =turns;
      return;
    }
    void setLB( int pLB )
    {
      LBN = pLB;
    }

protected:
	
    void publishGuts( ISostream & out )
	{
		out << runnumber << LBN << Completed << Starttime << Starttime_sec << Starttime_ns;
		out << Stoptime << Stoptime_sec << Stoptime_ns << Start_L1ID << Stop_L1ID << L1A_counter;
		out << L1A_turns << L1A_overflow << TriggerMask << Prescale_dividers << PIT_counters;
		out << TBP_counters << TAP_counters << TAV_counters << PIT_overflows << TBP_overflows;
		out << TAP_overflows << TAV_overflows << PIT_turns << TBP_turns << TAP_turns << TAV_turns;
    }

    void refreshGuts( ISistream & in )
	{
		in >> runnumber >> LBN >> Completed >> Starttime >> Starttime_sec >> Starttime_ns;
		in >> Stoptime >> Stoptime_sec >> Stoptime_ns >> Start_L1ID >> Stop_L1ID >> L1A_counter;
		in >> L1A_turns >> L1A_overflow >> TriggerMask >> Prescale_dividers >> PIT_counters;
		in >> TBP_counters >> TAP_counters >> TAV_counters >> PIT_overflows >> TBP_overflows;
		in >> TAP_overflows >> TAV_overflows >> PIT_turns >> TBP_turns >> TAP_turns >> TAV_turns;
    }

// <<EndUserCode>>

 private:
	void initialize()
    {
		LBN = 0;
		Completed = 0;
		PIT_overflows.resize( 1 );
		PIT_overflows[0] = 0;
		TBP_overflows.resize( 1 );
		TBP_overflows[0] = 0;
		TAP_overflows.resize( 1 );
		TAP_overflows[0] = 0;
		TAV_overflows.resize( 1 );
		TAV_overflows[0] = 0;

		
// <<BeginUserCode>>
		TBP_counters.reserve(256);
		TAP_counters.reserve(256);
		TAV_counters.reserve(256);
		
		for (int ii=0;ii<256;ii++)
		{
		  int ihelp = 0; //ii * 256;
		  TBP_counters.push_back( ihelp );
		  TAP_counters.push_back( ihelp );
		  TAV_counters.push_back( ihelp );
		}
		
		Starttime_sec = std::time(0);
		Stoptime_sec = std::time(0) + 2;
		
		L1A_counter = rand()%20;
// <<EndUserCode>>
    }


    
};

// <<BeginUserCode>>

// <<EndUserCode>>

inline std::ostream & operator<<( std::ostream & out, const IS_LB & info ) {
    info.print( out );
    return out;
}

#endif // IS_LB_H

