#ifndef OCLUMI_H
#define OCLUMI_H

#include <is/info.h>

#include <string>
#include <ostream>
#include <vector>


// <<BeginUserCode>>

// <<EndUserCode>>
/**
 * 
 * @author  generated by the IS tool
 * @version 29/07/16
 */

class OCLumi : public ISInfo {
public:

    /**
     * Raw Lumi, as received from the detectors
     */
    double                        RawLumi;

    /**
     * Error on Raw Lumi
     */
    double                        RawLumi_err;

    /**
     * Raw Lumi, normalized to number of bunch crossings
     */
    double                        RawLumiPerBX;

    /**
     * Error on Raw Lumi per BX
     */
    double                        RawLumiPerBX_err;

    /**
     * Mu, calibrated with polynomial function
     */
    double                        Mu;

    /**
     * Error on Mu
     */
    double                        Mu_err;

    /**
     * Calibrated  Lumi, i.e. (Mu*NbrBC)/(sigma*t_orb)
     */
    double                        CalibLumi;

    /**
     * Error on Calibrated  Lumi
     */
    double                        CalibLumi_err;

    /**
     * Specific  Lumi
     */
    double                        SpecLumi;

    /**
     * Error on Specific Lumi
     */
    double                        SpecLumi_err;

    /**
     * Type of calibration used
     */
    std::string                   CalibType;

    /**
     * Calibration parameters
     */
    std::vector<double>           CalibPars;

    /**
     * Run number the calculated luminosities refer to
     */
    unsigned int                  RunNumber;

    /**
     * Lumi block number the calculated luminosities refer to
     */
    unsigned int                  LBN;

    /**
     * Data quality: 0=GOOD
     */
    unsigned int                  Quality;

    /**
     * Reason for Quality value
     */
    std::string                   QualityReason;

    /**
     * Mask of active channels
     */
    unsigned int                  ChannelMask;

    /**
     * Length of integration time interval in number of LHC turns
     */
    unsigned int                  NumberOfTurns;

    /**
     * Number of filled BC per turn (if applicable)
     */
    unsigned int                  NumberOfFilledBunches;

    /**
     * BunchMask applied
     */
    std::vector<double>           BunchMask;

    /**
     * Name of algorithm
     */
    std::string                   AlgoName;

    /**
     * Start time (second) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       
     */
    double                        Starttime_sec;

    /**
     * Start time (nanosecond part) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       
     */
    double                        Starttime_nsec;

    /**
     * End time (second) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       
     */
    double                        Endtime_sec;

    /**
     * End time (nanosecond part) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       
     */
    double                        Endtime_nsec;


    static const ISType & type() {
	static const ISType type_ = OCLumi( ).ISInfo::type();
	return type_;
    }

    virtual std::ostream & print( std::ostream & out ) const {
	ISInfo::print( out );
	out << std::endl;
	out << "RawLumi: " << RawLumi << "\t// Raw Lumi, as received from the detectors" << std::endl;
	out << "RawLumi_err: " << RawLumi_err << "\t// Error on Raw Lumi" << std::endl;
	out << "RawLumiPerBX: " << RawLumiPerBX << "\t// Raw Lumi, normalized to number of bunch crossings" << std::endl;
	out << "RawLumiPerBX_err: " << RawLumiPerBX_err << "\t// Error on Raw Lumi per BX" << std::endl;
	out << "Mu: " << Mu << "\t// Mu, calibrated with polynomial function" << std::endl;
	out << "Mu_err: " << Mu_err << "\t// Error on Mu" << std::endl;
	out << "CalibLumi: " << CalibLumi << "\t// Calibrated  Lumi, i.e. (Mu*NbrBC)/(sigma*t_orb)" << std::endl;
	out << "CalibLumi_err: " << CalibLumi_err << "\t// Error on Calibrated  Lumi" << std::endl;
	out << "SpecLumi: " << SpecLumi << "\t// Specific  Lumi" << std::endl;
	out << "SpecLumi_err: " << SpecLumi_err << "\t// Error on Specific Lumi" << std::endl;
	out << "CalibType: " << CalibType << "\t// Type of calibration used" << std::endl;
	out << "CalibPars[" << CalibPars.size() << "]:\t// Calibration parameters" << std::endl;
	for ( size_t i = 0; i < CalibPars.size(); ++i )
	    out << i << " : " << CalibPars[i] << std::endl;
	out << "RunNumber: " << RunNumber << "\t// Run number the calculated luminosities refer to" << std::endl;
	out << "LBN: " << LBN << "\t// Lumi block number the calculated luminosities refer to" << std::endl;
	out << "Quality: " << Quality << "\t// Data quality: 0=GOOD" << std::endl;
	out << "QualityReason: " << QualityReason << "\t// Reason for Quality value" << std::endl;
	out << "ChannelMask: " << ChannelMask << "\t// Mask of active channels" << std::endl;
	out << "NumberOfTurns: " << NumberOfTurns << "\t// Length of integration time interval in number of LHC turns" << std::endl;
	out << "NumberOfFilledBunches: " << NumberOfFilledBunches << "\t// Number of filled BC per turn (if applicable)" << std::endl;
	out << "BunchMask[" << BunchMask.size() << "]:\t// BunchMask applied" << std::endl;
	for ( size_t i = 0; i < BunchMask.size(); ++i )
	    out << i << " : " << BunchMask[i] << std::endl;
	out << "AlgoName: " << AlgoName << "\t// Name of algorithm" << std::endl;
	out << "Starttime_sec: " << Starttime_sec << "\t// Start time (second) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       " << std::endl;
	out << "Starttime_nsec: " << Starttime_nsec << "\t// Start time (nanosecond part) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       " << std::endl;
	out << "Endtime_sec: " << Endtime_sec << "\t// End time (second) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       " << std::endl;
	out << "Endtime_nsec: " << Endtime_nsec << "\t// End time (nanosecond part) of the luminosity block or heart beat interval represented as number of UTC nanoseconds since 1970.       It is also the stop time of the previous block or heart beat interval.       This timestamp (seconds*1e9 + nanoseconds) is used as IOV for COOL.       ";
	return out;
    }

    OCLumi( )
      : ISInfo( "OCLumi" )
    {
	initialize();
    }

    ~OCLumi(){

// <<BeginUserCode>>

// <<EndUserCode>>
    }

protected:
    OCLumi( const std::string & type )
      : ISInfo( type )
    {
	initialize();
    }

    void publishGuts( ISostream & out ){
	out << RawLumi << RawLumi_err << RawLumiPerBX << RawLumiPerBX_err << Mu << Mu_err;
	out << CalibLumi << CalibLumi_err << SpecLumi << SpecLumi_err << CalibType << CalibPars;
	out << RunNumber << LBN << Quality << QualityReason << ChannelMask << NumberOfTurns;
	out << NumberOfFilledBunches << BunchMask << AlgoName << Starttime_sec << Starttime_nsec;
	out << Endtime_sec << Endtime_nsec;
    }

    void refreshGuts( ISistream & in ){
	in >> RawLumi >> RawLumi_err >> RawLumiPerBX >> RawLumiPerBX_err >> Mu >> Mu_err;
	in >> CalibLumi >> CalibLumi_err >> SpecLumi >> SpecLumi_err >> CalibType >> CalibPars;
	in >> RunNumber >> LBN >> Quality >> QualityReason >> ChannelMask >> NumberOfTurns;
	in >> NumberOfFilledBunches >> BunchMask >> AlgoName >> Starttime_sec >> Starttime_nsec;
	in >> Endtime_sec >> Endtime_nsec;
    }

private:
    void initialize()
    {
	Quality = 1;
	ChannelMask = 0x0;
	NumberOfTurns = 0;
	NumberOfFilledBunches = 0xffff;

// <<BeginUserCode>>

// <<EndUserCode>>
    }


// <<BeginUserCode>>

// <<EndUserCode>>
};

// <<BeginUserCode>>

// <<EndUserCode>>
inline std::ostream & operator<<( std::ostream & out, const OCLumi & info ) {
    info.print( out );
    return out;
}

#endif // OCLUMI_H
