#ifndef PMGPUBLISHEDPROCESSDATA_H
#define PMGPUBLISHEDPROCESSDATA_H

#include <is/info.h>

#include <string>
#include <ostream>


// <<BeginUserCode>>

// <<EndUserCode>>
/**
 * This class contains information about processes started by the PMG system for the partition under which the information server is running  
 * 
 * @author  generated by the IS tool
 * @version 03/09/09
 */

class PMGPublishedProcessData : public ISInfo {
public:

    /**
     * Process Owner
     */
    std::string                   user_name;

    /**
     * Process Handle
     */
    std::string                   handle;

    /**
     * PID
     */
    unsigned int                  pid;

    /**
     * Partition
     */
    std::string                   partition;

    /**
     * Node the process is running on
     */
    std::string                   host;

    /**
     * Node the request to start the process comes from
     */
    std::string                   host_requesting;

    /**
     * Application name
     */
    std::string                   app_name;

    /**
     * Process unique id
     */
    unsigned int                  app_id;

    /**
     * Process status (string)
     */
    std::string                   state;

    /**
     * Process status (code)
     */
    unsigned int                  state_code;

    /**
     * Process exit code
     */
    unsigned int                  exit;

    /**
     * Process exit signal
     */
    std::string                   signal;

    /**
     * Process error message
     */
    std::string                   error_msg;

    /**
     * Process start time
     */
    std::string                   start;

    /**
     * Process stop time
     */
    std::string                   stop;

    /**
     * Process RM token
     */
    unsigned int                  token;

    /**
     * Process binary
     */
    std::string                   executable;

    /**
     * Process LD_LIBRARY_PATH
     */
    std::string                   library_paths;

    /**
     * Process working directory
     */
    std::string                   workingdir;

    /**
     * Process arguments
     */
    std::string                   arguments;

    /**
     * Process environment
     */
    std::string                   environment;

    /**
     * Process standard output
     */
    std::string                   std_out;

    /**
     * Process standard error
     */
    std::string                   std_err;

    /**
     * Process standard input
     */
    std::string                   std_in;

    /**
     * The number of minor faults the process has made which have not required loading a memory page from disk
     */
    uint64_t                      minflt;

    /**
     * The number of minor faults that the process's waited-for children have made
     */
    uint64_t                      cminflt;

    /**
     * The number of major faults the process has made which have required loading a memory page from disk
     */
    uint64_t                      majflt;

    /**
     * The number of major faults that the process's waited-for children have made
     */
    uint64_t                      cmajflt;

    /**
     * Amount of time that this process has been scheduled in user mode (ms)
     */
    uint64_t                      utime;

    /**
     * Amount of time that this process has been scheduled in kernel mode (ms)
     */
    uint64_t                      stime;

    /**
     * Amount of time that this process's waited-for children have been scheduled in user mode (ms)
     */
    int64_t                       cutime;

    /**
     * Amount of time that this process's waited-for children have been scheduled in kernel mode (ms)
     */
    int64_t                       cstime;

    /**
     * Number of threads in this process
     */
    int64_t                       num_threads;

    /**
     * Number of pages swapped
     */
    uint64_t                      nswap;

    /**
     * Cumulative nswap for child processes
     */
    uint64_t                      cnswap;

    /**
     * Total program size (kB)
     */
    int64_t                       size;

    /**
     * Resident set size (kB)
     */
    int64_t                       resident;

    /**
     * Shared pages (from shared mappings) (kB)
     */
    int64_t                       share;

    /**
     * Text (code) size (kB)
     */
    int64_t                       text;

    /**
     * Data + stack size (kB)
     */
    int64_t                       data;

    /**
     * Current soft limit on the rss of the process (kB)
     */
    uint64_t                      rsslim;


    static const ISType & type() {
	static const ISType type_ = PMGPublishedProcessData( ).ISInfo::type();
	return type_;
    }

    virtual std::ostream & print( std::ostream & out ) const {
	ISInfo::print( out );
	out << "user_name: " << user_name << "	//Process Owner" << std::endl;
	out << "handle: " << handle << "	//Process Handle" << std::endl;
	out << "pid: " << pid << "	//PID" << std::endl;
	out << "partition: " << partition << "	//Partition" << std::endl;
	out << "host: " << host << "	//Node the process is running on" << std::endl;
	out << "host_requesting: " << host_requesting << "	//Node the request to start the process comes from" << std::endl;
	out << "app_name: " << app_name << "	//Application name" << std::endl;
	out << "app_id: " << app_id << "	//Process unique id" << std::endl;
	out << "state: " << state << "	//Process status (string)" << std::endl;
	out << "state_code: " << state_code << "	//Process status (code)" << std::endl;
	out << "exit: " << exit << "	//Process exit code" << std::endl;
	out << "signal: " << signal << "	//Process exit signal" << std::endl;
	out << "error_msg: " << error_msg << "	//Process error message" << std::endl;
	out << "start: " << start << "	//Process start time" << std::endl;
	out << "stop: " << stop << "	//Process stop time" << std::endl;
	out << "token: " << token << "	//Process RM token" << std::endl;
	out << "executable: " << executable << "	//Process binary" << std::endl;
	out << "library_paths: " << library_paths << "	//Process LD_LIBRARY_PATH" << std::endl;
	out << "workingdir: " << workingdir << "	//Process working directory" << std::endl;
	out << "arguments: " << arguments << "	//Process arguments" << std::endl;
	out << "environment: " << environment << "	//Process environment" << std::endl;
	out << "std_out: " << std_out << "	//Process standard output" << std::endl;
	out << "std_err: " << std_err << "	//Process standard error" << std::endl;
	out << "std_in: " << std_in << "	//Process standard input" << std::endl;
	out << "minflt: " << minflt << "	//The number of minor faults the process has made which have not required loading a memory page from disk" << std::endl;
	out << "cminflt: " << cminflt << "	//The number of minor faults that the process's waited-for children have made" << std::endl;
	out << "majflt: " << majflt << "	//The number of major faults the process has made which have required loading a memory page from disk" << std::endl;
	out << "cmajflt: " << cmajflt << "	//The number of major faults that the process's waited-for children have made" << std::endl;
	out << "utime: " << utime << "	//Amount of time that this process has been scheduled in user mode (ms)" << std::endl;
	out << "stime: " << stime << "	//Amount of time that this process has been scheduled in kernel mode (ms)" << std::endl;
	out << "cutime: " << cutime << "	//Amount of time that this process's waited-for children have been scheduled in user mode (ms)" << std::endl;
	out << "cstime: " << cstime << "	//Amount of time that this process's waited-for children have been scheduled in kernel mode (ms)" << std::endl;
	out << "num_threads: " << num_threads << "	//Number of threads in this process" << std::endl;
	out << "nswap: " << nswap << "	//Number of pages swapped" << std::endl;
	out << "cnswap: " << cnswap << "	//Cumulative nswap for child processes" << std::endl;
	out << "size: " << size << "	//Total program size (kB)" << std::endl;
	out << "resident: " << resident << "	//Resident set size (kB)" << std::endl;
	out << "share: " << share << "	//Shared pages (from shared mappings) (kB)" << std::endl;
	out << "text: " << text << "	//Text (code) size (kB)" << std::endl;
	out << "data: " << data << "	//Data + stack size (kB)" << std::endl;
	out << "rsslim: " << rsslim << "	//Current soft limit on the rss of the process (kB)" << std::endl;
	return out;
    }

    PMGPublishedProcessData( )
      : ISInfo( "PMGPublishedProcessData" )
    {
	initialize();
    }

    ~PMGPublishedProcessData(){

// <<BeginUserCode>>

// <<EndUserCode>>
    }

protected:
    PMGPublishedProcessData( const std::string & type )
      : ISInfo( type )
    {
	initialize();
    }

    void publishGuts( ISostream & out ){
	out << user_name << handle << pid << partition << host << host_requesting << app_name;
	out << app_id << state << state_code << exit << signal << error_msg << start << stop;
	out << token << executable << library_paths << workingdir << arguments << environment;
	out << std_out << std_err << std_in << minflt << cminflt << majflt << cmajflt << utime;
	out << stime << cutime << cstime << num_threads << nswap << cnswap << size << resident;
	out << share << text << data << rsslim;
    }

    void refreshGuts( ISistream & in ){
	in >> user_name >> handle >> pid >> partition >> host >> host_requesting >> app_name;
	in >> app_id >> state >> state_code >> exit >> signal >> error_msg >> start >> stop;
	in >> token >> executable >> library_paths >> workingdir >> arguments >> environment;
	in >> std_out >> std_err >> std_in >> minflt >> cminflt >> majflt >> cmajflt >> utime;
	in >> stime >> cutime >> cstime >> num_threads >> nswap >> cnswap >> size >> resident;
	in >> share >> text >> data >> rsslim;
    }

private:
    void initialize()
    {
	minflt = 0;
	cminflt = 0;
	majflt = 0;
	cmajflt = 0;
	utime = 0;
	stime = 0;
	cutime = 0;
	cstime = 0;
	num_threads = 0;
	nswap = 0;
	cnswap = 0;
	size = 0;
	resident = 0;
	share = 0;
	text = 0;
	data = 0;
	rsslim = 0;

// <<BeginUserCode>>

// <<EndUserCode>>
    }


// <<BeginUserCode>>

// <<EndUserCode>>
};

// <<BeginUserCode>>

// <<EndUserCode>>
inline std::ostream & operator<<( std::ostream & out, const PMGPublishedProcessData & info ) {
    info.print( out );
    return out;
}

#endif // PMGPUBLISHEDPROCESSDATA_H
