#ifndef DBPSKWRITER_H
#define DBPSKWRITER_H

#include <string>
#include <iostream>

namespace coral {
   class ISessionProxy;
//    class ISession;
//    class IConnection;
}

namespace TrigConf {

   class DbPskWriter {
   public:
      DbPskWriter( const std::string& cs,
                   const std::string& user = "",
                   const std::string& password = "", 
                   int retrialPeriod = 10,
                   int retrialTimeout = 35,
                   int connectionTimeout = 5,
                   std::ostream& o = std::cout);
      ~DbPskWriter();
      
      bool writeLBPS(unsigned int lb, unsigned int hltPrescaleKey, unsigned int run=0);
      
      bool clearTable(bool keepLastEntry=true);

      void printTableContent();

      // to handle the connection at every stage. they are inclusive,
      // so one can call createAndStartSession() and then
      // deleteConnection(), to go all the way up and down
      void createConnection();
      void openConnection();
      void createAndStartSession();
      void endAndDeleteSession();
      void closeConnection();
      void deleteConnection();

   private:
      // methods
      bool readTableContent(bool printoutput=false);

      void buildConnectionString();

      // data

      // transaction parameters
      static const bool READONLY=true, UPDATE=false;

      // last entry in DB
      unsigned int m_lastLb { 0 };
      unsigned int m_lastPsk { 0 };

      // db connection parameter
      std::string    m_cs { "" };
      std::string    m_user { "" };
      std::string    m_password { "" };
      std::string    m_schemaName { "" };
      int            m_retrialPeriod { 0 };
      int            m_retrialTimeout { 0 };
      int            m_connectionTimeout { 0 };

      // db connection
//       coral::ISession*    m_session { nullptr };
//       coral::IConnection* m_connection { nullptr };
      coral::ISessionProxy* m_sessionproxy { nullptr };
      
      // message output stream
      std::ostream&  m_ostream;
   };

}



#endif
