
#include "TriggerDB/DbPskWriter.h"

#include <iostream>
#include <cstdlib>

#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Exception.h"

#include "CoralKernel/Context.h"

#include "RelationalAccess/SchemaException.h"

#include "RelationalAccess/IDatabaseServiceDescription.h"
#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/ILookupService.h"
#include "RelationalAccess/IAuthenticationCredentials.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/IRelationalService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/ConnectionService.h"
//#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/IQuery.h"



TrigConf::DbPskWriter::DbPskWriter(const std::string& connection,
                                   const std::string& user,
                                   const std::string& password,
                                   int retrialPeriod,
                                   int retrialTimeout,
                                   int connectionTimeout,
                                   std::ostream& o) :
   m_cs(connection),
   m_user(user),
   m_password(password),
   m_retrialPeriod(retrialPeriod),
   m_retrialTimeout(retrialTimeout),
   m_connectionTimeout(connectionTimeout),
   m_ostream(o)
{
   //createConnection();   
   readTableContent(false);
}


TrigConf::DbPskWriter::~DbPskWriter() {
   endAndDeleteSession();
   //deleteConnection();
}

bool
TrigConf::DbPskWriter::writeLBPS(unsigned int lb, unsigned int hltPrescaleKey, unsigned int run) {

   // get the partition name from environment variable TDAQ_PARTITION
   std::string partition("NOPART");
   const char* part = getenv("TDAQ_PARTITION");
   if(part) { 
      partition=std::string(part);
   } else {
      const char* user = getenv("USER");
      if(user) partition=std::string(user);
   }

   
   if(lb<=m_lastLb) {
      m_ostream << "Can't insert key for LB " << lb << ". Lumiblock must be larger than " << m_lastLb << std::endl;
      return false;
   }

   if(hltPrescaleKey==m_lastPsk) {
      m_ostream << "Won't insert key " << hltPrescaleKey << " since it is already valid" << std::endl;
      return false;
   }

   createAndStartSession();

   if(!m_sessionproxy->transaction().isActive())
      m_sessionproxy->transaction().rollback();

   m_sessionproxy->transaction().start(UPDATE);

   coral::ITableDataEditor* tEditor;
   try {
      tEditor = &m_sessionproxy->nominalSchema().tableHandle( "HLT_PRESCALE_SET_COLL").dataEditor();
   }
   catch(coral::TableNotExistingException e) {
      m_ostream << "Table 'HLT_PRESCALE_SET_COLL' does not exist!" << std::endl;
      m_sessionproxy->transaction().commit();
      return false;
   }
   
   coral::AttributeList data;
   data.extend<std::string>( "HPSC_PARTITION" );
   data.extend<int>( "HPSC_RUN" );
   data.extend<int>( "HPSC_LB" );
   data.extend<int>( "HPSC_HLTPSK" );

   data["HPSC_PARTITION"].data<std::string>() = partition;
   data["HPSC_RUN"].data<int>() = run;
   data["HPSC_LB"].data<int>() = lb;
   data["HPSC_HLTPSK"].data<int>() = hltPrescaleKey;

   try {
      tEditor->insertRow(data);
   }
   catch(coral::DataEditorException e) {
      m_ostream << "Can not insert entry for LB " << lb << " (CORAL Exception)" << std::endl;
      m_sessionproxy->transaction().commit();   
      return false;
   }

   m_sessionproxy->transaction().commit();   
   m_lastLb  = lb;
   m_lastPsk = hltPrescaleKey;
   return true;
}

bool
TrigConf::DbPskWriter::clearTable(bool keepLastEntry) {
   createAndStartSession();
   if(keepLastEntry) {
      if(!readTableContent()) return false;
   }

   if(m_sessionproxy->transaction().isActive())
      m_sessionproxy->transaction().rollback();
   m_sessionproxy->transaction().start(UPDATE);

   std::string partition("NOPART");
   const char* part = getenv("TDAQ_PARTITION");
   if(part) { 
      partition=std::string(part);
   } else {
      const char* user = getenv("USER");
      if(user) partition=std::string(user);
   }

   coral::ITableDataEditor* tEditor;
   try {
      tEditor = &m_sessionproxy->nominalSchema().tableHandle( "HLT_PRESCALE_SET_COLL").dataEditor();
   }
   catch(coral::TableNotExistingException e) {
      m_ostream << "Table 'HLT_PRESCALE_SET_COLL' does not exist!" << std::endl;
      m_sessionproxy->transaction().commit();
      return false;
   }

   coral::AttributeList data;
   data.extend<std::string>("part");
   data[0].data<std::string>() = partition;
   tEditor->deleteRows("HPSC_PARTITION = :part", data);
   m_sessionproxy->transaction().commit();

   if(keepLastEntry) {
      m_lastLb  = 0;
      int lastOfPreviousRun = m_lastPsk;
      m_lastPsk = 0;
      writeLBPS(1, lastOfPreviousRun);
   } else {
      m_lastLb  = 0;
      m_lastPsk = 0;
   }

   return true;
}

void
TrigConf::DbPskWriter::printTableContent() {
   readTableContent(true);
}

bool
TrigConf::DbPskWriter::readTableContent(bool printoutput) {
   createAndStartSession();

   m_sessionproxy->transaction().start(READONLY);

   std::string partition("NOPART");
   const char* part = getenv("TDAQ_PARTITION");
   if(part) { 
      partition=std::string(part);
   } else {
      const char* user = getenv("USER");
      if(user) partition=std::string(user);
   }

   coral::IQuery* query = 0;
   try {
      query = m_sessionproxy->nominalSchema().tableHandle( "HLT_PRESCALE_SET_COLL").newQuery();
   }
   catch(coral::TableNotExistingException e) {
      m_ostream << "Table 'HLT_PRESCALE_SET_COLL' does not exist!" << std::endl;
      m_sessionproxy->transaction().commit();
      return false;
   }

   //Bind list
   coral::AttributeList bindList;
   query->setCondition( "", bindList );
   
   //Output data and types
   coral::AttributeList attList;
   attList.extend<std::string>( "HPSC_PARTITION" );
   attList.extend<int>( "HPSC_LB" );
   attList.extend<int>( "HPSC_HLTPSK" );
   query->defineOutput(attList);
   query->addToOutputList( "HPSC_PARTITION" );
   query->addToOutputList( "HPSC_LB" );
   query->addToOutputList( "HPSC_HLTPSK" );

   query->addToOrderList( "HPSC_PARTITION" );
   query->addToOrderList( "HPSC_LB" );

   // process the query
   coral::ICursor& cursor = query->execute();

   int entrycount = 0;
   while (cursor.next()) {
      const coral::AttributeList& row = cursor.currentRow();
      std::string readpart = row["HPSC_PARTITION"].data<std::string>();
      int lb               = row["HPSC_LB"       ].data<int>();
      int psk              = row["HPSC_HLTPSK"   ].data<int>();
      if(printoutput) std::cout << "Partition: " << readpart << ", start Lb: " << lb << ", psk:" << psk << std::endl;

      if(readpart==partition) {
         m_lastLb = lb;
         m_lastPsk = psk;
      }
      entrycount++;
   }

   if(printoutput) std::cout << "Table contains " << entrycount << " entries." << std::endl;

   delete query;
   m_sessionproxy->transaction().commit();
   return true;
}


/*
void
TrigConf::DbPskWriter::createConnection() {
   coral::Context& context = coral::Context::instance();
   // =================================
   // user-less login
   // =================================

   if(m_user == "" && m_cs.find("sqlite", 0) == std::string::npos  ) {

      // if no ':' is found, the connection string is an ALIAS
      if( m_cs.find(':') == std::string::npos ) {
         // =================================
         // load the lookup service
         // =================================
         const std::string lookSvcStr("CORAL/Services/XMLLookupService");
         coral::IHandle<coral::ILookupService> lookupsvc = context.query<coral::ILookupService>();
         if (!lookupsvc.isValid()) {
            context.loadComponent(lookSvcStr);
            lookupsvc = context.query<coral::ILookupService>();
         }
         if (!lookupsvc.isValid())
            throw std::runtime_error( "Could not locate any lookup service" );
       
         const coral::IDatabaseServiceSet * svcSet = lookupsvc->lookup( m_cs, coral::ReadOnly);
         if(svcSet->numberOfReplicas()==0)
            throw std::runtime_error( "Alias has no entries in dblookup.xml" );
       
         // take the first database service described
         const coral::IDatabaseServiceDescription & desc = svcSet->replica(0);
       
         // change the alias to the physical connection string
         m_ostream << "Connection alias '" << m_cs << "' resolved to '" << desc.connectionString() << "'" << std::endl;
         m_cs = desc.connectionString();
      }
     
      // =================================
      // load the authentification service
      // =================================
      const std::string authSvcStr("CORAL/Services/XMLAuthenticationService");
      coral::IHandle<coral::IAuthenticationService> authsvc = context.query<coral::IAuthenticationService>();
      if (!authsvc.isValid()) {
         context.loadComponent(authSvcStr);
         authsvc = context.query<coral::IAuthenticationService>();
      }
      if (!authsvc.isValid())
         throw std::runtime_error( "Could not locate any authentication service" );
      const coral::IAuthenticationCredentials& crds = authsvc->credentials( m_cs );
      m_user     = crds.valueForItem("user");
      m_password = crds.valueForItem("password");
   }

   coral::ConnectionService conSvcH;
   coral::IConnectionServiceConfiguration& csConfig = conSvcH.configuration();
   csConfig.setConnectionRetrialPeriod( m_retrialPeriod );    //time interval between two connection retrials
   // time out for the connection retrials before the connection service fails over to the next available replica or quits
   csConfig.setConnectionRetrialTimeOut( m_retrialTimeout );
   csConfig.setConnectionTimeOut( m_connectionTimeout );
   
   // m_ostream << "Load relational service with connection string " << m_cs << std::endl;
   
   // ===============================
   // load the relational service
   // ===============================
   const std::string relSvcStr("CORAL/Services/RelationalService");
   coral::IHandle<coral::IRelationalService> relsvc = context.query<coral::IRelationalService>();
   if (!relsvc.isValid()) {
      context.loadComponent(relSvcStr);
      relsvc = context.query<coral::IRelationalService>();
   }
   if (!relsvc.isValid())
      throw std::runtime_error( "Could not locate any relational service" );
   coral::IRelationalDomain& domain = relsvc->domainForConnection( m_cs );
   
   //    m_ostream << "Picked up a relational domain object" << std::endl;
   //    m_ostream << "  type           : " << domain.flavorName() << std::endl;
   //    m_ostream << "  implementation : " << domain.implementationName() << std::endl;
   //    m_ostream << "  version        : " << domain.implementationVersion() << std::endl;
   
   // Sanity check for TNS_ADMIN
   if(domain.flavorName()=="ORACLE" && getenv("TNS_ADMIN")==0)
      throw std::runtime_error("TNS_ADMIN not defined - oracle connections will not work!");

   // (DB, schemaName)
   std::pair<std::string,std::string> cs = domain.decodeUserConnectionString( m_cs );
   m_schemaName = cs.second;
   // ===============================
   // connect to the database
   // ===============================
   m_connection = domain.newConnection( cs.first );
}
*/

/*
void
TrigConf::DbPskWriter::openConnection() {
   if(m_connection==0) createConnection();
   m_connection->connect();   
}
*/


void
TrigConf::DbPskWriter::createAndStartSession() {

   if( m_sessionproxy ) return;

   coral::ConnectionService connSvc;
   coral::IConnectionServiceConfiguration& csc = connSvc.configuration();
   csc.setConnectionRetrialPeriod( m_retrialPeriod );
   csc.setConnectionRetrialTimeOut( m_retrialTimeout );
   csc.setConnectionTimeOut( m_connectionTimeout );

   m_sessionproxy = connSvc.connect(m_cs, coral::AccessMode::Update);
   std::cout << "Opening session " << m_cs << std::endl;

//    if( m_session && m_session->isUserSessionActive() ) return;

//    if( m_connection==0 || !m_connection->isConnected( true ) )
//       openConnection();

//    if(m_session==0)
//       m_session = m_connection->newSession( m_schemaName, coral::Update ); // other option coral::Read
   
//    if(!m_session->isUserSessionActive() )
//       m_session->startUserSession( m_user, m_password );
}

void
TrigConf::DbPskWriter::endAndDeleteSession() {

   if(m_sessionproxy) {
      try{
         delete m_sessionproxy;
         m_sessionproxy = nullptr;
         std::cout << "Closing session " << m_cs << std::endl;
      }
      catch ( coral::Exception& e ) {
         std::cout << "CORAL exception " << e.what() << std::endl;
         throw;
      }
   }

}

// void
// TrigConf::DbPskWriter::closeConnection() {
//    endAndDeleteSession();

//    try {
//       if( m_connection->isConnected( true ) )
//          m_connection->disconnect();
//    }
//    catch ( coral::Exception& e ) {
//       m_ostream << "CORAL exception " << e.what() << std::endl;
//       throw;
//    }
//    catch ( std::exception& e ) {
//          m_ostream << "Standard C++ exception " << e.what() << std::endl;
//          throw;
//    }
//    catch( ... ) {
//       m_ostream << "Unknown exception ..." << std::endl;
//       throw;
//    }
// }


// void
// TrigConf::DbPskWriter::deleteConnection() {
//    if( m_connection==0 ) return;
//    if( m_connection->isConnected( true ) )
//       closeConnection();

//    delete m_connection;
//    m_connection=0;
// }
