
#include <iostream>

#include "TriggerDB/DbPskWriter.h"

using std::cout;
using std::endl;
using std::cin;

void print(TrigConf::DbPskWriter* pskwriter) {
   pskwriter->printTableContent();
}

void update(TrigConf::DbPskWriter* pskwriter) {
   int lb, psk;
   cout << "Lumiblock : "; cin >> lb;
   cout << "PSK       : "; cin >> psk;
   pskwriter->writeLBPS(lb, psk);
}

void clear(TrigConf::DbPskWriter* pskwriter, bool keepLastEntry=true) {
   pskwriter->clearTable(keepLastEntry);
}



int main(int argc, char** argv) {
   if(argc!=2) {
      cout << "Usage:" << endl
           << argv[0] << " <triggerdb connection>" << endl;
      cout << "connection examples: " << endl
           << "   TRIGGERDB" << endl
           << "   oracle://INTR/ATLAS_CONF_DEV1" << endl
           << "   oracle://DEVDB10/ATLAS_TRIG_STELZER;user=atlas_trig_stelzer;passwd=...  [feel free to ask for pw and use it]" << endl;
      return 1;
   }

   std::string dbconn(argv[1]);
   
   std::string::size_type posSemicolon = dbconn.find(';');
   std::string cs(dbconn.substr(0,posSemicolon));
   std::string user("");
   std::string passwd("");

   if(posSemicolon!=std::string::npos) {
      std::string::size_type firstpos = dbconn.find("user",posSemicolon+1);
      if(firstpos!=std::string::npos) {
         firstpos += 5;
         posSemicolon = dbconn.find(";",firstpos);
         user = dbconn.substr(firstpos,posSemicolon-firstpos);
      }
      firstpos = dbconn.find("passwd",posSemicolon+1);
      if(firstpos!=std::string::npos) {
         firstpos += 7;
         posSemicolon = dbconn.find(";",firstpos);
         passwd = dbconn.substr(firstpos,posSemicolon-firstpos);
      }
   }

   TrigConf::DbPskWriter pskw(cs, user, passwd);

   char c;
   while(c!='0') {  
      c='0';
      cout << endl;
      cout << "1) ..... Print table" << endl;
      cout << "2) ..... Insert <LB,PSK>" << endl;
      cout << "3) ..... Clear table (keep last entry)" << endl;
      cout << "4) ..... Clear table completely" << endl << endl;
      cout << "0) ..... Quit" << endl << endl;
      cout << "Choice: "; cin >> c;
      switch(c){
      case '1': print(&pskw); break; 
      case '2': update(&pskw); break; 
      case '3': clear(&pskw); break; 
      case '4': clear(&pskw, false); break; 
      default: break; 
      }
   }

   return 0;
}
