/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package triggerpanel;

import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SwingUtilities;


public class L1PSSelectionDialogImpl extends L1PSSelectionDialog {

    private int partition = 0;
    private int currentSMK = 0;
    private int currentPSK = 0;
    private KeyDescriptor selectedPSK = null;
    private boolean cancelled = false;

    public KeyDescriptor getSelectedPSK() {
        return selectedPSK;
    }

    public boolean isCancelled() {
        return cancelled;
    }

    
    public L1PSSelectionDialogImpl(boolean modal, final int currentSMK, final int currentPSK, final int partition) {
        super(null, modal);
        
        keySelectionComboBox.addItem(new KeyDescriptor(0,"Loading ...",0,"",false));

        this.partition = partition;
        this.currentSMK = currentSMK;
        this.currentPSK = currentPSK;
        currentPSKLabel.setText(Integer.toString(this.currentPSK));
        title.setText("Select L1 prescale key for CTP partition " + partition);
        setTitle("L1 Prescale Selection for SMK " + currentSMK + " and partition " + partition);
        (new L1PSKLoader()).execute();
        
        showHidden.addActionListener((java.awt.event.ActionEvent evt) -> {
            SwingUtilities.invokeLater(() -> {
                fillDropDownBox();
            });
        });

        selectButton.addActionListener((ActionEvent e) -> {
            dispose();
        });
        
        cancelButton.addActionListener((ActionEvent e) -> {
            cancelled = true;
            dispose();
        });
    
        keySelectionComboBox.addItemListener((java.awt.event.ItemEvent evt) -> {
            if(evt.getStateChange()==ItemEvent.SELECTED) {
                selectedPSK = (KeyDescriptor) evt.getItem();
                setButtonStatus();
            }
        });
        
        selectedPSK = (KeyDescriptor) keySelectionComboBox.getSelectedItem();
    }

    private void setButtonStatus() {
        boolean enableSelectButton =
            selectedPSK != null && // something selected
            selectedPSK.key > 0 && // proper key
            selectedPSK.key != currentPSK; // has changed

        selectButton.setEnabled(enableSelectButton);
        cancelButton.setEnabled(true);
    }

    private class L1PSKLoader extends javax.swing.SwingWorker<Void, Void> {

        @Override
        public Void doInBackground() {
            try {
                TPGlobal.getDbKeysLoader().FlagL1PskAsNotLoaded();
                TPGlobal.getDbKeysLoader().UpdateL1Prescales(currentSMK);
            } catch (SQLException ex) {
                Logger.getLogger(L1PSSelectionDialogImpl.class.getName()).log(Level.SEVERE, null, ex);
            }
            return null;
        }

        @Override
        public void done() {
            fillDropDownBox();
            setVisible(true);
        }
    }

    private void fillDropDownBox() {
        TPGlobal.getDbKeysLoader().FlagL1PskAsNotLoaded();
        
        keySelectionComboBox.removeAllItems();
        for(KeyDescriptor kd : TPGlobal.getDbKeysLoader().GetL1Prescales(partition)) {
            if(!showHidden.isSelected() && kd.hidden) {
                continue;
            }
            keySelectionComboBox.addItem(kd);
            if(kd.key==currentPSK) { // mark current key as selected
                keySelectionComboBox.setSelectedItem(kd);
            }
        }
        if( 0 == keySelectionComboBox.getItemCount() ) {
            keySelectionComboBox.addItem(KeyDescriptor.NoPSKforPartition);
        }
    }
    
    
}
