package triggerpanel;

import Igui.IguiLogger;
import java.util.ArrayList;


class OKSHandler {

    private SMKVariable          SMKVar = null;
    private L1PSKVariable        L1PSKVar = null;
    private HLTPSKVariable       HLTPSKVar = null;
    private BGSKVariable         BGSKVar = null;
    private DBConnectionVariable DBConnVar = null;
    private CTPPartitionVariable CTPPartVar = null;
    private CTPDeadtimeVariable  CTPDTVar = null;
    private final ArrayList<OKSVariable<?> > varList;
    private final ArrayList<TPCallable> toCall;
    
    void addVar(OKSVariable<?> v) {
        varList.add(v);
    }
    
    void bind(TPCallable c) {
        toCall.add(c);
    }
    
    public OKSHandler() {
        this.toCall = new ArrayList<>();
        this.varList = new ArrayList<>();
    }
    
    void createVariables() {
        SMKVar    = new SMKVariable();
        L1PSKVar  = new L1PSKVariable();
        HLTPSKVar = new HLTPSKVariable();
        BGSKVar   = new BGSKVariable();
        DBConnVar = new DBConnectionVariable();
        CTPPartVar= new CTPPartitionVariable();
        CTPDTVar  = new CTPDeadtimeVariable();
    }

    public void readOKS() {
        IguiLogger.info("Reading OKS now:" + this.toString());
        for(TPCallable c : toCall) {
            c.call();
        }
    } 

    SMKVariable          getSMKVar()    { return SMKVar; }
    L1PSKVariable        getL1PSKVar()  { return L1PSKVar; }
    HLTPSKVariable       getHLTPSKVar() { return HLTPSKVar; }
    BGSKVariable         getBGSKVar()   { return BGSKVar; }
    DBConnectionVariable getDBConnVar() { return DBConnVar; }
    CTPPartitionVariable getCTPPartitionVar() { return CTPPartVar; }
    CTPDeadtimeVariable  getCTPDeadtimeVar() { return CTPDTVar; }
    
    public void print() { System.out.println(toString()); }

    @Override
    public String toString() {
        StringBuilder msg = new StringBuilder();
        for(OKSVariable<?> var: varList) {
            msg.append("\n   ").append(var.toString());
        }
        return msg.toString();
    } 

    public boolean uncommitedDbChanges() {
        boolean allCommitted = true;
        for(OKSVariable<?> var: varList) {
            if(! var.isCommitted() ) { allCommitted = false; break; }
        }
        return ! allCommitted;
    }

    public void abortRdbChanges() {
        for(OKSVariable<?> var: varList) {
            var.abortRdbChanges();
        }
    }

}
