/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package triggerpanel;

import Igui.IguiLogger;
import TTCInfo.TrigConfL1PsKeyNamed;
import is.InfoNotCompatibleException;
import is.RepositoryNotFoundException;
import java.awt.Frame;
import java.awt.event.ItemEvent;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.SwingUtilities;


class PSSelectionDialogImpl extends PSSelectionDialog {
    private static final long serialVersionUID = 1L;

    /** Trigger Configuration panel */
    final private TriggerPanel triggerPanel;

    // Are we modifying standby or physics keys?
    private TP.RS statetype;

    // does the user want to modify something?
    private boolean return_status=false;

    // IS Named objects, provide access to the current keys
    private TrigConfL1PsKeyNamed l1IS = null;
    private TrigConfHltPsKeyNamed hltIS = null;
    private KeyDescriptor currentl1 = null;
    private KeyDescriptor currenthlt = null;

    // the selected keys
    private KeyDescriptor selectedl1  = null;
    private KeyDescriptor selectedhlt = null;

    private boolean modifiedl1  = false;
    private boolean modifiedhlt = false;
    private boolean applyalwaysactive = false;

    boolean returnStatus() {
        return return_status;
    }

    boolean isCancelled() {
        return ! return_status;
    }

    KeyDescriptor selectedL1PS() {
        return selectedl1; 
    }
    KeyDescriptor selectedHLTPS() {
        return selectedhlt; 
    }

    private KeyDescriptor getKey(final String item, final TP.LVL level) {
        int key = Integer.parseInt(item.split(":")[0]);
        return triggerPanel.getKeyDescriptor(key, level);
    }


    
    
    public PSSelectionDialogImpl(Frame parent, boolean modal, TriggerPanel tp, TP.RS state) {
        super(parent, modal);
        this.statetype = state;
        this.triggerPanel = tp;
        
        TPGlobal.getDbKeysLoader().FlagL1PskAsNotLoaded();
        TPGlobal.getDbKeysLoader().FlagHLTPskAsNotLoaded();
        
        // set up the panel
        try {
            // read the prescales for the current SMK - will block EDT, but need to because can't do anything
            // till drop down is ready
            TPGlobal.getDbKeysLoader().UpdatePrescales(triggerPanel.getCurrentSMK());
        } catch (SQLException ex) {
            Logger.getLogger(PSSelectionDialog.class.getName()).log(Level.SEVERE, null, ex);
            // TODO write to panel that it can't connect to database
        }

        // get the current keys from IS (no need to subscribe, since they can't change underneath)
        readPSKeysFromIS();

        
        if(GUIState.ReadyState() == statetype && !GUIState.emittanceState || (GUIState.emittanceState == true) && statetype == TP.RS.EMITTANCE) {
            if( (currentl1!=null  && (triggerPanel.getCurrentL1PSK()  != currentl1.key ) ) ||
                (currenthlt!=null && (triggerPanel.getCurrentHLTPSK() != currenthlt.key) ) ) 
                {
                    // Current prescales do not match current state so we activate the OK button
                    IguiLogger.info("triggerPanel.OKS " + TPGlobal.OKS);
                    IguiLogger.info("triggerPanel.getCurrentL1PSK() " + triggerPanel.getCurrentL1PSK());
                    IguiLogger.info("triggerPanel.getCurrentHLTPSK() " + triggerPanel.getCurrentHLTPSK());
                    IguiLogger.info("currentl1.key " + currentl1.key);
                    IguiLogger.info("currenthlt.key " + currenthlt.key);
                    applyalwaysactive = true;
                }
        }

        fillDropDownBox();
        
        title.setText("Select " + statetype + " prescale keys");

        setButtonStatus();
        setVisible(true);
        
    }

    /**
     * Function reads keys from IS.
     */
    private void readPSKeysFromIS() {
        
        String firstpart = "";
        if(statetype == TP.RS.EMITTANCE){
            firstpart = "RunParams.Emittance.";
        }
        else{
            firstpart = "RunParams." + ( statetype==TP.RS.STANDBY ? "Standby." : "Physics.");
        }
        l1IS  = new TrigConfL1PsKeyNamed (TPGlobal.igui.getPartition(),firstpart+"L1PsKey");
        hltIS = new TrigConfHltPsKeyNamed(TPGlobal.igui.getPartition(),firstpart+"HltPsKey");

        try { l1IS.checkout();  } catch( is.InfoNotFoundException | RepositoryNotFoundException | InfoNotCompatibleException ex) {
            IguiLogger.warning("L1PS IS object not available." + ex.getMessage() );
            l1IS.L1PrescaleKey=0;
            l1IS.L1PrescaleComment="";
        }
        try { hltIS.checkout(); } catch(is.InfoNotFoundException | RepositoryNotFoundException | InfoNotCompatibleException ex) {
            IguiLogger.warning("HLTPS IS object not available." + ex.getMessage() );
            hltIS.HltPrescaleKey=0;
            hltIS.HltPrescaleComment="";
        }

        currentl1  = triggerPanel.getKeyDescriptor(l1IS.L1PrescaleKey, TP.LVL.L1);
        currenthlt = triggerPanel.getKeyDescriptor(hltIS.HltPrescaleKey, TP.LVL.HLT);

        if(currentl1!=null) {
            curL1Label.setText (Integer.toString(currentl1.key));
        }
        if(currenthlt!=null) {
            curHLTLabel.setText(Integer.toString(currenthlt.key));
        }
    }

    private void fillDropDownBox() {
        StandbyL1CB.removeAllItems();
        StandbyHLTCB.removeAllItems();
        for(KeyDescriptor kd : TPGlobal.getDbKeysLoader().GetL1Prescales(0)) {
            if(!showHidden.isSelected() && kd.hidden) {
                continue;
            }
            StandbyL1CB.addItem(kd);
            if(kd.key==l1IS.L1PrescaleKey) { // mark current key as selected
                StandbyL1CB.setSelectedItem(kd);
            }
        }
        for(KeyDescriptor kd : TPGlobal.getDbKeysLoader().GetHLTPrescales()) {
            if(!showHidden.isSelected() && kd.hidden) {
                continue;
            }
            // String el = kd.key + ":  " + kd.name + "  v." + kd.version;
            StandbyHLTCB.addItem(kd);
            if(kd.key==hltIS.HltPrescaleKey) { // mark current key as selected
                StandbyHLTCB.setSelectedItem(kd);
            }
        }
        
    }
    
    private void setButtonStatus() {
        IguiLogger.info("Set button state L1 " + modifiedl1 + " HLT " + modifiedhlt + " always active " + applyalwaysactive );

        if(modifiedl1 || modifiedhlt) {
            selectButton.setEnabled(true);
            cancelButton.setEnabled(true);
        } else {
            selectButton.setEnabled(false);
            cancelButton.setEnabled(true);
        }

        if(applyalwaysactive) {
            selectButton.setEnabled(true);            
        }
    }
    
    @Override
    void StandbyL1CBItemStateChangedImpl(java.awt.event.ItemEvent evt) {
        if(evt.getStateChange()==ItemEvent.SELECTED) {
            selectedl1 = (KeyDescriptor) evt.getItem();
            modifiedl1 = (selectedl1.key != l1IS.L1PrescaleKey);
            IguiLogger.info("Check modified L1 = " + selectedl1.key + " != " + l1IS.L1PrescaleKey + " = " + modifiedl1 );
            setButtonStatus();
        }
    }

    @Override
    void StandbyHLTCBItemStateChangedImpl(java.awt.event.ItemEvent evt){
        if(evt.getStateChange()==ItemEvent.SELECTED) {
            selectedhlt = (KeyDescriptor) evt.getItem();
            modifiedhlt = (selectedhlt.key != hltIS.HltPrescaleKey);
            IguiLogger.info("Check modified HLT = " + selectedhlt.key + " != " + hltIS.HltPrescaleKey + " = " + modifiedhlt );
            setButtonStatus();
        }
    }

    @Override
    void showHiddenActionPerformedImpl(java.awt.event.ActionEvent evt){
        Thread worker = new Thread() {
            @Override
            public void run() {
                fillDropDownBox();
                // Report the result using invokeLater().
                SwingUtilities.invokeLater( new Runnable() {
                    @Override
                    public void run() {
                        pack();
                    }
                });
            }
        };
        worker.start();
    };

    @Override
    void selectButtonActionPerformedImpl(java.awt.event.ActionEvent evt) {
        return_status = true;
        dispose();
    }                                            
    
    @Override
    void cancelButtonActionPerformedImpl(java.awt.event.ActionEvent evt) {
        return_status=false;
        // reset
        selectedl1  = currentl1;
        selectedhlt = currenthlt;
        dispose();
    }                                            

}
