/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package triggerpanel;

import Igui.Igui;
import Igui.IguiConstants;
import Igui.IguiException;
import Igui.IguiLogger;
import Igui.IguiPanel;
import Igui.RunControlFSM;
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import triggerdb.Entities.L1.L1Menu;
import triggerdb.Entities.L1.L1Prescale;
import static triggerpanel.TPGlobal.IS;
import static triggerpanel.TPGlobal.OKS;

public class TriggerPanelMP extends IguiPanel {

    private TriggerPanelMPGUIImpl panel;

    public TriggerPanelMP(Igui igui) {
        super(igui);

        TPGlobal.init(igui);

        TPGlobal.setRoDb(this.getDb());
        TPGlobal.setRwDb(this.getRwDb());        
        
        OKS.createVariables();

        MasterTriggerInterface.instance().setMasterTrigger( getDb() );

        setLayout(new BorderLayout());
        this.panel = new TriggerPanelMPGUIImpl(this);
        add(this.panel);

        
        this.panel.psk.setText(OKS.getL1PSKVar().value().toString());
        this.panel.bgk.setText(OKS.getBGSKVar().value().toString());
                
        // define actions for buttons
        this.panel.selectSMKButton.addActionListener((ActionEvent e) -> {
            SMKSelectionDialogImpl dialog = new SMKSelectionDialogImpl(true, OKS.getSMKVar().value());
            dialog.setVisible(true);
            if(dialog.isCancelled()) {
                return;
            }
            new SMKUpdateWorker( dialog.getSelectedSMK() ).execute();
        });
        
        this.panel.selectPSKButton.addActionListener((ActionEvent e) -> {
            L1PSSelectionDialogImpl dialog = null;
            if(GUIState.currentState == RunControlFSM.State.RUNNING){
                dialog = new L1PSSelectionDialogImpl(true, OKS.getSMKVar().value(), IS.CurrentL1Psk.getinfo().L1PrescaleKey, OKS.getCTPPartitionVar().value());
            }
            else{
                dialog = new L1PSSelectionDialogImpl(true, OKS.getSMKVar().value(), OKS.getL1PSKVar().value(), OKS.getCTPPartitionVar().value());                
            }
            dialog.setVisible(true);
            if(dialog.isCancelled()) {
                return;
            }
            boolean OKSCommitAllowed = ! GUIState.currentState.follows(RunControlFSM.State.NONE);
            if(OKSCommitAllowed) {
                new L1PskOKSUpdateWorker( dialog.getSelectedPSK() ).execute();
            } else {
                new L1PskSendToTriggerUpdateWorker( dialog.getSelectedPSK() ).execute();
            }
            new L1PsTableUpdateWorker(OKS.getSMKVar().value(), dialog.getSelectedPSK().key).execute();
        });
        
        this.panel.selectBGKButton.addActionListener((ActionEvent e) -> {
            BGSelectionDialogImpl dialog = null;
            if(GUIState.currentState == RunControlFSM.State.RUNNING){
                dialog = new BGSelectionDialogImpl(true, IS.CurrentBGSk.getinfo().L1BunchGroupKey);
            }
            else{
                dialog = new BGSelectionDialogImpl(true, OKS.getBGSKVar().value());
            }
            dialog.setVisible(true);
            if(dialog.isCancelled()) {
                return;
            }
            boolean OKSCommitAllowed = ! GUIState.currentState.follows(RunControlFSM.State.NONE);
            if(OKSCommitAllowed) {
                new BgskOKSUpdateWorker( dialog.getSelectedBGSK() ).execute();            
            } else {
                new BgskSendToTriggerUpdateWorker( dialog.getSelectedBGSK() ).execute();
            }
        });
        
    }

    private int getCTPPartition() {
        return OKS.getCTPPartitionVar().value();
    }
    
    private int getActiveL1PSK() {
        return IS.CurrentL1Psk.getinfo().L1PrescaleKey;
    }
    
    private class SMKUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        private final int smk;
        SMKUpdateWorker(final int smk) {
            super();
            this.smk = smk;
        }
        @Override
        public Void doInBackground() throws SQLException {

            // write to OKS
            TPGlobal.OKS.getSMKVar().setValue(smk);
            // KeyDescriptor kdSMK = TPGlobal.getDbKeysLoader().getKeyDescriptor(smk, TP.LVL.MENU);

            // update prescales
            TPGlobal.getDbKeysLoader().UpdateL1Prescales(OKS.getSMKVar().value());

            // check if l1 psk is still valid for current smk. If not, set to 0
            final int l1psk = OKS.getL1PSKVar().value();
            KeyDescriptor kdL1PSK = TPGlobal.getDbKeysLoader().getKeyDescriptor(l1psk, TP.LVL.L1);
            if( kdL1PSK == KeyDescriptor.NotValidForSMK ) {
                OKS.getL1PSKVar().setValue(0);
            }

            return null;
        }
        @Override
        public void done() {
            IguiLogger.info("SMKVar OKS " + OKS.getSMKVar().oksvalue() + " RDB " + OKS.getSMKVar().rdbvalue());
            // update panel
            panel.UpdateFromOKS();
            panel.UpdateCommitted();
        
        }
    }

    private class L1PskOKSUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        private final KeyDescriptor l1psk;
        private L1Prescale l1ps = null;
        L1PskOKSUpdateWorker(final KeyDescriptor l1psk) {
            super();
            this.l1psk = l1psk;
        }
        @Override
        public Void doInBackground() throws SQLException {
            // write to OKS
            OKS.getL1PSKVar().setValue( this.l1psk.key );
            return null;
        }
        @Override
        public void done() {
            // update panel
            panel.UpdateFromOKS();
            panel.UpdateCommitted();
            
            panel.psk.setToolTipText(l1psk.name + "<br><it>" + l1psk.comment + "</it>");
        }
    }
    
    private class L1PskSendToTriggerUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        private final KeyDescriptor l1psk;
        L1PskSendToTriggerUpdateWorker(final KeyDescriptor l1psk) {
            super();
            this.l1psk = l1psk;
        }
        @Override
        public Void doInBackground() throws SQLException {

            if(this.l1psk.key == 0) {
                IguiLogger.warning("Not sending L1Psk 0 to MasterTrigger");
                return null;
            }

            if( l1psk.key == getActiveL1PSK() ) {
                IguiLogger.warning("Not sending " + l1psk.key + " to MasterTrigger as it is already active");
                return null;
            }

            // sending the L1 prescales to the CTP
            boolean success = MasterTriggerInterface.instance().setL1Prescales(l1psk);

            // increase lb
            if (success && (TPGlobal.igui.getRCState() == RunControlFSM.State.RUNNING) ) {
                MasterTriggerInterface.instance().increaseLB();
            }
            return null;
        }
    }
    
    private class L1PsTableUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        final int l1psk;
        final int smk;
        private L1Prescale l1ps = null;
        private L1Menu l1menu = null;
        L1PsTableUpdateWorker(final int smk, final int l1psk) {
            super();
            this.l1psk = l1psk;
            this.smk = smk;
        }
        @Override
        public Void doInBackground() throws SQLException {

            Logger.getLogger(TriggerPanelMP.class.getName()).log(Level.INFO, "Loading L1 Prescales PSK " + this.l1psk);
            
            try {
                TPGlobal.getDbKeysLoader().DbConnect();
                this.l1ps = new L1Prescale( this.l1psk );
                IguiLogger.info("Loaded L1 Prescales " + this.l1ps);
                int menuid = TPGlobal.getDbKeysLoader().GetL1MenuId(this.smk);
                this.l1menu = new L1Menu( menuid );
                IguiLogger.info("Loaded L1 Prescales " + this.l1menu);
                TPGlobal.getDbKeysLoader().DbDisconnect();
                //l1psKD = new KeyDescriptor( l1ps.get_id(), l1ps.get_name(), l1ps.get_version(), l1ps.get_comment(), false);
            } catch (SQLException ex) {
                Logger.getLogger(TriggerPanelMP.class.getName()).log(Level.SEVERE, null, ex);
            }
            
            
            return null;
        }
        @Override
        public void done() {
            panel.FillPrescaleTable(this.l1menu, this.l1ps, OKS.getCTPPartitionVar().value());
        }
    }

    private class BgskOKSUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        private final KeyDescriptor bgsk;
        BgskOKSUpdateWorker(final KeyDescriptor bgsk) {
            super();
            this.bgsk = bgsk;
        }
        @Override
        public Void doInBackground() throws SQLException {
            // write to OKS
            OKS.getBGSKVar().setValue( bgsk.key );
            return null;
        }
        @Override
        public void done() {
            // update panel
            panel.UpdateFromOKS();
            panel.UpdateCommitted();
            panel.psk.setToolTipText(bgsk.name + "<br><it>" + bgsk.comment + "</it>");
        }
    }
    
    private class BgskSendToTriggerUpdateWorker extends javax.swing.SwingWorker<Void,Void> {
        private final KeyDescriptor bgsk;
        public BgskSendToTriggerUpdateWorker(final KeyDescriptor bgsk) {
            this.bgsk = bgsk;
        }
        @Override
        protected Void doInBackground() throws Exception {

            // sending the bunch group set key to the CTP
            boolean success = MasterTriggerInterface.instance().sendBunchGroupSetToTrigger(bgsk);

            // increase lb
            if (success && (TPGlobal.igui.getRCState() == RunControlFSM.State.RUNNING) ) {
                MasterTriggerInterface.instance().increaseLB();
            }
            return null;
        }
    }
    
    @Override
    public String getPanelName() {
        return "Trigger MP";
    }

    @Override
    public String getTabName() {
        return "Trigger MP";
    }

    @Override
    public void panelInit(RunControlFSM.State state, IguiConstants.AccessControlStatus acs) throws IguiException {
        GUIState.currentState = state;
        GUIState.currentAccess = acs;
        TPGlobal.setRwDb(this.getRwDb());

        OKS.readOKS();

        // subscribe to IS
        boolean silent = false;
        IS.subscribe(silent);
        
        this.panel.UpdateStateAccessControl();

        new L1PsTableUpdateWorker(OKS.getSMKVar().value(), OKS.getL1PSKVar().value()).execute();
    }

    @Override
    public void accessControlChanged(IguiConstants.AccessControlStatus acs) {
        super.accessControlChanged(acs);
        GUIState.currentAccess = acs;
        TPGlobal.setRwDb(this.getRwDb());        
        this.panel.UpdateStateAccessControl();
        this.panel.UpdateCommitted(); // when access in control we have access to the rw DB to check if changes have been committed
    }

    @Override
    public void rcStateChanged(RunControlFSM.State state, RunControlFSM.State state1) {
        super.rcStateChanged(state, state1); //To change body of generated methods, choose Tools | Templates.
        IguiLogger.info("Change state from " + state + " to " + state1);
        GUIState.currentState = state1;
        if(GUIState.currentState == state.RUNNING){
                IS.CurrentL1Psk.bind(() -> {
                    this.panel.psk.setText(Integer.toString(IS.CurrentL1Psk.getinfo().L1PrescaleKey) );
            });
        
            IS.CurrentBGSk.bind(() -> {
                this.panel.bgk.setText(Integer.toString(IS.CurrentBGSk.getinfo().L1BunchGroupKey) );
            });

        }
        else if(GUIState.currentState == state.NONE){
         //Go back to OKS - how do you unbined from IS?
        }
        this.panel.UpdateStateAccessControl();
    }
    
    @Override
    public void panelTerminated() {
    }

    @Override
    public boolean rcStateAware() {
        return true;
    }

    @Override
    public void dbDiscarded() {
        super.dbDiscarded(); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void dbReloaded() {
        OKS.readOKS();
    }

    @Override
    public void panelDeselected() {
        super.panelDeselected(); //To change body of generated methods, choose Tools | Templates.
    }

    @Override
    public void panelSelected() {
        super.panelSelected(); //To change body of generated methods, choose Tools | Templates.
    }

}
