/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package triggerpanel;

import Igui.IguiConstants.AccessControlStatus;
import Igui.IguiLogger;
import Igui.RunControlFSM;
import TTCInfo.BaseClassReservation;
//import L1CT.L1CTISData.BaseClassReservation;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;
import triggerdb.Entities.L1.L1Item;
import triggerdb.Entities.L1.L1Menu;
import triggerdb.Entities.L1.L1Prescale;
import static triggerpanel.TPGlobal.OKS;
import static triggerpanel.TPGlobal.committedclr;
import static triggerpanel.TPGlobal.uncommittedclr;


public final class TriggerPanelMPGUIImpl extends TriggerPanelMPGUI {

    enum CTPMode {
        NONE("None", "none"), SINGLEPARTITION("SinglePartition", "single partition mode"), MULTIPARTITION("MultiPartition", "multi partition mode");
        final private String name;
        final private String desc;
        public String getName() {
            return name;
        }
        private CTPMode(final String name, final String desc) {
            this.name = name;
            this.desc = desc;
        }
        @Override
        public String toString() {
            return this.desc;
        }
    }

    TriggerPanelMP panel;
    private CTPMode ctpModeEnum;
    
    TriggerPanelMPGUIImpl(TriggerPanelMP panel) {
        this.ctpModeEnum = CTPMode.NONE;
        this.panel = panel;
        
        TPGlobal.IS.CtpConfiguration.bind(() -> {
            UpdateCTPConfiguration();
        });
     
        TPGlobal.OKS.bind(() -> {
            UpdateFromOKS();
            UpdateCommitted();
        });

        UpdateCTPConfiguration();
        UpdateFromOKS();
        UpdateStateAccessControl();
        UpdateCommitted();
    }

    void UpdateFromOKS() {
        SwingUtilities.invokeLater(() -> {
            int partNumber = OKS.getCTPPartitionVar().value();
            partition.setText(Integer.toString(partNumber));
            
            int smkVar = OKS.getSMKVar().value();
            smk.setText(Integer.toString(smkVar));
            
            int pskVar = OKS.getL1PSKVar().value();
            psk.setText(Integer.toString(pskVar));
            
            int bgkVar = OKS.getBGSKVar().value();
            bgk.setText(Integer.toString(bgkVar));
        });
    }

    void UpdateCTPConfiguration() {
        SwingUtilities.invokeLater(() -> {
            TPL.info("Updating CTP configuration");
            BaseClassReservation reservation = TPGlobal.IS.CtpConfiguration.getinfo();
            if(reservation.reserved_partition.startsWith("primary")) {
                ctpModeEnum = CTPMode.SINGLEPARTITION;
            } else if (reservation.reserved_partition.startsWith("secondary")) {
                ctpModeEnum = CTPMode.MULTIPARTITION;
            } else {
                ctpModeEnum = CTPMode.NONE;
            }
            ctpMode.setText(ctpModeEnum.toString());
            
            int smkReserved = reservation.super_master_key;
            ctpState.setText(reservation.status);
            reservedSMK.setText(Integer.toString(smkReserved));
            String p[] = {"0","none","none","none"};
            for(int i=0; i<reservation.reserved_partitions.length; i++) {
                p[reservation.reserved_partitions[i]] = reservation.reserved_partition_names[i];
            }
            partuser1.setText(p[1]);
            partuser2.setText(p[2]);
            partuser3.setText(p[3]);
            
            int smkVar = OKS.getSMKVar().value();
            String instr = "";
            switch(ctpModeEnum) {
                case SINGLEPARTITION:
                    instr = "CTP configured in single-partition mode, Run Control is in charge of the system. This partition can not configure with the Segment L1CentralTrigger enabled.";
                    break;
                case MULTIPARTITION:
                    instr = "The CTP is configured in multi-partition mode with SMK " + smkReserved + ". ";
                    if( smkReserved != smkVar) {
                        instr += "This partition can only configure with the same SMK. Please select it or disable the Segment L1CentralTrigger.";
                    }
                    break;
                case NONE:
                    instr = "The CTP is free to use. Please coordinate with the Shift Leader if you want to use it.";
                    break;
            }
            instructions.setText(instr);
        });
    }

    void UpdateStateAccessControl() {
        SwingUtilities.invokeLater(() -> {
            boolean controlEnabled = (GUIState.currentAccess == AccessControlStatus.CONTROL);
            boolean OKSCommitAllowed = ! GUIState.currentState.follows(RunControlFSM.State.NONE);
            boolean isConnected = GUIState.currentState.follows(RunControlFSM.State.CONNECTED);
            //IguiLogger.info("Control " + controlEnabled);
            //IguiLogger.info("State " + controlEnabled);
            selectSMKButton.setEnabled(controlEnabled && OKSCommitAllowed);
            selectPSKButton.setEnabled(controlEnabled && (OKSCommitAllowed || isConnected));
            selectBGKButton.setEnabled(controlEnabled && (OKSCommitAllowed || isConnected));
        });
    }
    
/**
 * Code copied from triggertool/L1Records/CutToValueConverter.java
 *
 * The trigger item is accepted if R>=C. The prescale (PS) corresponding 
 * to a cutoff C is PS = (2**24-1)/(2**24-C)
 *
 */
    private static float calculatePrescaleFromCut(int dbValue) {
        // PS = (2**24-1)/(2**24-C)
        float sign = dbValue>=0 ? 1 : -1;
        int validCut = Math.abs(dbValue);
        if(validCut==0) {
            validCut = 1;
        }
        if(validCut>0xFFFFFF) {
            validCut=0xFFFFFF;
        }
        return sign * 0xFFFFFF / (0x1000000 - validCut);
    }
   
    void FillPrescaleTable( final L1Menu menu, final L1Prescale l1ps, final int partition) {
        IguiLogger.info("Filling table with " + menu.getItems().size() + " prescales");
        DefaultTableModel tModel = (DefaultTableModel) prescaleTable.getModel();
        tModel.setRowCount(0);
        for( L1Item item : menu.getItems() ) {
            if( partition > 0 && item.get_partition() != partition ) {
                continue;
            }
            int ctpid = item.get_ctp_id(); // ctp id goes from 0 to 511
            int cutval = l1ps.get_val(ctpid+1); // VAL goes from 1 to 512
            boolean enabled = cutval>0;
            float psval = calculatePrescaleFromCut(cutval);
            Object[] entry = {ctpid, item.get_name(), psval, enabled};
            tModel.addRow(entry);
        }
        pssName.setText(l1ps.get_name());
        pssName.setToolTipText(l1ps.get_comment());
        pssID.setText("Prescale set "+l1ps.get_id());
    }

    void UpdateCommitted() {
        final boolean SMKcommitted = OKS.getSMKVar().isCommitted();
        final boolean L1PSKcommitted = OKS.getL1PSKVar().isCommitted();
        final boolean BGSKcommitted = OKS.getBGSKVar().isCommitted();

        final boolean uncommitted = ! ( SMKcommitted && L1PSKcommitted && BGSKcommitted );

        SwingUtilities.invokeLater(() -> {
            smk.setForeground(SMKcommitted ? committedclr : uncommittedclr );
            psk.setForeground(L1PSKcommitted ? committedclr : uncommittedclr );
            bgk.setForeground(BGSKcommitted ? committedclr : uncommittedclr );
            commitMsg.setVisible(uncommitted);
        });
    }
}
