/*
 * LogViewer.java
 *
 * Created on March 30, 2008, 4:04 PM
 */
package triggertool.Components;

import triggerdb.Connections.ConnectionManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;
import javax.swing.JDialog;
import javax.swing.table.DefaultTableModel;
import javax.swing.JFrame;

///Simple dialog box that shows all the entries in the trigger logs.
/**
 * Display all the entries from the trigger logs table, for atonr.  In the
 * future this table could contain bug reports and information about trigger
 * tool crashes.
 * 
 * @author  Simon Head
 */
public class LogViewer extends JDialog {

    /**
     * Constructor creates and displays the form.  It also loads all the
     * records from the database.
     * 
     * @param parent The parent form, can be null.
     * @param modal True means that whatever called this waits for it to be closed.
     */
    public LogViewer(JFrame parent) {
        super((JDialog)null);
        initComponents();
        setTitle("Log Viewer");
        load();
        setLocationRelativeTo(parent);
        setVisible(true);
    }

    /**
     * Load all the log records from the database.  This is a very simple
     * SQL query that is modified for atonr.
     */
    private void load() {
        try {
            String query = "SELECT TLOG_USERNAME, TLOG_MODIFIED_TIME, TLOG_MESSAGE, TLOG_SHORT FROM TRIGGER_LOG";
            query = ConnectionManager.getInstance().fix_schema_name(query);
            //System.out.println(query);
            
            PreparedStatement ps = ConnectionManager.getInstance().getConnection().prepareStatement(query);
            ResultSet rset = ps.executeQuery(query);

            DefaultTableModel mod = (DefaultTableModel) tableDetails.getModel();
            mod.setNumRows(0);

            while (rset.next()) {
                Vector<String> values = new Vector<>();
                values.add(rset.getString("TLOG_SHORT"));
                values.add(rset.getString("TLOG_USERNAME"));
                values.add(rset.getTimestamp("TLOG_MODIFIED_TIME").toString());
                values.add(rset.getString("TLOG_MESSAGE"));
                mod.addRow(values);
            }

            rset.close();
            ps.close();
        } catch (SQLException ex) {
            //System.out.println(ex.getMessage());
            ex.printStackTrace();
        }
    }

    /** 
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jButton1 = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        tableDetails = new javax.swing.JTable();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(420, 290));

        jButton1.setText("Done");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        tableDetails.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Type", "User", "Time", "Message"
            }
        ) {
            boolean[] canEdit = new boolean [] {
                true, false, false, false
            };

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jScrollPane1.setViewportView(tableDetails);
        if (tableDetails.getColumnModel().getColumnCount() > 0) {
            tableDetails.getColumnModel().getColumn(1).setResizable(false);
        }

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(layout.createSequentialGroup()
                        .addContainerGap()
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
                    .add(layout.createSequentialGroup()
                        .add(329, 329, 329)
                        .add(jButton1)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 201, Short.MAX_VALUE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(jButton1)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents
    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        dispose();
    }//GEN-LAST:event_jButton1ActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButton1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JTable tableDetails;
    // End of variables declaration//GEN-END:variables
}
