/*
 * PasswordDialog.java
 *
 * Created on April 11, 2008, 11:16 AM
 */
package triggertool.Components;

import javax.swing.JOptionPane;
import triggerdb.Connections.ConnectionManager;
import triggerdb.Connections.UserMode;

///Dialog for atonr for switching user and/or user level
/**
 * Dialog box for atonr users to change and alter their user level.
 * 
 * @author Simon Head
 */
public class PasswordDialog extends javax.swing.JDialog {

    /** New mode that the user has requested */
    private UserMode newUserMode = UserMode.UNKNOWN;

    /**
     * Construct and show the dialog box
     * 
     * @param parent Parent form.
     * @param modal True means wait till this form is destroyed.
     * @param newUserMode The new mode the user wants.
     */
    public PasswordDialog(java.awt.Frame parent, boolean modal, UserMode newUserMode) {
        super(parent, modal);
        initComponents();

        this.newUserMode = newUserMode;
        String level = ConnectionManager.getInstance().getInitInfo().getUserMode().toString();
        String user = ConnectionManager.getInstance().getInitInfo().getSystemUsername();

        labelInfo.setText("Currently: " + level + " " + user);

        if (newUserMode == UserMode.USER && ConnectionManager.getInstance().getInitInfo().getUserMode().ordinal() > UserMode.USER.ordinal()) {
            Object[] options = {"Yes, please", "No, thanks"};

            int n = JOptionPane.showOptionDialog(null,
                    "Change to readonly mode - Are you sure?",
                    "Change Mode?",
                    JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null,
                    options,
                    options[1]);

            if (n == 0) {
                ConnectionManager.getInstance().getInitInfo().setUserMode(newUserMode);
            }
            dispose();
            return;
        } else if (newUserMode.ordinal() <= UserMode.USER.ordinal()) {
            dispose();
            return;
        }

        textUsername.setText(user);
        buttonDone.setEnabled(false);
        textPassword.setText("");
        setVisible(true);
    }

    /** 
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        buttonDone = new javax.swing.JButton();
        buttonCancel = new javax.swing.JButton();
        textUsername = new javax.swing.JTextField();
        labelInfo = new javax.swing.JLabel();
        textPassword = new javax.swing.JPasswordField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        jLabel1.setText("Username:");

        jLabel2.setText("Password");

        buttonDone.setText("Ok");
        buttonDone.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonDoneActionPerformed(evt);
            }
        });

        buttonCancel.setText("Cancel");
        buttonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonCancelActionPerformed(evt);
            }
        });

        textUsername.setText("jTextField1");

        labelInfo.setText("jLabel3");

        textPassword.setText("jPasswordField1");
        textPassword.addFocusListener(new java.awt.event.FocusAdapter() {
            public void focusGained(java.awt.event.FocusEvent evt) {
                textPasswordFocusGained(evt);
            }
        });
        textPassword.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                textPasswordMouseClicked(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(jLabel1)
                            .add(jLabel2))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(textPassword, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE)
                            .add(textUsername, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 301, Short.MAX_VALUE))
                        .add(3, 3, 3))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(buttonCancel)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(buttonDone)
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .add(labelInfo)
                        .addContainerGap(335, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(labelInfo)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 18, Short.MAX_VALUE)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(textUsername, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(textPassword, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .add(33, 33, 33)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(buttonDone)
                    .add(buttonCancel))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * The user has entered their password.  Try and change to the new mode and
     * close the window.
     * 
     * @param evt The event that caused this to run.
     */
private void buttonDoneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonDoneActionPerformed
    ConnectionManager.getInstance().changeUser(textUsername.getText(), new String(textPassword.getPassword()), newUserMode);
    dispose();
}//GEN-LAST:event_buttonDoneActionPerformed

    /**
     * Just close the window. 
     * 
     * @param evt The event that caused this to run.
     */
private void buttonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonCancelActionPerformed
    dispose();
}//GEN-LAST:event_buttonCancelActionPerformed

private void textPasswordMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_textPasswordMouseClicked
}//GEN-LAST:event_textPasswordMouseClicked

    /**
     * The password box has been clicked.  Assume that they've written in their
     * password.  So we can enable the ok button.
     * 
     * @param evt The event that fires this action.
     */
private void textPasswordFocusGained(java.awt.event.FocusEvent evt) {//GEN-FIRST:event_textPasswordFocusGained
    buttonDone.setEnabled(true);
}//GEN-LAST:event_textPasswordFocusGained

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton buttonCancel;
    private javax.swing.JButton buttonDone;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel labelInfo;
    private javax.swing.JPasswordField textPassword;
    private javax.swing.JTextField textUsername;
    // End of variables declaration//GEN-END:variables
}
