package triggertool.Components;

import triggerdb.Connections.ConnectionManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

///Dialog box to change atonr user rights.
/**
 * Trial version of a dialog box to enabled trigger meisters to change the 
 * access rights of other trigger tool users on atonr. Also allows users to be 
 * added and removed.
 * 
 * @author Simon Head
 */
public class UsersDialogExpert extends JDialog implements ListSelectionListener {

    /**
     *
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");
    ///List of current users.
    private DefaultListModel<String> items = new DefaultListModel<>();

    ///Creates new form UsersDialog.
    /**
     * Create and display the form to add / modify user access rights.
     * 
     * @param parent Parent form can be null.
     */
    public UsersDialogExpert(java.awt.Frame parent) {
        super((JDialog)null);
        initComponents();
        listUsers.setModel(items);
        listUsers.addListSelectionListener(this);
        comboLevel.setSelectedIndex(-1);

        buttonClearFieldsActionPerformed(null);
        load();

        setVisible(true);
    }

    ///Load all the users from the database,.
    /**
     * Load all the users and add them to the list on the left.
     */
    private void load() {
        items.clear();

        try {
            String query = "SELECT TT_LEVEL, TT_USER FROM TT_USERS ORDER BY TT_USER ASC";
            query = ConnectionManager.getInstance().fix_schema_name(query);
            //System.out.println(query);
            PreparedStatement ps = ConnectionManager.getInstance().getConnection().prepareStatement(query);
            ResultSet rset = ps.executeQuery();

            while (rset.next()) {
                items.addElement(rset.getString("TT_USER") + " -- " + rset.getString("TT_LEVEL"));
            }

            rset.close();
            ps.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            logger.log(Level.SEVERE, "Exception: {0}", ex);
        }
    }

    ///Netbeans. Don't edit.
    /** 
     * This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonDone = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        listUsers = new javax.swing.JList<String>();
        jPanel1 = new javax.swing.JPanel();
        textUsername = new javax.swing.JTextField();
        comboLevel = new javax.swing.JComboBox<String>();
        textPassword = new javax.swing.JPasswordField();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        buttonSave = new javax.swing.JButton();
        buttonClearFields = new javax.swing.JButton();
        buttonDelete = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);

        buttonDone.setText("Done");
        buttonDone.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonDoneActionPerformed(evt);
            }
        });

        jScrollPane1.setViewportView(listUsers);

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Selected User Details"));

        textUsername.setText("jTextField1");

        comboLevel.setModel(new javax.swing.DefaultComboBoxModel<String>(new String[] {"Shifter", "Expert", "DBA"}));

        textPassword.setText("jPasswordField1");

        jLabel1.setText("Username");

        jLabel2.setText("Level");

        jLabel3.setText("Password");

        org.jdesktop.layout.GroupLayout jPanel1Layout = new org.jdesktop.layout.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel1)
                    .add(jLabel2)
                    .add(jLabel3))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(textUsername, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 161, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, comboLevel, 0, 161, Short.MAX_VALUE)
                    .add(textPassword, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 161, Short.MAX_VALUE)))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(textUsername, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jLabel2)
                    .add(comboLevel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jPanel1Layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel3)
                    .add(textPassword, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(114, Short.MAX_VALUE))
        );

        buttonSave.setText("Save User");
        buttonSave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonSaveActionPerformed(evt);
            }
        });

        buttonClearFields.setText("Clear Fields");
        buttonClearFields.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonClearFieldsActionPerformed(evt);
            }
        });

        buttonDelete.setText("Delete User");
        buttonDelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonDeleteActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .add(buttonClearFields)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(buttonDelete)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(buttonSave)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(buttonDone))
                    .add(layout.createSequentialGroup()
                        .add(jScrollPane1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 151, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(jPanel1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 259, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(buttonDone)
                    .add(buttonSave)
                    .add(buttonClearFields)
                    .add(buttonDelete))
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    ///Done button, close the form.
    /**
     * Close the form.
     * 
     * @param evt Event that started this function.
     */
private void buttonDoneActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonDoneActionPerformed
    dispose();
}//GEN-LAST:event_buttonDoneActionPerformed

    ///Save the user information.
    /**
     * Save the user.
     * 
     * @param evt Event that started this function.
     */
private void buttonSaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonSaveActionPerformed
    if (comboLevel.getSelectedIndex() == -1 || textUsername.getText().isEmpty() || new String(textPassword.getPassword()).isEmpty()) {
        JOptionPane.showMessageDialog(this, "Fill in the username, access level and password first.", "Information Incomplete", JOptionPane.INFORMATION_MESSAGE);
        return;
    }

    String original_user = "";

    if (listUsers.getSelectedIndex() >= 0) {
        original_user = listUsers.getSelectedValue().split(" -- ")[0];
    }

    try {
        String query = "SELECT COUNT(TT_USER) FROM TT_USERS WHERE TT_USER=?";
        query = ConnectionManager.getInstance().fix_schema_name(query);
        PreparedStatement ps = ConnectionManager.getInstance().getConnection().prepareStatement(query);
        ps.setString(1, textUsername.getText());

        ResultSet rset = ps.executeQuery();
        int count = 0;
        while (rset.next()) {
            count = rset.getInt("TT_USER");
        }
        rset.close();
        ps.close();

        int n = 1;

        if (count == 0) {
            query = "INSERT INTO TT_USERS (TT_LEVEL, TT_USER, TT_PASSWORD) VALUES (?,?,?)";
        } else {
            Object[] options = {"Yes, please", "No, thanks"};

            n = JOptionPane.showOptionDialog(null,
                    "This user already exists, do you want to update their information?",
                    "User exists",
                    JOptionPane.YES_NO_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    null,
                    options,
                    options[1]);

            if (n == 0) {
                query = "UPDATE TT_USERS SET TT_LEVEL=?, TT_USER=?, TT_PASSWORD=? WHERE TT_USER=?";
            } else {
                return;
            }
        }

        query = ConnectionManager.getInstance().fix_schema_name(query);

        ps = ConnectionManager.getInstance().getConnection().prepareStatement(query);

        if (n == 0) {
            ps.setString(4, original_user);
        }

        ps.setString(1, comboLevel.getSelectedItem().toString());
        ps.setString(2, textUsername.getText());

        String pw = ConnectionManager.getInstance().getInitInfo().encrypt(new String(textPassword.getPassword()));

        ps.setString(3, pw);
        ps.executeUpdate();

        ps.close();
    } catch (SQLException ex) {
        ex.printStackTrace();
        logger.log(Level.SEVERE, "Exception: {0}", ex);
    }

    load();
}//GEN-LAST:event_buttonSaveActionPerformed

    ///Clear all the fields for the user.
    /**
     * Clear the user fields.
     * 
     * @param evt The event that starts this function.
     */
private void buttonClearFieldsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonClearFieldsActionPerformed
    textUsername.setText("");
    //comboLevel.setSelectedItem("User");
    comboLevel.setSelectedIndex(-1);
    textPassword.setText("");
}//GEN-LAST:event_buttonClearFieldsActionPerformed

    ///Delete button.
    /**
     * Remove a user from the database. First prompt the user.
     * @param evt
     */
private void buttonDeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonDeleteActionPerformed
    Object[] options = {"Yes, please", "No, thanks"};

    int n = JOptionPane.showOptionDialog(null, "Delete user " + textUsername.getText() + "?", "Delete User?", JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE, null, options, options[1]);

    if (n != 0) {
        return;
    }

    String query = "DELETE TT_USERS WHERE TT_USER=?";
    query = ConnectionManager.getInstance().fix_schema_name(query);

    try {
        PreparedStatement ps = ConnectionManager.getInstance().getConnection().prepareStatement(query);//GEN-LAST:event_buttonDeleteActionPerformed
        ps.setString(1, textUsername.getText());
        ps.executeUpdate();
        ps.close();
        load();
    }catch (SQLException ex) {
        logger.log(Level.SEVERE, "Exception: {0}", ex);
    }
}
    // Variables declaration - do not modify//GEN-BEGIN:variables
    ///Clear fields button.
    private javax.swing.JButton buttonClearFields;
    ///Remove user button.
    private javax.swing.JButton buttonDelete;
    ///Close the window button.
    private javax.swing.JButton buttonDone;
    ///Save user button.
    private javax.swing.JButton buttonSave;
    ///Access level.
    private javax.swing.JComboBox<String> comboLevel;
    ///Username label.
    private javax.swing.JLabel jLabel1;
    ///Level label.
    private javax.swing.JLabel jLabel2;
    ///Password label.
    private javax.swing.JLabel jLabel3;
    ///Box.
    private javax.swing.JPanel jPanel1;
    ///Scrolly Pane
    private javax.swing.JScrollPane jScrollPane1;
    ///User list.
    private javax.swing.JList<String> listUsers;
    ///Password.
    private javax.swing.JPasswordField textPassword;
    ///Username.
    private javax.swing.JTextField textUsername;
    // End of variables declaration//GEN-END:variables

    ///Run when list is changed.
    /**
     * When a value in the list of users has changed run this code.
     * 
     * @param e Event Selection.
     */
    @Override
    public void valueChanged(ListSelectionEvent e) {

        if (!e.getValueIsAdjusting() && listUsers.getSelectedIndex() >= 0) {
            String username = listUsers.getSelectedValue().split(" -- ")[0];
            String level = listUsers.getSelectedValue().split(" -- ")[1];
            textUsername.setText(username);
            comboLevel.setSelectedItem(level);
            textPassword.setText("abcdef");
        }
    }
}
