/*
 * DiffDisplayDialog.java
 *
 * Created on April 8, 2009, 4:43 PM
 */

package triggertool.Diff;

import java.util.logging.Logger;
import java.awt.Cursor;
import java.sql.SQLException;
import java.util.logging.Level;
import javax.swing.JDialog;
import javax.swing.JOptionPane;
import triggerdb.Entities.AbstractTable;
import triggerdb.Entities.SMT.SuperMasterTable;

/**
 *
 * @author  markowen
 */
public final class DiffDisplayDialog extends JDialog {

    /// The tree of differences
    private DiffTree diffTree = null;
    
    /// Super master key selection dialog
    DiffSMTSelDialog seldialog;
    
    ///Message Log.

    /**
     *
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");
    
    /** Creates new form DiffDisplayDialog
     *  The tables to be compated are passed as arguments & the results are displayed. 
     * @param parent
     * @param modal
     * @param t1
     * @param t2
     */
    public DiffDisplayDialog(java.awt.Frame parent, boolean modal, AbstractTable t1, AbstractTable t2) {
        super((JDialog)null);
        diffTree = new DiffTree();
	try {
	    this.updateTree(t1,t2);
	} catch(java.lang.OutOfMemoryError ex) {
	    logger.warning("Out of memory exception, quitting diffing");
	    dispose();
	} catch (SQLException ex) {
            String message = "Error occurred while creating the Diff Display Dialog panel.";
            logger.log(Level.SEVERE, message, ex);
            JOptionPane.showMessageDialog(parent, "SQL Error" , message +
            " '\n Please report this error attaching the log file. This can be found in " + triggertool.TriggerTool.logFileName, JOptionPane.ERROR_MESSAGE);
        }
        initComponents();
        setTitle("Diff Results");
        setLocationRelativeTo(parent);
    }
    
    /** Creates new form DiffDisplayDialog
     * @param parent
     * @throws java.sql.SQLException */
    public DiffDisplayDialog(java.awt.Frame parent) throws SQLException {
        super((JDialog)null);
        diffTree = new DiffTree();
        seldialog = new DiffSMTSelDialog(parent, true);
        
        if(seldialog.doDiff()) {
            int smt1_id = seldialog.GetSMK1();
            int smt2_id = seldialog.GetSMK2();
            SuperMasterTable t1 = new SuperMasterTable(smt1_id);
            SuperMasterTable t2 = new SuperMasterTable(smt2_id);
            initComponents();
            setLocationRelativeTo(parent);
            setVisible(true);
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            try {
		this.updateTree(t1,t2);
	    } catch(java.lang.OutOfMemoryError ex) {
		logger.warning("Out of memory exception, quitting diffing");
		dispose();
	    } catch (SQLException ex) {
                String message = "Error occurred while creating the Diff Display Dialog panel.";
                logger.log(Level.SEVERE, message, ex);
                JOptionPane.showMessageDialog(parent, "SQL Error" , message +
                " '\n Please report this error attaching the log file. This can be found in " + triggertool.TriggerTool.logFileName, JOptionPane.ERROR_MESSAGE);
            } finally {
		setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            }
        }
        else {
            dispose();
        }
    }

    /**
     *
     * @param t1
     * @param t2
     * @throws SQLException
     */
    public void updateTree(AbstractTable t1, AbstractTable t2) throws SQLException {
        diffTree.update(t1,t2);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane(diffTree);
        jLabel1 = new javax.swing.JLabel();
        doneButton = new javax.swing.JButton();
        expandAllButton = new javax.swing.JButton();
        collapseAllButton = new javax.swing.JButton();
        xmlOutButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(697, 372));

        jLabel1.setFont(new java.awt.Font("Lucida Grande", 0, 18)); // NOI18N
        jLabel1.setText("Diff Results");

        doneButton.setText("Done");
        doneButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                doneButtonActionPerformed(evt);
            }
        });

        expandAllButton.setText("Expand All");
        expandAllButton.setDefaultCapable(false);
        expandAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                expandAllButtonActionPerformed(evt);
            }
        });

        collapseAllButton.setText("Collapse All");
        collapseAllButton.setDefaultCapable(false);
        collapseAllButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                collapseAllButtonActionPerformed(evt);
            }
        });

        xmlOutButton.setText("Output XML");
        xmlOutButton.setEnabled(false);
        xmlOutButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                xmlOutButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 650, Short.MAX_VALUE)
                    .add(layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, 205, Short.MAX_VALUE)
                        .add(xmlOutButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                        .add(collapseAllButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(expandAllButton)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(doneButton)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(doneButton)
                    .add(expandAllButton)
                    .add(collapseAllButton)
                    .add(xmlOutButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane1, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

private void doneButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_doneButtonActionPerformed
    dispose();
}//GEN-LAST:event_doneButtonActionPerformed

private void expandAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_expandAllButtonActionPerformed
    setCursor(new Cursor(Cursor.WAIT_CURSOR));
    logger.log(Level.FINE, "Expanding {0} rows", diffTree.getRowCount());
    for(int i=0; i<diffTree.getRowCount(); ++i) {
	logger.log(Level.FINE, "Expanding {0}", i);
	diffTree.expandRow(i);
    }
    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
}//GEN-LAST:event_expandAllButtonActionPerformed

private void collapseAllButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_collapseAllButtonActionPerformed
    setCursor(new Cursor(Cursor.WAIT_CURSOR));
    int row=diffTree.getRowCount()-1;
    int rowcount=diffTree.getRowCount();
    while(row>-1) {
        diffTree.collapseRow(row);
        if(diffTree.getRowCount()==rowcount) { // no shrinking, move o next row
            row--;
        } else {
            row = diffTree.getRowCount()-1;
        }
        rowcount=diffTree.getRowCount();
    }
    //for(int i=0; i<diffTree.getRowCount(); ++i) diffTree.collapseRow(i);
    setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
}//GEN-LAST:event_collapseAllButtonActionPerformed

private void xmlOutButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_xmlOutButtonActionPerformed
    diffTree.outputXML("diff.xml");
}//GEN-LAST:event_xmlOutButtonActionPerformed


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton collapseAllButton;
    private javax.swing.JButton doneButton;
    private javax.swing.JButton expandAllButton;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton xmlOutButton;
    // End of variables declaration//GEN-END:variables

}
