/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package triggertool.Diff;

import java.awt.Point;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.StringSelection;
import java.awt.datatransfer.Transferable;
import java.awt.event.*;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.TimeZone;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.tree.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Comment;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import triggerdb.Diff.DiffNewResult;
import triggerdb.Diff.TopNode;
import triggerdb.DiffMutableTreeNode;
import triggerdb.Entities.AbstractTable;
import triggerdb.Entities.HLT.HLTComponent;

/**
 *  Class for the tree view of the results of diffing two objects.
 *
 * @author markowen
 */
public class DiffTree extends JTree implements ActionListener,
        TreeExpansionListener, TreeWillExpandListener, TreeSelectionListener,
        MouseListener, MouseMotionListener {

    /**
     *
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");
    private static final long serialVersionUID = 1L;
    private final DefaultMutableTreeNode top;
    private final DefaultTreeModel myTreeModel;

    private final ResultsTransfer toClipboard = new ResultsTransfer();

    /// selection rectangle
    private Rectangle selRec = null;
    /// starting point of selection rectangle
    private Point recStart = null;

    /**
     *
     */
    public DiffTree() {
        top = new DefaultMutableTreeNode("");
        myTreeModel = new DefaultTreeModel(top);
     
        initDiff();
    }

    private void initDiff() {
        setModel(myTreeModel);
        
        getSelectionModel().setSelectionMode(TreeSelectionModel.CONTIGUOUS_TREE_SELECTION);

        addTreeSelectionListener(this);
        addTreeExpansionListener(this);
        addTreeWillExpandListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
    }
    
    /**
     *
     * @param table1
     * @param table2
     * @throws SQLException
     */
    public void update(AbstractTable table1, AbstractTable table2) throws SQLException {
        myTreeModel.reload();
        DefaultMutableTreeNode treeNode = new DefaultMutableTreeNode(new TopNode(table1,table2));
        myTreeModel.setRoot(treeNode);
        //Runtime r = Runtime.getRuntime();
	//System.out.println("Free Memory  " + (r.freeMemory()/(1024.0*1024.0)));
	table1.doDiff(table2,treeNode,null);
        //System.out.println("Diff done:");
	//System.out.println("Free Memory  " + (r.freeMemory()/(1024.0*1024.0)));
	fireTreeExpanded(null);
    }
    
    @Override
    public void treeCollapsed(TreeExpansionEvent event) {
    }

    @Override
    public void treeExpanded(TreeExpansionEvent event) {
    }

    @Override
    public void treeWillCollapse(TreeExpansionEvent event) throws ExpandVetoException {
    }

    @Override
    public void treeWillExpand(TreeExpansionEvent event) throws ExpandVetoException {
        DefaultMutableTreeNode nd = (DefaultMutableTreeNode) event.getPath().getLastPathComponent();

        if (nd != null && nd.getUserObject() instanceof AbstractTable) {

        }
    }

    /**
     * Function to send differences to an xml file.
     * @param diff_filename - name of the xml file.
     * @throws java.sql.SQLException
     */
    public void outputXML(String diff_filename)  {

        /// Put date at top of file
        String DATE_FORMAT = "yyyy-MM-dd HH:mm:ss";
        java.text.SimpleDateFormat sdf = new java.text.SimpleDateFormat(DATE_FORMAT);

        Calendar cal = Calendar.getInstance(TimeZone.getDefault());
        sdf.setTimeZone(TimeZone.getDefault());

        /// New way
        try {
            DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
            DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
            Document doc = docBuilder.newDocument();
            // add some comments
            Comment comment1 = doc.createComment("This file is generated by the TriggerTool");
            doc.appendChild(comment1);
            Comment comment2 = doc.createComment( "On " + sdf.format(cal.getTime()) );
            doc.appendChild(comment2);

            // pass the head object from the tree to the writing function
            DefaultMutableTreeNode root = (DefaultMutableTreeNode)myTreeModel.getRoot();
            writeNodeNew(root,null,doc);
            
            /// output to file
            java.io.FileOutputStream ofilestream = new java.io.FileOutputStream( diff_filename );
            DOMSource domSource = new DOMSource(doc);
            TransformerFactory tFactory = TransformerFactory.newInstance();
            tFactory.setAttribute("indent-number", 4); // number of spaces per indent

            Transformer transformer = tFactory.newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT , "yes");
            transformer.transform(domSource, new StreamResult(new java.io.OutputStreamWriter(ofilestream, "utf-8")));

        } catch(ParserConfigurationException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        } catch (DOMException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        }
        catch (FileNotFoundException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        }
        catch (IllegalArgumentException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        }
        catch (UnsupportedEncodingException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        }
        catch (TransformerException parsex) {
            logger.severe("Caught Exception when outputing xml");
            parsex.printStackTrace();
        }
    }

    /**
     * This function takes a node of the tree and puts it into the xml
     * document
     *
     * @param node - node to write out
     * @param root - the parent element in the xml file (can be null for top level element)
     * @param doc - xml document to write into
     */
    private void writeNodeNew(DefaultMutableTreeNode node, Element root, Document doc)  {

        // Get the string from the Tree node
        String nodeline = node.getUserObject().toString();
        String[] splitnodeline = nodeline.split(" ");

        Element diffele = null;

        /// Special case for HLT Trigger Signature, since the names have spaces in.
        if(splitnodeline[0].compareTo((new triggerdb.Entities.HLT.HLTTriggerSignature()).getTableName())==0) {
            String name = nodeline.substring((new triggerdb.Entities.HLT.HLTTriggerSignature()).getTableName().length()+1);
            //out.write("<" + splitnodeline[0] + " name=\"" + name + "\">\n");
            diffele = doc.createElement(splitnodeline[0]);
            diffele.setAttribute("name", name);
        } else { //not hlt signature

            if(splitnodeline.length>4) {
                logger.warning("Unexpected length in writeNode, xml file will be missing something (see next line)");
                logger.warning(nodeline);
            }
            if(splitnodeline.length==4) { //special cases
                if(splitnodeline[0].compareTo((new triggerdb.Entities.HLT.HLTTE_TE()).getTableName())==0) { // HLT TE_TE
                    //out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[3] + "\">\n");
                    diffele = doc.createElement(splitnodeline[0]);
                    diffele.setAttribute("name", splitnodeline[3]);
                }
            }//length==4
            if(splitnodeline.length==3) { // special cases
                if(splitnodeline[0].compareTo((new triggerdb.Entities.SMT.SuperMasterTable()).getTableName())==0 || root==null) { //super master or first node
                    String[] id1 = splitnodeline[1].split("=");
                    String[] id2 = splitnodeline[2].split("=");
                    //out.write("<" + splitnodeline[0] + " " + id1[0] + "=\"" + id1[1] + "\" " + id2[0] + "=\"" + id2[1] + "\">\n");
                    diffele = doc.createElement(splitnodeline[0]);
                    diffele.setAttribute(id1[0], id1[1]);
                    diffele.setAttribute(id2[0], id2[1]);                    
                }
                if(splitnodeline[0].compareTo((new HLTComponent()).getTableName())==0) { // hlt comp
                    //out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[1] + "\" alias=\"" + splitnodeline[2] + "\">\n");
                    diffele = doc.createElement(splitnodeline[0]);
                    diffele.setAttribute("name", splitnodeline[1]);
                    diffele.setAttribute("alias", splitnodeline[2]);
                }
            }//length==3
            if(splitnodeline.length==2) {
                //out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[1] + "\">\n");
                diffele = doc.createElement(splitnodeline[0]);
                diffele.setAttribute("name", splitnodeline[1]);
            }
            if(splitnodeline.length==1) {
                //out.write("<" + nodeline + ">\n");
                diffele = doc.createElement(nodeline);
            }
        }//not HLT signature

        if(diffele==null) {
            logger.log(Level.WARNING, "Did not fill xml correctly for {0}", nodeline);
            logger.warning("XML diff file likely to be incomplete - please contact triggertool developers.");
            return;
        }

        if(root!=null) root.appendChild(diffele);
        else doc.appendChild(diffele);

        // Loop over the children of this node
        for(int i=0; i<node.getChildCount(); ++i) {

            DefaultMutableTreeNode child = (DefaultMutableTreeNode)node.getChildAt(i);

            //If terminating leaf, then write out the differences.
            if(child.isLeaf()) {
                String childline = child.getUserObject().toString();
                String[] childlinesplit = childline.split(": ");

                if(child instanceof DiffMutableTreeNode) {
                    DiffMutableTreeNode ver = (DiffMutableTreeNode)child;
                    //out.write("<" + ver.getKey() + " val1=\"" + ver.getVal1() + "\" val2=\"" + ver.getVal2() + "\"/>\n");
                    Element childele = doc.createElement(ver.getKey());
                    childele.setAttribute("val1", ver.getVal1());
                    childele.setAttribute("val2", ver.getVal2());
                    diffele.appendChild(childele);
                } else { // should be a 'new' case
                    if(childlinesplit.length==2) { //this case works for most things where these is only one :
                        /// Check it's NEW
                        if(childlinesplit[0].split(" ")[0].compareTo("New")==0) {
                            //out.write("<NEW type=\"" + childlinesplit[0].split(" ")[1] + "\" name=\"" + childlinesplit[1] + "\"/>\n");
                            Element childele = doc.createElement("NEW");
                            childele.setAttribute("type", childlinesplit[0].split(" ")[1]);
                            childele.setAttribute("name", childlinesplit[1]);
                            diffele.appendChild(childele);
                        }
                        else logger.log(Level.WARNING, "Item was not a NEW one: {0}", childline);

                    }
                    else logger.log(Level.WARNING, "Problem writing out child in split 1: {0}", childline);
                }// not DiffMutableTreeNode
                
            }
            else { // Not a terminating leaf, so re-call this function on the child node
                writeNodeNew(child,diffele,doc);
            }
        }//loop over children
    }

    /**
     * This function takes a node of the tree and sends it to a file in xml format.
     *
     * @param node - node to write out
     * @param out - the BufferedWriter to write to
     * @param tablevel - current tab level in the output file
     * @throws java.io.IOException
     */
    private void writeNode(DefaultMutableTreeNode node, BufferedWriter out, int tablevel) throws IOException, SQLException {
        
        String nodeline = node.getUserObject().toString();
        String[] splitnodeline = nodeline.split(" ");
        
        for(int j=0; j<tablevel; ++j) out.write("  ");

        /// Special case for HLT Trigger Signature, since the names have spaces in.
        if(splitnodeline[0].compareTo((new triggerdb.Entities.HLT.HLTTriggerSignature(-1)).getTableName())==0) {
            String name = nodeline.substring((new triggerdb.Entities.HLT.HLTTriggerSignature(-1)).getTableName().length()+1);
            out.write("<" + splitnodeline[0] + " name=\"" + name + "\">\n");
        } else { //not hlt signature

            if(splitnodeline.length>4) {
                logger.warning("Unexpected length in writeNode, xml file will be missing something (see next line)");
                logger.warning(nodeline);
            }
            if(splitnodeline.length==4) { //special cases
                if(splitnodeline[0].compareTo((new triggerdb.Entities.HLT.HLTTE_TE(-1)).getTableName())==0) { // HLT TE_TE
                    out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[3] + "\">\n");
                }
            }//length==4
            if(splitnodeline.length==3) { // special cases
                if(splitnodeline[0].compareTo((new triggerdb.Entities.SMT.SuperMasterTable(-1)).getTableName())==0) { //super master
                    String[] id1 = splitnodeline[1].split("=");
                    String[] id2 = splitnodeline[2].split("=");
                    out.write("<" + splitnodeline[0] + " " + id1[0] + "=\"" + id1[1] + "\" " + id2[0] + "=\"" + id2[1] + "\">\n");
                }
                if(splitnodeline[0].compareTo((new HLTComponent()).getTableName())==0) { // hlt comp
                    out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[1] + "\" alias=\"" + splitnodeline[2] + "\">\n");
                }
            }//length==3
            if(splitnodeline.length==2) {
                out.write("<" + splitnodeline[0] + " name=\"" + splitnodeline[1] + "\">\n");
            }
            if(splitnodeline.length==1) {
                out.write("<" + nodeline + ">\n");
            }
        } //not HLT signature
        String closer = splitnodeline[0];

        // Loop over the children of this node
        for(int i=0; i<node.getChildCount(); ++i) {

            DefaultMutableTreeNode child = (DefaultMutableTreeNode)node.getChildAt(i);

            //If terminating leaf, then write out the differences.
            if(child.isLeaf()) {
                for(int j=0; j<tablevel+1; ++j) out.write("  ");
                String childline = child.getUserObject().toString();
                String[] childlinesplit = childline.split(": ");
                
                /// Version changes are stored in the specific DiffMutableTreeNode class
                if(child instanceof DiffMutableTreeNode) {
                    DiffMutableTreeNode ver = (DiffMutableTreeNode)child;
                    out.write("<" + ver.getKey() + " val1=\"" + ver.getVal1() + "\" val2=\"" + ver.getVal2() + "\"/>\n");
                } else { // should be a 'new' case
                    if(childlinesplit.length==2) { //this case works for most things where these is only one :
                        /// Check it's NEW
                        if(childlinesplit[0].split(" ")[0].compareTo("New")==0) {
                            out.write("<NEW type=\"" + childlinesplit[0].split(" ")[1] + "\" name=\"" + childlinesplit[1] + "\"/>\n");
                        }
                        else logger.log(Level.WARNING, "Item was not a NEW one: {0}", childline);
                    
                    }
                    else logger.log(Level.WARNING, "Problem writing out child in split 1: {0}", childline);
                }// not DiffMutableTreeNode
                
            }
            else { // Not a terminating leaf, so re-call this function on the child node, increasing tab level by 1.
                //out.write("NOT LEAF " + child.getUserObject().toString() + "\n");
                writeNode(child,out,tablevel+1);
            }
        }
        for(int j=0; j<tablevel; ++j) out.write("  ");
        // Close this node
        out.write("</" + closer + ">\n");
        
    }
    
    @Override
    public void valueChanged(TreeSelectionEvent e) {

    }

    @Override
    public void mouseClicked(MouseEvent e) {

        // for right click we display a pop-up menu to allow copying to clipboard & text file
        if(e.getButton()==java.awt.event.MouseEvent.BUTTON3) { //right click

            // Show pop-up menu
            JPopupMenu popupmenu = new JPopupMenu();
            JMenuItem copyClipIt = new JMenuItem("Copy selected to clipboard");
            copyClipIt.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {

                    int selrows[] = DiffTree.this.getSelectionRows();
                    if(selrows==null) return;
                    if(selrows.length==0) return;
                    String forclipboard = new String();
                    for(int row : selrows) {
                        DefaultMutableTreeNode node = (DefaultMutableTreeNode)DiffTree.this.getPathForRow(row).getLastPathComponent();

                        forclipboard += node.toString();
                        forclipboard += "\n";
                    }
                    toClipboard.setClipboardContents(forclipboard);
                }
            });
            popupmenu.add(copyClipIt);

            JMenuItem copyTextIt = new JMenuItem("Copy selected to text file");
            copyTextIt.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {

                    int selrows[] = DiffTree.this.getSelectionRows();
                    if(selrows==null) return;
                    if(selrows.length==0) return;

                    boolean gotgoodname = false;

                    String filename = null;
                    while(!gotgoodname) {
                        filename = JOptionPane.showInputDialog("Enter file name for text dump");

                        //If user selects cancel, give up
                        if(filename==null) return;

                        // try again for empty filename
                        if(filename.length()==0) continue;

                        // see if the file exists
                        try{
                            FileReader fileReader = new FileReader(filename);
                            // if file exists - warn user
                            final int result = JOptionPane.showConfirmDialog(null, "Warning file " + filename + " exists - are you sure you want to override it?");
                            if(result == JOptionPane.OK_OPTION) {
                                gotgoodname=true;
                            }

                        } catch (FileNotFoundException ex) { // good file name if it does not exist
                            gotgoodname=true;
                        }
                    }//!good file name

                    /// Open file for writing
                    FileWriter outputStream;
                    try {
                        outputStream = new FileWriter(filename);
                    } catch(IOException exception) {
                        JOptionPane.showMessageDialog(null, "File " + filename + " could not be opened", "Error", JOptionPane.ERROR_MESSAGE);
                        logger.warning(exception.getMessage());
                        return;
                    }

                    /// Write results to file & close file
                    try {
                        for(int row : selrows) {
                            DefaultMutableTreeNode node = (DefaultMutableTreeNode)DiffTree.this.getPathForRow(row).getLastPathComponent();

                            outputStream.write(node.toString());
                            outputStream.write("\n");


                        }
                        outputStream.close();
                    } catch(IOException exception) {
                        logger.warning(exception.getMessage());
                    }
                    
                }
            });
            popupmenu.add(copyTextIt);

            JMenuItem openInMain = new JMenuItem("Open selected in main panel");
            openInMain.addActionListener(new ActionListener() {

                @Override
                public void actionPerformed(ActionEvent e) {

                    int selrows[] = DiffTree.this.getSelectionRows();

                    if (selrows == null) {
                        return;
                    }
                    if (selrows.length == 0) {
                        return;
                    }
                    DefaultMutableTreeNode node = (DefaultMutableTreeNode) DiffTree.this.getPathForRow(selrows[0]).getLastPathComponent();
                    if (node.getUserObject() instanceof DiffNewResult) {
                        logger.info("selected a node that has a new item");
                        DiffNewResult newres = (DiffNewResult) node.getUserObject();
                        String tabname = newres.getTableName();
                        int id = newres.getID();
                        String combobox = "NONE";
                        /// Unfortunately hard code tabname -> combobox correspondance
                        if (tabname.contains("L1")) { //L1 stuff
                            if (tabname.contains("THRESHOLD")) {
                                combobox = "L1Threshold";
                            }
                            if (tabname.contains("THRESHOLD_VALUE")) {
                                combobox = "L1ThresholdValue";
                            }
                            if (tabname.contains("ITEM")) {
                                combobox = "L1Item";
                            }
                            if (tabname.equals("L1_TM_TO_TT")) {
                                return; //currently do not display this in the main panel
                            }
                        }
                        if (tabname.contains("HLT")) { //HLT stuff
                            if (tabname.contains("TRIGGER_CHAIN")) {
                                combobox = "HLTTriggerChain";
                            }
                            if (tabname.contains("TRIGGER_SIGNATURE")) {
                                combobox = "HLTTriggerSignature";
                            }
                            if (tabname.contains("TRIGGER_ELEMENT")) {
                                combobox = "HLTTriggerElement";
                            }
                            if (tabname.contains("TRIGGER_TYPE")) {
                                combobox = "HLTTriggerType";
                            }
                            if (tabname.contains("TRIGGER_STREAM")) {
                                combobox = "HLTTriggerStream";
                            }
                            if (tabname.contains("TRIGGER_GROUP")) {
                                combobox = "HLTTriggerGroup";
                            }
                            if (tabname.contains("COMPONENT")) {
                                combobox = "HLTComponent";
                            }
                            if (tabname.contains("PARAMETER")) {
                                combobox = "HLTParameter";
                            }
                            if (tabname.contains("RELEASE")) {
                                combobox = "HLTRelease";
                            }
                        }

                        if (!combobox.equals("NONE")) {
                            triggertool.Panels.MainPanel panel = triggertool.Panels.MainPanel.getMainPanel();
                            panel.comboTable.setSelectedItem(combobox);
                            if (panel.comboTable.getSelectedItem().equals(combobox)) { // check the combobox was set correctly
                                panel.editSearchText.setText(String.valueOf(id));
                                panel.comboFieldSelect.setSelectedItem("ID");
                                panel.doSearch(true);
                                panel.toFront();
                            } else {
                                logger.log(Level.SEVERE, "Combobox setting {0} is not valid - needs to be fixed.", combobox);
                            }
                        } else {
                            logger.log(Level.WARNING, "Table of type {0} not dealt with - should be added to DiffTree code", tabname);
                        }

                    }
                }
            });

            popupmenu.add(openInMain);

            popupmenu.show(e.getComponent(), e.getX(), e.getY());
        }
    }

    @Override
    protected void paintComponent(java.awt.Graphics g) {
            super.paintComponent(g); //paints the background and image

            //If currentRect exists, paint a box on top.
            if (selRec != null) {
                //Draw a rectangle on top of the image.
                g.setColor(java.awt.Color.BLACK);
                g.drawRect(selRec.x, selRec.y,
                           selRec.width - 1, selRec.height - 1);

            }
    }//paintComponent

    /**
     * Update size of the selection rectangle
     * @param e
     */
    private void updateRecSize(MouseEvent e) {
                int x = e.getX();
                int y = e.getY();
                int xsize= java.lang.Math.abs(x - recStart.x);
                int ysize= java.lang.Math.abs(y - recStart.y);
                if(x > recStart.x && y > recStart.y) {
                    selRec.setSize(xsize,ysize);
                    return;
                }
                int x0=recStart.x;
                int y0=recStart.y;
                if(x < x0) x0 = x;
                if(y < y0) y0 = y;
                selRec.setLocation(x0,y0);
                selRec.setSize(xsize,ysize);
    }

    /**
     * See which rows are within the selection rectangle
     */
    private void updateSelected() {
        if( (selRec.getSize().height * selRec.getSize().width) != 0) {
            this.getSelectionModel().clearSelection();
            for(int irow = 0; irow<this.getRowCount(); ++irow) {
                if(this.getRowBounds(irow).intersects(selRec)) {
                    this.getSelectionModel().addSelectionPath(this.getPathForRow(irow));
                }
            }
        }
    }

    @Override
    public void mouseDragged(MouseEvent e) {
        if(e.getButton()==java.awt.event.MouseEvent.BUTTON1) {// update drag rectangle if left click
            updateRecSize(e);
            updateSelected();
            this.repaint();
        }
    }

    @Override
    public void mouseMoved(MouseEvent e) {
    }

    @Override
    public void mousePressed(MouseEvent e) {
        if(e.getButton()==java.awt.event.MouseEvent.BUTTON1) {// set up drag rectangle if left click
            selRec = new Rectangle(e.getX(),e.getY(),0,0);
            recStart = new Point(e.getX(),e.getY());
        }
    }

    @Override
    public void mouseReleased(MouseEvent e) {
        if(e.getButton()==java.awt.event.MouseEvent.BUTTON1) {// update drag rectangle if left click
            updateRecSize(e);
            updateSelected();
            selRec.setSize(0,0); // make the rectangle disappear
            this.repaint();
        }       
    }

    @Override
    public void mouseEntered(MouseEvent e) {
    }

    @Override
    public void mouseExited(MouseEvent e) {
    }

    @Override
    public void actionPerformed(ActionEvent e) {

    }
    

    
}

/**
 * Class to handle copying of info to the clipboard.
 * @author markowen
 */
final class ResultsTransfer implements ClipboardOwner {


    /**
    * Empty implementation of the ClipboardOwner interface.
    */
    @Override
    public void lostOwnership( Clipboard aClipboard, Transferable aContents) {
        //do nothing
    }

    /**
      * Place a String on the clipboard, and make this class the
      * owner of the Clipboard's contents.
      */
    public void setClipboardContents( String aString ){
        StringSelection stringSelection = new StringSelection( aString );
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        clipboard.setContents( stringSelection, this );
    }
}



