package triggertool.L1BunchGroupPanels;

import java.io.Serializable;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import triggerdb.Entities.AbstractTable;
import triggertool.Components.TableSorter;
import triggerdb.Entities.L1.L1BunchGroup;
import triggerdb.Entities.L1.L1BunchGroupSet;
import triggertool.TTExceptionHandler;

/**
 * BunchLeftPanel.java
 *
 * A sub-panel for the Bunch Group Editor. It displays TITLE FILTER BOXES MAIN
 * BUNCH GROUP or BUNCH GROUP SET TABLE
 *
 * It contains all the functionality and listeners.
 *
 * @author Tiago Perez <tperez@cern.ch>
 * @date 2011-06-08, 11:21:43 AM
 */
public final class BunchLeftPanel extends javax.swing.JPanel
        implements Serializable {

    /**
     * The logger.
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");
    /**
     * Needed by Serializable.
     */
    private static final long serialVersionUID = 1L;
    /**
     * Constant id for the BunchGroup class
     */
    public static final int BUNCHGROUP = 0;
    /**
     * Constant id for the BunchGroupSet class
     */
    public static final int BUNCHGROUPSET = 1;

    String testVar = "";
    /**
     * The class of this panel
     */
    private int mainClass = 0;
    /**
     * the parent component
     */
    private BunchGroupSubpanel parent;
    /**
     * List with all Bunch Groups, this holds the complete list of bunch groups
     * independently of the filtering.
     */
    private List<AbstractTable> blpList;

    private int ctpPartition;

    /**
     * Creates new BunchGroupSetEditor Panel
     */
    public BunchLeftPanel() {
        //super();
        initComponents();
    }

    /**
     * Creates new BunchGroupSetEditor Pane
     *
     * @param parent
     */
    public BunchLeftPanel(BunchGroupSubpanel parent) {
        super(true);
        this.BunchLeftPanelInit(parent, 0);
    }

    /**
     * Creates new BunchGroupSetEditor Pane
     *
     * @param newClass
     */
    public BunchLeftPanel(final int newClass) {
        super(true);
        this.BunchLeftPanelInit(null, newClass);
    }

    /**
     * Creates new BunchGroupSetEditor Pane
     *
     * @param parent
     * @param newClass
     */
    public BunchLeftPanel(BunchGroupSubpanel parent, final int newClass) {
        super(true);
        this.BunchLeftPanelInit(parent, newClass);
    }

    /**
     * Common init method shared by all constructors.
     *
     * @param parentPanel
     * @param newClass
     */
    public void BunchLeftPanelInit(BunchGroupSubpanel parentPanel, final int newClass) {
        ctpPartition = 0;
        InitPanel(parentPanel, newClass);
    }

    /**
     *
     * @param parentPanel
     * @param newClass
     * @param partition
     */
    public void BunchLeftPanelInit(BunchGroupSubpanel parentPanel, final int newClass, final int partition) {
        ctpPartition = partition;
        InitPanel(parentPanel, newClass);
    }

    private void InitPanel(BunchGroupSubpanel parentPanel, final int newClass) {
        //
        this.parent = parentPanel;
        //
        this.mainClass = newClass;
        if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            testVar = "Name";
        } else {
            testVar = "Size";
        }
        initComponents();
        if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            this.titleJLabel.setText("BUNCH GROUP SET EDITOR");
        } else {
            this.titleJLabel.setText("BUNCH GROUP EDITOR");
        }

        // Init left tables ets
        this.initCutomTables();
        try {
            this.fillMainTable();
            //this.mainTable.setRowSelectionInterval(0, 0);
            //this.mainTable.setRowSelectionInterval(0, 0);
        } catch (SQLException ex) {
            TTExceptionHandler.HandleSqlException(ex, this, "Cannot create BunchLeft panel.");
        }
    }

    /**
     * Sets the class to use. See header file.
     *
     * @param classId the class to use.
     */
    public void setMainClass(final int classId) {
        this.mainClass = classId;
    }

    /**
     * Sets the parent. This panel will use the parent to send/receive info from
     * the other panels.
     *
     * @param p the panel containing this.
     */
    public void setParent(final BunchGroupSubpanel p) {
        this.parent = p;
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        titleJLabel = new javax.swing.JLabel();
        filterJPanel = new javax.swing.JPanel();
        idlabel = new javax.swing.JLabel("BGS Key");
        filterLabel = new javax.swing.JLabel();
        filterText = new javax.swing.JTextField();
        idSearch = new javax.swing.JTextField();
        jScrollPane3 = new javax.swing.JScrollPane();

        titleJLabel.setFont(new java.awt.Font("Lucida Grande", 0, 24)); // NOI18N
        titleJLabel.setForeground(java.awt.Color.blue);
        titleJLabel.setText("BUNCH GROUP SET EDITOR");

        idlabel.setText("BGS Key");

        filterLabel.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
        filterLabel.setText("Name filter");

        filterText.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                filterTextKeyTyped(evt);
            }
        });

        idSearch.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                idSearchActionPerformed(evt);
            }
        });
        idSearch.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                idSearchKeyReleased(evt);
            }
        });

        org.jdesktop.layout.GroupLayout filterJPanelLayout = new org.jdesktop.layout.GroupLayout(filterJPanel);
        filterJPanel.setLayout(filterJPanelLayout);
        filterJPanelLayout.setHorizontalGroup(
            filterJPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(filterJPanelLayout.createSequentialGroup()
                .add(idlabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 71, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .add(0, 0, 0)
                .add(idSearch, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 56, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .add(filterLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 92, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(filterText, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 140, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
        );
        filterJPanelLayout.setVerticalGroup(
            filterJPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, filterJPanelLayout.createSequentialGroup()
                .add(filterJPanelLayout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(filterText, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(idSearch, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(filterLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 27, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(idlabel))
                .add(0, 0, 0))
        );

        filterJPanelLayout.linkSize(new java.awt.Component[] {filterLabel, filterText, idSearch, idlabel}, org.jdesktop.layout.GroupLayout.VERTICAL);

        mainTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {

            }
        ));
        mainTable.setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        mainTable.setDoubleBuffered(true);
        mainTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        mainTable.getTableHeader().setReorderingAllowed(false);
        jScrollPane3.setViewportView(mainTable);
        mainTable.getColumnModel().getSelectionModel().setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(filterJPanel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(org.jdesktop.layout.GroupLayout.LEADING, titleJLabel, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 448, Short.MAX_VALUE))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(titleJLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(filterJPanel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(jScrollPane3, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 375, Short.MAX_VALUE)
                .addContainerGap())
        );
    }// </editor-fold>//GEN-END:initComponents

    private void idSearchKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_idSearchKeyReleased
        JTextField idTF = (JTextField) evt.getComponent();
        int bgId;
        try {
            bgId = Integer.parseInt(idTF.getText());
        } catch (NumberFormatException e) {
            bgId = 0;
        }
        BunchLeftPanel.setSelectedId(bgId, this.mainTable);
    }//GEN-LAST:event_idSearchKeyReleased

    private void idSearchActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_idSearchActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_idSearchActionPerformed

    private void filterTextKeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_filterTextKeyTyped
        char k = evt.getKeyChar();
        if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            String filter = this.filterText.getText();
            this.filterBGbyName(filter + k);
        } else if (mainClass == BunchLeftPanel.BUNCHGROUP) {
            int filter;
            try {
                filter = Integer.parseInt(this.filterText.getText() + k);
            } catch (NumberFormatException e) {
                filter = -1;
            }
            this.filterBGbySize(filter);
        }
        this.setSelectedObject();
    }//GEN-LAST:event_filterTextKeyTyped

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel filterJPanel;
    protected javax.swing.JLabel filterLabel;
    protected javax.swing.JTextField filterText;
    private javax.swing.JTextField idSearch;
    public javax.swing.JLabel idlabel;
    private javax.swing.JScrollPane jScrollPane3;
    private final javax.swing.JTable mainTable = new javax.swing.JTable();
    private javax.swing.JLabel titleJLabel;
    // End of variables declaration//GEN-END:variables

    ///////////////////////////////////////////////////////
    // GENERAL METHODS    
    /**
     * Sets the name and title of the component to the specified string.
     *
     * @param name the new name and title for this panel.
     */
    public void setTitle(final String name) {
        this.titleJLabel.setText(name);
    }

    /**
     * Select (highlight) the row with the given ID from a BunchGroup or a
     * BunchGroupSet table.
     *
     * @param bgId the DB ID of the row to select.
     */
    public void setSelectedId(final int bgId) {
        if (bgId > 0) {
            BunchLeftPanel.setSelectedId(bgId, this.mainTable);
        }
    }

    /**
     * Select (highlight) the row with the given ID from a BunchGroup or a
     * BunchGroupSet table.
     *
     * @param the ID of the row to select.
     */
    private static void setSelectedId(final int bgId, JTable tab) {
        TableModel _mod = tab.getModel();
        int _rows = _mod.getRowCount();
        int selectedRowId = 0;
        if (_rows > 0) {
            for (int rowNumber = 0; rowNumber < _rows; rowNumber++) {
                int rowId = (Integer) _mod.getValueAt(rowNumber, 0);
                if (rowId == bgId) {
                    selectedRowId = rowNumber;
                }
            }
            tab.setRowSelectionInterval(selectedRowId, selectedRowId);
            tab.scrollRectToVisible(tab.getCellRect(selectedRowId, 2, false));
        }
    }

    /**
     * Initializes the bunch group editor left subpanel.
     */
    private void initCutomTables() {
        BGLTableListener listener = new BGLTableListener(
                this.mainTable,
                null);
        // Add selection listener to bunch groups table
        this.mainTable.getSelectionModel().addListSelectionListener(listener);
        this.mainTable.getColumnModel().getSelectionModel().addListSelectionListener(listener);
        this.mainTable.setRowSelectionAllowed(true);
        // set col widths
        // / this.bunchGroupTable.getColumnModel().getColumn(0).setWidth(15);
        switch (mainClass) {
            case BunchLeftPanel.BUNCHGROUPSET:
                {
                    this.mainTable.setModel(new BunchGroupSetLeftPanelTableModel());
                    TableColumnModel tcm = this.mainTable.getColumnModel();
                    // Remove vertical lines.
                    tcm.setColumnMargin(0);
                    //id column is smaller
                    tcm.getColumn(0).setPreferredWidth(75);
                    //name column is bigger
                    tcm.getColumn(1).setPreferredWidth(250);
                    //ver column is smaller
                    tcm.getColumn(2).setPreferredWidth(75);
                    break;
                }
            case BunchLeftPanel.BUNCHGROUP:
                {
                    this.mainTable.setModel(new BunchGroupLeftPanelTableModel());
                    TableColumnModel tcm = this.mainTable.getColumnModel();
                    // Remove vertical lines.
                    tcm.setColumnMargin(0);
                    //id column is smaller
                    tcm.getColumn(0).setPreferredWidth(75);
                    //size column is smaller
                    tcm.getColumn(1).setPreferredWidth(75);
                    break;
                }
            default:
                logger.severe(" Error in the Bunch Group Left Panel Config.");
                break;
        }
        // Sorter
        TableSorter bunchGroupTableSorter = new TableSorter(this.mainTable.getModel());
        this.mainTable.setModel(bunchGroupTableSorter);
        JTableHeader headerHLT = this.mainTable.getTableHeader();
        bunchGroupTableSorter.setTableHeader(headerHLT);
    }

    /**
     * Fills bunch group table and List with values from DB.
     */
    private void fillMainTable() throws SQLException {
        this.blpList = this.getTableList();
        if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            BunchGroupSetLeftPanelTableModel mod = (BunchGroupSetLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
            mod.clearRows();
            this.filterBGbyName();
        } else if (mainClass == BunchLeftPanel.BUNCHGROUP) {
            BunchGroupLeftPanelTableModel mod = (BunchGroupLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
            mod.clearRows();
            this.filterBGbySize(-1);
        }

    }

    /**
     * Gets the selected bunch group from the bunch group table.
     *
     * @return the selected bunch group.
     */
    public AbstractTable getSelectedRow() {
        AbstractTable at = null;
        int rowIdx = this.mainTable.getSelectedRow();
        if (!(rowIdx < 0)) {
            int tableId = (Integer) this.mainTable.getValueAt(rowIdx, 0);
            if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
                BunchGroupSetLeftPanelTableModel model = (BunchGroupSetLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
                at = model.getTable(tableId);
            } else if (mainClass == BunchLeftPanel.BUNCHGROUP) {
                BunchGroupLeftPanelTableModel model = (BunchGroupLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
                at = model.getTable(tableId);
            }

        }
        return at;
    }

    /**
     * Selects the row with the ID in the ID text field, if any.
     */
    public void setSelectedObject() {
        JTextField idTF = idSearch;
        int bgId;
        try {
            bgId = Integer.parseInt(idTF.getText());
        } catch (NumberFormatException e) {
            bgId = 0;
        }
        BunchLeftPanel.setSelectedId(bgId, this.mainTable);
    }

    /**
     * Filters the table based in the filter text box.
     */
    private void filterBGbyName() {
        this.filterBGbyName(this.filterText.getText());
    }

    /**
     * Filters out of the the table all row which do not contain the filter
     * word. This is case independent.
     *
     * @param filterName the name to filter.
     */
    private void filterBGbyName(final String filterName) {
        if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            BunchGroupSetLeftPanelTableModel mod = (BunchGroupSetLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
            mod.clearRows();
            String _filter = filterName.trim().toLowerCase();
            for (AbstractTable tab : this.blpList) {
                String name = tab.get_name().toLowerCase();
                if (name.contains(_filter)) {
                    mod.add(tab);
                }
            }

        }

    }

    private static AbstractTableModel getBunchGroupTableModel(final JTable table) {
        AbstractTableModel mod = null;
        TableModel tm = table.getModel();
        if (tm instanceof TableSorter) {
            mod = (AbstractTableModel) ((TableSorter) tm).getTableModel();
        } else {
            mod = (AbstractTableModel) tm;
        }
        return mod;
    }

    /**
     * Get the list of db objects for the main jtable.
     *
     * @return the list of AbstractTables.
     */
    private List<AbstractTable> getTableList() throws SQLException {
        List<AbstractTable> theList = new ArrayList<>();
        if (this.mainClass == BunchLeftPanel.BUNCHGROUP) {
            long t1 = System.currentTimeMillis();
            theList.addAll(L1BunchGroup.getBunchGroups());
            t1 = System.currentTimeMillis() - t1;
            logger.log(Level.FINER, " Time to load Bunch Groups = {0} sec.", t1 / 1e3);
        } else if (this.mainClass == BunchLeftPanel.BUNCHGROUPSET) {
            long t1 = System.currentTimeMillis();
            theList.addAll(L1BunchGroupSet.getBunchGroupSetsPartition(ctpPartition));
            t1 = System.currentTimeMillis() - t1;
            logger.log(Level.FINER, " Time to load Sets = {0} sec.", t1 / 1e3);
        }
        return theList;
    }

    /**
     * Reload table info from DB and select element with the given id.
     *
     * @param id The id of the object to select.
     * @throws java.sql.SQLException
     */
    public void updatePanel(int id) throws SQLException {
        this.fillMainTable();
        if (id > 0) {
            BunchLeftPanel.setSelectedId(id, this.mainTable);
        }
    }

    private void filterBGbySize(int filter) {
        if (mainClass == BunchLeftPanel.BUNCHGROUP) {
            BunchGroupLeftPanelTableModel mod = (BunchGroupLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.mainTable);
            mod.clearRows();

            for (AbstractTable tab : this.blpList) {
                int size = ((L1BunchGroup) tab).get_size();
                if (size == filter || filter == -1) {
                    mod.add(tab);
                }
            }
        }
    }

    /**
     * listen to event in the main left table.
     */
    private class BGLTableListener implements ListSelectionListener {

        JTable table;
        JComponent rightComp;

        // It is necessary to keep the table since it is not possible
        // to determine the table from the event's source
        BGLTableListener(JTable table, JComponent right) {
            this.table = table;
            this.rightComp = right;
        }

        @Override
        public void valueChanged(ListSelectionEvent e) {
            if (e.getSource() == table.getSelectionModel()
                    && table.getRowSelectionAllowed()) {
                try {
                    this.updateRightPanel();
                } catch (SQLException ex) {
                    TTExceptionHandler.HandleSqlException(ex, table, "Value changed in buch group caused a SQLException.");
                }
            }
        }

        private void updateRightPanel() throws SQLException {
            AbstractTable row = null;
            int rowId = this.table.getSelectedRow();
            if (rowId >= 0) {
                TableModel tm = this.table.getModel();
                int tableId = (Integer) tm.getValueAt(rowId, 0);
                if (mainClass == BunchLeftPanel.BUNCHGROUPSET) {
                    BunchGroupSetLeftPanelTableModel mod = (BunchGroupSetLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.table);
                    if (mod != null) {
                        row = mod.getTable(tableId);
                    } else {
                        logger.severe(" Error in the Bunch Group Table Model.");
                    }
                } else if (mainClass == BunchLeftPanel.BUNCHGROUP) {
                    BunchGroupLeftPanelTableModel mod = (BunchGroupLeftPanelTableModel) BunchLeftPanel.getBunchGroupTableModel(this.table);
                    if (mod != null) {
                        row = mod.getTable(tableId);
                    } else {
                        logger.severe(" Error in the Bunch Group Table Model.");
                    }
                }

            }
            parent.updateRight(row);
        }
    }
}
