package triggertool.Panels;

import java.util.logging.*;
import java.util.Vector;
import java.awt.Color;
import java.awt.EventQueue;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.JFrame;
import triggertool.Auxiliary.LoggingWindowHandler;

/**
 * Creates and formats the Window in which the Logging Messages are to be
 * displayed.
 *
 * @author Isabella Soldner-Rembold & Mark Owen
 */
public class LoggerWindow extends JFrame {

    /// vector to hold all the log records we should display
    private Vector<LogRecord> logmsgs;

    /// this integer is the limit on the size of the vector of log messages
    private final int messagelimit = 5000;

    /// level we are currently displaying (default to INFO)
    private Level currentlevel = Level.INFO;

    // /Message Log.
    /**
     *
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");

    /// flag to keep track of whether we initialized the graphics
    private boolean displayinit;

    /**
     * Default constructor. Do not start the graphics - only do this if the
     * display is requested (with setVisible).
     */
    public LoggerWindow() {
        displayinit = false;
        logmsgs = new Vector<>();      
        
        setTitle("Log");
        setType(javax.swing.JFrame.Type.UTILITY);
    }

    /**
     * Override the standard setVisible function. If graphical components not
     * initialized, then initialize them. Then call base class setVisible
     *
     * @param flag
     */
    @Override
    public void setVisible(boolean flag) {
        if (!displayinit) {
            initComponents();
            redrawForce(currentlevel);
            displayinit = true;
        }
        super.setVisible(flag);
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        InfoTextPane = new javax.swing.JTextPane();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        SevereMenu = new javax.swing.JMenuItem();
        WarningMenu = new javax.swing.JMenuItem();
        InfoMenu = new javax.swing.JMenuItem();
        ConfigMenu = new javax.swing.JMenuItem();
        FineMenu = new javax.swing.JMenuItem();
        FinerMenu = new javax.swing.JMenuItem();
        FinestMenu = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(600, 426));

        jScrollPane1.setViewportView(InfoTextPane);

        jMenu1.setText("Choose Level");

        SevereMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.ALT_MASK));
        SevereMenu.setText("Severe");
        SevereMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                SevereMenuActionPerformed(evt);
            }
        });
        jMenu1.add(SevereMenu);

        WarningMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, java.awt.event.InputEvent.ALT_MASK));
        WarningMenu.setText("Warning");
        WarningMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                WarningMenuActionPerformed(evt);
            }
        });
        jMenu1.add(WarningMenu);

        InfoMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_I, java.awt.event.InputEvent.ALT_MASK));
        InfoMenu.setText("Info");
        InfoMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                InfoMenuActionPerformed(evt);
            }
        });
        jMenu1.add(InfoMenu);

        ConfigMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, java.awt.event.InputEvent.ALT_MASK));
        ConfigMenu.setText("Config");
        ConfigMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ConfigMenuActionPerformed(evt);
            }
        });
        jMenu1.add(ConfigMenu);

        FineMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, java.awt.event.InputEvent.ALT_MASK));
        FineMenu.setText("Fine");
        FineMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FineMenuActionPerformed(evt);
            }
        });
        jMenu1.add(FineMenu);

        FinerMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, java.awt.event.InputEvent.ALT_MASK));
        FinerMenu.setText("Finer");
        FinerMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FinerMenuActionPerformed(evt);
            }
        });
        jMenu1.add(FinerMenu);

        FinestMenu.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_T, java.awt.event.InputEvent.ALT_MASK));
        FinestMenu.setText("Finest");
        FinestMenu.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                FinestMenuActionPerformed(evt);
            }
        });
        jMenu1.add(FinestMenu);

        jMenuBar1.add(jMenu1);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 600, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * When the Menu Item 'Choose Level-Warning' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void WarningMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_WarningMenuActionPerformed
        redraw(Level.WARNING);
    }//GEN-LAST:event_WarningMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Severe' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void SevereMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_SevereMenuActionPerformed
        redraw(Level.SEVERE);
}//GEN-LAST:event_SevereMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Info' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void InfoMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_InfoMenuActionPerformed
        redraw(Level.INFO);
    }//GEN-LAST:event_InfoMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Config' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void ConfigMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ConfigMenuActionPerformed
        redraw(Level.CONFIG);
    }//GEN-LAST:event_ConfigMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Fine' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void FineMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FineMenuActionPerformed
        redraw(Level.FINE);
    }//GEN-LAST:event_FineMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Finest' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void FinestMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FinestMenuActionPerformed
        redraw(Level.FINEST);
    }//GEN-LAST:event_FinestMenuActionPerformed

    /**
     * When the Menu Item 'Choose Level-Finer' is selected the Internal Frame
     * WarningFrame is made visible, while all the other Internal Frames are
     * made invisible.
     *
     * @param evt The event that fires the Action
     */
    private void FinerMenuActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_FinerMenuActionPerformed
        redraw(Level.FINER);
    }//GEN-LAST:event_FinerMenuActionPerformed

    private int getIndex(Level level) {
        int index = 0;
        if (level == Level.SEVERE) {
            index = 0;
        } else if (level == Level.WARNING) {
            index = 1;
        } else if (level == Level.INFO) {
            index = 2;
        } else if (level == Level.CONFIG) {
            index = 3;
        } else if (level == Level.FINE) {
            index = 4;
        } else if (level == Level.FINER) {
            index = 5;
        } else if (level == Level.FINEST) {
            index = 6;
        }
        return index;
    }

    private SimpleAttributeSet getSet(Level level) {
        int index = getIndex(level);

        if (sets[index] == null) {
            sets[index] = new SimpleAttributeSet();
            switch (index) {
                case 0: // SEVERE
                    StyleConstants.setForeground(sets[index], Color.red);
                    break;
                case 1: // WARNING    
                    StyleConstants.setForeground(sets[index], Color.orange);
                    break;
                case 2: // INFO
                    StyleConstants.setForeground(sets[index], Color.black);
                    break;
                case 3: // CONFIG
                    StyleConstants.setForeground(sets[index], Color.gray);
                    break;
                case 4: // FINE
                    StyleConstants.setForeground(sets[index], Color.green);
                    break;
                case 5: // FINER
                    StyleConstants.setForeground(sets[index], Color.green);
                    break;
                case 6: // FINEST
                    StyleConstants.setForeground(sets[index], Color.green);
                    break;
            }
        }
        return sets[index];
    }

    /**
     * Function inserts a a LogRecord into the displayed Document.
     *
     * @param record - the record to insert.
     */
    private void insertMsgIntoDoc(final LogRecord record) {
        StyledDocument doc3 = InfoTextPane.getStyledDocument();
        String message = null;
        //Format the data
        try {
            message = LoggingWindowHandler.getInstance().getFormatter().format(record);
        } catch (Exception e) {
            logger.warning("Problem formatting log record entry");
            return;
        }
        try {
            doc3.insertString(doc3.getLength(), message, getSet(record.getLevel()));
        } catch (javax.swing.text.BadLocationException ex) {
            logger.severe("Exception caught when inserting into logger document:");
            logger.severe(ex.toString());
            ex.printStackTrace();
        }

    }

    /**
     * Function adds another log message to the store & if necessary the window.
     *
     * @param record
     */
    public void addLogInfo(final LogRecord record) {
        // add next record to the store
        logmsgs.add(record);
        // if we exceed the max size, remove the earliest record
        if (logmsgs.size() > messagelimit) {
            logmsgs.remove(0);
        }

        // see if we need to display this message
        if (record.getLevel().intValue() >= currentlevel.intValue() && displayinit) {
            // need to put this message into the viewer
            EventQueue.invokeLater(new Runnable() {
                @Override
                public void run() {
                    insertMsgIntoDoc(record);
                }
            });
        }
    }

    /**
     * Function to trigger the re-drawing of the log window to show messages of
     * the specified level. Checks whether a redraw is needed based on the level
     *
     * @param level - the level to display
     */
    protected void redraw(final Level level) {

        if (level.intValue() != currentlevel.intValue()) {
            // complete redraw needed
            redrawForce(level);
        }//check level different to current
    }

    /**
     * Redraw the window at Level level. Redraws even if currentlevel==level.
     *
     * @param level
     */
    protected void redrawForce(final Level level) {

        logger.finest("Redrawing log window");
        EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {

                StyledDocument doc3 = InfoTextPane.getStyledDocument();
                try {
                    doc3.remove(0, doc3.getLength());
                } catch (javax.swing.text.BadLocationException ex) {
                    logger.severe("Exception caught when resetting logger document:");
                    logger.severe(ex.toString());
                    ex.printStackTrace();
                }
                for (LogRecord record : logmsgs) {
                    if (record.getLevel().intValue() >= level.intValue()) {
                        insertMsgIntoDoc(record);
                    }

                }//log record loop
                // set current level to level
                currentlevel = level;
            }
        });

    }

    final SimpleAttributeSet[] sets = new SimpleAttributeSet[10];
    SimpleAttributeSet set = new SimpleAttributeSet();


    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem ConfigMenu;
    private javax.swing.JMenuItem FineMenu;
    private javax.swing.JMenuItem FinerMenu;
    private javax.swing.JMenuItem FinestMenu;
    private javax.swing.JMenuItem InfoMenu;
    private javax.swing.JTextPane InfoTextPane;
    private javax.swing.JMenuItem SevereMenu;
    private javax.swing.JMenuItem WarningMenu;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JScrollPane jScrollPane1;
    // End of variables declaration//GEN-END:variables

}
