package triggertool.Panels.MenuWork;

import java.awt.Dimension;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import triggerdb.Entities.HLT.HLTPrescaleSet;
import triggerdb.Entities.L1.L1Prescale;
import triggerdb.Entities.SMT.SuperMasterTable;
import triggerdb.Entities.alias.HLTPrescaleSetAliasLumi;
import triggerdb.Entities.alias.L1PrescaleSetAliasLumi;
import triggerdb.Entities.alias.PrescaleSetAlias;
import triggertool.Panels.MainPanel;
import triggertool.TTExceptionHandler;

/**
 * List all Alias Sets present for an SMK and assign them to other SMK's as
 * needed.
 *
 * @author WPV
 */
public final class AssignAliasDialog extends JDialog {

    /**
     * The logger.
     */
    protected static final Logger logger = Logger.getLogger("TriggerTool");
    /**
     * The Super Master Key.
     */
    private int smk;
    SuperMasterTable smt;
    /**
     * The list with all Alias Sets.
     */
    private final DefaultListModel<String> AliasAllItems = new DefaultListModel<>();
    /**
     * The list with the Alias sets linked to the given SMT.
     */
    private final DefaultListModel<String> AliasLinkedItems = new DefaultListModel<>();

    MainPanel parent;
    
    /**
     * Create and display the dialog screen.
     *
     * @param smt the super master table to add alias sets to.
     * @param parent
     * @throws java.sql.SQLException
     */
    public AssignAliasDialog(final SuperMasterTable smt, final JFrame parent, Boolean otherAliasWindowActive)
            throws SQLException {

        super((JDialog) null);

        initComponents();

        try {
            this.smk = smt.get_id();
            this.smt = smt;
            this.parent = (MainPanel) parent;
            //popluate the sets table and the all groups table
            this.getLinkedSets();
        } catch (SQLException ex) {
            TTExceptionHandler.HandleSqlException(ex, rootPane, "Error while creating the Assign Prescale panel.");
        }

        //draw things
        populateSMKList(smt);
        jLabel6.setText("Current SMK " + smk + ": " + smt.get_name());
        this.setTitle("Assign Prescales");
        this.setLocationRelativeTo(parent);
        if (otherAliasWindowActive) {
            viewModeWarningLabel.setVisible(true);
            buttonAddAlias.setEnabled(false);
        } else {
            viewModeWarningLabel.setVisible(false);
        }
        this.parent.lockAliasAssign();

    }

    private void getSelectedSets(Integer smk, String type) throws SQLException {

        AliasAllItems.removeAllElements();

        for (String alias : PrescaleSetAlias.getAliasesForSMK(smk, type)) {
            AliasAllItems.addElement(alias);
        }
    }

    /**
     * find all alias sets in the DB linked to this SMT.
     *
     * @throws java.sql.SQLException
     */
    public void getLinkedSets() throws SQLException {

        AliasLinkedItems.removeAllElements();

        for (String alias : PrescaleSetAlias.getAliasesForSMK(this.smk)) {
            AliasLinkedItems.addElement(alias);
        }

    }

    ///Netbeans. Don't touch.
    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel2 = new javax.swing.JLabel();
        jScrollPane7 = new javax.swing.JScrollPane();
        listTableLinkedAliasSets = new javax.swing.JList<String>();
        buttonAddAlias = new javax.swing.JButton();
        jScrollPane9 = new javax.swing.JScrollPane();
        listTableAllAliasSets = new javax.swing.JList<String>();
        jLabel4 = new javax.swing.JLabel();
        buttonCancel = new javax.swing.JButton();
        jComboBox1 = new javax.swing.JComboBox<String>();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jComboBox2 = new javax.swing.JComboBox<>();
        jLabel7 = new javax.swing.JLabel();
        viewModeWarningLabel = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setMinimumSize(new java.awt.Dimension(1002, 511));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });

        jLabel2.setText("All Available Alias Sets:");

        listTableLinkedAliasSets.setModel(AliasLinkedItems);
        jScrollPane7.setViewportView(listTableLinkedAliasSets);

        buttonAddAlias.setText("Add");
        buttonAddAlias.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonAddAliasActionPerformed(evt);
            }
        });

        listTableAllAliasSets.setModel(AliasAllItems);
        jScrollPane9.setViewportView(listTableAllAliasSets);

        jLabel4.setText("Currently Linked Alias Sets:");

        buttonCancel.setText("Done");
        buttonCancel.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                buttonCancelActionPerformed(evt);
            }
        });

        jComboBox1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox1ActionPerformed(evt);
            }
        });

        jLabel5.setFont(new java.awt.Font("Liberation Sans", 1, 11)); // NOI18N
        jLabel5.setText("Select Origin SMK");

        jLabel6.setFont(new java.awt.Font("Liberation Sans", 1, 11)); // NOI18N
        jLabel6.setText("Current SMK");

        jComboBox2.setModel(new javax.swing.DefaultComboBoxModel<>(new String[] { "COSMIC", "STANDBY", "EMITTANCE", "OTHER" }));
        jComboBox2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jComboBox2ActionPerformed(evt);
            }
        });

        jLabel7.setFont(new java.awt.Font("Liberation Sans", 1, 11)); // NOI18N
        jLabel7.setText("Select Alias Type");

        viewModeWarningLabel.setFont(new java.awt.Font("Tahoma", 1, 13)); // NOI18N
        viewModeWarningLabel.setText("Another Instance Open - VIEW MODE");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(layout.createSequentialGroup()
                                .add(0, 0, Short.MAX_VALUE)
                                .add(buttonAddAlias, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 71, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .add(jLabel2)
                                .add(0, 0, Short.MAX_VALUE))
                            .add(org.jdesktop.layout.GroupLayout.LEADING, jScrollPane9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 484, Short.MAX_VALUE))
                        .add(10, 10, 10))
                    .add(layout.createSequentialGroup()
                        .add(8, 8, 8)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 127, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                            .add(jLabel7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 127, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING, false)
                            .add(jComboBox2, 0, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .add(jComboBox1, 0, 215, Short.MAX_VALUE))
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                                .add(jLabel6, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .add(92, 92, 92))
                            .add(org.jdesktop.layout.GroupLayout.TRAILING, jScrollPane7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 484, Short.MAX_VALUE)
                            .add(layout.createSequentialGroup()
                                .add(jLabel4)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .add(viewModeWarningLabel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 252, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                        .add(12, 12, 12))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .add(buttonCancel, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 74, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap())))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(23, 23, 23)
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                            .add(jLabel4)
                            .add(jLabel2))
                        .add(6, 6, 6))
                    .add(org.jdesktop.layout.GroupLayout.TRAILING, layout.createSequentialGroup()
                        .addContainerGap()
                        .add(viewModeWarningLabel)
                        .add(18, 18, 18)))
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.TRAILING)
                    .add(jScrollPane9, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE)
                    .add(jScrollPane7, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 210, Short.MAX_VALUE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(buttonAddAlias)
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel5, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jComboBox1, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel6, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.UNRELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jComboBox2, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel7, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, 28, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(buttonCancel))
                .add(0, 18, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * Associate HLT prescale set.
     *
     * @param evt
     */
private void buttonAddAliasActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonAddAliasActionPerformed

    //if nothing selected don't do anything
    if (listTableAllAliasSets.getSelectedIndex() < 0) {
        return;
    }

    try {
        SuperMasterTable incomingSMT = new SuperMasterTable(getSMKfromSelector());
        if ((incomingSMT.get_l1_master_table().get_menu_id().equals(smt.get_l1_master_table().get_menu_id())) && (incomingSMT.get_hlt_master_table().get_menu_id().equals(smt.get_hlt_master_table().get_menu_id()))) {
            JOptionPane.showMessageDialog(null,
                    "Selected SMK's have identical L1 and HLT menus, no need to assign aliases!.", "Assignment has no effect.",
                    JOptionPane.WARNING_MESSAGE);
            return;
        }
    } catch (SQLException ex) {
        Logger.getLogger(AssignAliasDialog.class.getName()).log(Level.SEVERE, null, ex);
    }

    JTextArea reportSummary = new JTextArea("Alias Assignment Report:\n\n");
    String reportText = "";
    Object[] sets = this.listTableAllAliasSets.getSelectedValues();
    for (Object set : sets) {
        //(HLTPrescaleSet) listTableAllHLT.getSelectedValue();
        int aliasInfo = Integer.parseInt(((String) set).split("\\:")[0]);
        logger.log(Level.FINE, " Adding Alias Set {0} to SMT {1}", new Object[]{aliasInfo, smk});
        try {
            reportText += (this.addAliasSet(aliasInfo) + "\n\n");
        } catch (SQLException ex) {
            TTExceptionHandler.HandleSqlException(ex, rootPane, "Error while performing Add Alias action.");
        }
        reportSummary.append(reportText);
        JScrollPane scrollPane = new JScrollPane(reportSummary);
        reportSummary.setLineWrap(true);
        reportSummary.setWrapStyleWord(true);
        reportSummary.setEditable(false);
        scrollPane.setPreferredSize(new Dimension(500, 500));
        JOptionPane.showMessageDialog(null, scrollPane, "Alias Assignment Report",
                JOptionPane.INFORMATION_MESSAGE);
    }
    try {
        getLinkedSets();
        getSelectedSets(getSMKfromSelector(), getAliasTypeFromSelector());
    } catch (SQLException ex) {
        Logger.getLogger(AssignAliasDialog.class.getName()).log(Level.SEVERE, null, ex);
    }
}//GEN-LAST:event_buttonAddAliasActionPerformed

    private String addAliasSet(final int aliasID) throws SQLException {

        String report = new String();
        PrescaleSetAlias alias = new PrescaleSetAlias(aliasID);

        PrescaleSetAlias newAlias = new PrescaleSetAlias(-1);
        newAlias.set_name(alias.get_name());
        if (alias.get_name_free().contains(("ASSIGNED"))) {
            String[] results = alias.get_name_free().split("\\_");
            int version = Integer.parseInt(results[results.length - 1]) + 1;
            String saveName = alias.get_name_free();
            String newName = saveName.substring(0, saveName.lastIndexOf("_") + 1) + version;
            newAlias.set_name_free(newName);
        } else {
            newAlias.set_name_free(alias.get_name_free() + "_ASSIGNED_1");
        }
        newAlias.set_comment(alias.get_comment() + " ASSIGNED FROM SMK " + getSMKfromSelector());

        while (newAlias.checkDup() != -1) {
            String[] results = newAlias.get_name_free().split("\\_");
            int version = Integer.parseInt(results[results.length - 1]) + 1;
            String saveName = newAlias.get_name_free();
            String newName = saveName.substring(0, saveName.lastIndexOf("_") + 1) + version;
            newAlias.set_name_free(newName);
        }
        newAlias.save();
        int new_id = newAlias.get_id();

        report += ("Assignment of alias " + aliasID + " from SMK " + getSMKfromSelector() + " to alias " + new_id + " from SMK " + smt.get_id() + "\n\n");

        SuperMasterTable incomingSMT = new SuperMasterTable(getSMKfromSelector());
        List<L1PrescaleSetAliasLumi> aliasListL1 = alias.getL1PrescaleSetAliases(incomingSMT.get_l1_master_table().get_menu_id());

        if (aliasListL1.isEmpty()) {
            report += "WARNING - NO L1 ALIASES!!!\n\n";
        }
        for (L1PrescaleSetAliasLumi aliasL1 : aliasListL1) {
            L1Prescale l1ps = aliasL1.get_prescale_set();

            Boolean doAssignmentL1 = true;
            for (L1Prescale set : smt.get_l1_master_table().get_menu().getPrescaleSets()) {
                if (l1ps.get_id() == set.get_id()) {
                    doAssignmentL1 = false;
                }
            }

            L1PrescaleSetAliasLumi newElement = new L1PrescaleSetAliasLumi(-1);

            if (doAssignmentL1) {
                ArrayList<Object> results = AssignPrescales.linkL1PrescaleSet(l1ps.get_id(), getSMKfromSelector(), smk, smt.get_l1_master_table().get_menu_id(), false);
                if (results.size() < 2) {
                    report += ("Assignment aborted for L1 Prescale: " + l1ps.get_id() + "\n\n");
                    newElement.set_prescale_set_id(-1);
                } else {
                    report += (String) results.get(1);
                    L1Prescale newL1Ps = (L1Prescale) results.get(0);
                    newElement.set_prescale_set(newL1Ps);
                    newElement.set_prescale_set_id(newL1Ps.get_id());
                    report += ("Assigned L1 Prescale: " + l1ps.get_id() + " to " + newL1Ps.get_id() + "\n\n");
                }
            } else {
                report += ("L1 Prescale: " + l1ps.get_id() + " already present in menu, no assignment needed\n\n");
                newElement.set_prescale_set(l1ps);
                newElement.set_prescale_set_id(l1ps.get_id());
            }
            newElement.set_menu(smt.get_l1_master_table().get_l1_menu());
            newElement.set_menu_id(smt.get_l1_master_table().get_l1_menu().get_id());
            newElement.set_lum_max(aliasL1.get_lum_max());
            newElement.set_lum_min(aliasL1.get_lum_min());
            newElement.set_comment(aliasL1.get_comment());
            newElement.set_alias_id(new_id);
            newElement.save();
            report += ("Saved new L1 Alias: " + newElement.get_id() + " \n\n");

        }

        //Process HLT Prescale in set
        List<HLTPrescaleSetAliasLumi> aliasListHLT = alias.getHLTPrescaleSetAliases(incomingSMT.get_hlt_master_table().get_menu_id());

        if (aliasListHLT.isEmpty()) {
            report += "WARNING - NO HLT ALIASES!!!\n\n";
        }

        for (HLTPrescaleSetAliasLumi aliasHLT : aliasListHLT) {
            HLTPrescaleSet hltps = aliasHLT.get_prescale_set();
            ArrayList<Integer> listOfIDs = new ArrayList<>();

            Boolean doAssignmentHLT = true;
            for (HLTPrescaleSet set : smt.get_hlt_master_table().get_menu().getPrescaleSets()) {
                if (hltps.get_id() == set.get_id()) {
                    doAssignmentHLT = false;
                }
                listOfIDs.add(set.get_id());
            }
            HLTPrescaleSetAliasLumi newElement = new HLTPrescaleSetAliasLumi(-1);

            ArrayList<Object> results = null;
            if (doAssignmentHLT) {
                results = AssignPrescales.addHLTset(hltps.get_id(), false, new SuperMasterTable(getSMKfromSelector()), smt.get_hlt_master_table().get_menu_id(), listOfIDs);
                if (results.size() < 2) {
                    report += ("Assignment aborted for HLT Prescale: " + hltps.get_id() + "\n\n");
                    newElement.set_prescale_set_id(-1);
                } else {
                    report += (String) results.get(1);
                    HLTPrescaleSet newHLTPs = (HLTPrescaleSet) results.get(0);
                    newElement.set_prescale_set(newHLTPs);
                    newElement.set_prescale_set_id(newHLTPs.get_id());
                    report += ("Assigned HLT Prescale: " + hltps.get_id() + " to " + newHLTPs.get_id() + "\n\n");
                }
            } else {
                report += ("HLT Prescale: " + hltps.get_id() + " already present in menu, no assignment needed\n\n");
                newElement.set_prescale_set(hltps);
                newElement.set_prescale_set_id(hltps.get_id());
            }
            newElement.set_menu(smt.get_hlt_master_table().get_menu());
            newElement.set_menu_id(smt.get_hlt_master_table().get_menu().get_id());
            newElement.set_lum_max(aliasHLT.get_lum_max());
            newElement.set_lum_min(aliasHLT.get_lum_min());
            newElement.set_comment(aliasHLT.get_comment());
            newElement.set_alias_id(new_id);
            newElement.save();
            report += ("Saved new HLT Alias: " + newElement.get_id() + " \n\n");
        }
        return report;
    }

    private void populateSMKList(SuperMasterTable smt) throws SQLException {
        ArrayList<String> keys = smt.getAllSMKsAndNames();
        jComboBox1.removeAllItems();
        Collections.sort(keys);
        for (String key : keys) {
            //Remove destination SMK from selector
            String selection[] = key.split("\\:");
            //System.out.println(selection[0] + " " + smk);
            if (!(Integer.parseInt(selection[0]) == smk)) {
                jComboBox1.addItem(key);
            }
        }

    }

///Close the screen
private void buttonCancelActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_buttonCancelActionPerformed
    this.parent.unlockAliasAssign();
    dispose();
}//GEN-LAST:event_buttonCancelActionPerformed

    private void jComboBox1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox1ActionPerformed
        try {
            getSelectedSets(getSMKfromSelector(), getAliasTypeFromSelector());
        } catch (SQLException ex) {
            TTExceptionHandler.HandleSqlException(ex, rootPane, "Error while populating alias sets for selected SMK.");
        }
    }//GEN-LAST:event_jComboBox1ActionPerformed

    private void jComboBox2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jComboBox2ActionPerformed
        try {
            getSelectedSets(getSMKfromSelector(), getAliasTypeFromSelector());
        } catch (SQLException ex) {
            TTExceptionHandler.HandleSqlException(ex, rootPane, "Error while populating alias sets for selected SMK.");
        }
    }//GEN-LAST:event_jComboBox2ActionPerformed

    private void formWindowClosing(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_formWindowClosing
        this.parent.unlockAliasAssign();
    }//GEN-LAST:event_formWindowClosing

    private Integer getSMKfromSelector() {
        String selection[] = jComboBox1.getSelectedItem().toString().split("\\:");
        return Integer.parseInt(selection[0]);
    }

    private String getAliasTypeFromSelector() {
        String selection = jComboBox2.getSelectedItem().toString();
        return selection;
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    ///Cancel and close.
    private javax.swing.JButton buttonAddAlias;
    ///Cancel and close.
    private javax.swing.JButton buttonCancel;
    private javax.swing.JComboBox<String> jComboBox1;
    private javax.swing.JComboBox<String> jComboBox2;
    ///Bunch Group Name.
    private javax.swing.JLabel jLabel2;
    ///Bunch Group Name.
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JScrollPane jScrollPane7;
    private javax.swing.JScrollPane jScrollPane9;
    private javax.swing.JList<String> listTableAllAliasSets;
    private javax.swing.JList<String> listTableLinkedAliasSets;
    private javax.swing.JLabel viewModeWarningLabel;
    // End of variables declaration//GEN-END:variables
}
