<?php

/**
 *	ATLAS Point 1 roles retrieved from LDAP.
 *	Uses: phpTreeGraph
 *	
 * 	@author Marius Leahu
**/

// get the parameters
$ldap_host = "atlas-ldap.cern.ch";
$ldap_role_prefix = "";
$top_role_filter = "LAR";
$test_mode = "";
//$test_mode = $_GET['test_mode'];
//$test_mode = "on";

if ($test_mode == "") {
	$ldap_host = $_GET['ldap_host'];
	$ldap_role_prefix = $_GET['ldap_role_prefix'];
	$top_role_filter = $_GET['top_role_filter'];
}

if ( $ldap_host == "" ){
    $ldap_host = "atlas-ldap.cern.ch";
}

function print_test($msg){
	if ( $test_mode != "" ) echo $msg;
}

//include GD rendering class
require_once('./classes/GDRenderer.php');

//create new GD renderer, optinal parameters: LevelSeparation,  SiblingSeparation, SubtreeSeparation, defaultNodeWidth, defaultNodeHeight
#$objTree = new GDRenderer(35, 10, 120, 150, 30);
$objTree = new GDRenderer(35, 10, 120);


//include LDAP roles class
require_once('./LDAP_roles.php');

$roles_obj = new LDAP_roles($ldap_host, "ou=atlas,o=cern,c=ch");


$roles = $roles_obj->getRoles($ldap_role_prefix);

//add nodes to the tree, parameters: id, parentid optional text, width, height, image(path)
//print_r($roles);
//echo "COUNT[".count($roles)."]\n";
$current_node_id = 0;
while (count($roles) > 0 ){
	//echo "ITERATION\n";

	foreach($roles as $role_name => $role){
		// skip the node if it has some seniors because its seniors must be processed first
		if (count($role[LDAP_roles::SENIORS_ARRAY]) > 0) continue;

		// check if there is a parent node id, if not, then add the default one which is 0
		if (count($role[LDAP_roles::PARENT_NODE_IDS_ARRAY]) == 0) {
			$role[LDAP_roles::PARENT_NODE_IDS_ARRAY][] = 0;

			// this is a top level role, so let's see if it matches the filter
			if ( $top_role_filter != "" && ! preg_match("/" . $top_role_filter . "/", $role_name) ) {
			    //echo "THE ROLE ". $role_name . " DOESN'T MATCH THE FILTER [". $top_role_filter ."]\n";

			    // this role doesn't match the filter, so let's remove it and then delete it from children's seniors list
			    unset($roles[$role_name]);
			    foreach($role[LDAP_roles::JUNIORS_ARRAY] as $junior) {
				// remove the current role from the its junior's seniors array
				foreach($roles[$junior][LDAP_roles::SENIORS_ARRAY] as $j => $jsenior_name){
					if ($jsenior_name == $role_name) { 
						unset($roles[$junior][LDAP_roles::SENIORS_ARRAY][$j]);
						//echo "remove ". $role_name . " from the seniors of " . $junior . "\n";
					}
				}
			    }
			    // continue with the next role
			    continue;
			}

		}
		
		// now, for each parent node id, create a node object with new id and add each of this ids to the juniors' parent node ids array
		foreach($role[LDAP_roles::PARENT_NODE_IDS_ARRAY] as $parent_node_id) {
			// create the node object
			$current_node_id++;
			$img_name = "roles_na.png";
			$node_text = $role_name;
			
			if ($role[LDAP_roles::ROLE_ASSIGNABLE] == 'true'){
				$img_name = "roles_a.png";
				$node_text = $node_text . " " .
						"(".
						    count($role[LDAP_roles::MEMBERS_ENABLED_ARRAY]). "/" .
						    count($role[LDAP_roles::MEMBERS_ASSIGNED_ARRAY]).
						")";
			}
			$node_width = strlen($node_text) * 8 + 4;
			$node_width = max($node_width, 100);
			$objTree->add($current_node_id, $parent_node_id, $node_text, $node_width, 20, $img_name);
			//echo "ADD: ID[".$current_node_id."] PARENT[".$parent_node_id."] ROLE[".$role_name."]\n";
			foreach($role[LDAP_roles::JUNIORS_ARRAY] as $junior) {
				$roles[$junior][LDAP_roles::PARENT_NODE_IDS_ARRAY][] = $current_node_id;
				// remove the current role from the its junior's seniors array
				foreach($roles[$junior][LDAP_roles::SENIORS_ARRAY] as $j => $jsenior_name){
					if ($jsenior_name == $role_name) 
						unset($roles[$junior][LDAP_roles::SENIORS_ARRAY][$j]);
				}
			}
		}
		unset($roles[$role_name]);
	}
//	print_r($roles);
}

//$objTree->setNodeLinks(GDRenderer::LINK_BEZIER);

$objTree->setBGColor(array(255, 255, 255));
$objTree->setNodeColor(array(0, 128, 255));
$objTree->setLinkColor(array(0, 64, 128));
//$objTree->setNodeLinks(GDRenderer::LINK_BEZIER);
//$objTree->setNodeBorder(array(0, 128, 255), 3);
$objTree->setNodeBorder(array(0, 0, 0), 2);
$objTree->setFTFont('/usr/share/fonts/bitstream-vera/Vera.ttf', 10, 0, GDRenderer::CENTER | GDRenderer::TOP);

if ($test_mode == "") {
	$objTree->stream();
}

?>
