/************************************************************************/
/*									*/
/*  This is the CMEM_RCC driver	 					*/
/*  Its purpose is to provide user applications with contiguous data 	*/
/*  buffers for DMA operations. It is not based on any extensions to 	*/
/*  the Linux kernel like the BigPhysArea patch.			*/
/*									*/
/*  12. Dec. 01  MAJO  created						*/
/*									*/
/*******C 2005 - The software with that certain something****************/


#include <linux/config.h>
#if defined(CONFIG_MODVERSIONS) && !defined(MODVERSIONS)
  #define MODVERSIONS
#endif
//MJ-SMP:#ifdef CONFIG_SMP
//MJ-SMP:  #define __SMP__
//MJ-SMP:#endif

#if defined(MODVERSIONS)
  #include <linux/modversions.h>
#endif

#ifdef BPA
  #include <linux/bigphysarea.h>
#endif
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/fs.h>
#include <linux/string.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/vmalloc.h>
#include <linux/mman.h>
#include <linux/wrapper.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>
#include <linux/string.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include "cmem_rcc/cmem_rcc_drv.h"

#ifdef MODULE
  MODULE_PARM (debug, "i");
  MODULE_PARM_DESC(debug, "1 = enable debugging   0 = disable debugging");
  MODULE_DESCRIPTION("Allocation of contiguous memory");
  MODULE_AUTHOR("Markus Joos, CERN/EP");
  #ifdef MODULE_LICENSE
    MODULE_LICENSE("Private: Contact markus.joos@cern.ch");
  #endif
#endif


// Globals
static int major;
static int debug = 0;
static char *proc_read_text;
static struct proc_dir_entry *cmem_rcc_file;
struct cmem_proc_data_t cmem_proc_data;
buffer_t *buffer_table;
#ifdef BPA
  static u_long* sram_p_addr;
  static u_long* sram_k_addr;
#endif

//Prototypes
int cmem_rcc_drv_open(struct inode *inode, struct file *file);
int cmem_rcc_drv_release(struct inode *inode, struct file *file);
int cmem_rcc_drv_ioctl(struct inode *inode, struct file *file, u_int cmd, u_long arg);
int cmem_rcc_drv_mmap(struct file *file, struct vm_area_struct *vma);
void cmem_rcc_drv_vclose(struct vm_area_struct *vma);

// The ordinary device operations
static struct file_operations cmem_rcc_drv_fops =
{
  owner:   THIS_MODULE,
  mmap:    cmem_rcc_drv_mmap,
  open:    cmem_rcc_drv_open,
  ioctl:   cmem_rcc_drv_ioctl,
  release: cmem_rcc_drv_release,
};

// memory handler functions
static struct vm_operations_struct cmem_rcc_drv_vm_ops = 
{
  close:  cmem_rcc_drv_vclose,   // mmap-close
};

/*****************************************************************************************/
static int proc_write_cmem(struct file *file, const char *buffer, u_long count, void *data)
/*****************************************************************************************/
{
  int len, loop, loop2;
  struct cmem_proc_data_t *fb_data = (struct cmem_proc_data_t *)data;
  struct page *page_ptr;

  kdebug(("cmem_rcc_drv(proc_write_cmem): proc_write_cmem called\n"));

  if(count > 99)
    len = 99;
  else
    len = count;

  if (copy_from_user(fb_data->value, buffer, len))
  {
    kdebug(("cmem_rcc_drv(proc_write_cmem): error from copy_from_user\n"));
    return(-EFAULT);
  }

  kdebug(("cmem_rcc_drv(proc_write_cmem): len = %d\n", len));
  fb_data->value[len - 1] = '\0';
  kdebug(("cmem_rcc_drv(proc_write_cmem): text passed = %s\n", fb_data->value));
 
  if (!strcmp(fb_data->value, "debug"))
  {
    debug = 1;
    kdebug(("cmem_rcc_drv(proc_write_cmem): debugging enabled\n")); 
  }
 
  if (!strcmp(fb_data->value, "nodebug"))
  {
    kdebug(("cmem_rcc_drv(proc_write_cmem): debugging disabled\n")); 
    debug = 0; 
  }

  if (!strcmp(fb_data->value, "freelock"))
  {
    kdebug(("cmem_rcc_drv(proc_write_cmem): releasing all locked segments\n")); 
    for(loop = 0; loop < MAX_BUFFS; loop++)
    {
      if (buffer_table[loop].size && buffer_table[loop].locked)
      {
        kdebug(("cmem_rcc_drv(proc_write_cmem): releasing locked buffer: type=%d  paddr=0x%08x  size=0x%08x  name=%s\n", buffer_table[loop].type, buffer_table[loop].paddr, buffer_table[loop].size, buffer_table[loop].name));
	if (buffer_table[loop].type == TYPE_GFP)
	{
	  page_ptr = virt_to_page(buffer_table[loop].kaddr);  // unreserve all pages
	  for (loop2 = (1 << buffer_table[loop].order); loop2 > 0; loop2--, page_ptr++)
            clear_bit(PG_reserved, &page_ptr->flags);
	  free_pages(buffer_table[loop].kaddr, buffer_table[loop].order);  // free the area 
	}
#ifdef BPA
	else
	  bigphysarea_free_pages((void *)buffer_table[loop].kaddr);
#endif
	// clear the entry in the buffer table
	buffer_table[loop].paddr = 0;
	buffer_table[loop].kaddr = 0;
	buffer_table[loop].size  = 0;
	buffer_table[loop].type  = 0;
	buffer_table[loop].pid   = 0;
	buffer_table[loop].order = 0;
	buffer_table[loop].used  = 0;
      }
    }
  }

  return len;
}


/***********************************************************************************************/
static int proc_read_cmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
/***********************************************************************************************/
{
  int loop, nchars = 0;
  static int len = 0;

  kdebug(("cmem_rcc_drv(proc_read_cmem): Called with buf    = 0x%08x\n", (u_int)buf));
  kdebug(("cmem_rcc_drv(proc_read_cmem): Called with *start = 0x%08x\n", (u_int)*start));
  kdebug(("cmem_rcc_drv(proc_read_cmem): Called with offset = %d\n", (u_int)offset));
  kdebug(("cmem_rcc_drv(proc_read_cmem): Called with count  = %d\n", count));
    
  if (offset == 0)
  {
    kdebug(("cmem_rcc_drv(proc_read_cmem): Creating text....\n"));
    len = 0;

    len += sprintf(proc_read_text, "\n");
  #ifdef BPA
    len += sprintf(proc_read_text + len, "0x%08x\n", (u_int)sram_p_addr);
  #else
    len += sprintf(proc_read_text + len, "0x00000000\n");
  #endif 
    len += sprintf(proc_read_text + len, "CMEM RCC driver for release %s (based on CVS tag %s)\n", RELEASE_NAME, CVSTAG);

    len += sprintf(proc_read_text + len, "__get_free_pages\n");
    len += sprintf(proc_read_text + len, "  PID | Handle | Phys. address |          Size | Locked | Order| Name\n");
    for(loop = 0; loop < MAX_BUFFS; loop++)
    {
      if (buffer_table[loop].used && buffer_table[loop].type == TYPE_GFP)
      {
	len += sprintf(proc_read_text + len, "%5d |", buffer_table[loop].pid);
	len += sprintf(proc_read_text + len, "%7d |", loop);
	len += sprintf(proc_read_text + len, "    0x%08x |", buffer_table[loop].paddr);
	len += sprintf(proc_read_text + len, "    0x%08x |", buffer_table[loop].size);
	len += sprintf(proc_read_text + len, "    %s |", buffer_table[loop].locked ? "yes" : " no");
	len += sprintf(proc_read_text + len, "    %d |", buffer_table[loop].order);
	len += sprintf(proc_read_text + len, " %s\n", buffer_table[loop].name);
      }
    }

  #ifdef BPA
    len += sprintf(proc_read_text + len, "BigPhysArea\n");
    len += sprintf(proc_read_text + len, "  PID | Handle | Phys. address |          Size | Locked | Name\n");
    for(loop = 0; loop < MAX_BUFFS; loop++)
    {
      if (buffer_table[loop].used && buffer_table[loop].type == TYPE_BPA)
      {
	len += sprintf(proc_read_text + len, "%5d |", buffer_table[loop].pid);
	len += sprintf(proc_read_text + len, "%7d |", loop);
	len += sprintf(proc_read_text + len, "    0x%08x |", buffer_table[loop].paddr);
	len += sprintf(proc_read_text + len, "    0x%08x |", buffer_table[loop].size);
	len += sprintf(proc_read_text + len, "    %s |", buffer_table[loop].locked ? "yes" : " no");
	len += sprintf(proc_read_text + len, " %s\n", buffer_table[loop].name);
      }
    }
  #endif

    len += sprintf(proc_read_text + len, " \n");
    len += sprintf(proc_read_text + len, "The command 'echo <action> > /proc/cmem_rcc', executed as root,\n");
    len += sprintf(proc_read_text + len, "allows you to interact with the driver. Possible actions are:\n");
    len += sprintf(proc_read_text + len, "debug    -> enable debugging\n");
    len += sprintf(proc_read_text + len, "nodebug  -> disable debugging\n");
    len += sprintf(proc_read_text + len, "freelock -> release all locked segments\n");
  }
  kdebug(("cmem_rcc_drv(proc_read_cmem): number of characters in text buffer = %d\n", len));

  if (count < (len - offset))
    nchars = count;
  else
    nchars = len - offset;
  kdebug(("cmem_rcc_drv(proc_read_cmem): min nchars         = %d\n", nchars));
  
  if (nchars > 0)
  {
    for (loop = 0; loop < nchars; loop++)
      buf[loop + (offset & (PAGE_SIZE - 1))] = proc_read_text[offset + loop];
    *start = buf + (offset & (PAGE_SIZE - 1));
  }
  else
  {
    nchars = 0;
    *eof = 1;
  }
 
  kdebug(("cmem_rcc_drv(proc_read_cmem): returning *start   = 0x%08x\n", (u_int)*start));
  kdebug(("cmem_rcc_drv(proc_read_cmem): returning nchars   = %d\n", nchars));
  return(nchars);
}


/*******************/
int init_module(void)
/*******************/
{
  int loop;

  kdebug(("cmem_rcc_drv(init_module): calling register_chrdev\n"));
  if ((major = register_chrdev(0, "cmem_rcc", &cmem_rcc_drv_fops)) < 0) 
  {
    printk("cmem_rcc_drv(init_module): unable to register character device\n");
    return (-EIO);
  }
  kdebug(("cmem_rcc_drv(init_module): major = %d\n", major));
 
  // Allocate memory for buffer table
  kdebug(("cmem_rcc_drv(init_module): MAX_BUFFS        = %d\n", MAX_BUFFS));
  kdebug(("cmem_rcc_drv(init_module): sizeof(buffer_t) = %d\n", sizeof(buffer_t)));
  kdebug(("cmem_rcc_drv(init_module): need %d bytes for __get_free_pages buffer table\n", MAX_BUFFS * sizeof(buffer_t)));
  buffer_table = (buffer_t *)kmalloc(MAX_BUFFS * sizeof(buffer_t), GFP_KERNEL);
  if (buffer_table == NULL)
  {
    printk("cmem_rcc_drv(init_module): unable to allocate memory for __get_free_pages buffer table\n");
    return (-ENOMEM);
  }
  
  // Clear the buffer table
  for(loop = 0; loop < MAX_BUFFS; loop++)
  {
    buffer_table[loop].paddr  = 0;
    buffer_table[loop].size   = 0;
    buffer_table[loop].used   = 0;
    buffer_table[loop].locked = 0;
    buffer_table[loop].type   = 0;
    buffer_table[loop].pid    = 0;
  }

#ifdef BPA    
  // Temporarily allocate the first page to get the PCI base address for the auto-map feature in vmeconfig
  sram_k_addr = (u_long *) bigphysarea_alloc_pages(1, 0, GFP_KERNEL);
  if (sram_k_addr)
  {
    sram_p_addr = (u_long *) virt_to_bus(sram_k_addr);
    kdebug(("cmem_rcc_drv(init_module): sram_k_addr = 0x%08x\n", (u_long)sram_k_addr));
    kdebug(("cmem_rcc_drv(init_module): sram_p_addr = 0x%08x\n", (u_long)sram_p_addr));
    bigphysarea_free_pages((void *)sram_k_addr);
  }
  else
  {
    kdebug(("cmem_rcc_drv(init_module): Failed to determine the BPA base address\n"));
    kdebug(("cmem_rcc_drv(init_module): Check if BPA memory has been reserved at boot time\n"));
    sram_p_addr = 0;
  }
#endif
      
   proc_read_text = (char *)kmalloc(MAX_PROC_TEXT_SIZE, GFP_KERNEL);
  if (proc_read_text == NULL)
  {
    kdebug(("cmem_rcc_drv(init_module): error from kmalloc\n"));
    return(-EFAULT);
  }
      
  // Install /proc entry
  cmem_rcc_file = create_proc_entry("cmem_rcc", 0644, NULL);
  if (cmem_rcc_file == NULL)
  {
    kdebug(("cmem_rcc_drv(init_module): error from call to create_proc_entry\n"));
    return (-ENOMEM);
  }

  strcpy(cmem_proc_data.name, "cmem_rcc");
  strcpy(cmem_proc_data.value, "cmem_rcc");
  cmem_rcc_file->data       = &cmem_proc_data;
  cmem_rcc_file->read_proc  = proc_read_cmem;
  cmem_rcc_file->write_proc = proc_write_cmem;
  cmem_rcc_file->owner      = THIS_MODULE;

  return(0);
}


/***********************/
void cleanup_module(void)
/***********************/
{
  int loop, loop2;
  struct page *page_ptr;

  // Release orphaned buffers
  for(loop = 0; loop < MAX_BUFFS; loop++)
  {
    if (buffer_table[loop].size)
    {
      if (buffer_table[loop].locked)
      {
        kdebug(("cmem_rcc_drv(cleanup_module): releasing locked buffer: type=%d  paddr=0x%08x  size=0x%08x  name=%s\n",
          buffer_table[loop].type, buffer_table[loop].paddr, buffer_table[loop].size, buffer_table[loop].name));
      }
      else
      {
        kdebug(("cmem_rcc_drv(cleanup_module): Releasing orphaned buffer: type=%d  paddr=0x%08x  size=0x%08x  name=%s\n",
          buffer_table[loop].type, buffer_table[loop].paddr, buffer_table[loop].size, buffer_table[loop].name));
      } 	
      
      if (buffer_table[loop].type == TYPE_GFP)
      {
	// unreserve all pages
	page_ptr = virt_to_page(buffer_table[loop].kaddr);

	for (loop2 = (1 << buffer_table[loop].order); loop2 > 0; loop2--, page_ptr++)
          clear_bit(PG_reserved, &page_ptr->flags);

	// free the area 
	free_pages(buffer_table[loop].kaddr, buffer_table[loop].order);
      }
#ifdef BPA
      else
	bigphysarea_free_pages((void *)buffer_table[loop].kaddr);
#endif
    }
  }
  
  // Return the buffer table
  kfree (buffer_table); 

  // Remove /proc entry
  remove_proc_entry("cmem_rcc", NULL);
  kfree(proc_read_text);

  // Unregister the device 
  unregister_chrdev(major, "cmem_rcc");
 
  return;
}


/***********************************************************/
int cmem_rcc_drv_open(struct inode *inode, struct file *file)
/***********************************************************/
{
  char *buf;
  private_stuff *pptr; 
  int loop;
  
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_open): function called for file at 0x%08x\n", file))
  //reserve space to store information about the memory buffers managed by this "file"
  buf = (char *)kmalloc(sizeof (private_stuff), GFP_KERNEL);
  if (buf == NULL)
  {
    kdebug(("cmem_rcc_drv(cmem_rcc_drv_open): error from kmalloc\n"));
    return(-EFAULT);
  }

  //Initialize the space
  pptr = (private_stuff *) buf;
  for (loop = 0; loop < MAX_BUFFS; loop++)
    pptr->buffer[loop] = 0;

  file->private_data = buf;
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_open): private_data = 0x%08x\n", (u_long)file->private_data));

  return(0);
}


/**************************************************************/
int cmem_rcc_drv_release(struct inode *inode, struct file *file)
/**************************************************************/
{
  int loop, loop2;
  struct page *page_ptr;
  private_stuff *pptr; 

  kdebug(("cmem_rcc_drv(cmem_rcc_drv_release): function called from process %d for file at 0x%08x\n", current->pid, file));

  pptr = (private_stuff *) file->private_data;

  // Release orphaned buffers of the current process
  for(loop = 0; loop < MAX_BUFFS; loop++)
  {
    if ((pptr->buffer[loop] == 1) && (!buffer_table[loop].locked == 1))
    {
      if (buffer_table[loop].type == TYPE_GFP)
      {
	// unreserve all pages
	page_ptr = virt_to_page(buffer_table[loop].kaddr);

	for (loop2 = (1 << buffer_table[loop].order); loop2 > 0; loop2--, page_ptr++)
          clear_bit(PG_reserved, &page_ptr->flags);

	// free the area 
	free_pages(buffer_table[loop].kaddr, buffer_table[loop].order);
      } 
#ifdef BPA
      else
        bigphysarea_free_pages((void *)buffer_table[loop].kaddr);
#endif
      
      kdebug(("cmem_rcc_drv(cmem_rcc_drv_release): Releasing orphaned buffer of process %d: type=%d  paddr=0x%08x  size=0x%08x  name=%s\n",
      buffer_table[loop].pid, buffer_table[loop].type, buffer_table[loop].paddr, buffer_table[loop].size, buffer_table[loop].name));

      // clear the entry in the buffer table
      buffer_table[loop].paddr = 0;
      buffer_table[loop].kaddr = 0;
      buffer_table[loop].size  = 0;
      buffer_table[loop].type  = 0;
      buffer_table[loop].pid   = 0;
      buffer_table[loop].order = 0;
      buffer_table[loop].used  = 0;
      pptr->buffer[loop] = 0;
    }
  }
    
  kfree(file->private_data);  
  return(0);
}


/***********************************************************************************/
int cmem_rcc_drv_ioctl(struct inode *inode, struct file *file, u_int cmd, u_long arg)
/***********************************************************************************/
{
  private_stuff *pptr; 

  pptr = (private_stuff *) file->private_data;

  switch (cmd)
  {
    case CMEM_RCC_GET:
    {
      u_int loop, tnum, ok, pagecount;
      cmem_rcc_t uio_desc;
      struct page *page_ptr;
            
      if (copy_from_user(&uio_desc, (void *)arg, sizeof(cmem_rcc_t)) !=0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GET): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      }   
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.size = 0x%08x\n", uio_desc.size)); 
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.type = 0x%08x\n", uio_desc.type)); 

//MJ-SMP: protect this fragment (preferrably with a spinlock)
      // Look for a free slot in the buffer table
      ok = 0;
      for(tnum = 0; tnum < MAX_BUFFS; tnum++)
      {
        if (buffer_table[tnum].used == 0)
        {
          buffer_table[tnum].used = 1;  //This is to reserve the entry 
          pptr->buffer[tnum] = 1;       //Remember which file this buffer will belong to
          uio_desc.handle = tnum;
          ok = 1;
          kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): tnum = %d\n", tnum));
          break;
        }
      }
//MJ-SMP: end of protected zone

      if (!ok)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GET): all buffers are in use\n");
        return(-CMEM_RCC_OVERFLOW);
      }
     
      if(uio_desc.type == TYPE_GFP)
      {
	uio_desc.kaddr = 0;
	// Get a physically contigous memory area with __get_free_pages 
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): calling __get_free_pages\n"));
	uio_desc.kaddr = __get_free_pages(GFP_ATOMIC, uio_desc.order);
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): __get_free_pages returns address 0x%08x\n", (u_long)uio_desc.kaddr));
	if (!uio_desc.kaddr)
	{
          printk("cmem_rcc_drv(ioctl,CMEM_RCC_GET): error from __get_free_pages for order=%d\n", uio_desc.order);
          buffer_table[tnum].used = 0;  // No longer required
          pptr->buffer[tnum] = 0;
          return(-CMEM_RCC_GFP);
	}

	// Reserve all pages to make them remapable
	page_ptr = virt_to_page(uio_desc.kaddr);

	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): reserving pages\n")); 
	for (loop = (1 << uio_desc.order); loop > 0; loop--, page_ptr++)
          set_bit(PG_reserved, &page_ptr->flags);

	uio_desc.size = PAGE_SIZE * (1 << uio_desc.order);
      } 
#ifdef BPA
      else
      {
	pagecount = (int)((uio_desc.size - 1) / PAGE_SIZE + 1); // pages
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): requested number of BPA pages = %d\n", pagecount));

	uio_desc.kaddr = (long)bigphysarea_alloc_pages(pagecount, 0, GFP_KERNEL);
        uio_desc.size = PAGE_SIZE * pagecount;
	if (uio_desc.kaddr == 0) 
	{
	 printk("cmem_rcc_drv(ioctl,CMEM_RCC_GET): error on bigphysarea_alloc_pages\n"); 
         buffer_table[tnum].used = 0;  //Not required any more
         pptr->buffer[tnum] = 0;
	 return(-CMEM_RCC_BPA);
	} 
      }
#endif
      
      uio_desc.paddr = virt_to_bus((void *) uio_desc.kaddr);

      // Complete the entry in the buffer table
      buffer_table[tnum].size   = uio_desc.size;
      buffer_table[tnum].paddr  = uio_desc.paddr;
      buffer_table[tnum].kaddr  = uio_desc.kaddr;
      buffer_table[tnum].pid    = current->pid;
      buffer_table[tnum].order  = uio_desc.order;
      buffer_table[tnum].type   = uio_desc.type;
      buffer_table[tnum].locked = 0;
      strcpy(buffer_table[tnum].name, uio_desc.name);
  
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): PAGE_SIZE       = 0x%08x\n", (u_int)PAGE_SIZE));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.kaddr  = 0x%08x\n", (u_long)uio_desc.kaddr));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.paddr  = 0x%08x\n", (u_long)uio_desc.paddr));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.size   = 0x%08x\n", (u_int)uio_desc.size));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.name   = %s\n", uio_desc.name));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.order  = 0x%08x\n", (u_int)uio_desc.order));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.type   = 0x%08x\n", (u_int)uio_desc.type));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): uio_desc.handle = 0x%08x\n", (u_int)uio_desc.handle));

      if (copy_to_user((void *)arg, &uio_desc, sizeof(cmem_rcc_t)) != 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GET): error in from copy_to_user\n");
        return(-CMEM_RCC_CTU);
      }

      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GET): done\n"));
      break;
    }    
       
    case CMEM_RCC_FREE:
    {
      u_int handle, loop;
      struct page *page_ptr;
      
      if (copy_from_user(&handle, (void *)arg, sizeof(int)) !=0 )
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      } 
         
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): handle = 0x%08x\n", handle));

      // Check if the handle makes sense
      if (buffer_table[handle].used == 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): Invalid handle\n");
        return(-CMEM_RCC_ILLHAND);
      }
      
      if (buffer_table[handle].type == TYPE_GFP)
      {
	// unreserve all pages
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): unreserving GFP pages\n"));
	page_ptr = virt_to_page(buffer_table[handle].kaddr);

	for (loop = (1 << buffer_table[handle].order); loop > 0; loop--, page_ptr++)
          clear_bit(PG_reserved, &page_ptr->flags);

	// free the area 
	free_pages(buffer_table[handle].kaddr, buffer_table[handle].order);
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): pages freed\n")); 
      }
#ifdef BPA
      else
      {   
        bigphysarea_free_pages((void *)buffer_table[handle].kaddr);
        kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): BPA memory freed @ address 0x%8x\n", buffer_table[handle].kaddr));
      }            
#endif
 
      // Delete the entry in the buffer table
      buffer_table[handle].paddr  = 0;
      buffer_table[handle].locked = 0;
      buffer_table[handle].pid    = 0;
      buffer_table[handle].kaddr  = 0;
      buffer_table[handle].type   = 0;
      buffer_table[handle].used   = 0;
      buffer_table[handle].order  = 0;
      buffer_table[handle].size   = 0;  //This enables the entry to be re-used
      pptr->buffer[handle] = 0;

      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_FREE): done\n"));
      break;
    }
    
  case CMEM_RCC_LOCK:
    {
      u_int handle;
      
      if (copy_from_user(&handle, (void *)arg, sizeof(int)) !=0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_LOCK): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      } 
      
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_LOCK): handle = 0x%08x\n", handle));
      
      // Check if the handle makes sense
      if (buffer_table[handle].used == 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_LOCK): Invalid handle\n");
        return(-CMEM_RCC_ILLHAND);
      }

      buffer_table[handle].locked = 1;
      
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_LOCK): done\n"));
      break;
    }
 
  case CMEM_RCC_UNLOCK:
    {
      u_int handle;
      
      if (copy_from_user(&handle, (void *)arg, sizeof(int)) !=0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_UNLOCK): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      } 
      
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_UNLOCK): handle = 0x%08x\n", handle));
      
      // Check if the handle makes sense
      if (buffer_table[handle].used == 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_UNLOCK): Invalid handle\n");
        return(-CMEM_RCC_ILLHAND);
      }

      buffer_table[handle].locked = 0;
      
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_UNLOCK): done\n"));
      break;
    }
    
  case CMEM_RCC_GETPARAMS:
    {
      cmem_rcc_t uio_desc;
            
      if (copy_from_user(&uio_desc, (void *)arg, sizeof(cmem_rcc_t)) !=0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GETPARAMS): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      }
      
      // Check if the handle makes sense
      if (buffer_table[uio_desc.handle].used == 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GETPARAMS): Invalid handle\n");
        return(-CMEM_RCC_ILLHAND);
      }	
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GETPARAMS): called for handle %d\n", uio_desc.handle));
      uio_desc.paddr  = buffer_table[uio_desc.handle].paddr;
      uio_desc.uaddr  = buffer_table[uio_desc.handle].uaddr;
      uio_desc.kaddr  = buffer_table[uio_desc.handle].kaddr;
      uio_desc.size   = buffer_table[uio_desc.handle].size;
      uio_desc.order  = buffer_table[uio_desc.handle].order;
      uio_desc.locked = buffer_table[uio_desc.handle].locked;
      uio_desc.type   = buffer_table[uio_desc.handle].type;
      strcpy(uio_desc.name, buffer_table[uio_desc.handle].name);
      
      if (copy_to_user((void *)arg, &uio_desc, sizeof(cmem_rcc_t)) != 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_GETPARAMS): error in from copy_to_user\n");
        return(-CMEM_RCC_CTU);
      }
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_GETPARAMS): done\n"));
      break;
    } 
       
    case CMEM_RCC_SETUADDR:
    {
      cmem_rcc_t uio_desc;
            
      if (copy_from_user(&uio_desc, (void *)arg, sizeof(cmem_rcc_t)) !=0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_SETUADDR): error in from copy_from_user\n");
        return(-CMEM_RCC_CFU);
      }
      
      // Check if the handle makes sense
      if (buffer_table[uio_desc.handle].used == 0)
      {
        printk("cmem_rcc_drv(ioctl,CMEM_RCC_SETUADDR): Invalid handle\n");
        return(-CMEM_RCC_ILLHAND);
      }	
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_SETUADDR): called for handle %d\n", uio_desc.handle));
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_SETUADDR): uaddr = 0x%08x\n", uio_desc.uaddr));
      buffer_table[uio_desc.handle].uaddr = uio_desc.uaddr;
      
      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_SETUADDR): done\n"));
      break;
    } 
             
  case CMEM_RCC_DUMP:
    {
      char *buf;
      int len, loop;

      kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_DUMP): called\n"));

      buf = (char *)kmalloc(TEXT_SIZE, GFP_KERNEL);
      if (buf == NULL)
      {
        kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_DUMP): error from kmalloc\n"));
        return(-CMEM_RCC_KMALLOC);
      }

      len = 0;
      len += sprintf(buf + len, "Memory allocated by __get_free_pages\n");
      len += sprintf(buf + len, "  PID | Phys. address |          Size | Order | Locked | Name\n");
      for(loop = 0; loop < MAX_BUFFS; loop++)
      {
        if (buffer_table[loop].size && buffer_table[loop].type == TYPE_GFP)
        {
          len += sprintf(buf + len, "%5d |", buffer_table[loop].pid);
          len += sprintf(buf + len, "    0x%08x |", buffer_table[loop].paddr);
          len += sprintf(buf + len, "    0x%08x |", buffer_table[loop].size);
          len += sprintf(buf + len, "     %d |", buffer_table[loop].order);
          len += sprintf(buf + len, "    %s |", buffer_table[loop].locked ? "yes" : " no");
          len += sprintf(buf + len, " %s\n", buffer_table[loop].name);
        }
      }

#ifdef BPA
      len += sprintf(buf + len, "Memory allocated by BigPhysArea\n");
      len += sprintf(buf + len, "  PID | Phys. address |          Size | Locked | Name\n");
      for(loop = 0; loop < MAX_BUFFS; loop++)
      {
        if (buffer_table[loop].size && buffer_table[loop].type == TYPE_BPA)
        {
          len += sprintf(buf + len, "%5d |", buffer_table[loop].pid);
          len += sprintf(buf + len, "    0x%08x |", buffer_table[loop].paddr);
          len += sprintf(buf + len, "    0x%08x |", buffer_table[loop].size);
          len += sprintf(buf + len, "    %s |", buffer_table[loop].locked ? "yes" : " no");
          len += sprintf(buf + len, " %s\n", buffer_table[loop].name);
        }
      }
#endif

      if (copy_to_user((void *)arg, buf, TEXT_SIZE * sizeof(char)) != 0)
      {
	kdebug(("cmem_rcc_drv(ioctl,CMEM_RCC_DUMP): error from copy_to_user\n"));
	return(-CMEM_RCC_CTU);
      }
            
      kfree(buf);
      break;
    }
  }
  return(0);
}


// 2.4.x: this method is called from do_mmap_pgoff, from
// do_mmap, from the syscall. The caller of do_mmap grabs
// the mm semaphore. So we are protected from races here.
/******************************************************************/
int cmem_rcc_drv_mmap(struct file *file, struct vm_area_struct *vma)
/******************************************************************/
{
  u_long offset, size;

  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): cmem_rcc_drv_mmap called\n"));
  
  offset = vma->vm_pgoff << PAGE_SHIFT;
  
  size = vma->vm_end - vma->vm_start;

  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): offset = 0x%08x\n", (u_long)offset));
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): size   = 0x%08x\n", (u_long)size));

  if (offset & ~PAGE_MASK)
  {
    printk("cmem_rcc_drv(cmem_rcc_drv_mmap): offset not aligned: %ld\n", offset);
    return -ENXIO;
  }

  // we only support shared mappings. "Copy on write" mappings are
  // rejected here. A shared mapping that is writeable must have the
  // shared flag set.
  if ((vma->vm_flags & VM_WRITE) && !(vma->vm_flags & VM_SHARED))
  {
    printk("cmem_rcc_drv(cmem_rcc_drv_mmap): writeable mappings must be shared, rejecting\n");
    return(-EINVAL);
  }

  vma->vm_flags |= VM_RESERVED;
  
  // we do not want to have this area swapped out, lock it
  vma->vm_flags |= VM_LOCKED;

  // we create a mapping between the physical pages and the virtual
  // addresses of the application with remap_page_range.
  // enter pages into mapping of application
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): Parameters of remap_page_range()\n"));
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): Virtual address  = 0x%08x\n", (u_long)vma->vm_start));
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): Physical address = 0x%08x\n", (u_long)offset));
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): Size             = 0x%08x\n", (u_long)size));
  if (remap_page_range(vma, vma->vm_start, offset, size, vma->vm_page_prot))
  {
    printk("cmem_rcc_drv(cmem_rcc_drv_mmap): remap page range failed\n");
    return -ENXIO;
  }
  
  vma->vm_ops = &cmem_rcc_drv_vm_ops;  
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_mmap): cmem_rcc_drv_mmap done\n"));
  return(0);
}


/**************************************************/
void cmem_rcc_drv_vclose(struct vm_area_struct *vma)
/**************************************************/
{  
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_vclose): Virtual address  = 0x%08x\n", (u_long)vma->vm_start));
  kdebug(("cmem_rcc_drv(cmem_rcc_drv_vclose): mmap released\n"));
}
