#ifndef COCA_CLIENTISSUES_H
#define COCA_CLIENTISSUES_H

//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id$
//
// Description:
//	Class Issues.
//
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------

//----------------------
// Base Class Headers --
//----------------------
#include "ers/ers.h"

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "coca/common/Issues.h"

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//		---------------------
// 		-- Class Interface --
//		---------------------

// Void Structure, Just For SemiColon; to make doxygen happier
#define VSJFSC struct _VSJFSC_

namespace daq {
namespace coca {

/// @addtogroup coca_client

/**
 *  @ingroup coca_client
 *
 *  @namespace daq::coca::DBClientIssues
 *  @brief Collection of issue classes raised by DBClient class
 */

/**
 *  @ingroup coca_client
 *
 *  @namespace daq::coca::RegisterIssues
 *  @brief Collection of issue classes raised by Register class
 */

/**
 *  @class daq::coca::DBClientIssues::CannotReadDB
 *  @brief Issue generated when client cannot access CoCa database.
 */
ERS_DECLARE_ISSUE_BASE (DBClientIssues, CannotReadDB, Issues::Exception,
                        "database unreadable or corrupted: " << session,
                        , ((std::string) session)) VSJFSC;

/**
 *  @class daq::coca::DBClientIssues::DatasetNotFound
 *  @brief Issue generated when client provides dataset name which does not exist.
 */
ERS_DECLARE_ISSUE_BASE (DBClientIssues, DatasetNotFound, Issues::Exception,
                        "dataset not found: " << dataset,
                        , ((std::string) dataset)) VSJFSC;

/**
 *  @class daq::coca::DBClientIssues::ArchiveNotFound
 *  @brief Issue generated when client provides archive name which does not exist.
 */
ERS_DECLARE_ISSUE_BASE (DBClientIssues, ArchiveNotFound, Issues::Exception,
                        "archive not found: " << archive,
                        , ((std::string) archive)) VSJFSC;

/**
 *  @class daq::coca::DBClientIssues::FileNotFound
 *  @brief Issue generated when client provides file name which does not exist.
 */
ERS_DECLARE_ISSUE_BASE (DBClientIssues, FileNotFound, Issues::Exception,
                        "file not found in dataset `" << dataset
                        << "': " << file,
                        , ((std::string) dataset) ((std::string) file)) VSJFSC;

/**
 *  class daq::coca::RegisterIssues::Exception
 *  @brief Base class for the exception-type issues generated by Register class.
 */
ERS_DECLARE_ISSUE (RegisterIssues, Exception, , ) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::IPCException
 *  @brief Class for issues generated for communication problems with CoCa IPC server.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, IPCException, Exception,
    "IPC error: " << message, , ((std::string) message)) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::MissingFile
 *  @brief Issue generated when file being registered cannot be found.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, MissingFile, Exception,
    "File does not exist: " << file, , ((std::string) file)) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::PathNameError
 *  @brief Issue generated when file being registered cannot be found.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, PathNameError, Exception,
        "Error in path names: path='" << pathname << "' basedir='" << basedir << "'", ,
        ((std::string) basedir) ((std::string) pathname)) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::SetupError
 *  @brief Base class for issues generated when something in the setup is not right.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, SetupError, Exception, , , ) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::PartitionNotFound
 *  @brief Issue generated when specified partition name does not exist.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, PartitionNotFound, SetupError,
    "partition not found: '" << partition << "'", , ((std::string) partition)) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::ServerNotFound
 *  @brief Issue generated when CoCa server cannot be found.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, ServerNotFound, SetupError,
    "CoCa server not found: " << server << " (caught: " << exception << ")",
    , ((std::string) server) ((std::string) exception)) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::CoCaException
 *  @brief Base class for issues generated when CoCa server cannot register files.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, CoCaException, Exception, , , ) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::RegistrationRefused
 *  @brief Issue generated when CoCa server cannot accept new registrations (typically
 *    when server shuts itself down).
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, RegistrationRefused, CoCaException,
    "new registration refused by server", , ) VSJFSC;

/**
 *  @class daq::coca::RegisterIssues::RegistrationError
 *  @brief Issue generated when CoCa server fails to register a file.
 */
ERS_DECLARE_ISSUE_BASE (RegisterIssues, RegistrationError, CoCaException,
 "registration failed with error: " << error << " (" << message << ")",
 , ((int) error) ((std::string) message)) VSJFSC;

} // namespace coca
} // namespace daq

#endif // COCA_CLIENTISSUES_H
