#ifndef COCA_ISSUES_H
#define COCA_ISSUES_H

//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id$
//
// Description:
//	Class Issues.
//
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------

//----------------------
// Base Class Headers --
//----------------------
#include "ers/ers.h"

//-------------------------------
// Collaborating Class Headers --
//-------------------------------

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//		---------------------
// 		-- Class Interface --
//		---------------------

// Void Structure, Just For SemiColon; to make doxygen happier
#define VSJFSC struct _VSJFSC_

namespace daq {
namespace coca {

/// @addtogroup coca_common

/**
 *  @ingroup coca_common
 */

    /**
     *  @namespace daq::coca::Issues
     *  @brief Collection of issue classes for coca package.
     *  
     *  The classes defined in this namespace whould generally be useful for both 
     *  client and server side (or coca_common package). For client- or server-specific
     *  issues define corresponding classes in client or server packages.
     *  
     *  @version $Id$
     *
     *  @author Federico Zema
     *  @author Andy Salnikov
     */

    /**
     *  @class daq::coca::Issues::Exception
     *  @brief Generic exception class, base for many exception classes in coca package.
     */
    ERS_DECLARE_ISSUE (Issues, Exception, , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::CoralException
     *  @brief Generic CORAL exception, generated for CORAL-related issues.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, CoralException, Exception, 
     "Generic CORAL exception", , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::DBConnectionFailure
     *  @brief Exception generated when database connection fails.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, DBConnectionFailure, CoralException,
     "Database connection failed: ", , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::TransactionException
     *  @brief Generic transaction-related error.
     */
    ERS_DECLARE_ISSUE_BASE(Issues, TransactionException, Exception, , , ) VSJFSC;
    
    /**
     *  @class daq::coca::Issues::TransactionActive
     *  @brief Exception generated when trying to start transaction when 
     *         transaction is already active.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, TransactionActive, TransactionException, 
      "cannot start transaction, another transaction is active", , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::TransactionInactive
     *  @brief Exception generated when trying to stop transaction when 
     *         transaction is not active.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, TransactionInactive, TransactionException, 
      "cannot close transaction, transaction already closed", , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::NoSuchRecord
     *  @brief Exception generated for the missing record in the database when 
     *  the code expects it to be there.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, NoSuchRecord, Exception,
      "no such record: " << PK, , ((std::string) PK)) VSJFSC;

    /**
     *  @class daq::coca::Issues::FileAlreadyRegistered
     *  @brief Exception generated while trying to register a file which is in 
     *  the database already.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, FileAlreadyRegistered, Exception,
      "file was already registered: " << file, , ((std::string) file)) VSJFSC;
                              
    /**
     *  @class daq::coca::Issues::Message
     *  @brief Generic message class, base for messages generated by coca.
     */
    ERS_DECLARE_ISSUE (Issues, Message, , ) VSJFSC;

    /**
     *  @class daq::coca::Issues::WaitForDatabase
     *  @brief Message generated when database connection fails.
     */
    ERS_DECLARE_ISSUE_BASE (Issues, WaitForDatabase, Message,
      "Database error detected, connection may have failed, will wait for re-connect", , ) VSJFSC;

} // namespace coca
} // namespace daq

#endif // COCA_ISSUES_H
