#ifndef COCA_SIZEMB_H
#define COCA_SIZEMB_H
//----------------------------------------------------------------------
// File and Version Information:
//      $Id$
//----------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "ers/ers.h"

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------


namespace daq {
namespace coca {

    // Issues declaration
    ERS_DECLARE_ISSUE (SizeMB, Exception, , )

    ERS_DECLARE_ISSUE_BASE (SizeMB, SizeMismatch, Exception,
                            "file: " << path
                            << ": expected " << exp_sizeMB
                            << "MB, got: " << act_sizeMB << "MB",
                            , ((std::string) path)
                            ((unsigned) exp_sizeMB)
                            ((unsigned) act_sizeMB))

/// @addtogroup coca_common

/**
 * @ingroup coca_common
 * @brief Namespace containing several helper methods to calculate
 * and check the sizes of files.
 */
namespace SizeMB {

    /**
     *  @brief Calculate size in Megabytes.
     *  
     *  Returns size divided by the number  of bytes in Megabyte, except
     *  that for non-zero sizes below 1MB it always returns 1.
     *  
     *  @param[in] size  Size in bytes.
     *  @return Size in megabytes.
     */
     unsigned getSizeMB (uint64_t size);
    
    /**
     *  @brief Returns file size in MB.
     *  
     *  Find a file, gets its size and uses getSizeMB() to calculate
     *  the the MB size of the file.
     *  
     *  @param[in] path  Path name of the file.
     *  @return Size of the file in megabytes.
     *  
     *  @throw SizeMB::Exception  Thrown if file is not found or cannot be accessed.
     */
    unsigned getSizeMB (const std::string &path);

    /**
     *  @brief Checks the size of the file.
     *  
     *  Finds the file, gets its size and compares to its expected size, throws
     *  an exception if sizes are different.
     *  
     *  @param[in] path  Path name of the file.
     *  @param[in] expectedSizeMB  Expected file size in MB.
     *  
     *  @throw SizeMB::SizeMismatch  If the file cannot be found or if its 
     *                               size differs from expected size.
     */
    void checkSizeMB (const std::string &path, unsigned expectedSizeMB);

} // namespace SizeMB

} // namespace coca
} // namespace daq

#endif // COCA_SIZEMB_H
