// $Id$
// $Name$

#include "coca/common/DBBase.h"

#include <CoralBase/Exception.h>
#include <RelationalAccess/ConnectionService.h>
#include <RelationalAccess/IConnectionServiceConfiguration.h>

#include "coca/common/Issues.h"
#include "ers/ers.h"

//#include <DB/TimeStamp.h>

using namespace daq::coca;


/*
 * DBBase
 */

DBBase::DBBase (const std::string& connStr)
    : DBBase(connStr, false)
{
}

DBBase::DBBase (const std::string& connStr, bool update)
    : m_connStr(connStr)
    , m_nextid("COCA_NEXTID")
    , m_s2i("COCA_STR2ID", m_nextid)
    , m_arName("COCA1_ARCHIVES")
    , m_caName("COCA1_CACHE")
    , m_update(update)
{
}

DBBase::~DBBase ()
    throw ()
{
}

/// Get the handle for archive database table
coral::ITable& 
DBBase::archiveTable(Transaction& tran) const
{
    return tran.session().nominalSchema().tableHandle(m_arName);
}

/// Get the handle for cache database table
coral::ITable& 
DBBase::cacheTable(Transaction& tran) const
{
    return tran.session().nominalSchema().tableHandle(m_caName);
}


std::string
DBBase::getStr (unsigned id, Transaction& tran)
{
    return m_s2i.getStr (id, tran);
}

unsigned
DBBase::getId (const std::string &str, Transaction& tran)
{
    return m_s2i.getId (str, tran);
}

unsigned
DBBase::insStr (const std::string &str, Transaction& tran)
{
    return m_s2i.insert (str, tran);
}

std::shared_ptr<coral::ISessionProxy>
DBBase::session()
{

    coral::ConnectionService conn_serv;

    static bool doDebug = getenv("COCA_CORAL_DEBUG") != nullptr;
    if (doDebug) {
        conn_serv.setMessageVerbosityLevel(coral::Debug);
    } else {
        conn_serv.setMessageVerbosityLevel(coral::Warning);
    }

    // set connection parameter so that connection failures happen sooner
    // and replicas are not excluded for too long
    coral::IConnectionServiceConfiguration& config = conn_serv.configuration();
    config.setMissingConnectionExclusionTime(0);
    config.setConnectionRetrialPeriod(2);
    config.setConnectionRetrialTimeOut(2);

    ERS_DEBUG (0, "database connection: \"" << m_connStr << '"');
    auto accessMode = m_update ? coral::Update : coral::ReadOnly;
    coral::ISessionProxy* session = conn_serv.connect (m_connStr, accessMode);

    return std::shared_ptr<coral::ISessionProxy>(session);

}

// Returns default CORAL connection string for CoCa database.
std::string 
DBBase::defConnStr()
{
  static const std::string connStr("COCA");
  return connStr;
}
