//----------------------------------------------------------------------
// File and Version Information:
// 	$Id$
//
// Description:
//	Class IncomingFile...
//
// Author List:
//      Andy Salnikov
//
//----------------------------------------------------------------------

//-----------------------
// This Class's Header --
//-----------------------
#include "coca/common/IncomingFile.h"

//-----------------
// C/C++ Headers --
//-----------------
#include <iostream>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "coca/common/FileSystem.h"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

//		----------------------------------------
// 		-- Public Function Member Definitions --
//		----------------------------------------

namespace daq {
namespace coca {

//----------------
// Constructors --
//----------------
IncomingFile::IncomingFile (const std::string& host,
                            const std::string& clientDir,
                            const std::string& relPath, 
                            unsigned sizeMB,
                            int prio,
                            unsigned dtcSec)
    : m_host(host)
    , m_clientDir(clientDir)
    , m_relPath(relPath)
    , m_sizeMB(sizeMB)
    , m_prio(prio)
    , m_dtcSec(dtcSec)
{
}

//--------------
// Destructor --
//--------------
IncomingFile::~IncomingFile () throw ()
{
}

/// Returns path name on client host.
std::string 
IncomingFile::clientPath () const
{
    return FileSystem::join(m_clientDir, m_relPath);
}

/// Stream insertion operator to print object contents
std::ostream& 
operator<<(std::ostream& out, const IncomingFile& f)
{
    out << "RelPath(" << f.relPath() << ") SizeMB(" << f.sizeMB() << "MB)"
        << " PRIO(" << f.prio() << ") DTC(" << f.dtcSec() << "sec)"
        << " Host(" << f.host() << ") DirClient(" << f.clientDir() << ")";
    return out;
}

} // namespace coca
} // namespace daq
