//----------------------------------------------------------------------
// File and Version Information:
//      $Id$
//----------------------------------------------------------------------

//-----------------------
// This Class's Header --
//-----------------------
#include "coca/common/SizeMB.h"

//-----------------
// C/C++ Headers --
//-----------------
#include <boost/filesystem.hpp>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

using namespace daq::coca;
namespace fs = boost::filesystem;


unsigned
SizeMB::getSizeMB (uint64_t size)
{
    unsigned sizeMB = size/1048576;
    if (sizeMB == 0 && size > 0) {
        sizeMB = 1;
    }
    return sizeMB;
}

unsigned
SizeMB::getSizeMB (const std::string &path)
{
    try {
        uint64_t size = fs::file_size(path);
        return getSizeMB (size);
    } catch (const fs::filesystem_error &ex) {
        throw Exception(ERS_HERE, ex);
    }
}


void
SizeMB::checkSizeMB (const std::string &path, unsigned expectedSizeMB)
{
    // find actual size
    unsigned actual_sizeMB = 0;
    try {
        actual_sizeMB = getSizeMB (path);
    } catch (const ers::Issue &ex) {
        // means file does not exist or other failure to access it
        throw SizeMismatch (ERS_HERE, path, expectedSizeMB, 0, ex);
    }
    
    // check sizes
    if (actual_sizeMB != expectedSizeMB) {
        throw SizeMismatch (ERS_HERE, path, expectedSizeMB, actual_sizeMB);
    }
}
