//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id$
//
// Description:
//	Test suite case for the coca::FileSystem.
//
//------------------------------------------------------------------------

//---------------
// C++ Headers --
//---------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "coca/common/FileSystem.h"

using namespace daq::coca ;

#define BOOST_TEST_MODULE coca_test_fs
#define BOOST_TEST_DYN_LINK
#include <boost/test/unit_test.hpp>

/**
 * Simple test suite for module coca_test_fs.
 * See http://www.boost.org/doc/libs/1_36_0/libs/test/doc/html/index.html
 */

// ==============================================================

BOOST_AUTO_TEST_CASE( test_join )
{
    BOOST_CHECK_EQUAL(FileSystem::join("a", "b"), "a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("a", "/b"), "a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("/a", "b"), "/a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("/a", "/b"), "/a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("/a", "////b"), "/a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("a/", "/b/"), "a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("a///", "/b///"), "a/b");
    BOOST_CHECK_EQUAL(FileSystem::join("/a/b////c/", "/d///e///f"), "/a/b/c/d/e/f");
}

// ==============================================================

BOOST_AUTO_TEST_CASE( test_split )
{
    std::pair<std::string, std::string> res;
    
    // check with absolute path and empty basedir
    res = FileSystem::split("/a/b/c/d/e", "");
    BOOST_CHECK_EQUAL(res.first, "/a/b/c/d");
    BOOST_CHECK_EQUAL(res.second, "e");

    // check with relative path and empty basedir
    res = FileSystem::split("d/e", "", "/a/b/c/");
    BOOST_CHECK_EQUAL(res.first, "/a/b/c/d");
    BOOST_CHECK_EQUAL(res.second, "e");

    // check with absolute path and basedir
    res = FileSystem::split("/a/b/c/d/e", "/a/b/c/");
    BOOST_CHECK_EQUAL(res.first, "/a/b/c");
    BOOST_CHECK_EQUAL(res.second, "d/e");

    // check with relative path and basedir
    res = FileSystem::split("e", "/a/b/c", "/a/b/c/d");
    BOOST_CHECK_EQUAL(res.first, "/a/b/c");
    BOOST_CHECK_EQUAL(res.second, "d/e");

    // This should throw because basedir is different from path
    BOOST_CHECK_THROW(res = FileSystem::split("/a/b/c/d/e", "/a/b/C"), FileSystem::BasedirMismatch);

    // This should throw because basedir is longer than path
    BOOST_CHECK_THROW(res = FileSystem::split("/a/b", "/a/b/c"), FileSystem::BasedirMismatch);

}
