"""Module containing MdaMainMenu class and related methods.
"""

#--------------------------------
#  Imports of standard modules --
#--------------------------------
import logging

#-----------------------------
# Imports for other modules --
#-----------------------------
import urwid
from ..AppPanel import AppPanel
from ..ui import UIListBoxWithHeader, UIColumns, UISelectableText

#----------------------------------
# Local non-exported definitions --
#----------------------------------

_log = logging.getLogger(__name__)

#------------------------
# Exported definitions --
#------------------------


class MdaMainMenu(UIListBoxWithHeader, AppPanel):
    """Widget class containing list of possible options in MDA main menu.

    Parameters
    ----------
    app : `MdaAdmin`
    """

    _selectable = True

    signals = ['selected']

    #----------------
    #  Constructor --
    #----------------
    def __init__(self, app):
        self._app = app

        menu = [("Paritions Table", "partitions"),
                ("Datasets table", "datasets"),
                ("Files not in CoCa", "files_not_archived"),
                ("LB Sequences Table", "lb_seq")]

        col_width = [30]
        header = UIColumns(['Data selection'], col_width, 2)

        items = []
        for selection, value in menu:
            item = UIColumns([UISelectableText(selection)], col_width, 2, 0)
            urwid.connect_signal(item, 'activated', self._itemActivated, user_args=[value])
            items.append(item)

        UIListBoxWithHeader.__init__(self, items, header=header)

    def title(self):
        return 'Information to browse'

    def hints(self):
        return [('Enter', "Select")]

    def status(self):
        return "Database information selector"

    def _itemActivated(self, value, item):
        _log.debug("emitting signal 'selected': %r", value)
        self._emit('selected', value)
