#ifndef COLDPIE_ENUMTYPE_H
#define COLDPIE_ENUMTYPE_H

//--------------------------------------------------------------------------
//
// Description:
//  Class EnumType.
//
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------
#include "Python.h"
#include <map>

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//      ---------------------
//      -- Class Interface --
//      ---------------------

namespace coldpie {

/// @addtogroup coldpie

/**
 *  @ingroup coldpie
 *
 *  @brief Trivial emulation of C++ enum type for Python.
 *
 *  Enum type created by this class is just a "namespace type" with a bunch
 *  of class attributes defining names for integer constants. It is more or
 *  less equivalent to this code:
 *
 *      class SomeEnum:
 *          """Docstring generated by the class"
 *          CONST1 = 1
 *          CONST2 = 2
 */

class EnumType {
public:

    /// Enum is a name plus integer value
    struct Enum {
        const char* name;
        int value;
    };

    /**
     *  This constructor takes two arguments: name which may be a dotted
     *  name including package and module name, and array of enums,
     *  last array entry must have zero name pointer
     */
    EnumType(const char* typeName, Enum* enums);
    EnumType(const char* typeName, Enum* enums, const char* modName);

    /**
     *  Returns type name
     */
    const char* typeName() const
    {
        return m_typeName;
    }

    // Destructor
    ~EnumType();

    // Returns _borrowed_ reference to Python type object
    PyObject* type() const
    {
        return (PyObject*)m_type;
    }

protected:

private:

    typedef std::map<long, PyObject*> Int2Enum;

    // Data members
    char* m_typeName;
    PyTypeObject* m_type;
    char* m_docString;

    EnumType(const EnumType&);
    EnumType& operator=(const EnumType&);
};

} // namespace coldpie

#endif // COLDPIE_ENUMTYPE_H
