//--------------------------------------------------------------------------
// File and Version Information:
//  $Id$
//
// Description:
//  Class EnumType...
//
// Author List:
//      Andrei Salnikov
//
//------------------------------------------------------------------------

//-----------------------
// This Class's Header --
//-----------------------
#include "coldpie/EnumType.h"

//-----------------
// C/C++ Headers --
//-----------------
#include <string>
#include <cstring>
#include <sstream>

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "coldpie/make_pyshared.h"

//-----------------------------------------------------------------------
// Local Macros, Typedefs, Structures, Unions and Forward Declarations --
//-----------------------------------------------------------------------

#if PY_MAJOR_VERSION >= 3
#define PY3MIG_STRING_FROM_STRING(X) PyUnicode_FromString(X)
#define PY3MIG_INT_FROM_LONG(X) PyLong_FromLong(X)
#else
#define PY3MIG_STRING_FROM_STRING(X) PyString_FromString(X)
#define PY3MIG_INT_FROM_LONG(X) PyInt_FromLong(X)
#endif

namespace {

char docString[] = "\
Python class which emulates C++ enumeration. It defines class attributes\n\
whose values are integer numbers, so that you can use EnumType.EnumName\n\
notation to access the values.\n\
";

}

//      ----------------------------------------
//      -- Public Function Member Definitions --
//      ----------------------------------------

//----------------
// Constructors --
//----------------
coldpie::EnumType::EnumType(const char* typeName, Enum* enums)
  : EnumType(typeName, enums, nullptr)
{
}

coldpie::EnumType::EnumType(const char* typeName, Enum* enums, const char* modName)
  : m_typeName(0)
  , m_type()
  , m_docString(0)
{
  PyObject* pyTypeName = PY3MIG_STRING_FROM_STRING(typeName);
  PyObject* bases = PyTuple_Pack(1, (PyObject*)&PyBaseObject_Type);
  PyObject* dict = PyDict_New();

  // define class attributes as enum values
  for (Enum* eiter = enums; eiter->name ; ++ eiter) {

    // convert value to Python integer
    PyObject* value = PY3MIG_INT_FROM_LONG(eiter->value);

    // store it in the class attribute
    PyDict_SetItemString(dict, eiter->name, value);

    Py_CLEAR(value);
  }

  // generate doc string
  std::ostringstream doc;
  doc << ::docString << "\nThis class defines following constants:";
  for (Enum* eiter = enums; eiter->name ; ++ eiter) {
    doc << "\n    - " << eiter->name << " = " << eiter->value;
  }
  PyObject* docstr = PY3MIG_STRING_FROM_STRING(doc.str().c_str());
  PyDict_SetItemString(dict, "__doc__", docstr);
  PyObject* modname = nullptr;
  if (modName != nullptr) {
    PyObject* modname = PY3MIG_STRING_FROM_STRING(modName);
    PyDict_SetItemString(dict, "__module__", modname);
  }

  PyObject* args = PyTuple_Pack(3, pyTypeName, bases, dict);
  m_type = (PyTypeObject*)PyObject_CallObject((PyObject*)&PyType_Type, args);
  Py_CLEAR(args);
  Py_CLEAR(modname);
  Py_CLEAR(docstr);
  Py_CLEAR(dict);
  Py_CLEAR(bases);
  Py_CLEAR(pyTypeName);
}

//--------------
// Destructor --
//--------------
coldpie::EnumType::~EnumType ()
{
}
