#ifndef DAQ_TOKENS_JSON_TRAITS_H_
#define DAQ_TOKENS_JSON_TRAITS_H_

#include <string>
#include <vector>
#include <map>
#include <chrono>

#include <nlohmann/json.hpp>
#include "jwt-cpp/jwt.h"

namespace daq {
  namespace tokens {

    // specialization for nlohmann::json
    struct nlohmann_traits {
      
      using json = nlohmann::json;
      using value_type = json;
      using object_type = json::object_t;
      using array_type = json::array_t;
      using string_type = std::string;
      using number_type = double;
      using integer_type = int64_t;
      using boolean_type = bool;

      static jwt::json::type get_type(const json &val);
      static json::object_t  as_object(const json &val);
      static std::string     as_string(const json &val);
      static json::array_t   as_array(const json &val);
      static int64_t         as_int(const json &val);
      static bool            as_bool(const json &val);
      static double          as_number(const json &val);
      static bool            parse(json &val, std::string str);
      static std::string     serialize(const json &val);
    };
  }
}

#endif // DAQ_TOKENS_JSON_TRAITS_H_
