
#include "daq_tokens/internal/json_traits.h"

namespace daq {
  namespace tokens {

    using nlohmann::json;
    
    jwt::json::type nlohmann_traits::get_type(const json &val) {
      using jwt::json::type;
      
      if (val.type() == json::value_t::boolean)
	return type::boolean;
      else if (val.type() == json::value_t::number_integer)
	return type::integer;
      else if (val.type() == json::value_t::number_float)
	return type::number;
      else if (val.type() == json::value_t::string)
	return type::string;
      else if (val.type() == json::value_t::array)
	return type::array;
      else if (val.type() == json::value_t::object)
	return type::object;
      else
	throw std::logic_error("invalid type");
    }
    
    json::object_t nlohmann_traits::as_object(const json &val) {
      if (val.type() != json::value_t::object)
	throw std::bad_cast();
      return val.get<json::object_t>();
    }

    std::string nlohmann_traits::as_string(const json &val) {
      if (val.type() != json::value_t::string)
	throw std::bad_cast();
      return val.get<std::string>();
    }

    json::array_t nlohmann_traits::as_array(const json &val) {
      if (val.type() != json::value_t::array)
	throw std::bad_cast();
      return val.get<json::array_t>();
    }

    int64_t nlohmann_traits::as_int(const json &val) {
      if (val.type() != json::value_t::number_integer &&
	  val.type() != json::value_t::number_unsigned)
	throw std::bad_cast();
      return val.get<int64_t>();
    }

    bool nlohmann_traits::as_bool(const json &val) {
      if (val.type() != json::value_t::boolean)
	throw std::bad_cast();
      return val.get<bool>();
    }

    double nlohmann_traits::as_number(const json &val) {
      if (val.type() != json::value_t::number_float)
	throw std::bad_cast();
      return val.get<double>();
    }

    bool nlohmann_traits::parse(json &val, std::string str) {
      val = json::parse(str.begin(), str.end());
      return true;
    }

    std::string nlohmann_traits::serialize(const json &val) {
      return val.dump();
    }

  }
}
