#ifndef DQM_ARCHIVE_DQMARESULTCONVERTER_H
#define DQM_ARCHIVE_DQMARESULTCONVERTER_H
//--------------------------------------------------------------------------
// File and Version Information:
// 	$Id$
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------
#include <string>
#include <map>
#include <vector>
#include <mutex>

//----------------------
// Base Class Headers --
//----------------------
#include "is/inforeceiver.h"

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "dqm_archive/archiver/ResultQueue.h"
#include "dqm_core/Result.h"
#include "dqmf/is/Result.h"

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------
class IPCPartition;

//		---------------------
// 		-- Class Interface --
//		---------------------

namespace dqm_archive {

/// @addtogroup dqm_archive

/**
 *  @ingroup dqm_archive
 *
 *  Class which is responsible for retrieving OH histograms for
 *  DQM results. It receives results from a queue populated by
 *  DqmaResultReceiver, retrieves histograms if necessary, adds
 *  them to result and pushes result to output queue. It has to
 *  run in a separate thread.
 *
 *  This software was developed for the ATLAS project.  If you use all or
 *  part of it, please give an appropriate acknowledgment.
 *
 *  @version $Id$
 *
 *  @author Andy Salnikov
 */

class DqmaResultConverter {
public:

    typedef ResultQueue<std::shared_ptr<dqmf::is::Result>> IsResultQueue;
    typedef ResultQueue<std::shared_ptr<dqm_core::Result>> CoreResultQueue;

    // constructor
    DqmaResultConverter(const IPCPartition& partition,
            const std::shared_ptr<IsResultQueue>& inQueue,
            const std::shared_ptr<CoreResultQueue>& outQueue,
            std::mutex& rootMutex);

    // operator called by thread
    void operator()();

protected:

private:

    IPCPartition m_partition;
    std::shared_ptr<IsResultQueue> m_inQueue;
    std::shared_ptr<CoreResultQueue> m_outQueue;
    std::mutex& m_rootMutex;      ///< Mutex to synchronize access to ROOT
};

} // namespace dqm_archive

#endif // DQM_ARCHIVE_DQMARESULTCONVERTER_H
