#ifndef DQM_ARCHIVE_ILTSARCHIVER_H
#define DQM_ARCHIVE_ILTSARCHIVER_H

//-----------------
// C/C++ Headers --
//-----------------
#include <string>

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "ers/ers.h"

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//		---------------------
// 		-- Class Interface --
//		---------------------

namespace dqm_archive {

/**
 *  @class LTSIssues::Exception
 *  @brief Base class for the exception-type issues generated by LTS archiver classes.
 */
ERS_DECLARE_ISSUE (LTSIssues, Exception, , )

/// @addtogroup dqm_archive

/**
 * @ingroup dqm_archive
 *
 * @brief Interface for classes implementing Long-Term Storage archiving methods.
 *
 * This is rather simple interface as it have just a single method called
 * `store()` which takes the name of the file to save in the archive. This
 * method is called by DQM archiver to send archived data to external storage,
 * for example CoCa.
 */

class ILTSArchiver {
public:

    // Destructor
    virtual ~ILTSArchiver() = default;

    /**
     *  @brief Send one more file to storage
     *
     *  @param[in] path     Full path name of the file to be stored.
     *
     *  @except LTSIssues::Exception if archiving failed
     */
    virtual void store(const std::string& path) = 0;

};

} // namespace dqm_archive

#endif // DQM_ARCHIVE_ILTSARCHIVER_H
