#ifndef _DQM_DISPLAY_GUI_CANVAS_WIDGET_H_
#define _DQM_DISPLAY_GUI_CANVAS_WIDGET_H_

/* \file CanvasWidget Declares the dqm_display::gui::CanvasWidget class.
 * \author Serguei Kolos
 */

#include <QImage>
#include <QWidget>

class TCanvas;
class TPad;

/*! \namespace dqm_display
 *  This is a wrapping namespace for all public classes of the Data Quality Monitoring framework.
 */
namespace dqm_display {
    /*! \namespace gui
     *  This is a wrapping namespace for all public classes of the Data Quality Monitoring Display GUI.
     */
    namespace gui {
        
        /*! \class dqm_display::gui::CanvasWidget
         *  Class stands to output histogram on widget
         */
        class CanvasWidget : public QWidget {
	    Q_OBJECT

        public:
            CanvasWidget(QWidget* parent = 0, int width = 600, int height = 400);

            virtual ~CanvasWidget();

            QImage getImage(const QSize & size = QSize());
	    
            void resize(const QSize & size);

            void refresh();

            void reset();

            virtual void paintCanvas() { }

        protected:
            void mouseMoveEvent(QMouseEvent* );
            void mousePressEvent(QMouseEvent* );
            void mouseReleaseEvent(QMouseEvent* );
            void changeEvent(QEvent* );
            void paintEvent(QPaintEvent* );
            void resizeEvent(QResizeEvent* );
            void showEvent(QShowEvent* );
            
        protected:
            TCanvas*     m_canvas;
            QImage       m_image;
        };
}}

#endif
