#ifndef _DQM_DISPLAY_GUI_HISTOGRAM_H_
#define _DQM_DISPLAY_GUI_HISTOGRAM_H_

/* \file Histogram Declares the dqm_display::gui::Histogram class.
 * \author Yurij Ilchenko, Kevin Slagle. Serguei Kolos
 */

#include <ctime>

#include <QMetaType>

#include <QTimer>
#include <dqm_display/gui/CanvasWidget.h>
#include <string.h>

class TGraph;
class TMultiGraph;
class TNamed;
class TH1;
class TLegend;
class TObject;
class TCanvas;
class TPad;

namespace dqm_display {
namespace core {
  class Parameter;
  class TreeNode;
}
}

/*! \namespace dqm_display
 *  This is a wrapping namespace for all public classes of the Data Quality Monitoring framework.
 */
namespace dqm_display {
    /*! \namespace gui
     *  This is a wrapping namespace for all public classes of the Data Quality Monitoring Display GUI.
     */
    namespace gui {
        
        /*! \class dqm_display::gui::Histogram
         *  Class stands to output histogram on widget
         */
        class Histogram : public CanvasWidget {
            Q_OBJECT

            enum HistoType { TH1_TYPE, TH2_TYPE, TH3_TYPE, TGraph_TYPE, TGraph2D_TYPE, OTHER_TYPE };

        public:
            Histogram(QWidget* parent = 0);

            Histogram(  const std::string & drawOption,
			const std::string & canvasOption,
			bool scaleRef,
			const core::Parameter * parameter);

            ~Histogram();
            
            void setNode(const core::TreeNode * node);
            
            std::time_t time() const { return m_time; }

            bool isNull() const { return m_histogram == 0; }
	    
            void reset(TObject* histo, std::time_t time, TObject* refHisto, bool forceReset);

            QString getName();

            void paintCanvas();

        private:
	    void plotReference();
	    void plotHistogram(bool noTitle);
            void plotTime();
            bool setHistogram(TObject *histo, TNamed*& m_h, HistoType& m_t);
            void setPadsOptions(TPad * pad1, TPad * pad2);
            
        private:            
            TNamed*	 m_histogram;
            std::time_t  m_time;
            TNamed*	 m_reference;
            TObject*     m_reference_pointer;
            TH1*         m_clone_reference;
	    std::string  m_draw_option;
	    std::string	 m_canvas_option;
	    bool	 m_scale_ref;
	    TMultiGraph* m_multiGraph;
            TLegend*	 m_legend;
            HistoType	 m_type;
            HistoType	 m_ref_type;
            bool	 m_split_canvas;

            const core::Parameter * m_parameter;
        };
}}

#endif
