/*! \file DetailsViewDelegate.cpp Implements the DetailsViewDelegate class.
 * \author Serguei Kolos
 * \version 1.0
 */
#include <dqm_display/gui/DetailsViewDelegate.h>
#include <dqm_display/gui/Histogram.h>
#include <dqm_display/gui/Status.h>

#include <dqm_display/core/Parameter.h>

#include <QBitmap>
#include <QDebug>
#include <QPainter>
#include <QTextBlock>
#include <QTextDocument>
#include <QTextLayout>
#include <QTextLine>
#include <QTextOption>

namespace dqm_display { 
namespace gui {

DetailsViewDelegate::DetailsViewDelegate(int row_height, int padding, QObject *parent) 
  : IconsViewDelegate(QSize(row_height, row_height), padding, parent)
{ ; }

void 
DetailsViewDelegate::paint( QPainter * painter,
	const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
    painter->save();

    if(option.state & QStyle::State_Selected)
    {
        painter->fillRect(option.rect, QBrush(QColor(112, 153, 204)));
    }

    IconsViewDelegate::doPaint(painter, option, index);

    if ( index.column() != 0 ) {
	QSize size(option.rect.size() - QSize(2*m_padding, 2*m_padding));
	QTextDocument doc;
	doc.setHtml(qvariant_cast<QString>(index.data(Qt::DisplayRole)));
        
	int y = 0;        
        for ( QTextBlock b = doc.begin(); b != doc.end(); b = b.next() )
        {        
	    QTextOption op;
	    op.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);

            QTextLayout * layout = b.layout();            
            layout->setTextOption(op);
	    layout->beginLayout();
	    while (1) {
		QTextLine line = layout->createLine();

		if (!line.isValid())
		    break;

		line.setLineWidth(size.width());

		if (size.height() >= y + 2 * option.fontMetrics.lineSpacing()) {
		    line.draw(painter, QPoint(0, y));
                    y += option.fontMetrics.lineSpacing();
		}
		else {
		    QString lastLine("...");
		    painter->drawText(0, y + option.fontMetrics.ascent(), lastLine);
		    break;
		}
	    }
	    layout->endLayout();
	}
    }

    painter->restore();
}

QSize
DetailsViewDelegate::sizeHint( const QStyleOptionViewItem & option, const QModelIndex & ) const
{    
    return QSize(option.rect.width(), m_size_hint.height());
}

} } // namespace dqm_display::gui
