/////////////////////////////////////////////////////////////////////////////////////////
//      object.h
//
//      Implementation of the IPC proxy application
//
//      Sergei Kolos August 2007
//
//      description:
//              This file contains implementation of the IPC proxy object interface
//
/////////////////////////////////////////////////////////////////////////////////////////

#ifndef IPC_PROXY_OBJECT_H
#define IPC_PROXY_OBJECT_H

#include <ipc/objectbase.h>
#include <ipc/exceptions.h>
#include <boost/type_traits.hpp>

namespace ipc
{
// this wrapper class converts protected destructor of the class T
// to the public one of the class wrapper<T>
template<class T>
struct wrapper: public T
{
};
}

template<class I, class POA_I, template<class > class POA_I_tie>
class IPCProxyObject: public IPCProxyObjectBase,
		      public IPCObject<POA_I, ipc::multi_thread, ipc::transient>,
		      public POA_I_tie<ipc::wrapper<typename boost::remove_pointer<typename I::_ptr_type>::type>>
{
    typedef ipc::wrapper<typename boost::remove_pointer<typename I::_ptr_type>::type> TargetType;

    typename I::_var_type target_;
    typename I::_var_type self_;

public:
    IPCProxyObject(const char * ior)
    : POA_I_tie<TargetType>( createTarget(ior) )
    {
        target_ = this -> _tied_object();
	if ( CORBA::is_nil( target_ ) )
	{
	    ERS_DEBUG( 1, "Got nil reference to the '" << ior << "' remote object" );
	    throw CORBA::INV_OBJREF( 0, CORBA::COMPLETED_NO );
	}
	this->_is_owner(false);
        self_ = this -> _this();
    }

private:
    static TargetType* createTarget(const char * ior)
    {
	try {
	    typename I::_ptr_type ref = I::_narrow( IPCCore::stringToObject( ior ) );
            return reinterpret_cast<TargetType*>( ref );
	}
	catch( CORBA::SystemException & ex ) {
	    ERS_DEBUG( 1, "Cannot contact the '" << ior
		    << "' remote object, got '" << ex << "' exception" );
	    throw;
	}
    }

    CORBA::Boolean _non_existent()
    {
	return !__target_exists__();
    }

    CORBA::Boolean _is_equivalent( CORBA::Object_ptr other_object )
    {
	return target_ -> _is_equivalent( other_object );
    }

    CORBA::Object_ptr __reference__()
    {
	return self_;
    }

    bool __target_exists__()
    {
	try {
	    if ( CORBA::is_nil(target_) || target_ -> _non_existent() )
	    return false;
	}
	catch(...) {
	    return false;
	}

	return true;
    }

    void __destroy__()
    {
	this -> _destroy();
    }

    PortableServer::POA_ptr _default_POA() {
	return IPCObject<POA_I,ipc::multi_thread,ipc::transient>::_default_POA();
    }
};

#endif
