#ifndef MDA_RUNPARAMETERS_H
#define MDA_RUNPARAMETERS_H
//--------------------------------------------------------------------------
// File and Version Information:
//      $Id$
//------------------------------------------------------------------------

//-----------------
// C/C++ Headers --
//-----------------

//----------------------
// Base Class Headers --
//----------------------

//-------------------------------
// Collaborating Class Headers --
//-------------------------------
#include "ers/ers.h"
#include "is/infodictionary.h"

//------------------------------------
// Collaborating Class Declarations --
//------------------------------------

//              ---------------------
//              -- Class Interface --
//              ---------------------

namespace daq {
namespace mda {

    /**
     *  @class RunParametersIssues::Exception
     *  @brief Base for exception classes generated by MDAConfig class.
     */
    ERS_DECLARE_ISSUE (RunParametersIssues, Exception, , )
            
    /**
     *  @class RunParametersIssues::EnvVarNotDefined
     *  @brief Issue generated when environment does not have required variable defined.
     */
    ERS_DECLARE_ISSUE_BASE (RunParametersIssues, EnvVarNotDefined, Exception,
            "environment variable " << envvar << " is not defined", ,
            ((std::string) envvar))

    /**
     *  @class RunParametersIssues::ISexception
     *  @brief Issue generated when environment does not have required variable defined.
     */
    ERS_DECLARE_ISSUE_BASE (RunParametersIssues, ISexception, Exception,
            "IS exception", , )

/// @addtogroup mda_archive

/**
 *  @ingroup mda_archive
 *
 *  @brief Class providing read-write access to IS server containing run parameters.
 *
 *  This software was developed for the ATLAS project.  If you use all or 
 *  part of it, please give an appropriate acknowledgment.
 *
 *  @version $Id$
 *
 *  @author Federico Zema
 *  @author Andy Salnikov
 */
class RunParameters {
public:
        
    /**
     *  @brief Default constructor
     *  
     *  @throw RunParametersIssues::EnvVarNotDefined Thrown if TDAQ_PARTITION
     *          envvar is not defined
     */
    RunParameters() ;

    // destructor
    ~RunParameters() throw ();
    
    /**
     *  @brief  Return run number from IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    unsigned getRunNumber ();
    
    /**
     *  @brief  Set run number in IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    void setRunNumber (unsigned runNumber);

    /**
     *  @brief  Return run type from IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    std::string getRunType ();
    
    /**
     *  @brief  Set run type in IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    void setRunType (const std::string& runType);

    /**
     *  @brief  Return LB type from IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    unsigned getLBNumber ();

    /**
     *  @brief  Set LB number in IS.
     *  
     *  @throw RunParametersIssues::ISexception
     */
    void setLBNumber (unsigned lbNumber);

private:
    
    ISInfoDictionary m_dict;
    
};

} // namespace mda
} // namespace daq

#endif // MDA_RUNPARAMETERS_H
