#include <functional>

#include <ers/ers.h>

#include "mts/ThreadPoolNQ.h"

using namespace daq::mts ;

// main working cycle of a thread
void WorkerThread::operator()()
{
ERS_DEBUG(1, "Thread " << std::this_thread::get_id() << " starting") ;  

    while(true)
    {
    if ( pool.stop )
		{
ERS_DEBUG(1, "Stopping thread " << std::this_thread::get_id()) ; 
		return ;
		}

	if ( pool.total_counter - pool.busy_counter < pool.total_counter * 0.3 && pool.expandable )
		{
		// size_t new_size = pool.increase_pool() ;
		// ERS_LOG("MTS thread pool is quite busy, total threads now: " << pool.total_counter) ;
		}

    try {
ERS_DEBUG(4, "Thread " << std::this_thread::get_id() << " seeking for a job") ;  
	Task t = pool.task_provider.get_next_task() ; 	// get next task or TaskProvider::NoTasks
	pool.busy_counter++ ;
	t() ;		// execute the task
	pool.busy_counter-- ;
ERS_DEBUG(4, "Thread " << std::this_thread::get_id() << " executed a job") ;
		}
    catch ( const TaskProvider::NoTasks& )
		{
	// TODO block on condition or just std::this_thread::yield() or better do micro sleep?
ERS_DEBUG(3, "Thread " << std::this_thread::get_id() << " idle waiting for a job") ;
	std::unique_lock<std::mutex> lock(pool.condition_mutex) ;
    	pool.ready_counter++ ;
    	pool.condition.wait(lock) ;
	// std::this_thread::yield() ; // ALT: 
	// std::this_thread::sleep_for(std::chrono::microseconds(500)) ; // in practice this consumes ~200% CPU in idle
		}
    catch ( std::exception& ex )
		{
		ERS_LOG("Unexpected exception from task: " << ex.what() ) ;  
		pool.busy_counter-- ;
		}
    catch ( ... )
		{
		ERS_LOG("Unexpected exception from task! Consider a BUG " ) ;  
		pool.busy_counter-- ;
		}
    }   
}

// the constructor just launches some amount of workers and waits for them to start and reach ready point
ThreadPoolNQ::ThreadPoolNQ(size_t nthreads, TaskProvider& tp, bool exp)
    :   expandable(exp), task_provider(tp), ready_counter(0), total_counter(0), busy_counter(0), stop(false)
{
ERS_DEBUG(1, "Constructing ThreadPool") ;
initial_size = nthreads ;
for(size_t i = 0; i < nthreads; ++i)
	{ worker_threads.push_back(std::thread(WorkerThread(*this))) ; total_counter++ ; }

ERS_DEBUG(2, "Waiting for all threads to be ready to process tasks") ;
while( true ) { if ( ready_counter == nthreads ) { break ; } else { std::this_thread::yield() ; } ; } ;

ERS_DEBUG(1, "ThreadPoolNQ constructed and ready") ;
}
 
// the destructor joins all threads
ThreadPoolNQ::~ThreadPoolNQ()
{
ERS_DEBUG(1, "Destroying ThreadPool") ;

	// stop all threads
  stop = true;
  condition.notify_all();
 
	// join them when they finish
  for (size_t i = 0; i < worker_threads.size() ; ++i)
	  { worker_threads[i].join() ; }

ERS_DEBUG(1, "ThreadPool destroyed, there shall not be any more tasks around!") ;
}

size_t
ThreadPoolNQ::increase_pool()
{
for(size_t i = 0; i < initial_size; ++i)
	{ worker_threads.push_back(std::thread(WorkerThread(*this))) ; total_counter++ ; }
ERS_DEBUG(0, "ThreadPool size increased, now " << total_counter << " threads running") ;
return total_counter ;
}
