
template <class T>
void
oh::GraphDataI<T>::get_errors( std::vector<const double*>& output ) const {
    output.resize( T::errors.size() );
    for ( unsigned int i = 0; i < T::errors.size(); i++ ) {
	output[i] = &T::errors[i*T::entries];
    }
}

template <class T>
void
oh::GraphDataI<T>::set_errors(const std::vector<const double*>& input ) {
    T::errors.resize( input.size()*T::entries );
    for (unsigned int i = 0; i < input.size(); i++ ) {
	memcpy( &T::errors[i*T::entries], input[i], sizeof( double ) * T::entries );
    }
}

template <class T>
void
oh::GraphDataI<T>::set_error_style(typename T::ErrorStyle es) {
    T::errorStyle=es;
}

template <class T>
typename T::ErrorStyle
oh::GraphDataI<T>::get_error_style() const {
    return T::errorStyle;
}

template <class T>
void
oh::GraphDataI<T>::get_points(std::vector<const double*>& valueArrays) const {
    assert( T::points.size() >= (T::dimension+1)*T::entries );
    valueArrays.resize(T::dimension+1);
    for (int i=0;i<=T::dimension;i++) {
	valueArrays[i]=&T::points[i*T::entries];
    }
}

template <class T>
void
oh::GraphDataI<T>::get_data(std::vector<const double*>& data) const {
    data.resize( T::dimension + T::errors.size() + 1 );
    for (unsigned int i=0;i<=T::dimension;i++) {
	data[i]=&T::points[i*T::entries];
    }
    for (unsigned int i=0;i<T::errors.size();i++) {
	data[i+T::dimension+1]=&T::errors[i*T::entries];
    }
}

template <class T>
void
oh::GraphDataI<T>::set_points(const std::vector<const double*>& valueArrays) {
    assert( valueArrays.size() >= static_cast<unsigned>(T::dimension+1) );
    T::points.resize( (T::dimension+1)*T::entries );
    for (unsigned int i=0;i<T::dimension+1;i++) {
	memcpy(&T::points[i*T::entries], valueArrays[i], sizeof(double) * T::entries);
    }
}

