/*
 * oh::ObjectBase.h - interface to the base Object class in IS
 * Serguei Kolos, Aug 2006
 */

#ifndef OH_OBJECT_BASE_H
#define OH_OBJECT_BASE_H

#include <oh/core/Axis.h>

namespace oh
{
    template <class T>
    struct ObjectBase : public T
    {        
	ObjectBase( )
	{ ; }
       
	ObjectBase( const std::string & class_name )
          : T( class_name )
	{ ; }
       
        /** Set title */
	void set_title(const std::string& t);

	/** Get title */
	const std::string & get_title() const;

	//! Add new annotation
	void add_annotation( const std::string & key, const std::string & value );

	/** Set annotations */
	void set_annotations(const std::vector< std::pair< std::string, std::string > > & ann );

	/** get annotations */
	void get_annotations( std::vector< std::pair< std::string, std::string > > & ann ) const;

	//! Retrieve the number of entries
	unsigned int get_entries( ) const;

	//! Update the number of entries
	void set_entries( unsigned int entries );

	/**  Update title of axis */
	void set_axis_title( oh::Axis::ID axis, const std::string & label );

	/** Retrieve number of axes */
	unsigned int get_axes_number( ) const;
        
	/** Retrieve title of axis */
	const std::string & get_axis_title( oh::Axis::ID axis ) const;

	/** Get reference to the indices of axes labels */
	std::vector<int> & indices( oh::Axis::ID axis );

	/** Get const reference to the indices of axes labels */
	const std::vector<int> & get_indices( oh::Axis::ID axis ) const;

	/** Returns reference to the array of indices of axes labels.
	 *  \param axis Must be X, Y or Z.
	 *  \return Refernce to the vector of indices.
	 */
	std::vector<std::string> & labels( oh::Axis::ID axis );

	/** Get const reference to the labels of the axes */
	const std::vector<std::string> & get_labels( oh::Axis::ID axis ) const;

	/** Retrieve axis type */
	oh::Axis::Kind get_axis_type( oh::Axis::ID axis ) const;

	/** Retrieve the number of bins for the axis */
	int get_bin_count( oh::Axis::ID axis ) const;

	/** Retrieve variable axis partition
	 * This function may only be called if axis_type() returns VariableAxis.
	 * \param axis Must be X, Y or Z.
	 * \return A double std::vector which will be filled with the bin edges.*/
	const std::vector<double> & get_axis_range( oh::Axis::ID axis ) const;

	/** Retrieve a fixed axis partition
	 * This function may only be called if axis_type() returns FixedAxis.
	 * \param axis Must be X, Y or Z.
	 * \param low Lower edge of the first bin.
	 * \param width Bin width.*/
	void get_axis_range( oh::Axis::ID axis, double & low, double & binwidth ) const;

	/** Create a variable axis partition
	 * This function stores bins range for a particular axis and sets axis type to VariableAxis.
	 * \param axis Must be X, Y or Z.
	 * \param xbins A double std::vector of bin edges.*/
	void set_axis_range( oh::Axis::ID axis, const std::vector<double> & xbins );

	/** Create a variable axis partition
	 * This function stores bins range for a particular axis and sets axis type to VariableAxis.
	 * \param axis Must be X, Y or Z.
	 * \param edges A double arary of bin edges.
	 * \param size Size of the bin edges array.*/
	void set_axis_range( oh::Axis::ID axis, const double * edges, unsigned int size );

	/** Create a fixed axis partition
	 * This function sets axis type to FixedAxis.
	 * \param axis Must be X, Y or Z.
	 * \param low Lower edge of the first bin.
	 * \param width Bin width.
	 * \param nbins Number of bins. */
	void set_axis_range( oh::Axis::ID axis, double low, double binwidth, unsigned int nbins );
    };
}

#include <oh/core/ObjectBase.i>

#endif

