template <class T>
const std::string &
oh::ObjectBase<T>::get_title( ) const
{
    return T::title;
}

template <class T>
void
oh::ObjectBase<T>::set_title( const std::string & t )
{
    T::title = t;
}

// just the same as in HistogramData.i
template <typename T>
void 
oh::ObjectBase<T>::add_annotation( const std::string & key, const std::string & value )
{
    this->setAnnotation( key, value );
}

// just the same as in HistogramData.i
template <class T>
void
oh::ObjectBase<T>::get_annotations( std::vector< std::pair< std::string, std::string > > & ann ) const
{
    ann.reserve(this->annotations().size());
    for (auto a: this->annotations()) {
    	ann.push_back(std::make_pair(a.first, a.second.data()));
    }
}

// just the same as in HistogramData.i
template <class T>
void
oh::ObjectBase<T>::set_annotations( const std::vector< std::pair< std::string, std::string > > & ann )
{
    for (auto a: ann) {
        this->setAnnotation(a.first, a.second);
    }
}

/*!
    Return number of histogram entries.
    \return number of entries.
*/
template <typename T>
unsigned int
oh::ObjectBase<T>::get_entries( ) const
{
    return T::entries;
}

/*!
    Set number of histogram entries.
*/
template <typename T>
void
oh::ObjectBase<T>::set_entries( unsigned int ee ) {
    T::entries = ee;
}

/*!
    Returns number of axes descriptions. It not necessarely the same as
    histogram dimension. For example 1D histogram may have Y axis title set.
    \return unsigned int number of axes descriptions
*/
template <typename T>
unsigned int
oh::ObjectBase<T>::get_axes_number( ) const
{
    return T::axes.size();
}

template <class T>
const std::string &
oh::ObjectBase<T>::get_axis_title( oh::Axis::ID axis ) const
{
    assert( (unsigned int)axis < T::axes.size() );
    return T::axes[axis].title;
}

template <class T>
void
oh::ObjectBase<T>::set_axis_title( oh::Axis::ID axis, const std::string & title )
{
    if ( (unsigned int)axis >= T::axes.size() )
    	T::axes.resize( axis + 1 );
    T::axes[axis].title = title;
}

template <class T>
const std::vector<int> &
oh::ObjectBase<T>::get_indices( oh::Axis::ID axis ) const
{
    assert( oh::Axis::X <= axis && axis <= oh::Axis::Z );
    assert( (unsigned int)axis < T::axes.size() );
    return T::axes[axis].indices;
}

template <class T>
std::vector<int> &
oh::ObjectBase<T>::indices( oh::Axis::ID axis )
{
    assert( oh::Axis::X <= axis && axis <= oh::Axis::Z );

    if ( (unsigned int)axis >= T::axes.size() )
    	T::axes.resize( axis + 1 );
    return T::axes[axis].indices;
}

template <class T>
const std::vector<std::string> &
oh::ObjectBase<T>::get_labels( oh::Axis::ID axis ) const
{
    assert( oh::Axis::X <= axis && axis <= oh::Axis::Z );
    assert( (unsigned int)axis < T::axes.size() );
    return T::axes[axis].labels;
}

template <class T>
std::vector<std::string> &
oh::ObjectBase<T>::labels( oh::Axis::ID axis )
{
    assert( oh::Axis::X <= axis && axis <= oh::Axis::Z );
    
    if ( (unsigned int)axis >= T::axes.size() )
    	T::axes.resize( axis + 1 );
    return T::axes[axis].labels;
}

template <class T>
oh::Axis::Kind
oh::ObjectBase<T>::get_axis_type( oh::Axis::ID axis ) const
{
    if ( (unsigned int)axis >= T::axes.size() )
    	return oh::Axis::NoAxis;
    return T::axes[axis].kind;
}

template <class T>
int
oh::ObjectBase<T>::get_bin_count( oh::Axis::ID axis ) const
{
    if ( (unsigned int)axis < (unsigned int)T::dimension )
	return T::axes[axis].nbins;
    else
    	return -1;
}

template <class T>
const std::vector<double> &
oh::ObjectBase<T>::get_axis_range( oh::Axis::ID axis ) const
{
    assert( get_axis_type( axis ) == oh::Axis::Variable );    
    assert( (unsigned int)axis < T::axes.size() );
    return T::axes[axis].range;
}

template <class T>
void
oh::ObjectBase<T>::get_axis_range( oh::Axis::ID axis, double & low, double & width ) const
{
    assert( get_axis_type( axis ) == oh::Axis::Fixed );
    
    low   = T::axes[axis].range[ 0 ];
    width = T::axes[axis].range[ 1 ];
}

template <class T>
void
oh::ObjectBase<T>::set_axis_range( oh::Axis::ID axis, const std::vector<double> & xbins ) {
    set_axis_range( axis, &xbins[0], xbins.size() );
}

template <class T>
void
oh::ObjectBase<T>::set_axis_range( oh::Axis::ID axis, const double * edges, unsigned int size ) {
    if ( (unsigned int)axis >= T::axes.size() )
    	T::axes.resize( axis + 1 );
    
    T::axes[axis].kind = oh::Axis::Variable;
    
    //
    // We have vector of bin edges ,number of bins is ( size - 1 )
    //
    T::axes[axis].nbins = size - 1;
    T::axes[axis].range.resize( size );
    memcpy( &T::axes[axis].range[0], edges, sizeof( double ) * size );
}

template <class T>
void
oh::ObjectBase<T>::set_axis_range( oh::Axis::ID axis, double low, double width, unsigned int nbins ) {
    if ( (unsigned int)axis >= T::axes.size() )
    	T::axes.resize( axis + 1 );
    
    T::axes[axis].kind = oh::Axis::Fixed;
    
    T::axes[axis].nbins = nbins;
    T::axes[axis].range.resize( 2 );
    T::axes[axis].range[0] = low;
    T::axes[axis].range[1] = width;
}   

