/*
  FILE:        InfrastructureChecker.h
  AUTHOR:      Danilo Cimino [cimino@cli.di.unipi.it]
  DESCRIPTION: A Thread that periodically checks whether the IPCPartition 
               and the IS server are up and running.
  MODIFIED:    Roberto Agostino Vitillo [vitillo@cli.di.unipi.it] (Early 2008) -- Multiple server support added
*/

#ifndef INFRASTRUCTURECHECKER_H
#define INFRASTRUCTURECHECKER_H

#include <string>   
#include <boost/scoped_ptr.hpp>   

#include <unistd.h>
#include <ipc/alarm.h>
#include <ipc/partition.h>

namespace ohp {

  class CoreInterface;
  
  class InfrastructureChecker
  {
  public:
    /*
     * Set the list of connected servers
     * @param serversName list of connected servers
     */
    InfrastructureChecker(const IPCPartition & p, const std::vector<std::string>& servers);
    
  private:
    bool periodic_action();
        
  private:
    std::vector<std::string> 	m_servers; 	/** List of checked servers */
    std::vector<std::string> 	m_down_servers;	/** List of checked servers */
    IPCPartition		m_partition;
    bool			m_up_sent;
    bool			m_down_sent;
    IPCAlarm			m_alarm;
  };
}
#endif // INFRASTRUCTURECHECKER_H
