/*
  FILE:		ohp::FileManager.h
  AUTHOR:	Danilo Cimino [cimino@cli.di.unipi.it]
  Modified:	Serguei Kolos
*/

#include <ohp/FileManager.h>
#include <ohp/CoreInterface.h>

#include <TFile.h>
#include <TObject.h>

ohp::FileManager::FileManager(const std::vector<std::string>& v)
{  
    std::vector<std::string>::const_iterator it;
    std::vector<std::string>::const_iterator end = v.end();
    for(it = v.begin(); it != end; ++it) {
	addFile(*it);
    }
}

void 
ohp::FileManager::addFile(const std::string& file_name)
{
    boost::mutex::scoped_lock lock(m_mutex);
    boost::shared_ptr<TFile> file(TFile::Open(file_name.c_str()));
    if (file) {
	m_files.push_back(file);
    }
}

TObject* 
ohp::FileManager::readHistogram(const std::string& name)
{
    boost::mutex::scoped_lock lock(m_mutex);
  
    std::vector<boost::shared_ptr<TFile> >::iterator it;
    std::vector<boost::shared_ptr<TFile> >::iterator end = m_files.end();
    for(it = m_files.begin(); it != end; ++it) {
	TObject* h = (*it)->Get(name.c_str());
	if (h) {
	    ERS_DEBUG(2, "Found '"<<name<<"' histogram in the '"<<(*it)->GetName()<<"' file");
            return CoreInterface::cloneObject(h);
	}
    }
    ERS_DEBUG(2, "Histogram '"<<name<<"' is not found in existing files");
    return 0;
}
