/** ------------------------------------------------------------
 *  merger.cxx - main file of opmon_merger application
 *
 *  merges two or more files generated by opmon_monitor
 *  using the RCTFileList interface
 *
 *  author: Herbert Kaiser, Sep 2006
 *  ------------------------------------------------------------
 */

#include <string>
#include <vector>
#include <iostream>  
#include "RCTFileList.h"
#include "RCTFile.h"

using std::string;
using std::vector;
using std::cout;
using std::cerr;
using std::endl;


int main( int argc, char ** argv )
{
    bool wrongArgs=false;
    bool help=false;
    vector<string> list;
    char* outFile=NULL;
    bool force=false;
    for (int i=1;i<argc;i++) {
	if (strcmp("-h",argv[i])==0 || strcmp("--help",argv[i])==0) {
	    cout << "Command line tool to merge files produced by opmon_monitor"
	            " containing logged values of run control transition times of the controllers"
		    " in a monitored partition." << endl;
	    help=true;
	    break;
	}
	if (strcmp("-o",argv[i])==0) {
	    if (++i>=argc || outFile) {
		cout << "Bad parameters!" << endl;
		wrongArgs=true;
		break;
	    } else
		outFile=argv[i];
	    continue;
	}
	if (strcmp("-f",argv[i])==0) {
	    force=true;
	    continue;
	}

	list.push_back(argv[i]);
    }
    
    if (help || wrongArgs || argc<2 || !outFile || list.size()<1 ) {
	if ( (!outFile) && (!help) )
	    cout << "No output file specified!" << endl;
	if ( (list.size()<1) && (!help) )
	    cout << "No input files specified!" << endl;
	cout << "\nUsage: opmon_merger [-f] -o output.root input1.root input2.root input3.root ...\n"
		"The file name after '-o' is the name for the output file.\n"
		"Use '-f' to force the merging even if the output file already exists or"
		" the files come from different partitions.\n"
		"The given files must be ROOT files produced by opmon_monitor." << endl;
	return 1;
    }
    
    try {
	RCTFileList fileList(list);
	fileList.mergeFiles(outFile,force);
    } catch (RCTFileException) {
	cerr << "ERROR: Problems opening the files. Canceled." << endl;
	return 1;
    }

    return 0;
}

