#include <is/infoiterator.h>

#include "application_exceptions.h"
#include "server_monitoring.h"

std::string pbeast::srv::Monitoring::s_is_name_prefix = "RunParams.pbeast-server.UserMon.";
std::string pbeast::srv::Monitoring::s_is_corba_name = "SUM_CORBA";
std::string pbeast::srv::Monitoring::s_is_rest_name = "SUM_REST";
std::string pbeast::srv::Monitoring::s_is_api_name = "SUM_API";


pbeast::srv::UserStat::UserStat() :
  m_num_of_requests(0),
  m_size_of_data(0),
  m_max_size_of_data(0),
  m_processing_time(0),
  m_max_processing_time(0),
  m_last_non_null(m_obj.time().c_time())
{
  m_obj.num_of_requests = 0;
  m_obj.average_size_of_data_per_inteval = 0;
  m_obj.max_size_of_data_per_inteval = 0;
  m_obj.average_processing_time_per_inteval = 0;
  m_obj.max_processing_time_per_inteval = 0;
  m_obj.total_num_of_requests = 0;
  m_obj.total_size_of_data = 0;
  m_obj.max_size_of_data = 0;
  m_obj.total_processing_time = 0;
  m_obj.max_processing_time = 0;
}

pbeast::srv::UserStat::UserStat(const PBeastServerUserMon& obj) :
  m_num_of_requests(0),
  m_size_of_data(0),
  m_max_size_of_data(0),
  m_processing_time(0),
  m_max_processing_time(0),
  m_obj(obj),
  m_last_non_null(m_obj.time().c_time())
{
}

pbeast::srv::Monitoring::Monitoring() :
  m_api(),
  m_rest(),
  m_corba()
{
  // read backup

  try
    {
      IPCPartition partition;
      ISInfoIterator it(partition, "RunParams", PBeastServerUserMon::type());

      while (it())
        {
          PBeastServerUserMon obj;
          it.value(obj);

          std::string name = it.name();
          if (name.find(s_is_name_prefix) == 0)
            {
              name.erase(0, s_is_name_prefix.length());

              ERS_LOG("restore object \"" << it.name() << '\"');

              if (name == s_is_api_name)
                m_api.m_obj = obj;
              else if (name == s_is_rest_name)
                m_rest.m_obj = obj;
              else if (name == s_is_corba_name)
               m_corba.m_obj = obj;
              else
                m_users.emplace(name, new UserStat(obj));
            }
        }
    }
  catch (ers::Issue& ex)
    {
      throw daq::pbeast::MonitoringPublicationError(ERS_HERE, "iterate", "of PBeastServerUserMon class", ex);
    }
}

std::string
pbeast::srv::UserStat::to_string(const char * prefix) const
{
  std::ostringstream s;

  if(m_obj.total_num_of_requests)
    s << '\n'
      << prefix << "number of requests          : " << m_obj.total_num_of_requests << '\n'
      << prefix << "total size of data          : " << m_obj.total_size_of_data << '\n'
      << prefix << "max size of data            : " << m_obj.max_size_of_data << '\n'
      << prefix << "total processing time       : " << m_obj.total_processing_time << '\n'
      << prefix << "max request processing time : " << m_obj.max_processing_time;
  else
    s << " (null)";

  return s.str();
}


void
pbeast::srv::Monitoring::publish_calls_stats(UserStat& stat, const std::string& name, uint64_t publication_interval, bool obvilion_flag)
{
  uint64_t num_of_requests = stat.m_num_of_requests.exchange(0);
  uint64_t size_of_data = stat.m_size_of_data.exchange(0);
  uint64_t max_size_of_data = stat.m_max_size_of_data.exchange(0);
  uint64_t processing_time = stat.m_processing_time.exchange(0);
  uint64_t max_processing_time = stat.m_max_processing_time.exchange(0);

  bool need_to_publish = false;

  if(num_of_requests)
    {
      stat.m_obj.num_of_requests = average_rate(num_of_requests, publication_interval);
      stat.m_obj.average_size_of_data_per_inteval = average_size(size_of_data, num_of_requests);
      stat.m_obj.max_size_of_data_per_inteval = max_size_of_data;
      stat.m_obj.average_processing_time_per_inteval = average_size(processing_time, num_of_requests);
      stat.m_obj.max_processing_time_per_inteval = max_processing_time;

      stat.m_obj.total_num_of_requests += num_of_requests;
      stat.m_obj.total_size_of_data += size_of_data;
      stat.m_obj.max_size_of_data = std::max(stat.m_obj.max_size_of_data, max_size_of_data);
      stat.m_obj.total_processing_time += processing_time;
      stat.m_obj.max_processing_time = std::max(stat.m_obj.max_processing_time, max_processing_time);

      need_to_publish = true;
    }
  else if(stat.m_obj.num_of_requests != 0)
    {
      stat.m_obj.num_of_requests = 0;
      stat.m_obj.average_size_of_data_per_inteval = 0;
      stat.m_obj.max_size_of_data_per_inteval = 0;
      stat.m_obj.average_processing_time_per_inteval = 0;
      stat.m_obj.max_processing_time_per_inteval = 0;
   }

  const std::string full_name(s_is_name_prefix + name);


  if (need_to_publish == false && obvilion_flag)
    {
      time_t now;
      time(&now);

      ERS_DEBUG(0, full_name << " was published last time " << (now - stat.m_obj.time().c_time()) << " seconds ago");

      if((now - stat.m_last_non_null) > put_user_info_into_oblivion)
        {
          try
            {
              ERS_DEBUG(0, "put into oblivion monitoring object " << full_name);
              m_is_info_dict.remove(full_name);
            }
          catch (std::exception& ex)
            {
              ers::error(daq::pbeast::MonitoringPublicationError(ERS_HERE, "remove", full_name));
            }

          return;
        }
    }

  try
    {
      ERS_DEBUG(1, "publish calls stats for " << full_name);
      m_is_info_dict.checkin(full_name, stat.m_obj);
    }
  catch (std::exception& ex)
    {
      ers::error(daq::pbeast::MonitoringPublicationError(ERS_HERE, "publish", name));
    }

  if (need_to_publish)
    stat.m_last_non_null = stat.m_obj.time().c_time();
}


pbeast::srv::Monitoring::~Monitoring()
{
  publish_calls_stats(true);

  ERS_LOG("API requests:" << m_api.to_string());
  ERS_LOG("REST requests:" << m_rest.to_string());
  ERS_LOG("CORBA requests:" << m_corba.to_string());

  for (const auto& x : m_users)
    ERS_LOG(x.first << " user requests:" << x.second->to_string());
}

void
pbeast::srv::Monitoring::publish_calls_stats(bool obvilion_flag)
{
  static uint64_t counter(0);
  counter++;

  publish_calls_stats(m_api, s_is_api_name, mon_sleep_interval, false);
  publish_calls_stats(m_rest, s_is_rest_name, mon_sleep_interval, false);
  publish_calls_stats(m_corba, s_is_corba_name, mon_sleep_interval, false);

  if (!(counter % (user_sleep_inteval / mon_sleep_interval)) || obvilion_flag)
    for (const auto& x : m_users)
      publish_calls_stats(*x.second, x.first, user_sleep_inteval, obvilion_flag);
}

std::shared_ptr<pbeast::srv::UserStat>
pbeast::srv::Monitoring::get_user_stat(const std::string& name)
{
    {
      std::shared_lock lock(m_users_mutex);

      auto it = m_users.find(name);

      if (it != m_users.end())
        return it->second;
    }

  std::unique_lock lock(m_users_mutex);

  auto it = m_users.find(name);

  if (it != m_users.end())
    return it->second;
  else
    return m_users.emplace(name, new UserStat()).first->second;
}
