define USER = ATLAS_PVSSAFP -- name of the owner account
define VERSION = 5.1        -- schema version

-- Create attributes table
CREATE TABLE PVSS2COOL_DB_ATTRIBUTES (
	DB_ATTRIBUTE_NAME   VARCHAR2(255),
	DB_ATTRIBUTE_VALUE  VARCHAR2(4000)
);

ALTER TABLE PVSS2COOL_DB_ATTRIBUTES ADD (
	CONSTRAINT PVSS2COOL_DB_ATTRIBUTES_PK PRIMARY KEY (DB_ATTRIBUTE_NAME));

-- Create Folders table
CREATE TABLE PVSS2COOL_FOLDERS_R2 (
	FOLDERID          NUMBER(20) NOT NULL,
	FOLDERNAME        VARCHAR2(4000) NOT NULL,
	FOLDERDESC        VARCHAR2(4000),
	FOLDERGROUP       NUMBER(10) DEFAULT 0 NOT NULL,
	GROUPTABLE        VARCHAR2(255),
	DPTYPENAME        VARCHAR2(255),
	DPSUBTYPENAME     VARCHAR2(255),
	DPTINFO           NUMBER(10),
	USEALIAS          NUMBER(1),
	USELIST           NUMBER(1),
	DPPATTERN         VARCHAR2(255),
	CHANNELIDTYPE     NUMBER(10),
	CHANNELIDINFO     VARCHAR2(255),
	LASTUPDATEBEFORE  VARCHAR2(25),
	LASTUPDATEAFTER   VARCHAR2(25),
	MECHANISM         NUMBER(10),
	UPDATEINTERVAL    NUMBER(10),
	INTERVALCOUNT     NUMBER(10),
	STATUSREQ         NUMBER(10),
	STATUS            NUMBER(10),
	LISTCHANGE        NUMBER(1),
	ALIASDPE	  VARCHAR2(255),
	TIMEPRECISION	  VARCHAR2(63)
);

ALTER TABLE PVSS2COOL_FOLDERS_R2 ADD (
	CONSTRAINT PVSS2COOL_FOLDERS_R2_PK PRIMARY KEY (FOLDERID),
	UNIQUE (FOLDERNAME)
);

-- Create Fields table
CREATE TABLE PVSS2COOL_FIELDS (
  FOLDERID       NUMBER(20) NOT NULL,
  POSITION       NUMBER(10) NOT NULL,
  DPELEMENTNAME  VARCHAR2(1000),
  FIELDNAME      VARCHAR2(35),
  NEWFIELDNAME   VARCHAR2(35),
  COOLFIELDTYPE  VARCHAR2(50),
  PVSSFIELDTYPE  VARCHAR2(50),
  ISID           NUMBER(1),
  ISDYN          NUMBER(1),
  STATUS         NUMBER(10),
  DEADBAND	 BINARY_DOUBLE,
  TIMEOUT	 NUMBER(10),
  CHNAME	 VARCHAR2(127)
);

ALTER TABLE PVSS2COOL_FIELDS ADD (
	CONSTRAINT PVSS2COOL_FIELDS_PK PRIMARY KEY (FOLDERID, POSITION, CHNAME),
	CONSTRAINT PVSS2COOL_FIELDS_FID_FK FOREIGN KEY (FOLDERID) REFERENCES PVSS2COOL_FOLDERS_R2 (FOLDERID));

-- Create List table
CREATE TABLE PVSS2COOL_LISTS (
  FOLDERID       NUMBER(20) NOT NULL,
  POSITION       NUMBER(10) NOT NULL,
  DPNAME	 VARCHAR2(1000),
  NEWDPNAME 	 VARCHAR2(1000),
  CHANNELID	 NUMBER(10),
  CHANNELNAME	 VARCHAR2(255),
  DESCRIPTION	 VARCHAR2(255),
  STATUS	 NUMBER(10)
);

ALTER TABLE PVSS2COOL_LISTS ADD (
	CONSTRAINT PVSS2COOL_LISTS_PK PRIMARY KEY (FOLDERID, POSITION),
	CONSTRAINT PVSS2COOL_LISTS_FID_FK FOREIGN KEY (FOLDERID) REFERENCES PVSS2COOL_FOLDERS_R2 (FOLDERID));

-- Create sequence for folder IDs
CREATE SEQUENCE PVSS2COOL_FOLDERS_R2_SEQ NOCACHE START WITH 1;

-- Insert schema version information
INSERT INTO PVSS2COOL_DB_ATTRIBUTES (DB_ATTRIBUTE_NAME, DB_ATTRIBUTE_VALUE)
VALUES ('SCHEMA_VERSION', '&VERSION');

-- Output grant statements required
select 'GRANT SELECT, INSERT, UPDATE, DELETE ON ' || object_name || ' TO &USER._W;'
  from ALL_OBJECTS
  where object_type in ('TABLE','VIEW')
    and owner = '&USER'
    and object_name like 'PVSS2COOL%'
union
select 'GRANT SELECT ON ' || object_name || ' TO &USER._W;'
  from ALL_OBJECTS
  where object_type in ('SEQUENCE')
    and owner = '&USER'
    and object_name like 'PVSS2COOL%'
union
select 'GRANT EXECUTE ON ' || object_name || ' TO &USER._W;'
  from ALL_OBJECTS
  where object_type in ('TYPE','PROCEDURE','PACKAGE','FUNCTION')
    and owner = '&USER'
    and object_name like 'PVSS2COOL%';

-- then execute generated GRANTs,
-- then 
COMMIT;
