#include <iostream>
#include <sstream>
#include <string>

#include "src/common_utils.h"

static void
ref2stream(std::ostringstream& s, CORBA::Long recursion_depth, const rdb::RDBNameList& cnames)
{
  if(recursion_depth) {
    s << '@' << recursion_depth;

    if(cnames.length()) {
      s << '@';
      for(unsigned long idx = 0; idx < cnames.length(); idx++) {
        if(idx) s << ':';
	s << cnames[idx];
      }
    }
  }
}

std::string
rdb::query_params_to_string(const char * class_name, const char * query, CORBA::Long recursion_depth, const rdb::RDBNameList& cnames)
{
  std::ostringstream s;
  s << class_name << '@' << query;
  
  ref2stream(s, recursion_depth, cnames);

  return s.str();
}

std::string
rdb::object_id_params_to_string(const char * class_name, bool look_in_sclasses, const char * objectid, CORBA::Long recursion_depth, const rdb::RDBNameList& cnames)
{
  std::ostringstream s;
  s << class_name;

  if(look_in_sclasses == false) {
    s << "@N";
  }

  s << '@' << objectid;

  ref2stream(s, recursion_depth, cnames);

  return s.str();
}

rdb::RDBObject *
rdb::make_rdb_object(const std::string& class_name, const char * object_id)
{
  rdb::RDBObject * object = new rdb::RDBObject();
  object->classid = CORBA::string_dup(class_name.c_str());
  object->name    = CORBA::string_dup(object_id);
  return object;
}

void
rdb::destroy(rdb::ObjectIDCache *& qc)
{
  for(rdb::ObjectIDCache::iterator i = qc->begin(); i != qc->end(); ++i) {
    delete i->second.second;
  }

  delete qc;
  qc = 0;
}

void
rdb::destroy(rdb::QueryCache *& qc)
{
  for(rdb::QueryCache::iterator i = qc->begin(); i != qc->end(); ++i) {
    delete i->second.first;
    delete i->second.second;
  }

  delete qc;
  qc = 0;
}

// FIXME 2020-07-30 copied form OksKernel constructor
// create OKS method instead of this
void
rdb::parse_config_version (const char *version, std::string &param, std::string &val)
{
  if (version)
    {
      if (*version)
        {
          if (const char *value = strchr (version, ':'))
            {
              param.assign (version, value - version);
              val.assign (value + 1);
            }
          else
            {
              std::ostringstream text;
              text << "bad OKS repository version value \"" << version << "\" expecting parameter:value format";
              throw std::runtime_error (text.str ().c_str ());
            }
        }
    }
}
