#include <iostream>
#include <ipc/partition.h>

#include <config/Errors.h>

#include <rdb/errors.h>
#include <rdb/rdb_info.h>

#include "rdbconfig/rdb_server.h"

#include "report_exceptions.h"

namespace rdbconfig {

  rdb_server::rdb_server(const std::string& partition_name, const std::string& server_name) :
    m_partition   (partition_name.empty() ? new IPCPartition() : new IPCPartition(partition_name.c_str())),
    m_server_name (server_name),
    m_inited      (false),
    m_session     (0),
    m_ping_cb     (0)
  {
    lookup();
  }

  rdb_server::~rdb_server()
  {
    clean();
  }

  bool
  rdb_server::is_valid() const
  {
    return ( m_partition != 0 && ( m_is_writer ? !CORBA::is_nil(m_write_cursor) : !CORBA::is_nil(m_read_cursor) ) );
  }

  void
  rdb_server::clean()
  {
    if(m_inited && m_is_writer) {
      try {
        m_write_cursor->close_session( m_session, m_proc_info );
      }
      catch ( rdb::CannotProceed & ex ) {
        ers::error( daq::config::Generic( ERS_HERE, rdbconfig::mk_rdb_server_exception_text( "rdb_server::clean()" , "close_session" , ex.text ).c_str() ) );
      }
      catch ( const CORBA::SystemException & ex ) {
        ers::error( daq::config::Generic( ERS_HERE, rdbconfig::mk_corba_exception_text( "rdb_server::clean()" , "close_session" , ex ).c_str() ) );
      }

      m_ping_cb->_destroy();
      m_ping_cb = 0;
    }

    delete m_partition;
    m_partition = 0;

    m_inited = false;
  }

  void
  rdb_server::lookup()
  {
    if(!m_inited) {
      try {
        m_read_cursor = m_partition->lookup<rdb::cursor>(m_server_name.c_str());
        m_is_writer = false;
      }
      catch( ers::Issue& ex1 ) {
        try {
          m_write_cursor = m_partition->lookup<rdb::writer>(m_server_name.c_str());
          m_is_writer = true;
          rdb::ProcessInfoHelper::set(m_proc_info);
          m_ping_cb = new RDBWriterPingCB();
          m_session = m_write_cursor->open_session( m_proc_info, m_ping_cb->_this() );
          m_ping_cb->set_session_id(m_session);
        }
        catch ( CORBA::SystemException & ex ) {
          throw;  // to be caught by caller
        }
        catch( ers::Issue& ex2 ) {
          throw daq::rdb::LookupFailed(ERS_HERE, m_server_name.c_str(), m_partition->name().c_str(), ex2);
	}
      }

      m_inited = true;
    }

    else {
      if(m_is_writer == false) {
        try {
          m_read_cursor = m_partition->lookup<rdb::cursor>(m_server_name.c_str());
        }
        catch( ers::Issue& ex ) {
          throw daq::rdb::LookupFailed(ERS_HERE, m_server_name.c_str(), m_partition->name().c_str(), ex);
	}
      }
      else {
        try {
          m_write_cursor = m_partition->lookup<rdb::writer>(m_server_name.c_str());
        }
        catch( ers::Issue& ex ) {
          throw daq::rdb::LookupFailed(ERS_HERE, m_server_name.c_str(), m_partition->name().c_str(), ex);
	}
      }
    }
  }

}
