#ifndef _rdbconfig_report_exceptions_
#define _rdbconfig_report_exceptions_

#include <ipc/exceptions.h>

#include <ers/ers.h>

namespace rdbconfig {
  std::string mk_no_loaded_db(const char * config_method);
  std::string mk_no_server_exception_text(const char * config_method, const char * rdb_method);
  std::string mk_rdb_server_exception_text(const char * config_method, const char * rdb_method, const char * reason);
  std::string mk_session_exception_text(const char * config_method, const char * rdb_method, int64_t session);
  std::string mk_wrong_server_exception_text(const char * config_method, const std::string& server_name);
  std::string mk_corba_exception_text(const char * config_method, const char * rdb_method, const CORBA::SystemException& ex);
}

#define RDB_SERVER_METHOD( CODE , WHERE , WHAT )                                                                              \
{                                                                                                                             \
  try {                                                                                                                       \
    if(!m_server || !m_server->partition()) {                                                                                 \
      throw ( daq::config::Generic( ERS_HERE, rdbconfig::mk_no_loaded_db( WHERE ).c_str() ) );                                \
    }                                                                                                                         \
    try {                                                                                                                     \
      m_server->lookup();                                                                                                     \
    }                                                                                                                         \
    catch(daq::rdb::LookupFailed & ex) {                                                                                      \
      throw ( daq::config::Generic( ERS_HERE, rdbconfig::mk_no_server_exception_text( WHERE , WHAT ).c_str() ) );             \
    }                                                                                                                         \
    CODE ;                                                                                                                    \
  }                                                                                                                           \
  catch ( const rdb::SessionNotFound & ex ) {                                                                                 \
    throw ( daq::config::Generic( ERS_HERE, rdbconfig::mk_session_exception_text( WHERE , WHAT , ex.session_id ).c_str() ) ); \
  }                                                                                                                           \
  catch ( const rdb::CannotProceed & ex ) {                                                                                   \
    throw ( daq::config::Generic( ERS_HERE, rdbconfig::mk_rdb_server_exception_text( WHERE , WHAT , ex.text ).c_str() ) );    \
  }                                                                                                                           \
  catch ( const rdb::NotFound & ex ) {                                                                                        \
    const char * type = (                                                                                                     \
      (ex.why == rdb::object_not_exist) ? "object" :                                                                          \
      (ex.why == rdb::class_not_exist ? "class" :                                                                             \
      (ex.why == rdb::attribute_not_exist ? "attribute" :                                                                     \
      "relationship"))                                                                                                        \
    );                                                                                                                        \
    throw ( daq::config::NotFound( ERS_HERE, type, ex.item ) );                                                               \
  }                                                                                                                           \
  catch ( const CORBA::SystemException & ex ) {                                                                               \
    throw ( daq::config::Generic( ERS_HERE, rdbconfig::mk_corba_exception_text( WHERE , WHAT , ex ).c_str() ) );              \
  }                                                                                                                           \
}

#endif
