/*
 * ROBFragmentProcessor.h
 *
 *  Created on: May 29, 2019
 *      Author: kolos
 */

#ifndef SWROD_ROBFRAGMENTPROCESSOR_H_
#define SWROD_ROBFRAGMENTPROCESSOR_H_

#include <memory>

#include <is/infodynany.h>

#include <swrod/ROBFragmentConsumerBase.h>
#include <swrod/CustomProcessor.h>

namespace swrod {
    class Core;

    class ProcessorPool {
        protected:
            ProcessorPool(const boost::property_tree::ptree & config, const Core& core);

        protected:
            std::vector<std::unique_ptr<CustomProcessor>> m_processors;
    };

    class ROBFragmentProcessor : public ProcessorPool,
                                 public ROBFragmentConsumerBase {
    public:
        ROBFragmentProcessor(const boost::property_tree::ptree & config, const Core& core);

        void linkDisabled(const InputLinkId & link_id) {
            for_each(m_processors.begin(), m_processors.end(),
                    [&link_id](auto & w){ w->linkDisabled(link_id); });
        }

        void linkEnabled(const InputLinkId & link_id) {
            for_each(m_processors.begin(), m_processors.end(),
                    [&link_id](auto & w){ w->linkEnabled(link_id); });
        }

        void runStarted(const RunParams& run_params) override {
            for_each(m_processors.begin(), m_processors.end(),
                    [&run_params](auto & w){ w->runStarted(run_params); });
            ROBFragmentConsumerBase::runStarted(run_params);
        }

        void runStopped() override {
            ROBFragmentConsumerBase::runStopped();
            for_each(m_processors.begin(), m_processors.end(),
                    [](auto & w){ w->runStopped(); });
        }

        void userCommand(const daq::rc::UserCmd & cmd) override {
            for_each(m_processors.begin(), m_processors.end(),
                    [&cmd](auto & w){ w->userCommand(cmd); });
        }

        ISInfo * getStatistics() override;

    private:
        ISInfoDynAny m_statistics;
    };
}

#endif /* SWROD_ROBFRAGMENTPROCESSOR_H_ */
