/*
 * ClearSession.h
 *
 *  Created on: Jan 26, 2018
 *      Author: kolos
 */

#ifndef SWROD_HLT_CLEARSESSION_H_
#define SWROD_HLT_CLEARSESSION_H_

#include <assert.h>
#include <cstdint>

#include <memory>
#include <queue>
#include <vector>

#include "ClearMessage.h"
#include "Session.h"

namespace swrod {
    namespace hlt {

        class ClearSession {
        public:

            ClearSession(const std::vector<uint32_t> & rob_ids, uint32_t clear_group_size = 100) :
                    m_rob_ids(rob_ids), m_clear_group_size(clear_group_size) {
                ;
            }

            void addSession(std::shared_ptr<Session> session) {
                m_sessions.push_back(session);
            }

            uint32_t getClearsCounter() const {
                return m_clears_counter;
            }

            void recordL1ID(uint32_t l1id) {
            }

            void skipL1ID(uint32_t l1id) {
                std::unique_lock<std::mutex> lock(m_mutex);
                m_skipped_l1ids.push(l1id);
            }

            void receivedL1ID(uint32_t l1id) {
                std::unique_lock<std::mutex> lock(m_mutex);
                m_clear_l1ids.push_back(l1id);
                if (m_clear_l1ids.size() == m_clear_group_size) {
                    sendClears();
                }
                while (!m_skipped_l1ids.empty()) {
                    uint32_t skipped_l1id = m_skipped_l1ids.front();
                    if (skipped_l1id < l1id || (((l1id>>24) <= 1) && ((skipped_l1id>>24) >= 0xfe))) {
                        m_skipped_l1ids.pop();
                        m_clear_l1ids.push_back(skipped_l1id);
                        if (m_clear_l1ids.size() == m_clear_group_size) {
                            sendClears();
                        }
                    } else {
                        break;
                    }
                }
            }

            void adjustRates() {
                static const size_t threshold = 10000;
                static const size_t timeout = 10000;
                size_t counter = 0;
                while (m_skipped_l1ids.size() > threshold && counter++ < timeout) {
                    usleep(10);
                }
            }

        private:
            void sendClears() {
                std::unique_ptr<ClearMessage> message(
                        new ClearMessage(m_transaction_id, m_clear_l1ids));
                m_sessions[m_transaction_id++ % m_sessions.size()]->asyncSend(std::move(message));
                m_clears_counter += m_clear_l1ids.size();
                m_clear_l1ids.clear();
            }

        private:
            typedef std::queue<uint32_t> L1IDS;

            const std::vector<uint32_t> m_rob_ids;
            const uint32_t m_clear_group_size;
            uint32_t m_transaction_id = 0;
            uint32_t m_clears_counter = 0;
            std::mutex m_mutex;
            L1IDS m_skipped_l1ids;
            std::vector<uint32_t> m_clear_l1ids;
            std::vector<std::shared_ptr<Session>> m_sessions;
        };
    }
}

#endif /* SWROD_HLT_CLEARSESSION_H_ */
